################################################################################
# PastView 
# Copyright © 2018 Chevenet F. francois.chevenet@ird.fr
# MIVEGEC/IBC IRD/CNRS
################################################################################


################################################################################
# INTERFACE
################################################################################
# MAIN PANEL
proc MainPanel {} {
	global IDT A T S tcl_platform
	# INIT
	set S(version) "PastView v.46"
	set S(MapNumber) 0
	set S(tool) {}
	set S(SurfaceMax) 0
	set S(SurfaceMaxBubble) 6.0
	set S(probathreshold) 0.0
	set S(camenbdisplayreshold) 0.9
	set S(piechartdisplaynumber) hidden
	set S(subtreebgseuil) 0.95
	set S(subtreebgseuil2) 0.95
	set S(supportthreshold) 0.75
	set S(floodthreshold) 0.9
	set S(criteria-proba-threshold) 0.65
	set S(criteriacheck,pt) 1 
	set S(criteriacheck,sp) 0
	set S(criteriacheck,sz) 1
	set S(criteriacheck,df) 0
	set S(criteriacheck,ps) 1
	set S(criteriacheck,sl) 0
	set S(criteriacheck,szdf) 1
	set S(criteriacheck,dv) 0
	set S(criteriacheck,sldv) 0
	set S(sz-threshold) 2
	set S(df-threshold) 1
	set S(ps-threshold) 1
	set S(sl-threshold) ?
	set S(szdf-threshold) 1
	set S(dv-threshold) ?
	set S(sldv-threshold) ?
	set S(SizeCirclePropor) 0
	set S(lnodessansmap) {}
	set S(includingoutroupcheck) 0
	set S(font) {Arial 8 normal}
	set S(SurfaceMax) 10
	set S(MapList) {}
	set S(UpdateColorCible) BGtree
	set S(ColorSaturation) 120 
	set S(ColorValue) 120
	set S(ColorSaturation,LeafLabel) 50 
	set S(ColorValue,LeafLabel) 150
	set S(ColorSaturation,BGtree) 50 
	set S(ColorValue,BGtree) 150
	set S(ColorSaturation,FGtree) 100 
	set S(ColorValue,FGtree) 100
	set S(ColorSaturation,MaxLikpiechart) 50 
	set S(ColorValue,MaxLikpiechart) 200
	set S(ColorSaturation,PupkoAnnot) 50 
	set S(ColorValue,PupkoAnnot) 200
	set S(ColorSaturation,TipsAnnot) 100 
	set S(ColorValue,TipsAnnot) 250
	set S(ProbaCurveMode) 2
	set S(F81frequenceannotMODE) 2
	set S(cammenbdisplay) normal
	set S(treefgprobathreshold) 0.00
	set S(CanvasBackground) #999999
	set S(TransitionMapLayoutType) multi
	set S(TransitionMapLayoutView) circular
	set S(TransitionMapLayoutAnnotation) 1
	set S(TransitionMapLayoutransitionSize) 1
	set S(TransitionMapLayoutGeneration) 1
	set S(TransitionHistory) {}
	set S(TooltipStringOnOff) 1
	set S(TooltipString) ?
	set S(CurrentMapColor) #AABBCC
	set S(CurrentPanelColor) #FFCC66
	set S(filtersymboltype) rectangle
	set S(filtersymboltypewidth) 10
	set S(filtersymboltypeheight) 10
	set S(filtersymbolcoloroutline) yellow
	set S(PhyloTypeWhat) majority
	set S(PhyloTypeHi) bg
	set S(PhyloTypeDisplay) map
	set S(AAtreeColorOperation) fg
	set S(AAtreeColorWhat) majority
	set S(aatreecolorscale1) 0.0 
	set S(aatreecolorscale2) 0.95 
	set S(SurfaceBubbleTransitionMap) 12
	set S(TransitionMapFontSize) 8 
	set S(TransitionMapWhat) majority
	set S(tool) moveall
	set S(TransitionMapLayoutViewCluster) bubble
	set S(TransitionMapFactorYuser) 3.0
	set S(TransitionMapFactorXuser) 1.5
	set S(TransitionMapFont) [list Helvetica $S(TransitionMapFontSize) normal]
	set S(TransitionMapMonoLayout) rectangular
	set S(TransitionMapLayoutView) bra
	set S(TransitionMapMonoNode) 0
	set S(TransitionMapMonoAnnotation) 1
	set S(TransitionMapMonoArrow) 0
	set S(TransitionMapMultiSize) 0
	set S(TransitionMapMultiAnnotation) 1
	set S(TransitionMapMultiAnnotationTransition) 0
	set S(TransitionMapMultiArrow) 0
	set S(TransitionMapMultiLevel) 1 
	set S(TransitionMapMultiLayout) slanted
	set S(AAbubbleType) majority 
	set S(TransitionQueryWhat)  majority
	set S(ColorSetMode) hierarchic
	set S(deltaPieCompar) 6
	
	### TOP
	wm withdraw .
	destroy .pan
	set p [toplevel .pan -relief raised -borderwidth 2  -background "#AAA"]
	wm protocol .pan WM_DELETE_WINDOW "pamexit"
	wm title .pan $S(version) 
	set x [expr ([winfo screenwidth $p]  /2) - 1000/2]
	set y [expr ([winfo screenheight $p] /2) - 1000/2]
	wm geometry $p "1200x1000+$x+$y"
	# HELP
	makeTooltip
	
	############## TABNOTEBOOK
	set w11 [ttk::notebook .pan.n]
	$w11 add [ttk::frame .pan.n.t] -image inout -text "File"  -compound top ;# I/O, preferences
	$w11 add [ttk::frame .pan.n.a] -image edition -text "Edit"  -compound top ;# basic tree edition 
	$w11 add [ttk::frame .pan.n.aa] -image ancestralannotation -text "AA" -compound top ;# computation of ancestral annotation
	$w11 add [ttk::frame .pan.n.s] -image transitions -text "T" -compound top  ;# transition of ancestral annotation
	$w11 add [ttk::frame .pan.n.co] -image compare -text "C"  -compound top ;# comparaison of AA and transition
	ttk::notebook::enableTraversal $w11
	
	############### PANEDWINDOW GRAPHICS #DCDAD5
	set w12 [ttk::notebook .pan.g]
	$w12 add [ttk::frame .pan.g.g] -text "Maps"
	$w12 add [ttk::frame .pan.g.i] -text "Info"
	#$w12 add [ttk::frame .pan.g.h] -text "Help"
	
	### INFO
	set wi [frame .pan.g.i.info ]
	text $wi.edit -yscrollcommand "$wi.edit.scroll set"  -background "#DDDDDD" -height 3
	scrollbar $wi.edit.scroll -command "$wi.edit yview" 
	pack $wi.edit -expand true -fill both
	pack $wi.edit.scroll -side right -fill y
	set S(infopath) $wi.edit
	pack $wi -expand true -fill both
	
	### HELP
	set S(mainnotebook) .pan.g
	set m .pan.g.h.manual
	$S(mainnotebook) select .pan.g.g
	set comptI 0
	set S(SCRIPTdir) [pwd]
	set directory [file join $S(SCRIPTdir) manual]
	foreach file [glob -nocomplain -directory $directory *.jpg] {
		image create photo $comptI -file $file
		$m image create end -image $comptI
		incr comptI
	}
	
	############### GRID
	grid $w11 -row 0 -column 0 -rowspan 2 -sticky ns
	grid $w12 -row 0 -column 1 -rowspan 2 -sticky news
	grid columnconfigure .pan 1 -weight 1
	grid rowconfigure .pan 1 -weight 1
	############### BAR DE GESTION DES MAPS
	set w0map [labelframe .pan.g.g.map -text "Map"  ]
	$w0map configure -labelwidget [MakeFlagLabelFrame $w0map "Map" ]
	ttk::button $w0map.addm -text "+ Map" -command "MakeMap .pan.g.g.g" 
	ttk::menubutton $w0map.removemap -text "- Map" -menu $w0map.removemap.m 
	ttk::menubutton $w0map.target -text "Map selection" -menu $w0map.target.m 
	set m  [menu  $w0map.target.m -tearoff 0]
	$m configure -postcommand "MapTarget $m"
	set m  [menu  $w0map.removemap.m -tearoff 0]
	$m configure -postcommand "MapRemove $m"
	ttk::menubutton $w0map.display -text "Display" -menu $w0map.display.m 
	set m  [menu  $w0map.display.m -tearoff 0]
	$m configure -postcommand "MapDisplay $m"
	ttk::button $w0map.addb -text "+ Panel" -command "MakeBoard" 
	ttk::button $w0map.removed -text "- Panel" -command "RemoveBoard" 
	pack $w0map.target $w0map.addm $w0map.removemap $w0map.display -side left -expand true -fill x
	pack $w0map.addb $w0map.removed -side left -expand true -fill x
	###
	set gg [ttk::panedwindow .pan.g.g.g -orient vertical]
	set S(MapNumber) 0

	MakeMap $gg

	### pack
	grid $w0map -row 0 -column 0 -sticky ew
	grid $gg -row 1 -column 0  -sticky news
	grid columnconfigure .pan.g.g 0 -weight 1
	grid rowconfigure .pan.g.g 1 -weight 1
	
	# TABENOTEBOOK CONTROLS
	set w0 .pan.n.t
	
	#
	set S(messagetoolbox1) "Input / Output"
	message .pan.n.t.message -width 50c -font {Arial 14 normal} -textvariable S(messagetoolbox1) -background #003366  -foreground #DCDAD5
	pack .pan.n.t.message -expand false -fill x
			
	# block Load
	set w0loa [labelframe $w0.loa -text "Load"  ]
	$w0loa configure -labelwidget [MakeFlagLabelFrame $w0loa "Load"]
	ttk::button $w0loa.loadtree -text "Load tree..." -width 15 -command LoadTree
	ttk::button $w0loa.loadannotation -text "Load annotations..." -width 15 -command LoadAnnotation
	ttk::button $w0loa.loadsupport -text "Load support..." -width 15 -command LoadSupport
	pack $w0loa.loadtree $w0loa.loadannotation $w0loa.loadsupport -expand true -fill x
	pack $w0loa -expand true -fill x
	# HELP
	setTooltip $w0loa [HelpText FileLoad] 
	
	# block import 
	set w0imp [labelframe $w0.imp -text "Import" ]
	$w0imp configure -labelwidget [MakeFlagLabelFrame $w0imp "Import" ]
	set beastvariableslistbox [ttk::frame $w0imp.listbox ]
	set S(beastvariableslistbox) $w0imp.listbox.l
	lbs $beastvariableslistbox 5 6 single
	ttk::button $w0imp.e -text "Select NEXUS file" -command "SelectFiletoImport"
	ttk::button $w0imp.i -text "Import" -command "BEASTImport"	
	pack $w0imp.e -side top -expand true -fill x
	pack $beastvariableslistbox -expand true -fill both
	pack $w0imp.i -side top -expand true -fill x
	pack $w0imp -expand true -fill x
	# HELP
	setTooltip $w0.imp [HelpText FileBeastImport] 
	
	# block import ANNOTATION
	set w0imp2 [labelframe $w0.imp2 -text "Annotation format" ]
	$w0imp2 configure -labelwidget [MakeFlagLabelFrame $w0imp2 "Annotation format" ]
	set annotationvariableslistbox [ttk::frame $w0imp2.listbox ]
	set S(annotationvariableslistbox) $w0imp2.listbox.l
	lbs $annotationvariableslistbox 5 6 single
	ttk::button $w0imp2.e -text "Select annotation file" -command "SelectFiletoImportAnnotation"
	ttk::button $w0imp2.i -text "Import" -command "ANNOTATIONImport"
	pack $w0imp2.e -side top -expand true -fill x
	pack $annotationvariableslistbox -expand true -fill both
	pack $w0imp2.i -side top -expand true -fill x
	pack $w0imp2 -expand true -fill x
	# HELP
	setTooltip $w0.imp2 [HelpText FileAnnotationImport] 

	# output graphics *** passer en mode menu pour cibler une des 3 vues
	set w0b [labelframe $w0.saveg -text "Save" ]
	$w0b configure -labelwidget [MakeFlagLabelFrame $w0b "Save"]
	ttk::menubutton $w0b.savesvg -text "Save graphic as SVG..." -menu $w0b.savesvg.m
	set m  [menu $w0b.savesvg.m -tearoff 0]
	$m configure -postcommand "MapListExportSVG $m"
	ttk::menubutton $w0b.saveps -text "Save graphic as PS..." -menu $w0b.saveps.m
	set m  [menu $w0b.saveps.m -tearoff 0]
	$m configure -postcommand "MapListExportPS $m"
	ttk::menubutton $w0b.savecsv -text "Save annotations" -menu $w0b.savecsv.m
	set m  [menu $w0b.savecsv.m -tearoff 0]
	$m configure -postcommand "MapListExportAnnotation $m"
	pack $w0b.savesvg $w0b.saveps $w0b.savecsv -expand true -fill x
	pack $w0b -side top -expand 1 -fill x 
	# HELP
	setTooltip $w0.saveg [HelpText FileSave] 
	
	# Misc
	set misc  [labelframe $w0.misc -text "Preferences" ]
	$misc configure -labelwidget [MakeFlagLabelFrame $misc "Preferences" ]
	ttk::menubutton $misc.theme -text "Theme" -menu $misc.theme.m
	set m  [menu  $misc.theme.m -tearoff 0]
	$m add command -label "Clam" -command "ttk::style theme use clam"
	$m add command -label "Alt" -command "ttk::style theme use alt"
	$m add command -label "Default" -command "ttk::style theme use default"
	ttk::menubutton $misc.canvasbg -text "Background color" -menu $misc.canvasbg.m
	set m  [menu  $misc.canvasbg.m -tearoff 0]
	$m add command -label "Tree canvas backround color" -command "MapBackgroundColor"
	$m add command -label "Panel canvas background color" -command "BoardBackgroundColor"
	ttk::menubutton $misc.help -text "Help" -menu $misc.help.m
	set m  [menu  $misc.help.m -tearoff 0]
	$m add command -label "Manual" -command "pamhelp"
	$m add checkbutton -label "Tooltips" -variable S(TooltipStringOnOff) -command "makeTooltip"
	ttk::button $misc.exit -text "Exit" -command "pamexit"
	pack $misc.theme $misc.canvasbg $misc.help $misc.exit -side top -fill x
	pack $misc -side top -expand true -fill x
	# HELP
	setTooltip $w0.saveg [HelpText FilePreferences]
	
	##### TABNOTEBOOK EDIT 
	set wa .pan.n.a
	
	set S(messagetoolbox2) "Tree & Annotation Edition"
	message .pan.n.a.message -width 50c -font {Arial 14 normal} -textvariable S(messagetoolbox2) -background #003366 -foreground  #DCDAD5
	pack .pan.n.a.message -expand false -fill x
	
	# Zoom Toolbox  
	set w0a [labelframe $wa.toolbox -text "Global" -background #DCDAD5]
	$w0a configure -labelwidget [MakeFlagLabelFrame $w0a "Global" ] 
	set wt1i1 [frame $w0a.i1 ] ;# icons
	set wt1i2 [frame $w0a.i2] ;# icons
	set wt1i3 [frame $w0a.i3] ;# icons
	set wt1c [ttk::frame $w0a.c] ;# canvas
	set wt1t [ttk::frame $w0a.t] ;# tools
	# mini canvas
	set wmini [ttk::frame $w0a.minicanvas]
	canvas $wmini.c -width 150 -height 150 -highlightthickness 0 -background "#DCDAD5" -borderwidth 0 -relief flat
	set S(canvasmini) $wmini.c
	pack $wmini.c -fill both -expand true
	set S(minicanvas) $wmini.c
	pack $wmini -fill both -expand true
	if {$tcl_platform(os) == "Darwin"} {
		foreach tool {zoomw zoomf zoom- zoom+} {
			radiobutton $wt1i1.$tool -value $tool -text $tool -image $tool -variable S(tool)  \
			-highlightbackground  #DCDAD5 -indicatoron off -highlightthickness 0 -relief flat 
			pack $wt1i1.$tool -side left -expand true -fill x
		}
		foreach tool {zoomx- zoomx+ zoomy- zoomy+} {
			radiobutton $wt1i2.$tool -value $tool -text $tool -image $tool -variable S(tool) \
			-highlightbackground  #DCDAD5 -indicatoron off -highlightthickness 0 -relief ridge 
			pack $wt1i2.$tool -side left -expand true -fill x
		}
		foreach tool {moveall } {
			radiobutton $wt1i3.$tool -value $tool -text $tool -image $tool -variable S(tool) \
			-highlightbackground  #DCDAD5 -indicatoron off -highlightthickness 0 -relief ridge 
			pack $wt1i3.$tool -side left -expand true -fill x 
		}
	} else {
		foreach tool {zoomw zoomf zoom- zoom+} {
			radiobutton $wt1i1.$tool -value $tool -text $tool -image $tool -variable S(tool) \
			-highlightbackground  #DCDAD5 -indicatoron off -highlightthickness 0 -relief ridge 
			pack $wt1i1.$tool -side left -expand true -fill x
		}
		foreach tool {zoomx- zoomx+ zoomy- zoomy+} {
			radiobutton $wt1i2.$tool -value $tool -text $tool -image $tool -variable S(tool) \
			-highlightbackground  #DCDAD5 -indicatoron off -highlightthickness 0 -relief ridge 
			pack $wt1i2.$tool -side left -expand true -fill x
		}
		foreach tool {moveall } {
			radiobutton $wt1i3.$tool -value $tool -text $tool -image $tool -variable S(tool) \
			-highlightbackground  #DCDAD5 -indicatoron off -highlightthickness 0 -relief ridge 
			pack $wt1i3.$tool -side left -expand true -fill x
		}
	}
	# RESET
	ttk::menubutton $w0a.resetree -text "Reset" -menu $w0a.resetree.m
	set m  [menu  $w0a.resetree.m -tearoff 0]
	$m add command -label "Reset all" -command "ResetAll"
	$m add separator
	$m add command -label "Tree foreground color" -command "ResetTreeForegroundColor"
	$m add command -label "Tree background color" -command "ResetTreeBackgroundColor"
	$m add command -label "Line width" -command "ResetLineWidth"
	$m add separator
	$m add command -label "Leave foreground color" -command "ResetLeaveForegroundColor"
	$m add command -label "Leave annotation" -command "ResetLeaveAnnotation"
	$m add separator
	$m add command -label "Node annotation" -command "ResetNodeAnnotation"
	$m add separator
	$m add command -label "Map tree" -command "ResetMapCanvas"
	$m add command -label "Map views" -command "ResetMapBoards"
	pack $w0a.resetree -expand true -fill x 
	pack $w0a -side top -expand 1 -fill x 
	#
	setTooltip $wt1i1.zoomf "MAGNIFY - fit to window (x)"
	setTooltip $wt1i1.zoomf "MAGNIFY - frame"
	setTooltip $wt1i1.zoom- "MAGNIFY (Zoom Out)"
	setTooltip $wt1i1.zoom+ "MAGNIFY (Zoom In)"
	setTooltip $wt1i2.zoomy- "MAGNIFY Y (Zoom Out)"
	setTooltip $wt1i2.zoomy+ "MAGNIFY Y (Zoom In)"
	setTooltip $wt1i2.zoomx- "MAGNIFY X (Zoom Out)"
	setTooltip $wt1i2.zoomx+ "MAGNIFY X (Zoom In)"
	pack $wt1c $wt1i1 $wt1i2 $wt1i3 $wt1t -expand true -fill x -side top
	pack $w0a -side top -expand 1 -fill x
	
	# COLOR MANAGEMENT
	set w0d [labelframe $w0a.colorannot -text "Colors" -background #DCDAD5]
	ttk::menubutton $w0d.colorannot -text "Color set (H)" -menu $w0d.colorannot.m 
	set m  [menu  $w0d.colorannot.m -tearoff 1 ]
	set S(MenuColorAnnotation) $m
	$m configure -postcommand "ColorAnnotation $m"	
	frame $w0d.sca -background #DCDAD5
	scale $w0d.sca.bgscalesaturation -orient horizontal -from 50.0 -to 250.0 -resolution 1.0 -width 10 -length 20 -background #DCDAD5 -variable S(ColorSaturation) -command {catch UpdateColor}
	scale $w0d.sca.bgscalevalue -orient horizontal -from 50.0 -to 250.0 -resolution 1.0 -width 10 -length 20 -background #DCDAD5 -variable S(ColorValue) -command {catch UpdateColor}
	ttk::menubutton $w0d.cible -text "Color update (S & V)" -menu $w0d.cible.m
	set m  [menu  $w0d.cible.m -tearoff 0]
	$m add radiobutton -value BGtree -label "Tree background color" -variable S(UpdateColorCible) -command ColorUpdateScale
	$m add radiobutton -value FGtree -label "Tree foreground color" -variable S(UpdateColorCible) -command ColorUpdateScale
	$m add radiobutton -value MaxLikpiechart -label "Piechart" -variable S(UpdateColorCible) -command ColorUpdateScale
	$m add radiobutton -value PupkoAnnot -label "Bubble" -variable S(UpdateColorCible) -command ColorUpdateScale
	$m add radiobutton -value TipsAnnot -label "Tips" -variable S(UpdateColorCible) -command ColorUpdateScale
	pack $w0d.sca.bgscalesaturation $w0d.sca.bgscalevalue -side left -expand 1 -fill x
	pack $w0d.colorannot $w0d.cible $w0d.sca  -expand true -fill x
	pack $w0d -side top -expand 1 -fill x
	
	# EDITION 
	set w0c [labelframe $wa.toolbox3 -text "Tree" ]
	$w0c configure -labelwidget [MakeFlagLabelFrame $w0c "Tree"] 
	set wt1i1 [frame $w0c.i1] ;# icons
	set wt1i2 [frame $w0c.i2] ;# icons
	if {$tcl_platform(os) == "Darwin"} {
		foreach tool {s-swap  s-size-+ s-color infowhat} {
			radiobutton $wt1i1.$tool -value $tool -text $tool -image $tool -variable S(tool) \
			-highlightbackground  #DCDAD5 -indicatoron off -highlightthickness 0 -relief ridge 
			pack $wt1i1.$tool -side left -expand true -fill x
		}
		foreach tool {root} {
			radiobutton $wt1i2.$tool -value $tool -text $tool -image $tool -variable S(tool) \
			-highlightbackground  #DCDAD5 -indicatoron off -highlightthickness 0 -relief ridge 
			pack $wt1i2.$tool -side left -expand true -fill x
		}
		
	} else {
		foreach tool {s-swap  s-size-+ s-color infowhat} {
			radiobutton $wt1i1.$tool -value $tool -text $tool -image $tool -variable S(tool) -variable S(tool) \
			-highlightbackground  #DCDAD5 -indicatoron off -highlightthickness 0 -relief ridge 
			pack $wt1i1.$tool -side left -expand true -fill x
		}
		foreach tool {root} {
			radiobutton $wt1i2.$tool -value $tool -text $tool -image $tool -variable S(tool) -variable S(tool) \
			-highlightbackground  #DCDAD5 -indicatoron off -highlightthickness 0 -relief ridge 
			pack $wt1i2.$tool -side left -expand true -fill x
		}
	}
	setTooltip $wt1i1.s-swap "SWAP"
	setTooltip $wt1i1.s-size-+ "LINE (size, D1 & Shift-D1)"
	setTooltip $wt1i1.s-color "COLOR (foreground)"
	setTooltip $wt1i1.infowhat "INFO (note)"
	setTooltip $wt1i2.root "ROOT"
	ttk::menubutton $w0c.editiongenerique -text "Layout" -menu $w0c.editiongenerique.m 
	set m  [menu  $w0c.editiongenerique.m -tearoff 1 ]
	$m add command -label "Laderize up" -command "LaderizeSwap up"
	$m add command -label "Laderize down" -command "LaderizeSwap down"
	ttk::menubutton $w0c.editiongeneriqueleaf -text "Leaves" -menu $w0c.editiongeneriqueleaf.m 
	set m  [menu  $w0c.editiongeneriqueleaf.m -tearoff 1 ]
	$m add command -label "On/Off" -command "leafOnOff"
	$m add command -label "Font" -command "leafFont"
	$m add command -label "Color" -command "SleafForegroundColor"
	ttk::menubutton $w0c.editiongeneriqueline -text "Branches" -menu $w0c.editiongeneriqueline.m 
	set m  [menu  $w0c.editiongeneriqueline.m -tearoff 1 ]
	$m add command -label "Size -" -command "SizeLine- "
	$m add command -label "Size +" -command "SizeLine+"
	$m add separator
	$m add command -label "Size 1" -command "SizeLineSet 1"
	$m add command -label "Size 2" -command "SizeLineSet 2"
	$m add command -label "Size 3" -command "SizeLineSet 3"
	pack $w0c.editiongenerique $w0c.editiongeneriqueleaf $w0c.editiongeneriqueline -expand true -fill x -side top
	pack $wt1i1 $wt1i2 $wt1i3  -expand true -fill x -side top
	pack $w0c -side top -expand 1 -fill x
	setTooltip $w0c.editiongenerique "Edition"
	setTooltip $w0c.editiongeneriqueleaf "Edition"
	setTooltip $w0c.editiongeneriqueline "Edition"
	# ANNOTATION NODE
	set w1b [labelframe $wa.toolbox5 -text "Annotation"]
	$w1b configure -labelwidget [MakeFlagLabelFrame $w1b "Annotation"] 
	ttk::menubutton $w1b.annotationl -text "Leaf annotation" -menu $w1b.annotationl.m
	set m [menu $w1b.annotationl.m -tearoff 0]
	$m add command -label "Color labels" -command "LeafAnnotationColorLabels"
	$m add command -label "Color brackets with text" -command "set S(LeafAnnotationColorBracketsText) 1 ; LeafAnnotationColorBrackets"
	$m add command -label "Color brackets without text" -command "set S(LeafAnnotationColorBracketsText) 0 ; LeafAnnotationColorBrackets"
	$m add command -label "Clusters 100% background color" -command "ResetTreeBackgroundColor ;  Cluster100ColorCAP "
	set wna [labelframe $w1b.annotationn -text "Node annotation"]
	$wna configure -labelwidget [MakeFlagLabelFrame $wna "Node annotation"] 
	set wnant [ttk::notebook $wna.n]
	$wnant add [ttk::frame $wnant.sp] -text "Sp"
	$wnant add [ttk::frame $wnant.lb] -text "Lb"
	$wnant add [ttk::frame $wnant.id] -text "Id"
	pack $wnant -expand true -fill x
	### sp
	set S(SupportTresholdDown) 0.25 ;# affichage si superieur a cette valeur
	set S(SupportTresholdUp) 0.75 ;# affichage si inferieur a cette valeur
	set S(SupportColor) blue
	set S(SupportHightligth) antree
	set S(checksupportonoff) 0
	# sp seuils
	frame $wnant.sp.scalesp
	scale $wnant.sp.scalesp.tresholddown -orient horizontal -from 0 -to 1.0 -resolution 0.01 -width 10 -length 20 -background #DCDAD5 -variable S(SupportTresholdDown) -command {catch AnnotateNodeSupport}
	scale $wnant.sp.scalesp.tresholdup -orient horizontal -from 0 -to 1.0 -resolution 0.01 -width 10 -length 20 -background #DCDAD5 -variable S(SupportTresholdUp) -command {catch AnnotateNodeSupport}
	pack $wnant.sp.scalesp.tresholddown $wnant.sp.scalesp.tresholdup -side left -expand true -fill x
	# sp params
	ttk::menubutton $wnant.sp.params -text "Parameters" -menu $wnant.sp.params.m
	set m  [menu $wnant.sp.params.m -tearoff 0]
	$m add command -label "Support On/Off"  -command {SupportOnOff}
	$m add separator
	$m add command -label "Color" -command SupportColor
	$m add radiobutton -value fgtree -label "Tree foreground color" -variable S(SupportHightligth) -command AnnotateNodeSupport
	$m add radiobutton -value antree -label "Tree annotate" -variable S(SupportHightligth) -command AnnotateNodeSupport
	pack $wnant.sp.scalesp  $wnant.sp.params -expand true -fill x
	
	# bl
	set S(BrLenColor) orange
	ttk::button $wnant.lb.color -text "Color" -command BrLenColor
	ttk::button $wnant.lb.onoff -text "Display Br. Len. On/Off" -command AnnotateNodeBranchLength
	pack $wnant.lb.color $wnant.lb.onoff
	# id
	set S(IDColor) purple
	ttk::button $wnant.id.color -text "Color" -command IDColor
	ttk::button $wnant.id.onoff -text "Display ID" -command AnnotateNodeID
	pack $wnant.id.color $wnant.id.onoff
	pack $w1b.annotationl $w1b.annotationn -expand true -fill x -side top
	pack $w1b -side top -expand 1 -fill x

	##### A.A. TAB ANCESTRAL ANNOTATIONS
	set w1 .pan.n.aa
	
	set S(messagetoolbox3) "Compute & Display Ancestral Annotations"
	message .pan.n.aa.message -width 50c -font {Arial 14 normal} -textvariable S(messagetoolbox3) -background #003366  -foreground #DCDAD5
	pack .pan.n.aa.message -expand false -fill x
	

	# COMPUTE ANCESTRAL ANNOTATION parsimony
	set w0comp [labelframe $w1.comp -text "Compute" ]
	$w0comp configure -labelwidget [MakeFlagLabelFrame $w0comp "Compute"] 

	# parsimony
	set w0compm [frame $w0comp.mode]
	set S(parsimonyMode) 1
	ttk::radiobutton $w0compm.m1 -value 1 -text "ACCTRAN" -variable S(parsimonyMode)
	ttk::radiobutton $w0compm.m2 -value 2 -text "DELTRAN" -variable S(parsimonyMode)
	pack $w0compm.m1 $w0compm.m2 -side left -expand true -fill x
	ttk::button $w0comp.c -text "Parsimony" -command "AAcomputeparsimony"
	pack $w0comp.c $w0comp.mode  -expand true -fill x

	# max lik 
	ttk::button $w0comp.f81c -text "F81-like" -command "ALGO4"
	ttk::button $w0comp.pupkorun -text "Joint scenario" -command "ALGO5"

	ttk::radiobutton $w0comp.f81c1 -value 1 -text "priors: tree frequency" -variable S(F81frequenceannotMODE)
	ttk::radiobutton $w0comp.f81c2 -value 2 -text "priors: equiprob." -variable S(F81frequenceannotMODE)
	pack $w0comp.f81c $w0comp.pupkorun $w0comp.f81c1  $w0comp.f81c2 -expand true -fill x
	pack $w0comp -expand true -fill x
	
	# TREE COLOR
	set w1b [labelframe $w1.edition -text "AA Tree color" ]
	$w1b configure -labelwidget [MakeFlagLabelFrame $w1b "AA Tree color"] 
	# data
	ttk::menubutton $w1b.majopercent -text "Data" -menu $w1b.majopercent.m
	set m  [menu $w1b.majopercent.m -tearoff 0]
	$m add radio -label "Majority" -variable S(AAtreeColorWhat) -value majority -command "catch AAtreeColorCAP"
	$m add radio -label "Joint scenario" -variable S(AAtreeColorWhat) -value pupko -command "catch AAtreeColorCAP"
	$m add radio -label "Parsimony" -variable S(AAtreeColorWhat) -value parsimony -command "catch AAtreeColorCAP"
	pack $w1b.majopercent -expand true -fill x
	# Foreground color
	set w1efg [labelframe $w1b.fg -text "Fg" -background #DCDAD5]
	ttk::button $w1efg.go -text "Color" -command "catch AAtreeColorCAP"
	scale $w1efg.aatreecolorscale1 -orient horizontal -from 0.0 -to 1.0 -resolution 0.01 -width 10 -length 50 -background #DCDAD5 -variable S(aatreecolorscale1) -command {catch AAtreeColorCAP}
	pack  $w1efg.go $w1efg.aatreecolorscale1  -expand true -fill x -side top
	pack $w1efg -expand true -fill x
	# Background color using Sz
	set w1e [labelframe $w1b.phylotype -text "Bg" -background #DCDAD5]
	# Sz
	set sz [frame $w1e.sz -bg #DCDAD5]
	entry $sz.sz -width 5 -textvariable S(sz-threshold) -width 5 -font {Courrier 10 normal} -highlightthickness 0 -relief flat -bg #DCDAD5 -fg  #003366
	ttk::label $sz.szl -text "Sz ≥" 
	# Pt proba threshold
	set pt [frame $w1e.threshold -bg #DCDAD5]
	entry $pt.e -width 5 -textvariable S(criteria-proba-threshold) -width 5 -font {Courrier 10 normal} -highlightthickness 0 -relief flat -bg #DCDAD5 -fg  #003366
	ttk::checkbutton $pt.l -text "Pt ≥" -variable S(criteriacheck,pt)
	pack $sz.szl $sz.sz  -side left -expand true -fill x
	pack $pt.l $pt.e -side left -expand true -fill x


	# run
	ttk::button $w1e.run -text "Color" -command PhylotypeMasterCAP
	pack $sz $pt $w1e.run -expand true -fill x
	pack $w1e  -expand true -fill x
	pack $w1b -expand true -fill x

	# AA unique BUBBLE (majority/pupko/parsimony)
	set S(AAbubbleSimplify) 0
	set w1b [labelframe $w1.center -text "AA Bubble" ]
	$w1b configure -labelwidget [MakeFlagLabelFrame $w1b "AA Bubble"] 
	ttk::menubutton $w1b.data -text "Data" -menu $w1b.data.m
	set m  [menu $w1b.data.m -tearoff 0]
	$m add radiobutton -label "Majority" -variable S(AAbubbleType) -value majority -command "catch BubbleCAP"
	$m add radiobutton -label "Joint scenario" -variable S(AAbubbleType) -value pupko -command "catch BubbleCAP"
	$m add radiobutton -label "Parsimony" -variable S(AAbubbleType) -value parsimony -command "catch BubbleCAP"
	ttk::menubutton $w1b.draw -text "Display" -menu $w1b.draw.m
	set m  [menu $w1b.draw.m -tearoff 0]
	$m add command -label "On/Off" -command "BubbleOnOff"
	$m add separator
	$m add checkbutton -label "Simplify view" -variable S(AAbubbleSimplify) -command "BubbleSimplifyVisuCAP"
	ttk::menubutton $w1b.only -text "Annotation" -menu $w1b.only.m
	set m  [menu $w1b.only.m -tearoff 0]
	$m configure -postcommand "BubbleOnlyCommand $m"
	frame $w1b.zoom
	ttk::button $w1b.zoom.zbase- -text "-" -width 1 -command "BubbleZoom -"
	ttk::button $w1b.zoom.zbase+ -text "+" -width 1 -command "BubbleZoom +"
	pack  $w1b.zoom.zbase- $w1b.zoom.zbase+ -side left -expand true -fill both
	pack $w1b.data $w1b.only  $w1b.draw  $w1b.zoom -side top -expand true -fill both
	pack $w1b -expand true -fill x

	# AA PIECHARTS
	set w1b [labelframe $w1.piechart -text "AA Piechart" ]
	$w1b configure -labelwidget [MakeFlagLabelFrame $w1b "AA PieChart" ] 
	ttk::menubutton $w1b.make -text "Display" -menu $w1b.make.m
	set m  [menu $w1b.make.m -tearoff 0]
	$m add command -label "Display" -command "catch PREmakeMap"
	$m add command -label "Remove" -command "CammenbRemove"
	pack  $w1b.make -side top -expand 1 -fill x

	# MODE 1 seuils
	frame $w1b.mode1 -background #DCDAD5
	scale $w1b.mode1.camscale -orient horizontal -from 0.0 -to 1.0 -resolution 0.01 -width 10 -length 20 -background #DCDAD5 -variable S(probathreshold) -command {catch PREmakeMap}
	scale $w1b.mode1.camscale2 -orient horizontal -from 0.0 -to 1.01 -resolution 0.01 -width 10 -length 20 -background #DCDAD5 -variable S(camenbdisplayreshold) -command {catch PREmakeMap}
	pack $w1b.mode1.camscale $w1b.mode1.camscale2 -side left -expand 1 -fill x
	# MODE 2 affichage piecharts si presence d'annotation avec proba > majo-standard deviation
	#set S(piechartFilterDeviation) 0
	#frame $w1b.mode2 -background #DCDAD5
	#ttk::checkbutton $w1b.mode2.onoff -text "> M - ∂/2" -variable S(piechartFilterDeviation) -command {catch PREmakeMap}
	#pack $w1b.mode2.onoff -expand 1 -fill x
	# MODE 3 affichage piecharts si presence d'annotation avec proba > majo-majo * % (user)
	frame $w1b.mode3 -background #DCDAD5
	set S(piechartFilterPercent) 0
	set S(piechartFilterPercentTresh) 0.1
	ttk::checkbutton $w1b.mode3.onoff -text "> M - %M" -variable S(piechartFilterPercent) -command {catch PREmakeMap}
	ttk::entry $w1b.mode3.percent -width 7 -font {Helvetica 12 normal} -textvariable S(piechartFilterPercentTresh)
	pack $w1b.mode3.onoff $w1b.mode3.percent -side left -expand 0 -fill x
	# MODE 4 affichage piecharts si support >seuil (user)
	#frame $w1b.mode4 -background #DCDAD5
	#set S(piechartFilterSupportTresh) 0.75
	#set S(piechartFilterSupport) 0
	#ttk::checkbutton $w1b.mode4.onoff -text "< Sp" -variable S(piechartFilterSupport) -command {catch PREmakeMap}
	#ttk::entry $w1b.mode4.percent -width 7 -font {Helvetica 12 normal} -textvariable S(piechartFilterSupportTresh)
	#pack $w1b.mode4.onoff $w1b.mode4.percent -side left -expand 0 -fill x
	pack $w1b.mode1  $w1b.mode3  -side top -expand 1 -fill x
	frame $w1b.map
	#ttk::checkbutton $w1b.map.displaynumber -text "Values"  -onvalue normal -offvalue hidden -variable S(piechartdisplaynumber) -command PieValuesOnOff
	ttk::button $w1b.map.zbase- -text "-" -width 1 -command "ZoomCircle -"
	ttk::button $w1b.map.zbase+ -text "+" -width 1 -command "ZoomCircle +"
	pack $w1b.map.zbase- $w1b.map.zbase+ -side left -expand true -fill both
	pack $w1b.map -expand true -fill x
	pack $w1b -expand true -fill x

	# test FILTRES
	set w1b [labelframe $w1.filter -text "Filters" ]
	$w1b configure -labelwidget [MakeFlagLabelFrame $w1b "Filters" ] 
	ttk::menubutton $w1b.fil -text "Filter" -menu $w1b.fil.m
	set m  [menu $w1b.fil.m -tearoff 0]
	$m add command -label "> M - ∂/2" -command "Filter01"
	$m add command -label "Majo ≠ Pupk." -command "Filter02"
	$m add separator
	$m add command -label "A(n) ≠ A(father) (majo.)" -command "Filter04"
	$m add command -label "A(n) ≠ A(father) (joint scenario)" -command "Filter03"
	$m add separator
	$m add command -label "Symbol"  -command "FilterSymbolPanel"
	$m add separator
	$m add command -label "Reset"  -command "FilterReset"
	pack $w1b.fil -side top -expand true -fill both
	pack $w1b -expand true -fill x
	
	### TRANSITIONS
	set w1 .pan.n.s
	
	set S(messagetoolbox4) "Transitions of Ancestral Annotations"
	message .pan.n.s.message -width 50c -font {Arial 14 normal} -textvariable S(messagetoolbox4) -background  #003366  -foreground #DCDAD5
	pack .pan.n.s.message -expand false -fill x
	
	#  TRANSITION QUERY 
	set S(TransitionQueryString) ?
	set w1b [labelframe $w1.transition -text "Transitions Query" ]
	$w1b configure -labelwidget [MakeFlagLabelFrame $w1b "Transitions Query" ] 
	# insert
	ttk::menubutton $w1b.insert -text "Insert" -menu $w1b.insert.m
	set m [menu $w1b.insert.m -tearoff 0]
	$m configure -postcommand "TransitionQueryInsertList $m"
	# query string
	entry $w1b.scenario  -font {Helvetica 12 normal} -textvariable S(TransitionQueryString) -font {Courrier 10 normal} -highlightthickness 0 -relief flat -bg #DCDAD5 -fg  #003366
	set S(TransitionQueryStringPath) $w1b.scenario

	# Hi operation
	ttk::menubutton $w1b.hi -text "Data" -menu $w1b.hi.m
	set m  [menu $w1b.hi.m -tearoff 0]
	$m add radiobutton  -label "Majority" -variable S(TransitionQueryWhat) -value majority  
	$m add radiobutton  -label "Joint scenario" -variable S(TransitionQueryWhat) -value pupko 
	$m add radiobutton  -label "Parsimony" -variable S(TransitionQueryWhat) -value parsimony  

	# run
	ttk::button $w1b.run -text "Search" -command "TransitionQueryRun"
	frame $w1b.video
	ttk::button $w1b.video.first -text "|<" -width 1 -command "TransitionQueryFirst"
	ttk::button $w1b.video.previous -text "<" -width 1 -command "TransitionQueryPrevious"
	#ttk::button $w1b.video.play -text "@" -width 1 -command "TransitionQueryPlay"
	ttk::button $w1b.video.next -text ">" -width 1 -command "TransitionQueryNext"
	ttk::button $w1b.video.last -text ">|" -width 1 -command "TransitionQueryLast"
	pack $w1b.video.first $w1b.video.previous $w1b.video.next $w1b.video.last -side left -expand true -fill x
	ttk::menubutton $w1b.transitionhistory -text "Edit" -menu $w1b.transitionhistory.m 
	set m  [menu  $w1b.transitionhistory.m -tearoff 0]
	$m configure -postcommand "TransitionHistory $m"
	# pack
	pack $w1b.scenario $w1b.insert $w1b.hi $w1b.run $w1b.video $w1b.transitionhistory -expand true -fill x
	pack $w1b -expand true -fill x
	# ANNOTATION MAP 
	set w1b [labelframe $w1.map -text "Transitions Map"]
	$w1b configure -labelwidget [MakeFlagLabelFrame $w1b "Transitions Map" ] 
	ttk::menubutton $w1b.draw -text "Data " -menu $w1b.draw.m
	set m  [menu $w1b.draw.m -tearoff 0]
	$m add radiobutton  -label "Majority" -variable S(TransitionMapWhat) -value majority  
	$m add radiobutton  -label "Joint scenario" -variable S(TransitionMapWhat) -value pupko 
	$m add radiobutton  -label "Parsimony" -variable S(TransitionMapWhat) -value parsimony 
	$m add separator
	$m add radiobutton -label "Dynamic mode" -variable S(tool) -value tresholdbar  -command "TransitionMapDynamic"

	set fullmap [frame $w1b.layoutmono -bg #DCDAD5]
	ttk::button $w1b.layoutmono.draw -text "Full" -command {TransitionMap mono}
	ttk::menubutton $w1b.layoutmono.option -text "options..." -menu $w1b.layoutmono.option.m
	set m  [menu $w1b.layoutmono.option.m -tearoff 0]
	$m add radiobutton -label "Radial" -variable S(TransitionMapMonoLayout) -value radial  
	$m add radiobutton -label "Slanted" -variable S(TransitionMapMonoLayout) -value slanted  
	$m add radiobutton -label "Rectangular" -variable S(TransitionMapMonoLayout) -value rectangular  
	$m add separator
	$m add radiobutton -label "Generation" -variable S(TransitionMapLayoutView) -value gen  
	$m add radiobutton -label "Branch lengths" -variable S(TransitionMapLayoutView) -value bra  
	$m add separator
	$m add radiobutton -label "Bubble" -variable S(TransitionMapLayoutViewCluster) -value bubble  
	$m add radiobutton -label "Piecharts" -variable S(TransitionMapLayoutViewCluster) -value piechart  
	$m add separator
	$m add checkbutton -label "Tips % (annotation)" -variable S(TransitionMapMonoNode) 
	$m add checkbutton -label "Annotation (3 char.)" -variable S(TransitionMapMonoAnnotation) 
	$m add checkbutton -label "Arrow (on/off)" -variable S(TransitionMapMonoArrow) 
	pack $w1b.layoutmono.draw $w1b.layoutmono.option -side left -expand yes -fill x

	set condensedmap [frame $w1b.layoutmulti -bg #DCDAD5]
	ttk::button $w1b.layoutmulti.draw -text "Compressed" -command {TransitionMap multi}
	ttk::menubutton $w1b.layoutmulti.option -text "options..." -menu $w1b.layoutmulti.option.m	
	set m  [menu $w1b.layoutmulti.option.m -tearoff 0]
	$m add separator 
	$m add radiobutton -label "Compression level #1" -variable S(TransitionMapMultiLevel) -value 1  
	$m add radiobutton -label "Compression level #2" -variable S(TransitionMapMultiLevel) -value 2  
	$m add separator 
	$m add radiobutton -label "Radial" -variable S(TransitionMapMultiLayout) -value radial  
	$m add radiobutton -label "Slanted" -variable S(TransitionMapMultiLayout) -value slanted  
	$m add radiobutton -label "Rectangular" -variable S(TransitionMapMultiLayout) -value rectangular 
	$m add separator 
	$m add checkbutton -label "Number of transitions(s) - Edge size" -variable S(TransitionMapMultiSize) 
	$m add checkbutton -label "Number of transitions(s) - Text" -variable S(TransitionMapMultiAnnotationTransition) 
	$m add checkbutton -label "Annotation (3 char.)" -variable S(TransitionMapMultiAnnotation)
	$m add checkbutton -label "Arrow (on/off)" -variable S(TransitionMapMultiArrow)
	pack $w1b.layoutmulti.draw $w1b.layoutmulti.option -side left -expand yes -fill x

	ttk::button $w1b.save -text "Save (Newick)" -command "PupkoSaveNewick"
	pack $w1b.draw $w1b.layoutmono $w1b.layoutmulti  $w1b.save -expand true -fill x
	frame $w1b.zoomfactor1
	ttk::button $w1b.zoomfactor1.fx- -text "x-" -width 0 -image TMzoomx- -command "TransitionMapFactorXuser -"  
	ttk::button $w1b.zoomfactor1.fx+ -text "x+" -width 0 -image TMzoomx+ -command "TransitionMapFactorXuser +"
	ttk::button $w1b.zoomfactor1.fy- -text "y-" -width 0 -image TMzoomy- -command "TransitionMapFactorYuser -"
	ttk::button $w1b.zoomfactor1.fy+ -text "y+" -width 0 -image TMzoomy+ -command "TransitionMapFactorYuser +"
	frame $w1b.zoomfactor2 
	ttk::button $w1b.zoomfactor2.zbase- -text "o-" -width 0 -image TMzoom-  -command "TransitionMapZoom -"
	ttk::button $w1b.zoomfactor2.zbase+ -text "o+" -width 0 -image TMzoom+  -command "TransitionMapZoom +"
	ttk::button $w1b.zoomfactor2.ztext- -text "f-" -width 0 -image TMzoomxfont- -command "TransitionMapZoomText -"
	ttk::button $w1b.zoomfactor2.ztext+ -text "f+" -width 0 -image TMzoomxfont+ -command "TransitionMapZoomText +"
	
	pack  $w1b.zoomfactor1.fx- $w1b.zoomfactor1.fx+ $w1b.zoomfactor1.fy- $w1b.zoomfactor1.fy+ -side left -expand 1 -fill x
	pack  $w1b.zoomfactor2.zbase- $w1b.zoomfactor2.zbase+ $w1b.zoomfactor2.ztext- $w1b.zoomfactor2.ztext+ -side left -expand 1 -fill x
	pack  $w1b.zoomfactor1 $w1b.zoomfactor2  -expand true -fill x
	#pack  $w1b.zoomfactor $w1b.zoomlayout -expand true -fill x
	pack $w1b -expand true -fill x
	
	# MATRIX transitions
	set S(MatrixWhat) majority
	set w1b [labelframe $w1.bbm -text "Transitions Matrix" ]
	$w1b configure -labelwidget [MakeFlagLabelFrame $w1b "Transitions Matrix"] 
	ttk::menubutton $w1b.what -text "Data" -menu $w1b.what.m
	set m  [menu $w1b.what.m -tearoff 0] 
	$m add radiobutton -label "Majority" -variable S(MatrixWhat) -value majority  
	$m add radiobutton -label "Joint scenario" -variable S(MatrixWhat) -value pupko 
	$m add radiobutton -label "Parsimony" -variable S(MatrixWhat) -value parsimony  
	pack $w1b.what -expand true -fill x
	ttk::menubutton $w1b.transitionmatrix -text "Display" -menu $w1b.transitionmatrix.m
	set m  [menu $w1b.transitionmatrix.m -tearoff 0] 
	$m add radiobutton -label "Count" -variable S(MatrixType) -value 1 -command TransitionMatrixCAP
	$m add radiobutton -label "Relative rate" -variable S(MatrixType) -value 2 -command TransitionMatrixCAP

	pack $w1b.transitionmatrix -expand true -fill x
	pack $w1b -expand true -fill x


	######################################## TAB comparison of AAs and Transitions .pan.n.c
	
	### INTRA MAP comparaison major/parcimonie/pupko	
	
		set S(messagetoolbox5) "Comparative Analysis"
	message .pan.n.co.message -width 50c -font {Arial 14 normal} -textvariable S(messagetoolbox5) -background #003366 -foreground #DCDAD5 
	pack .pan.n.co.message -expand false -fill x
	
	
	set w1 [labelframe .pan.n.co.a -text "Intra MAP comparison"]
	$w1 configure -labelwidget [MakeFlagLabelFrame $w1 "Intra MAP Inconsistencies" ]
	ttk::button $w1.aa -text "Display On/Off" -command BubbleInconsistenciesCAP 
	pack $w1.aa  -expand true -fill x
	pack $w1 -expand true -fill x 
	frame $w1.zoom
	ttk::button $w1.zoom.z- -text "-" -width 1 -command "BubbleInconsistenciesZoom -"
	ttk::button $w1.zoom.z+ -text "+" -width 1 -command "BubbleInconsistenciesZoom +"
	pack  $w1.zoom.z- $w1.zoom.z+ -side left -expand true -fill both	
	pack  $w1.zoom -expand true -fill both	

	### INTER MAP compaison de deux distributions
	# affichage piechart quand la majoritire est differente entre deux distri
	set w2 [labelframe .pan.n.co.b -text "Inter MAP comparison"  ]
	$w2 configure -labelwidget [MakeFlagLabelFrame $w2 "Inter MAP Inconsistencies" ]
	ttk::menubutton $w2.aa -text "AA distribution from" -menu $w2.aa.m 
	set m  [menu  $w2.aa.m -tearoff 0]
	$m configure -postcommand "ComparAASelection $m"
	ttk::button $w2.bub -text "Bubble" -command AAcomparBubble
	ttk::button $w2.pie -text "Piecharts" -command AAcomparPieChart
	frame $w2.map
	ttk::button $w2.map.zbase- -text "-" -width 1 -command "ZoomAAcomparPieChart -"
	ttk::button $w2.map.zbase+ -text "+" -width 1 -command "ZoomAAcomparPieChart +"

	pack $w2.map.zbase- $w2.map.zbase+ -side left -expand true -fill both
	pack $w2.aa $w2.bub $w2.pie $w2.map -expand true -fill x
	pack $w2 -expand true -fill x 
}
#  
proc pamexit {} {
	set choix [tk_messageBox -type okcancel -default ok  -message "Exit PAELA ?" -icon question]
	if {$choix == "ok"} {
		exit
	}
}
#
proc MakeFlagLabelFrame  {w t {i boule} {bg #DCDAD5} {fg #003366} } {
	$w configure -background $bg -relief ridge
	set labwload [frame $w.w -background $bg ]
	set labwloadimg [label $w.w.i -image $i -background $bg]
	set labwloadlab [label $w.w.l -text $t -background $bg -foreground $fg]
	pack $labwloadimg $labwloadlab -side left -expand true -fill x
	return $labwload
}
proc lbs {w  wi he {mode multiple}} {
	global IDT A T S
	#ttk::scrollbar $w.yscroll  -command "$w.l yview" -width 7 -background #DCDAD5
	#ttk::scrollbar $w.xscroll  -orient horizontal -command "$w.l xview " -width 7 -background #DCDAD5
	ttk::scrollbar $w.yscroll  -command "$w.l yview" 
	ttk::scrollbar $w.xscroll  -orient horizontal -command "$w.l xview " 
	listbox  $w.l  -xscroll "$w.xscroll set" -yscroll "$w.yscroll set" -background #DCDAD5 \
		-selectmode $mode -width $wi -height $he -exportselection 0
	grid $w.l -row 0 -column 0 -sticky news
	grid $w.yscroll -row 0 -column 1 -sticky ns
	grid $w.xscroll -row 1 -column 0 -sticky ew
	grid rowconfigure $w 0 -weight 1
	grid columnconfigure $w 0 -weight 1
	return $w
}
proc pamhelp {} {
	global S
	destroy .panhelp
	set p [toplevel .panhelp -relief raised -borderwidth 2]
	wm title $p "PAELA Help" 
	set x [expr ([winfo screenwidth $p] /2) - 1000/2]
	set y [expr ([winfo screenheight $p] /2) - 1000/2]
	wm geometry $p "1200x1000+$x+$y"
	#OSX
	set S(SCRIPTdir) [file dirname [info script]]
	set comptI 0
	text $p.manual
	pack $p.manual -expand true -side top -fill both
	set m $p.manual
	set comptI 0
	set directory [file join $S(SCRIPTdir) manual]
	foreach file [glob -nocomplain -directory $directory *.jpg] {
		image create photo $comptI -file $file
		$m image create end -image $comptI
		incr comptI
	}	
}
################################################################################

################################################################################
# IMPORT ANNOTATION matrice croisant feuilles x variables
# vers matrice disjonctive pour une variable
################################################################################
proc SelectFiletoImportAnnotation {} {
	global S
	set S(annotationfiletoimport) [tk_getOpenFile -initialdir $S(pathuser) -title "Select ANNOTATION file... "]
	if {$S(annotationfiletoimport) != ""} {
		set S(pathuser) [file dirname $S(annotationfiletoimport)]
		update idletasks
		if [catch {open $S(annotationfiletoimport) r} fid] {
			return 0
		} else {
			set lvariables [gets $fid ] 
			regsub -all "," $lvariables " " lvariables
			regsub -all "'" $lvariables "" lvariables
			$S(annotationvariableslistbox) delete 0 end
			foreach v [lrange $lvariables 1 end] {
				$S(annotationvariableslistbox) insert end $v
			}
			close $fid
		}
	}
}
#
proc ANNOTATIONImport {} {
	global S
	set aindex [$S(annotationvariableslistbox) curselection]
	if {$aindex == ""} {
		tk_messageBox -type ok  -message "Please, select a variable of annotation" -icon info
		return
	} elseif {[llength $aindex] > 1}  {
		tk_messageBox -type ok  -message "Please, select only ONE variable of annotation" -icon info
		return
	} else {
		set variable [$S(annotationvariableslistbox) get $aindex]
	}
	set S(AnnotationImportVariable) $variable
	
	if [catch {open $S(annotationfiletoimport) r} fid] {
		return 0
	} else {
		set lvariables [gets $fid] 
		regsub -all "," $lvariables " " lvariables
		regsub -all "'" $lvariables "" lvariables
		set listeID {}
		while {[eof $fid] != 1} {
			set row  [gets $fid]
			regsub -all "," $row " " row
			regsub -all "'" $row "" row
			set id [lindex $row 0]
			foreach var [lrange $lvariables 1 end]  val [lrange $row 1 end] {
				if {$id != ""} {
					set AnnotationTranslate($id,$var) $val
					lappend listeID $id
				}
			}
			#
		}
		close $fid
		# domaine de valeur de la variable
		set ldomain {}
		foreach {k v} [array get AnnotationTranslate *,$S(AnnotationImportVariable)] {
			lappend ldomain $v
		}
		set ldomain [lsort -dictionary [DelRep $ldomain]]
		# SAVE ANNOTATION  for $S(AnnotationImportVariable)
		set GenName [lindex [split $S(annotationfiletoimport) /.] end-1] 
		set filenameAnnotation [file join $S(pathuser) [format "%s%s%s%s%s" PAELAexport- $GenName - $S(AnnotationImportVariable) -ANNOTATIONS.txt]] 
		set fid [open $filenameAnnotation w]
		# premiere ligne liste de toutes les annotations
		puts -nonewline $fid "ID"
		foreach v $ldomain {
			puts -nonewline $fid ",$v" 
		}
		puts $fid "\n"
		set listeID [lsort -dictionary -increasing $listeID]
		# export
		foreach n [DelRep $listeID] {
			puts -nonewline $fid "$n" 
			# respecter l'ordre des variables
			foreach v $ldomain {
				if {$AnnotationTranslate($n,$S(AnnotationImportVariable)) == $v} {
					puts -nonewline $fid ",1"
				} else {
					puts -nonewline $fid ",0" 
				} 
			}
			puts $fid "\n"
		}
		close $fid
	}
}
################################################################################


################################################################################
# IMPORT BEAST
################################################################################
#
proc SelectFiletoImport {} {
	global S TImport AImport NODEID
	set S(beastfiletoimport) [tk_getOpenFile -initialdir $S(pathuser) -title "Select BEAST file... "]
	if {$S(beastfiletoimport) != ""} {
		set S(pathuser) [file dirname $S(beastfiletoimport)]
		update idletasks
		if [catch {open $S(beastfiletoimport) r} fid] {
			return 0
		} else {
			set file_data [read $fid]
			close $fid
		}
		set data [split $file_data "\n"]
		set S(BeastNEXUSTree) ""
		set flag 0
		foreach row $data {
			if {[lsearch $row "trees;"] != -1} {
				set S(BeastNEXUSTree) [format "%s%s" $S(BeastNEXUSTree) $row]
				set flag 1
			} 
			if {[lsearch $row "end;"] != -1 && $flag == 1} {
				break
			} elseif {$flag == 1} {
				set S(BeastNEXUSTree) [format "%s%s" $S(BeastNEXUSTree) $row]
			}
		}
		# ICI inserer traitement des VRAIS labels feuilles
		# ils sont dans entete $S(BeastNEXUSTree) a ce stade
		#set fid [open z.txt w]
		#puts $fid [string range $S(BeastNEXUSTree) 0 [string first "(" $S(BeastNEXUSTree)]]  
		#close $fid
		#
		regsub -all " " $S(BeastNEXUSTree) "" S(BeastNEXUSTree) 
		regsub -all \" $S(BeastNEXUSTree) "" S(BeastNEXUSTree) 
		regsub -all "end" $S(BeastNEXUSTree) "" S(BeastNEXUSTree)
		regsub -all "End" $S(BeastNEXUSTree) "" S(BeastNEXUSTree)
		regsub -all ";" $S(BeastNEXUSTree) "" S(BeastNEXUSTree)
		set S(BeastNEXUSTree) [string range $S(BeastNEXUSTree) [string first "(" $S(BeastNEXUSTree)] end]
		set S(BeastNEXUSTree) [format "%s%s" $S(BeastNEXUSTree) ":0.0;"]
		regsub -all {\]} $S(BeastNEXUSTree) ">" S(BeastNEXUSTree)
		regsub -all {\[} $S(BeastNEXUSTree) "<" S(BeastNEXUSTree)
		catch {unset TImport} 
		catch {unset AImport}
		set NODEID 0
		DecomposeNewickBEAST $NODEID $S(BeastNEXUSTree)
		#DecomposeNewickBEAST $NODEID  "(A<V1={xa,xa},V3=za,V2={ya,ya,ya}>:1.0,B<V1={xb,xb},V3=zb,V2={yb,yb,yb}>:1.0)<V1={x0,x0},V3=z0,V2={y0,y0,y0}>:1.0"
		.pan.n.t.imp.listbox.l delete 0 end
		foreach {k v} [array get AImport $NODEID,*] {
			set var [lindex [split $k ,] end]
			.pan.n.t.imp.listbox.l insert end $var
		}
	}
}
# DecomposeNewickBEAST $NODEID  "('A'\[V1={xa,xa},V3=z,V2={ya,ya,ya}\]:1.0,'B'\[V1={xb,xb},V3=z,V2={yb,yb,yb}\]:1.0)\[V1={x0,x0},V3=z0,V2={y0,y0,y0}\]:1.0"
proc DecomposeNewickBEAST {node newick} {
	global TImport AImport NODEID
	if {[string first "(" $newick] == -1} {
		# FEUILLE 
		set tipListABranch [split $newick "':<>" ]
		set BranchLenght [lindex $tipListABranch end]
		set Label [lindex $tipListABranch 0]
		set Annotation [lrange $tipListABranch 1 end-2]
		set TImport($node,type) f
		set TImport($node,brlg) $BranchLenght
		set TImport($node,label) $Label
		ExtractAnnotationTip $node $Annotation
		return
	} else {
		# NODE
		set AnnotationAndBranchLenght [lindex [split $newick ')'] end]
		set index [string last ")" $newick ]
		set AnnotationAndBranchLenght [string range $newick [expr $index + 1] end]
		set newick [string range $newick 0 $index]
		set ListAnnotationAndBranchLenght [split $AnnotationAndBranchLenght :]
		set BranchLenght [lindex $ListAnnotationAndBranchLenght end]
		set Annotation [string range [lindex $ListAnnotationAndBranchLenght 0] 1 end-1]
		ExtractAnnotationNode $node $Annotation
		# decomposition et recursion bras droit , bras gauche
		# trouver la , separant les deux bras (attention les , dans la liste d'annotation ne doivent pas etre considerees)
		set i -1
		set id -1
		
		#set test 0
		set long [string length $newick ]
		for {set x 0} {$x <= $long} {incr x} {
			set id [expr $id +1]
			set c [string range $newick $x $x ]
			if {$c == "("} {
				set i [expr $i +1 ]
				continue
			} elseif {$c == ")"} {
				set i [expr $i -1 ]
				continue
			} 
			if {$c == "<"} {
				set test 0
				continue
			}
			if {$c == ">"} {
				set test 1
			}
			if {$c == ","} {
				if {$test == 1} {
					set test 0
					if {$i == 0} {
						set bg [string range $newick 1 $id-1]
						set bd [string range $newick [expr $id + 1] end-1]
						set NodeIDfilG [NODEIDincr]
						set NodeIDfilD [NODEIDincr]
						if {$node == 0} {
							set TImport($node,type) n
							set TImport($node,brlg) 0
							set TImport($node,fils1) $NodeIDfilG
							set TImport($node,fils2) $NodeIDfilD
						} else {
							set TImport($node,type) n
							set TImport($node,brlg) $BranchLenght
							set TImport($node,fils1) $NodeIDfilG
							set TImport($node,fils2) $NodeIDfilD
						}
						DecomposeNewickBEAST $NodeIDfilG $bg
						DecomposeNewickBEAST $NodeIDfilD $bd
						return
					}
				}
			} else {
				continue
			}
		}
	}
}
#
proc ExtractAnnotationTip {node annotation} {
	global AImport 
	regsub -all "," $annotation " " LA
	regsub -all "=" $LA " " LA
	regsub -all "%" $LA "" LA
	regsub -all "&" $LA "" LA
	foreach {va lv} $LA {
		set AImport($node,$va) [string map {\\ ""} $lv]
	}
}
#
proc ExtractAnnotationNode {node annotation} {
	global AImport 
	regsub -all "," $annotation " " LA
	regsub -all "=" $LA " " LA
	regsub -all "%" $LA "" LA
	regsub -all "&" $LA "" LA
	foreach {va lv} $LA {
		foreach vi $lv {
			lappend AImport($node,$va) $vi
		}
	}
}
#
proc BuiltNewick {n} {
	global TImport NewNewick
	set fils1 $TImport($n,fils1)
	set fils2 $TImport($n,fils2)
	set dbl $TImport($n,brlg)
	set NewNewick [format "%s%s%s%s%s%s%s%s%s" "ooo" $fils1 "," $fils2 "ccc" $n ":" $dbl ";" ]
	RebuildTreeNewickStringForFils1 $fils1
	RebuildTreeNewickStringForFils2 $fils2
	regsub -all "ooo" $NewNewick "(" NewNewick
	regsub -all "ccc" $NewNewick ")" NewNewick
	return $NewNewick
}
#
proc RebuildTreeNewickStringForFils1 {n} {
	global TImport NewNewick
	set typ $TImport($n,type)
	set dbl $TImport($n,brlg)
	if {$typ == "f"} {
		set adds1 [format "%s%s%s%s%s" "ooo" $n ":" $dbl ","]
		set p1 [format "%s%s%s" "ooo" $n ","]
		regsub -all  $p1 $NewNewick $adds1 NewNewick
		return
	} else {
		set fils1 $TImport($n,fils1)
		set fils2 $TImport($n,fils2)
		set adds1 [format "%s%s%s%s%s%s%s%s%s%s" "ooo" "ooo" $fils1 "," $fils2 "ccc" $n ":" $dbl "," ]
		set p1 [format "%s%s%s" "ooo" $n ","]
		regsub -all  $p1 $NewNewick $adds1 NewNewick
		RebuildTreeNewickStringForFils1 $fils1
		RebuildTreeNewickStringForFils2 $fils2
		return 
	}
}
#
proc RebuildTreeNewickStringForFils2 {n} {
	global TImport NewNewick
	set typ $TImport($n,type)
	set dbl $TImport($n,brlg)
	if {$typ == "f"} {
		set adds2 [format "%s%s%s%s%s" "," $n ":" $dbl "ccc"]
		set p2 [format "%s%s%s" "," $n "ccc"]
		regsub -all $p2 $NewNewick $adds2 NewNewick
		return
	} else {
		set fils1 $TImport($n,fils1)
		set fils2 $TImport($n,fils2)
		set adds2 [format "%s%s%s%s%s%s%s%s%s%s" "," "ooo" $fils1 "," $fils2 "ccc" $n ":" $dbl "ccc" ]
		set p2 [format "%s%s%s" "," $n "ccc"]
		regsub -all $p2 $NewNewick $adds2 NewNewick
		RebuildTreeNewickStringForFils1 $fils1
		RebuildTreeNewickStringForFils2 $fils2
		return 
	}
}
#
proc BEASTImportEtExport {} {
	global IDT TImport AImport S T AA A SA SApost provisoire
	# selection user variable d'annotation, si vide info
	set aindex [.pan.n.t.imp.listbox.l curselection]
	if {$aindex == ""} {
		tk_messageBox -type ok  -message "Please, select a variable of annotation" -icon info
		return
	} elseif {[llength $aindex] > 1}  {
		tk_messageBox -type ok  -message "Please, select only ONE variable of annotation" -icon info
		return
	} else {
		set variable [.pan.n.t.imp.listbox.l get $aindex]
	}
	set S(NewikBeastImportVariable) $variable
	# TREE (en premier) 
	set s [BuiltNewick 0]
	# nom des fichiers export (tree et annotation)
	# SAVE TREE
	set GenName [lindex [split $S(beastfiletoimport) /.] end-1] 
	set filenameTree [file join $S(pathuser) [format "%s%s%s%s%s" PAELAexport- $GenName - $S(NewikBeastImportVariable) -TREE.txt]] 
	set filenameAnnotation [file join $S(pathuser) [format "%s%s%s%s%s" PAELAexport- $GenName - $S(NewikBeastImportVariable) -ANNOTATION.txt]] 
	set filenameSupport [file join $S(pathuser) [format "%s%s%s%s%s" PAELAexport- $GenName - $S(NewikBeastImportVariable) -SUPPORT.txt]] 
	set fid [open $filenameTree w]
	puts $fid $s 
	close $fid
	# SAVE ANNOTATION
	set fid [open $filenameAnnotation w]
	# premiere ligne liste de toutes les annotations
	# ERROR : $AImport(0,$S(NewikBeastImportVariable)) n'a pas toujours toutes les annotations
	set lllaaa {}
	foreach {k v} [array get AImport *,$S(NewikBeastImportVariable) ] {
		foreach vv $v {
			lappend lllaaa $vv
		}
	}
	set listecompleteannotation [lsort -dictionary [DelRep $lllaaa]]
	puts -nonewline $fid "ID"
	foreach v $listecompleteannotation {
		puts -nonewline $fid ",$v" 
	}
	puts $fid "\n"
	# liste des nodes
	set lnode {}
	foreach {k v} [array get AImport *,$S(NewikBeastImportVariable)] {
		set node [lindex [split $k ,] 0]
		lappend lnode $node
	}
	set lnode [lsort -dictionary $lnode]
	# 
	catch {unset provisoire}
	foreach n $lnode {
		foreach v $AImport($n,$S(NewikBeastImportVariable)) p $AImport($n,$S(NewikBeastImportVariable).prob) {
			set provisoire($n,$v) $p
		}
	}
	foreach n $lnode {
		puts -nonewline $fid "$n" 
		# respecter l'ordre des variables
		foreach v $listecompleteannotation {
			if {[info exists provisoire($n,$v)] == 0} {
				puts -nonewline $fid ",0" 
			} else {
				puts -nonewline $fid ",$provisoire($n,$v)" 
			}
		}
		puts $fid "\n"
	}
	close $fid
	
	# SAVE SUPPORT
	set fid [open $filenameSupport w]
	#puts $fid "ID,support" 
	# liste des nodes : on l'a dejà
	foreach n $lnode {
		if {$TImport($n,type) != "f"} {
			puts $fid "$n,$AImport($n,posterior)"
		}
	}
	close $fid
	# message utilisateur 
	tk_messageBox -type ok  -message "Tree, Support & Annotations saved ($S(pathuser)) " -icon info
	# chargement direct
	LoadTreeGo $filenameTree
	LoadAnnotationGo $filenameAnnotation
	LoadSupportGo $filenameSupport
	
}
################################################################################


################################################################################
# MOUSE EVENTS
################################################################################
proc D3Switch {w x y} {
	if {[winfo exists $w.d3menu ] == 1} {destroy $w.d3menu}
	set m [menu $w.d3menu -tearoff 0]
	set i [$w find withtag current]
	set x [$w canvasx $x]
	set y [$w canvasy $y]
	if {$i != {}}  {
		set tags [$w gettags $i]
		if {[lsearch -glob $tags LEGEND] != -1} {
			D3legend $w $m $i
		} elseif  {[lsearch -glob $tags TMCIRCLE ] != -1} {
			D3TMCIRCLE $w $m $i
		} elseif  {[lsearch -exact $tags X ] != -1} {
			
		} elseif  {[lsearch -exact $tags Y] != -1} {
			
		}
		tk_popup $m [winfo pointerx $w] [winfo pointery $w]
	}
}

proc D3legend {w m} {
	global IDT S
	$m add command -label "Legend" 
	$m add separator
	$m add command -label "Delete" -command "$S(canvas) delete LEGEND"
}
#
proc D3TMCIRCLE {w m i} {
	global IDT S
	$m add command -label "Transition" 
	$m add separator
	$m add command -label "Highlight tree" -command "TransitionMapHiTreeNode"
}
proc AnyEnterCanvas {w x y} {
	global IDT A T S
	bindtags $w [list $S(tool) $w Canvas . all ]
	#  ToolIcon $w $x $y
}
proc AnyLeaveCanvas {w x y} {
	$w delete mousetool
	bindtags $w {}
}
proc ToolIcon {w x y} {
	global IDT A T S
	set x [$w canvasx $x]
	set y [$w canvasy $y]
	$w delete mousetool
	$w create image [expr $x +20] [expr $y +20] -image $S(tool)  -tags "mousetool"
}
#
proc CanvasBind {c} {
	global IDT S tcl_platform
	# TOOLBOX current tool
	bind $c <Any-Enter> "AnyEnterCanvas %W %x %y"
	bind $c <Any-Leave> "AnyLeaveCanvas %W %x %y"
	$c bind Z <Any-Enter> "AnyEnterNode %W %x %y"
	$c bind Z <Any-Leave> "AnyLeaveNode %W %x %y"
	bind $c <Motion> {ToolIcon %W %x %y}
	# MENU CONTEXTUELS
	if {$tcl_platform(os) == "Darwin"} {
		bind $c <Button-2> {D3Switch %W %x %y}
		bind $c <Control-Button-1> {D3Switch %W %x %y}
		bind $c <Button-3> {D3Switch %W %x %y}
	} else {
		bind $c <Button-3> {D3Switch %W %x %y}
	}
	# TOOLBOX 
	bind zoomw  <Button-1> "Sfittocontent %W"
	bind zoomf <Button-1> "zoomMark %W %x %y"
	bind zoomf <B1-Motion> "zoomStroke %W %x %y"
	bind zoomf <ButtonRelease-1> "zoomArea %W %x %y"
	#bind zoomf <Control-MouseWheel> "zoom %W 1.2"
	bind zoom+ <Button-1> "zoom %W 1.2"
	bind zoom- <Button-1> "zoom %W 0.8"
	bind zoomy+ <Button-1> "zoom2 %W 1 1.2"
	bind zoomy- <Button-1> "zoom2 %W 1 0.8"
	bind zoomx+ <Button-1> "zoom2 %W 1.2 1"
	bind zoomx- <Button-1> "zoom2 %W 0.8 1"
	bind g-move <Button-1> {ScenarioMakeGraphic %W %x %y}
	bind scenariobar <Button-1> {ScenarioBarMajoritaire %W %x %y}
	bind scenariobarpupko <Button-1> {ScenarioBarPupko %W %x %y}
	bind g-move2 <Button-1> {CompassMakeGraphic %W %x %y}
	bind g-expand <Button-1> {ScenarioMakeComparGraphic %W %x %y}
	#bind g-movesubtree <Button-1> {sb-movestart %W %x %y}
	bind g-movesubtree <Button-1> {SizeLineToolBox %W +}
	bind moveall <Button-1> "ToolboxMoveAll %W %x %y"
	bind moveall <ButtonRelease-1> "ToolboxMoveAllStop %W %x %y"
	bind s-swap <Button-1> {Swap %W }
	bind root <Button-1> {RootCap %W }
	bind s-size-+ <Button-1> "SizeLineToolBox %W +"
	bind s-size-+ <Shift-Button-1> "SizeLineToolBox %W -"
	bind s-color <Button-1> {S-color-ToolBox %W}
	bind s-color <Shift-Button-1> {S-color-ToolBox2 %W}
	$c bind bulle <B1-Motion> {BLLmove %W %x %y}
	bind what <Button-1> {BLLwhat %W %x %y}
	bind infowhat <Button-1> {infowhatMake %W %x %y}
	bind tresholdbar <B1-Motion> {TransitionMapDynamicMove %W %x %y}
	#bind tresholdbar <Button-1> {TransitionMapDynamicD1 %W %x %y}
	#bind TMCIRCLE <Button-1> {TransitionMapHiTreeNode %W %x %y}
	#bind TMCIRCLE <ButtonRelease-1> {TransitionMapHiTreeNodeRelease %W %x %y}
	bind fgcoloronthefly <Button-1> {FgTreeColorPath %W %x %y}
	bind fgcoloronthefly <Shift-Button-1> {FgTreeColorPathShift %W %x %y}
	$c bind TMCIRCLE <Any-Enter> "TransitionMapHiTreeNodeEnter %W %x %y"
	$c bind TMCIRCLE <Any-Leave> "TransitionMapHiTreeNodeLeave %W %x %y"
}
###
proc ToolboxMoveAll {w x y} {
	global S 
	set S(mox) $x
	set S(moy) $y
	set S(mov) all
	bind moveall <B1-Motion> "ToolboxMoveAllMotion %W %x %y"
}
###
proc ToolboxMoveAllMotion {w x y} {
	global S 
	$w move $S(mov) [expr $x - $S(mox)] [expr $y - $S(moy)]
	set S(mox) $x
	set S(moy) $y
}
###
proc ToolboxMoveAllStop {w x y} {
	global S 
	#$w configure -scrollregion [$w bbox all]
}
#
proc AnyLeaveNode {w x y} {
	global IDT S
	$w itemconfigure current  -fill $S(fgcolornodetransient)
}
#
proc AnyEnterNode {w x y} {
	global IDT S
	set S(fgcolornodetransient) [lindex [$w itemconfigure current -fill] end]
	$w itemconfigure current  -fill magenta
}
################################################################################


################################################################################
# BALLON HELP TOOLTIP
################################################################################
#
proc setTooltip {widget text} {
	if { $text != "" } {
		bind $widget <Any-Enter> [list after 100 [list showTooltip $text]]
		#bind $widget <Any-Leave> [list after 100 [list showTooltip "?"]]
	}
}
#
proc makeTooltip {} {
	global S 
	catch { destroy .help }
	if {$S(TooltipStringOnOff) == 1} {
		toplevel .help -bd 1 -bg black -width 200 -height 100
		#text .help.edit -yscrollcommand ".help.edit.scroll set" -width 200 -height 100  -background "#FAFAD2" 
		text .help.edit  -width 50 -height 20  -background "#FAFAD2" 
		set S(HelpPATH) .help.edit
		#scrollbar .help.edit.scroll -command "$S(HelpPATH) yview" 
		#set S(HelpPATHscroll) .help.edit.scroll
		pack $S(HelpPATH) -expand true -fill both
		#pack $S(HelpPATHscroll) -side right -fill y
		wm title .help "Paela help"
	}
}
#
proc showTooltip {text} {
	global S
	destroy $S(HelpPATH)
	if {[winfo exist .help]} {
		#destroy $S(HelpPATHscroll)
		#text $S(HelpPATH) -yscrollcommand "$S(HelpPATHscroll) set" -width 50 -background "#FAFAD2" -height 5
		text $S(HelpPATH)  -width 50 -background "#FAFAD2" -height 5
		#scrollbar $S(HelpPATHscroll) -command "$S(HelpPATH) yview" 
		pack $S(HelpPATH) -expand true -fill both
		#pack $S(HelpPATHscroll) -side right -fill y
		$S(HelpPATH) insert end $text
		$S(HelpPATH) see end
	}
}
proc HelpText {t} {
	switch -exact $t {
		FileLoad {
			return "\nLoad a Newick string (with or without node identifiers, see manual) \
					\nLoad primary annotations, with or without ancestral annotations (the tree newick string must contains node identifiers, see manual) \
					\nLoad supports (the tree newick string must contains node identifiers, see manual)"
	 	}
		FileBeastImport  {
			return "BEAST import 1) Clic the 'Select Beast file' button and select a BEAST output, wait for the parsing of the data then, 2) select the variable of annotation to extract and 3) clic the 'Import' button. The tree and the annotations will be automatically loaded but 3 files will also created in the same file’s path than the BEAST file: the tree, the annotations and the support values."
		}
		FileAnnotationImport {
			return "Select the Strains x Variables file then select one of its variable to extract, and clic the Import button"
		}
		FileSave {
			return "Save"
		}
		FilePreferences {
			return "Preferences"
		}
		EditGlobal {
			return "Edit Global"
		}
		EditTree {
			return "Edit Tree"
		}
		EditAnnotation {
			return "Edit Annotation"
		}
	}
}	
################################################################################

################################################################################
# GESTION MAP
################################################################################
#
proc MakeMap {pw} {
	global IDT A T S
	set S(MapNumber) [expr $S(MapNumber) +1]
	set id $S(MapNumber)
	set IDT $id
	lappend S(MapList) $id
	set S(BoardNumber,$id) 1
	set idboard1 [format "%s%s" $id .1]
	lappend S(BoardList,$id) $idboard1
	# MAP 
	set w [format "%s%s%s" $pw .m $id]
	set w [labelframe $w -text "Map #$id"  -foreground black ]
	set S($IDT,path) $w
	bind $w <Button-1> "MapCurrent $id"
	$pw add $w -weight 50 
	set m [ttk::panedwindow $w.p -orient horizontal ]
	set mMain [labelframe $m.main -text "Tree" -background #CCC -foreground black ]
	set mBoard [labelframe $m.board -text "Panels" -background #CCC -foreground black ]
	$m add $mMain -weight 80
	$m add $mBoard -weight 20
	pack $w.p -expand true -fill both
	# MAIN 
	set map $mMain.canvas
	ttk::frame $map
	ttk::scrollbar $map.xsbar -orient horizontal -command "$map.c xview"
	ttk::scrollbar $map.ysbar -orient vertical -command "$map.c yview"
	canvas $map.c -background white \
		-xscrollcommand "$map.xsbar set" -yscrollcommand "$map.ysbar set" \
		-highlightthickness 0 -background $S(CanvasBackground)
	grid $map.c -row 0 -column 0 -sticky news
	grid $map.xsbar -row 1 -column 0 -sticky ew
	grid $map.ysbar -row 0 -column 1 -sticky ns
	grid rowconfigure $map 0 -weight 1
	grid columnconfigure $map 0 -weight 1
	pack $map -expand true -fill both
	CanvasBind $map.c
	set S(canvas) $map.c
	# BOARD 
	set m [ttk::panedwindow $mBoard.p -orient vertical ]
	set m1 [labelframe $m.1 -text "#$IDT.1"  -foreground black ] ; # board 1
	set S($idboard1,path) $m1
	bind $m1 <Button-1> "BoardCurrent $idboard1"
	$m add $m1 -weight 50
	pack $m -expand true -fill both
	set map $m1.canvas
	ttk::frame $map
	ttk::scrollbar $map.xsbar -orient horizontal -command "$map.c xview"
	ttk::scrollbar $map.ysbar -orient vertical -command "$map.c yview"
	canvas $map.c -background white -height 10 \
		-xscrollcommand "$map.xsbar set" -yscrollcommand "$map.ysbar set" \
		-highlightthickness 0
	grid $map.c -row 0 -column 0 -sticky news
	grid $map.xsbar -row 1 -column 0 -sticky ew
	grid $map.ysbar -row 0 -column 1 -sticky ns
	grid rowconfigure $map 0 -weight 1
	grid columnconfigure $map 0 -weight 1
	pack $map -expand true -fill both
	CanvasBind $map.c
	set S(canvas2) $map.c
	UpdateMapColor $id
}
#
proc MakeBoard {} {
	global IDT A T S
	incr S(BoardNumber,$IDT)
	set idboard [format "%s%s%s" $IDT . $S(BoardNumber,$IDT)]
	lappend S(BoardList,$IDT) $idboard 
	# BOARD panedwindow
	set m [format "%s%s%s" .pan.g.g.g.m $IDT .p.board.p] ;# panedwindow
	set wi [format "%s%s%s" $m . $S(BoardNumber,$IDT)]
	set m1 [labelframe $wi -text "#$idboard"  -foreground black ]
	set S($idboard,path) $m1
	bind $m1 <Button-1> "BoardCurrent $idboard"
	
	$m add $m1 -weight 50
	# BOARD canvas  
	set map $m1.canvas
	ttk::frame $map
	ttk::scrollbar $map.xsbar -orient horizontal -command "$map.c xview"
	ttk::scrollbar $map.ysbar -orient vertical -command "$map.c yview"
	canvas $map.c -background white -width 500 -height 500 \
		-xscrollcommand "$map.xsbar set" -yscrollcommand "$map.ysbar set" \
		-highlightthickness 0
	grid $map.c -row 0 -column 0 -sticky news
	grid $map.xsbar -row 1 -column 0 -sticky ew
	grid $map.ysbar -row 0 -column 1 -sticky ns
	grid rowconfigure $map 0 -weight 1
	grid columnconfigure $map 0 -weight 1
	pack $map -expand true -fill both
	CanvasBind $map.c
	# UPADTE
	# par defaut ce board devient le board actif
	UpdateMapColor $IDT
	update idletasks
	BoardCurrent $idboard
}
#
proc RemoveBoard {} {
	global IDT A T S
	if {[llength $S(BoardList,$IDT)] >= 2} {
		set idboard [lindex [split $S(canvas2) . ] end-2]
		set m [format "%s%s%s" .pan.g.g.g.m $IDT .p.board.p] ;# panedwindow
		set wi [format "%s%s%s" $m . $idboard]
		$m forget $wi
		set ID [format "%s%s%s" $IDT . $idboard]
		set l $S(BoardList,$IDT)
		set S(BoardList,$IDT) {}
		foreach e $l {
			if {$e != $ID} {
				lappend S(BoardList,$IDT) $e
			}
		}
		unset S($ID,path)
		# UPADTE
		UpdateMapColor $IDT
		update idletasks
		BoardCurrent [lindex $S(BoardList,$IDT) end]
	}
}
#
proc MapTarget {m} {
	global IDT A T S
	if {[info exist S(MapList)] &&  $S(MapList) != ""} {
		$m delete 0 end
		foreach id $S(MapList) {
			$m add command -label "Map #$id" -command "MapCurrent $id"
		}
		$m add separator
		$m add command -label "Info" -command MapGlobalInfo
	} else {
		$m add command -label "No map"
	}
}
#
proc MapGlobalInfo {} {
	global S A T
	$S(mainnotebook) select 1
	foreach id $S(MapList) {
		$S(infopath) insert end "MAP $id :\n"
		$S(infopath) insert end  "\t Tree file: $S($id,filetree), [llength $T($id,ue_lab)] strains\n"
		$S(infopath) insert end  "\t Annotation file: $S($id,fileannotation), [llength $A($id,AnnotationList)] value(s):\n"
		# frequency
		foreach annotation [lsort -dictionary $A($id,AnnotationList)] {
			set A($id,f,$annotation) 0.0
			foreach ue $T($id,ue_cod) {
				set feuille $T($id,dbv,$ue)
				set A($id,f,$annotation) [expr $A($id,f,$annotation) + $A($id,$feuille,$annotation)]
			}
			set A($id,f,$annotation) [expr double($A($id,f,$annotation) / $T($id,nbue))]
			$S(infopath) insert end "\t\t $annotation : [expr 100.00 * [rnd $A($id,f,$annotation) 2]] %\n "
		}
	}
	$S(infopath) see end
}
#
proc MapTargetCompare {m} {
	global IDT A T S
	if {[info exist S(MapList)] &&  $S(MapList) != ""} {
		$m delete 0 end
		foreach id $S(MapList) {
			set i [lsearch $S(AAIDTlist) $id]
			if {$i == -1} {set s ""} else {set s ">"}
			if {$id != $IDT} {
				$m add command -label "$s Map #$id" -command "MapTargetCompareGo $id"
			}
		}
	} else {
		$m add command -label "No map"
	}
}
proc MapTargetCompareGo {id} {
	global IDT A T S
	if {[info exist S(AAIDTlist)]} {
		set i [lsearch $S(AAIDTlist) $id]
		if {$i == -1} {
			# ajout
			lappend S(AAIDTlist) $id
		} else {
			# retrait
			set l [concat [lrange $S(AAIDTlist) 0 [expr $i -1]] " " [lrange $S(AAIDTlist) [expr $i+1] end]]
			set S(AAIDTlist) $l
		}
	} 
}
#
proc MapRemove {m} {
	global IDT A T S
	if {[info exist S(MapList)] &&  $S(MapList) != ""} {
		$m delete 0 end
		foreach id $S(MapList) {
			$m add command -label "Map #$id" -command "MapRemoveGo $id"
		}
	} else {
		$m add command -label "No map"
	}
}
#
proc MapRemoveGo {id} {
	global IDT A T S
 	# mise a jour liste
	set lnew {}
	foreach mapi $S(MapList) {
		if {$mapi != $id} {
			lappend lnew $mapi
		}
	}
	set S(MapList) $lnew
	#
	destroy .pan.g.g.g.m$id
	# si la map supprimée etait en target remettre une map en target
	# par defafut la dernniere 
	set pathmapdelete [format "%s%s%s" .pan.g.g.g.m $id .p.main.canvas.c]
	if {$S(canvas) == $pathmapdelete} {
		MapCurrent [lindex $S(MapList) end]
	}
}
#
proc MapDisplay {m} {
	global IDT A T S
	if {[info exist S(MapList)] &&  $S(MapList) != ""} {
		$m delete 0 end
		foreach id $S(MapList) {
			set map [format "%s%s" .pan.g.g.g.m $id]
			set S(ismapped,$id) [winfo ismapped $map]
			$m add checkbutton -label "Map #$id" -variable S(ismapped,$id) -command "MapDisplayGo $id"
		}
	} else {
		$m add command -label "No map"
	}
}
#
proc MapDisplayGo {id} {
	global IDT A T S
	set map [format "%s%s" .pan.g.g.g.m $id]
	if {$S(ismapped,$id)} {
		.pan.g.g.g add $map
	} else {
		.pan.g.g.g forget $map
	}
}
# 
proc MapCurrent {id} {
	global IDT A T S
	set IDT $id
	set S(canvas) [format "%s%s%s" .pan.g.g.g.m $id .p.main.canvas.c]
	# par defaut le board 1
	set S(canvas2) [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. 1 .canvas.c]
	UpdateMapColor $id
	# update listbox liste des sequences pour transition curves
	#$S(strainlistpath) delete 0 end
	#foreach l [lsort -dictionary -increasing $T($IDT,ue_lab)] {
	#	$S(strainlistpath) insert end $l
	#}
}
#
proc BoardCurrent {id} {
	global IDT A T S
	set idmain [lindex [split $id "."] 0]
	set idboard [lindex [split $id "."] end]
	set IDT $idmain
	set S(canvas) [format "%s%s%s" .pan.g.g.g.m $idmain .p.main.canvas.c]
	set S(canvas2) [format "%s%s%s%s%s" .pan.g.g.g.m $idmain .p.board.p. $idboard .canvas.c]
	UpdateMapColorFromBoard $id
	
}
# id map
proc UpdateMapColor {id} {
	global IDT A T S
	# on eteint tout
	foreach idi $S(MapList) {
		$S($idi,path) configure -background "#DDD"
		foreach idb $S(BoardList,$idi) {
			$S($idb,path) configure -background "#DDD"
		}
	}
	# on active
	# main
	$S($id,path) configure -background $S(CurrentMapColor) 
	# par defaut le premier board
	set idi [lindex $S(BoardList,$id) 0]
	# "#FAFAD2"
	$S($idi,path) configure -background $S(CurrentPanelColor)
}
# id board
proc UpdateMapColorFromBoard {id} {
	global IDT A T S
	# on eteint tout
	foreach idi $S(MapList) {
		$S($idi,path) configure -background "#DDD"
		foreach idb $S(BoardList,$idi) {
			$S($idb,path) configure -background "#DDD"
		}
	}
	# on active
	# main
	set idmap [lindex [split $id "."] 0]
	$S($idmap,path) configure -background $S(CurrentMapColor) 
	$S($id,path) configure -background $S(CurrentPanelColor)
}
################################################################################

################################################################################
# IN/OUT
################################################################################
#
proc LoadTree {} {
	global IDT A T S
	set file [tk_getOpenFile -initialdir $S(pathuser) -title "Load Tree... "]
	if {$file != "" } {
		LoadTreeGo $file
	}
}
proc LoadTreeGo {file} {
	global IDT A T S
	$S(infopath) insert end "> $file\n"
	$S(infopath) see current
	if {$file != "" } {
		set S(pathuser) [file dirname $file]
		set S($IDT,filetree) [file tail $file]
		CreateTree $file
		# mini canvas brush
		PhyNJmini $S(canvasmini)
		zoomwindow $S(canvasmini) black
		#foreach l [lsort -dictionary -increasing $T($IDT,ue_lab)] {
		#	$S(strainlistpath) insert end $l
		#}
	}
}
#
proc LoadTreeFile {file} {
	global IDT A T S
	$S(infopath) insert end "> $file\n"
	$S(infopath) see current
	if {$file != "" } {
		set S(pathuser) [file dirname $file]
		set S($IDT,filetree) [file tail $file]
		CreateTree $file
		# mini canvas brush
		PhyNJmini $S(canvasmini)
		zoomwindow $S(canvasmini) black
	}
	#foreach l [lsort -dictionary -increasing $T($IDT,ue_lab)] {
	#	$S(strainlistpath) insert end $l
	#}
}
#
proc LoadAnnotation {} {
	global IDT A T S
	set file [tk_getOpenFile -initialdir $S(pathuser) -title "Load Annotations... "]
	if {$file != "" } {
		LoadAnnotationGo $file
	}
}
#
proc LoadAnnotationGo {file} {
	global IDT A T S
	$S(infopath) insert end "> $file\n"
	$S(infopath) see current
	if {$file != "" } {
		set S($IDT,fileannotation) [file tail $file]
		set S(pathuser) [file dirname $file]
		CreateTableAnnotation $file
		$S(infopath) insert end "> [llength $A($IDT,AnnotationList)] value(s): $A($IDT,AnnotationList)\n"
		$S(infopath) see current
	}
	
}
#
proc LoadAnnotationFile {file} {
	global IDT A T S
	$S(infopath) insert end "> $file\n"
	$S(infopath) see current
	if {$file != "" } {
		set S($IDT,fileannotation) [file tail $file]
		set S(pathuser) [file dirname $file]
		CreateTableAnnotation $file
	}
	$S(infopath) insert end "> [llength $A($IDT,AnnotationList)] value(s): $A($IDT,AnnotationList)\n"
	$S(infopath) see current
}
# 
proc CreateTableAnnotation {file} {
	global IDT A T S AA
	if [catch {open $file r} fid] {
		return 0
	} else {
		set lvariables [gets $fid ] 
		regsub -all "," $lvariables " " lvariables
		regsub -all "'" $lvariables "" lvariables
		set A($IDT,AnnotationList) [lrange $lvariables 1 end]
		while {[eof $fid] != 1} {
			set row  [gets $fid]
			regsub -all "," $row " " row
			regsub -all "'" $row "" row
			set id [lindex $row 0]
			#######
			foreach var $A($IDT,AnnotationList) val [lrange $row 1 end] {
				if {$id != ""} {
					set nodeinterne $T($IDT,vbd,$id)
					set AA($IDT,distribution,$nodeinterne,$var) $val
					if {$T($IDT,typ,$nodeinterne) == "f"} {
						set A($IDT,$id,$var) $val
					}
				}
			}
		}
		close $fid
		# affectation de couleurs
		# progressive colors, une couleur par annotation
		switch -exact $S(ColorSetMode) {
			linear {
				set annotations [lsort -dictionary $A($IDT,AnnotationList)]
				set lc [ColorProgressive [llength $annotations]]
				foreach valeur $annotations c $lc {
					set S(ColorAnnotation,$valeur) $c
				}
			}
			hierarchic {
				set lc [ColorProgressive2]
			}
			loadcolor {
				set colorfilename [tk_getOpenFile -initialdir $S(pathuser) -title "Select COLORS file... "]
				if {$colorfilename != ""} {
					if [catch {open $colorfilename r} fid] {
						return 0
					} else {
						while {[eof $fid] != 1} {
							set row  [gets $fid]
							regsub -all "," $row " " row
							set a [lindex $row 0]
							set c [lindex $row end]
							set S(ColorAnnotation,$a) $c
						}
						close $fid
					}
				}
			}
		}
	}
	return 
}
#
proc LoadSupport {} {
	global IDT A T S
	set file [tk_getOpenFile -initialdir $S(pathuser) -title "Load Support... "]
	if {$file != "" } {
		LoadSupportGo $file
	}
}
proc LoadSupportGo {file} {
	global IDT A T S
	set S(pathuser) [file dirname $file]
	if [catch {open $file r} fid] {
		return 0
	} else {
		while {[eof $fid] != 1} {
			set row  [gets $fid]
			regsub -all "," $row " " row
			set node [lindex $row 0]
			set support [lindex $row 1]
			# attention le code n est interne, l'ID referancant les annotations est dbv
			if {$node != ""} {
				set nodea $T($IDT,dbv,$node)
				set T($IDT,$T($IDT,vbd,$node),support) $support
				#set T($IDT,$nodea,support) $support
				#set T($IDT,$node,support) $support
			}
		}
		close $fid
		return
	}
}
#
proc MapListExportAnnotation {m} {
	global IDT A T S
	$m delete 0 end
	if {[info exist S(MapList)] &&  $S(MapList) != ""} {
		foreach id $S(MapList) {
			$m add command -label "Map #$id" -command "ExportAnnotation $id"
			$m add separator
		}
	} else {
		$m add command -label "No map"
	}
}
#
proc MapListExportPS {m} {
	global IDT A T S
	$m delete 0 end
	if {[info exist S(MapList)] &&  $S(MapList) != ""} {
		foreach id $S(MapList) {
			set path1 [format "%s%s" $S($id,path) .p.main.canvas.c]
			$m add command -label "Map #$id tree" -command "ExportPS $path1"
			foreach idb $S(BoardList,$id) {
				set idboard [lindex [split $idb "."] end]
				set pathb [format "%s%s%s%s" $S($id,path) .p.board.p. $idboard .canvas.c]
				$m add command -label "Map #$idb view #$idb" -command "ExportPS $pathb"
			}
			$m add separator
		}
	} else {
		$m add command -label "No map"
	}
}
#
proc MapListExportSVG {m} {
	global IDT A T S
	$m delete 0 end
	if {[info exist S(MapList)] &&  $S(MapList) != ""} {
		foreach id $S(MapList) {
			set path1 [format "%s%s" $S($id,path) .p.main.canvas.c]
			$m add command -label "Map #$id tree" -command "ExportSVG $path1"
			foreach idb $S(BoardList,$id) {
				set idboard [lindex [split $idb "."] end]
				set pathb [format "%s%s%s%s" $S($id,path) .p.board.p. $idboard .canvas.c]
				$m add command -label "Map #$idb view #$idb" -command "ExportSVG $pathb"
			}
			$m add separator
		}
	} else {
		$m add command -label "No map"
	}
}
#
proc ExportPS {w} {
	global IDT A T S
	#set c $S(canvas)
	set c $w
	set typelist {
		{"Postscript Files" {".ps"} }
		{"All Files" {*}}
	}
	set file [tk_getSaveFile -initialdir $S(pathuser) -defaultextension ".ps" \
			-filetypes $typelist -title "Save graphic as PostScript file..."]
	if {$file != "" } {
		set S(pathuser) [file dirname $file]
		set x0 0
		set y0 0
		set x1 [winfo width $c]
		set y1 [winfo height $c]
		foreach {x0 y0 x1 y1} [$c bbox all] {}
		set w [expr $x1 - $x0]
		set h [expr $y1 - $y0]
		$c postscript  -file  $file -colormode color \
			-x $x0 -y $y0 -width $w -height $h \
			-pagex 0 -pagey 0 -pagewidth 20.c -pageheight 30.c \
			-pageanchor nw
	}
}
#
proc ExportSVG {w} {
	global IDT A T S
	set typelist {
		{"SVG Files" {".svg"} }
		{"All Files" {*}}
	}
	set file [tk_getSaveFile  -initialdir $S(pathuser) -defaultextension ".svg" \
			-filetypes $typelist  -title "Save Graphic as SVG file..."]
	if {$file != "" } {
		set S(pathuser) [file dirname $file]
		::can2svg::canvas2file $w $file
	}
}
#
proc ExportAnnotation {IDT} {
	global A T S AA 
	set typelist {
		{"Text Files" {".txt"} }
		{"All Files" {*}}
	}
	set file [tk_getSaveFile  -initialdir $S(pathuser) -defaultextension ".txt" \
			-filetypes $typelist  -title "Save Annotations..."]
	if {$file != "" } {
		set S(pathuser) [file dirname $file]
		set fid [open $file w]
		# premiere ligne variable
		puts  -nonewline $fid "ID"
		foreach v $A($IDT,AnnotationList) {
			puts -nonewline $fid ",$v" 
		}
		puts $fid "\n"
		# lignes suivantes IDnode, valeurs
		foreach id $T($IDT,all_cod) {
			set n $T($IDT,dbv,$id)
			puts -nonewline $fid "$n" 
			foreach v $A($IDT,AnnotationList) {
				puts -nonewline $fid ",$AA($IDT,distribution,$id,$v)"
			}
			puts $fid "\n"
		}
		close $fid
	}
}
################################################################################


################################################################################
# TREE 
################################################################################
#
proc Drawtree {} {
	global IDT A T S
	$S(canvas) delete all
	TreeInit
	PhyNJ $S(canvas)
	$S(canvas) configure -scrollregion [$S(canvas) bbox all]
	update idletasks
}
# TreeInit
proc TreeInit {} {
	global IDT A T S
	
	set S(tree-height) 1400 ; # OK ou auto
	set S(tree-width) 800 ; # OK
	set S(tree-font) {-family Arial -size 10 -slant italic -weight normal}  ; # OK
	set S(annotation-font) {-family Arial -size 8 -slant roman -weight normal}  ; # OK
	set S(tree-fg) black ; # OK
	set S(tree-state) normal ; # OK
	set S(tree-linewidth) 1 ; # OK
	#
	set S($IDT,rotation) 0
	set S($IDT,tree-linewidth) 1
}
#
proc PhyNJ {w} {
	global IDT A T S
	global G
	catch {unset G}
	set y 0
	set nb $T($IDT,nbue)
	set largeur $S(tree-width)
	if {$S(tree-height) == "auto"} {set hauteur [expr 10 * $nb]} else  {set hauteur $S(tree-height)}
	set fy [expr double($hauteur)  / $nb ]
	set fx [expr double($largeur) / $T($IDT,xmax)]
	set S(fx) $fx ; # utiliser ulterieurement (dessin scale)
	set S($IDT,fx) $fx ; # utiliser ulterieurement (dessin scale)
	set n 0
	### LEAVES
	foreach i $T($IDT,ue_cod) {
		incr n
		set y [expr $n * $fy ]
		set G($i,x) [expr ($T($IDT,sox,$i) - $T($IDT,dbl,$i)) * $fx]
		set G($i,y) $y
		$w create line [expr $T($IDT,sox,$i) * $fx] $G($i,y) $G($i,x) $G($i,y) -tags "$i Z H$i N$i"
		$w create text [expr ($T($IDT,sox,$i) * $fx) + 2]  $G($i,y) -text  $T($IDT,ctl,$i) -tags "EUL$T($IDT,ctl,$i) L " -anchor w -font $S(tree-font)
	}
	### EDGES
	for {set i [expr $T($IDT,tot) - 1 ]} {$i >= 1} {incr i -1} {
		foreach b $T($IDT,cbg,$i) {
			set G($b,y) [expr ($G($T($IDT,fils1,$b),y) + $G($T($IDT,fils2,$b),y)) / 2.0]
			set G($b,x) [expr $G($T($IDT,fils1,$b),x) - ($T($IDT,dbl,$b) * $fx)]
			# Vertical
			$w create line $G($T($IDT,fils1,$b),x) $G($T($IDT,fils1,$b),y) $G($T($IDT,fils1,$b),x) $G($T($IDT,fils2,$b),y) -tags "Z V$b N$b"
			# Horizontal
			$w create line $G($T($IDT,fils1,$b),x) $G($b,y) $G($b,x) $G($b,y)  -tags "Z H$b N$b"
		}
	}
	### ROOT
	set ch [$w coords H1]
	set cb [$w coords H2]
	set yh [lindex $ch 1]
	set yb [lindex $cb 1]
	set x [lindex $ch 2]
	$w create line $x $yh $x $yb -tags "V0 H0 Z"
}
################################################################################

################################################################################
# EDITION 
################################################################################

# EDITION CANVAS BACKGROUND COLOR
proc MapBackgroundColor {} {
	global S IDT
	set color [tk_chooseColor]
	if {$color != ""} {
		$S(canvas) configure -background $color
	}
}
#
proc BoardBackgroundColor {} {
	global S IDT
	set color [tk_chooseColor]
	if {$color != ""} {
		$S(canvas2) configure -background $color
	}
}
# EDITION TREE COLOR
### les arretes du sous-arbre
proc S-color-ToolBox {w} {
	global S IDT
	set tags [$w gettags current]
	set n [string range [lindex $tags [lsearch -glob $tags N*]] 1 end]
	if {$n != ""} {
		set color [tk_chooseColor]
		#set lnode [Node2SubTreeNodes $n]
		set lnode [Node2NodesAll $n]
		
		foreach ni $lnode {
				$w itemconfigure N$ni -fill $color
			}
	}
}
### que les feuilles sous n pointé
proc S-color-ToolBox2 {w} {
	global S IDT T
	set tags [$w gettags current]
	set n [string range [lindex $tags [lsearch -glob $tags N*]] 1 end]
	if {$n != ""} {
		set color [tk_chooseColor]
		set lnode [Node2NodesLeaf $n]
		foreach ni $lnode {
				$w itemconfigure EUL$T($IDT,ctl,$ni) -fill $color
			}
	}
}
### toutes les feuilles de l'arbre
proc SleafForegroundColor {} {
	global S
	set color [tk_chooseColor]
	if {$color != ""} {
		$S(canvas) itemconfigure L -fill $color
	}
}
# EDITION TREE LINE SIZE
proc SizeLineSet {size} {
	global IDT A T S 
	set w $S(canvas)
	set S($IDT,tree-linewidth) $size
	$w itemconfigure Z -width $size
}
###
proc SizeLine+ {} {
	global IDT A T S 
	set S($IDT,tree-linewidth) [expr $S($IDT,tree-linewidth) + 1]
	SizeLineSet $S($IDT,tree-linewidth)
}
###
proc SizeLine- {} {
	global IDT A T S 
	set S($IDT,tree-linewidth) [expr $S($IDT,tree-linewidth) - 1]
	if {$S($IDT,tree-linewidth) <= 0 } {
		set S($IDT,tree-linewidth) 1
	}
	SizeLineSet $S($IDT,tree-linewidth)
}
# 
proc SizeLineToolBox {w mode} {
	global IDT A T S 
	set tags [$w gettags current]
	set n [string range [lindex $tags [lsearch -glob $tags N*]] 1 end]
	if {$n != ""} {
		#set lnode [Node2SubTreeNodes $n]
		set lnode [Node2NodesAll $n]

		SizeLineToolBoxGo $w $mode $n $lnode
	}
}
#
proc SizeLineToolBoxGo {w mode n lnode} {
	global IDT A T S 
	# tous les enfants de n
	switch -- $mode {
		+ {
			foreach ni $lnode {
				set width [expr  [lindex [$w itemconfigure N$ni -width] end] +1]
				$w itemconfigure N$ni -width $width
			}
		}
		- {
			foreach ni $lnode {
				set width [expr  [lindex [$w itemconfigure N$ni -width] end] -1]
				if {$width <=0} {set width 1}
				$w itemconfigure N$ni -width $width
			}
		}
	}
}
# TREE LINE WIDTH ANNOTATION MAJORITAIRE
# epaisseur arrete proportionelle à la valeur de la majoritaire
proc TreeForegrounfColorMajorityAnnotation2 {} {
	global IDT A T S 
	set w $S(canvas)
	
	# recherche de la majoritaire
	foreach n $T($IDT,all_cod) {
		set max 0
		# attention le code n est interne, l'ID referancant les annotations est dbv $T($IDT,dbv,$n)
		set node $T($IDT,dbv,$n)
		foreach {k v} [array get A $IDT,$node,*] {
			set a [lindex [split $k ,] end]
			if {$v > $max} {
				set max $v
				set A($IDT,majoritaire,$n) $a
			}
		}
	}
	# 4 classes de valeur de la majoritaire/epaisseur de trait
	# T($IDT,allint_cod)
	foreach id $T($IDT,all_cod) {
		set node $T($IDT,dbv,$id)
		set v $A($IDT,$node,$A($IDT,majoritaire,$id))
		# 4 classes
		if {$v >= 0.75} {
			set wid 7
		} elseif {$v >= 0.50 && $v < 0.75} {
			set wid 5
		} elseif {$v >= 0.25 && $v < 0.50} {
			set wid 3
		} else {
			set wid 1
		}
		$w itemconfigure H$id -width $wid
	}
}


################################################################################
# TRANSITION MATRIX
################################################################################
### matrix transition : parsimony, majority, pupko
### et run
proc TransitionMatrixCAP {{seuil 1}} {
	global IDT T S A AA TansitionMatrix
	# DATA
	catch {unset TansitionMatrix}
	switch -exact $S(MatrixWhat) {
		majority {
			foreach n $T($IDT,all_cod) {
				set max 0
				foreach {k v} [array get AA $IDT,distribution,$n,*] {
					set a [lindex [split $k ,] end]
					if {$v > $max} {
						set max $v
						set AA($IDT,majority,$n) $a
					}
				}
			}
			foreach n $T($IDT,all_cod) {
				set TansitionMatrix($IDT,$n) $AA($IDT,majority,$n)
			}
		}
		pupko {
			# transition AA vers Bubble
			foreach n $T($IDT,allint_cod) {
				set TansitionMatrix($IDT,$n) $AA($IDT,pupko,$n)
			}
			# ajout feuilles
			foreach n $T($IDT,ue_cod) {
				set max 0
				foreach {k v} [array get AA $IDT,distribution,$n,*] {
					set a [lindex [split $k ,] end]
					if {$v > $max} {
						set max $v
						set AA($IDT,pupko,$n) $a
					}
				}
				set TansitionMatrix($IDT,$n) $AA($IDT,pupko,$n)
			}
		}
		parsimony {
			# transition AA vers Bubble
			foreach n $T($IDT,all_cod) {
				set TansitionMatrix($IDT,$n) $AA($IDT,parsimony,$n)
			}
		}
	}
	# DISPLAY
	#switch -exact $S(MatrixType) {
	#	frequency {TansitionMatrixFrequency }
	#	flux {TansitionMatrixFlux }
	#	dispersion {TansitionMatrixDispersion }
	#	symetry {TansitionMatrixSymetry }
	#}
	switch -exact $S(MatrixType) {
		1 {TansitionMatrix1 }
		2 {TansitionMatrix2 }
		3 {TansitionMatrix3 }
		frequency {TansitionMatrixFrequency }
		flux {TansitionMatrixFlux }
		dispersion {TansitionMatrixDispersion }
		symetry {TansitionMatrixSymetry }
	}
}
########################
# BUBBLE MAP TYPE 1 Na➞b
########################
## BB(a,b) = nombre de transition de a vers b
proc TansitionMatrix1 {} {
	global IDT T A S BB TansitionMatrix
	catch {unset BB}
	set TAG [format "%s%s" OBJECT= [randomColor]]
	$S(canvas2) delete all
	# liste annotation triee
	set LA [lsort -dictionary $A($IDT,AnnotationList)] 
	# Initialisation de la matrice BB
	foreach a1 $LA {
		foreach a2 $LA {
			# represente la transition a1 vers a2 ou inverse
			set BB($a1,$a2) 0
		}
	}
	# Filling
	BubbleMapFill 0
	#  max
	set Vmax 0.0
	foreach a1 $LA {
		foreach a2 $LA {
		
			if {$a1 != $a2 && $BB($a1,$a2) > $Vmax} {set Vmax $BB($a1,$a2)}
		}
	}
	set delta 20
	set Smax [expr $delta / 2.0]
	set F [expr $Smax / $Vmax]
	set w $S(canvas2)
	set y 15
	$w create text 15 -5 -text "Count" -font {Arial 12 normal} -fill #DDD -anchor w
	foreach a1 $LA {
		set x 15 
		# a1 : l'annotation d'origine
		$w create text $x $y -text $a1 -font {Arial 10 normal} -fill $S(ColorAnnotation,$a1) -anchor e -tags "L $TAG"
		incr x 30
		foreach a2 $LA {
			if {$a1 == $a2} {
				# diagonale
				$w create text $x $y -text $BB($a1,$a2) -font {Arial 10 normal} -fill $S(ColorAnnotation,$a1) -tags "$TAG"
			} else {
				if {$BB($a1,$a2) == 0.0} {
					$w create oval $x $y $x $y -fill black -outline black -tags "$TAG"
				} else {
					set v [expr $F * $BB($a1,$a2)]
					$w create oval [expr $x - $v] [expr $y - $v] [expr $x + $v] [expr $y + $v] \
						-fill #AAA -outline #AAA -tags "$TAG"
					$w create text $x $y -text $BB($a1,$a2) -font {Arial 10 normal} -fill black -tags "$TAG"
				}
			}
			incr x $delta
		}
		incr y $delta
	}
	# 
	$w configure -scrollregion [$w bbox all]
}
########################
# BUBBLE MAP TYPE 2 Na➞b /(Na. * 𝝅b)
# ou Na. est le nb de transitions de a vers n'importe quoi , y compris a
########################
## BB(a,b) = nombre de transition de a vers b
proc TansitionMatrix2 {} {
	global IDT T A S BB TansitionMatrix
	catch {unset BB}
	set TAG [format "%s%s" OBJECT= [randomColor]]
	$S(canvas2) delete all
	# liste annotation triee
	set LA [lsort -dictionary $A($IDT,AnnotationList)] 
	# Initialisation de la matrice BB
	foreach a1 $LA {
		foreach a2 $LA {
			# $a1,$a2 represente la transition a1 vers a2 
			set BB($a1,$a2) 0
		}
	}
	# Filling
	BubbleMapFill 0
	#
	catch {unset BBB}
	foreach a1 $LA {
		set Sum 0.0
		foreach a2 $LA {
			set Sum [expr $Sum + $BB($a1,$a2)]
		}
		set BBB($a1) $Sum 
	}
	#  max (on ne considere que qd a1 different a2
	set Vmax 0.0
	catch {unset BBBB}
	foreach a1 $LA {
		foreach a2 $LA {
			set BBBB($a1,$a2) [expr $BB($a1,$a2) / ($BBB($a1) * $A($IDT,f,$a2))]
			if {$a1 != $a2 && $BBBB($a1,$a2) > $Vmax} {set Vmax $BBBB($a1,$a2)}
		}
	}
	#
	set delta 20
	set Smax [expr $delta / 2.0]
	set F [expr $Smax / $Vmax]
	set w $S(canvas2)
	set y 15
	$w create text 15 -5 -text "Relative rate"  -font {Arial 12 normal} -fill #DDD -anchor w 
	foreach a1 $LA {
		set x 15 
		# a1 : l'annotation d'origine
		$w create text $x $y -text $a1 -font {Arial 7 normal} -fill $S(ColorAnnotation,$a1) -anchor e -tags "L $TAG"
		incr x 30
		foreach a2 $LA {
			set v $BBBB($a1,$a2)
			set vtext  [rnd $v 1] 
			if {$a1 == $a2} {
				# diagonale
				$w create text $x $y -text $vtext -font {Arial 7 normal} -fill $S(ColorAnnotation,$a1) -tags "$TAG"
			} else {
				if {$v == 0.0} {
					$w create oval $x $y $x $y -fill black -outline black -tags "$TAG"
				} else {
					set vd [expr $F * $v]
					$w create oval [expr $x - $vd] [expr $y - $vd] [expr $x + $vd] [expr $y + $vd] \
						-fill #AAA -outline #AAA -tags "$TAG"
					$w create text $x $y -text $vtext -font {Arial 7 normal} -fill black -tags "$TAG"
				}
			}
			incr x $delta
		}
		incr y $delta
	}
	# 
	$w configure -scrollregion [$w bbox all]
}
########################
# BUBBLE MAP TYPE 3 (Na➞b + Nb➞a) / (2 * 𝝅a * 𝝅b)
########################
## BB(a,b) = nombre de transition de a vers b
proc TansitionMatrix3 {} {
	global IDT T A S BB TansitionMatrix
	catch {unset BB}
	set TAG [format "%s%s" OBJECT= [randomColor]]
	$S(canvas2) delete all
	# liste annotation triee
	set LA [lsort -dictionary $A($IDT,AnnotationList)] 
	# Initialisation de la matrice BB
	foreach a1 $LA {
		foreach a2 $LA {
			# represente la transition a1 vers a2 ou inverse
			set BB($a1,$a2) 0
		}
	}
	# Filling
	BubbleMapFill 0
	#  max
	set Vmax 0.0
	foreach a1 $LA {
		foreach a2 $LA {
			set v [expr ($BB($a1,$a2)+$BB($a2,$a1)) / (2*$A($IDT,f,$a1) * $A($IDT,f,$a2))]
			if {$a1 != $a2 && $v > $Vmax} {set Vmax $v}
		}
	}
	set delta 20
	set Smax [expr $delta / 2.0]
	set F [expr $Smax / $Vmax]
	set w $S(canvas2)
	set y 15
	$w create text 15 -5 -text "(Na➞b + Nb➞a) / (2 * πa * πb)" -font {Arial 12 normal} -fill #DDD -anchor w
	foreach a1 $LA {
		set x 15 
		# a1 : l'annotation d'origine
		$w create text $x $y -text $a1 -font {Arial 8 normal} -fill $S(ColorAnnotation,$a1) -anchor e -tags "L $TAG"
		incr x 30
		foreach a2 $LA {
			set v [expr ($BB($a1,$a2) + $BB($a2,$a1)) / (2*$A($IDT,f,$a1) * $A($IDT,f,$a2))]
			set vtext  [rnd $v 1] 

			if {$a1 == $a2} {
				# diagonale
				$w create text $x $y -text $vtext -font {Arial 8 normal} -fill $S(ColorAnnotation,$a1) -tags "$TAG"
			} else {
				if {$v == 0.0} {
					$w create oval $x $y $x $y -fill black -outline black -tags "$TAG"
				} else {
					set vd [expr $F * $v]
					$w create oval [expr $x - $vd] [expr $y - $vd] [expr $x + $vd] [expr $y + $vd] \
						-fill #AAA -outline #AAA -tags "$TAG"
					$w create text $x $y -text $vtext -font {Arial 6 normal} -fill black -tags "$TAG"
				}
			}
			incr x $delta
		}
		incr y $delta
	}
	# 
	$w configure -scrollregion [$w bbox all]
}
########################
# BUBBLE MAP FREQUENCE
########################
## BB(a,b) = nombre de transition de a vers b
proc TansitionMatrixFrequency {} {
	global IDT T A S BB TansitionMatrix
	catch {unset BB}
	set TAG [format "%s%s" OBJECT= [randomColor]]
	$S(canvas2) delete all
	# liste annotation triee
	set LA [lsort -dictionary $A($IDT,AnnotationList)] 
	# Initialisation de la matrice BB
	foreach a1 $LA {
		foreach a2 $LA {
			# represente la transition a1 vers a2 ou inverse
			set BB($a1,$a2) 0
		}
	}
	# Filling
	BubbleMapFill 0
	#  max
	set Vmax 0.0
	foreach a1 $LA {
		foreach a2 $LA {
		
			if {$a1 != $a2 && $BB($a1,$a2) > $Vmax} {set Vmax $BB($a1,$a2)}
		}
	}
	set delta 20
	set Smax [expr $delta / 2.0]
	set F [expr $Smax / $Vmax]
	set w $S(canvas2)
	set y 15
	$w create text 15 -5 -text "Frequency matrix" -font {Arial 12 normal} -fill #DDD -anchor e
	foreach a1 $LA {
		set x 15 
		# a1 : l'annotation d'origine
		$w create text $x $y -text $a1 -font {Arial 8 normal} -fill $S(ColorAnnotation,$a1) -anchor e -tags "L $TAG"
		incr x 30
		foreach a2 $LA {
			if {$a1 == $a2} {
				# diagonale
				$w create text $x $y -text $BB($a1,$a2) -font {Arial 8 normal} -fill $S(ColorAnnotation,$a1) -tags "$TAG"
			} else {
				if {$BB($a1,$a2) == 0.0} {
					$w create oval $x $y $x $y -fill black -outline black -tags "$TAG"
				} else {
					set v [expr $F * $BB($a1,$a2)]
					$w create oval [expr $x - $v] [expr $y - $v] [expr $x + $v] [expr $y + $v] \
						-fill #AAA -outline #AAA -tags "$TAG"
					$w create text $x $y -text $BB($a1,$a2) -font {Arial 6 normal} -fill black -tags "$TAG"
				}
			}
			incr x $delta
		}
		incr y $delta
	}
	# 
	$w configure -scrollregion [$w bbox all]
}
########################
# INDICE DE FLUX
########################
# proportion du nombre de transition de a vers b parmis toutes les transitions vers b
proc TansitionMatrixFlux {} {
	global IDT T A S BB TansitionMatrix
	catch {unset BB}
	$S(canvas2) delete all
	# liste annotation triee
	set LA [lsort -dictionary $A($IDT,AnnotationList)] 
	# majoritaire, tous les nodes
	foreach n $T($IDT,all_cod) {
		set max 0
		# l'ID referancant les annotations est dbv $T($IDT,dbv,$n)
		set node $T($IDT,dbv,$n)
		foreach {k v} [array get A $IDT,$node,*] {
			set a [lindex [split $k ,] end]
			if {$v > $max} {
				set max $v
				set A($IDT,majoritaire,$n) $a
			}
		}
	}
	# Initialisation de la matrice BB
	foreach a1 $LA {
		foreach a2 $LA {
			# represente la transition a1 vers a2 ou inverse
			set BB($a1,$a2) 0
		}
	}
	# Filling
	BubbleMapFill 0
	# ici Vmax est 1.0
	set delta 20
	set Smax [expr $delta / 2.0]
	set F $Smax
	set w $S(canvas2)
	set y 15
	$w create text 15 -5 -text "Flux matrix" -font {Arial 12 normal} -fill #CCC -anchor e
	foreach a1 $LA {
		set x 15 
		# a1 : l'annotation d'origine
		$w create text $x $y -text $a1 -font {Arial 8 normal} -fill $S(ColorAnnotation,$a1) -anchor e -tags "L"
		incr x 30
		# calculer le nombre total de transitions VERS a2
		set somme 0.0
		set somme2 0.0
		foreach a2 $LA {
			# set somme [expr $somme + $BB($a1,$a2)]
			# variante : on ne compte pas les transitions intra
			if {$a1 != $a2} {
				set somme [expr $somme + $BB($a1,$a2)]
			} else {
				set somme2 [expr $somme2 + $BB($a1,$a2)]
			}
		}
		# proportion nombre de transitions a1 vers a2 / somme (total de transitions VERS a2)
		foreach a2 $LA {
			if {$a1 == $a2} {
				# diagonale
				#$w create text $x $y -text "$somme\n$somme2" -font {Arial 8 normal} -justify center -fill $S(ColorAnnotation,$a1)
				#set si [expr $somme + $somme2]
				$w create text $x $y -text "$somme2" -font {Arial 8 normal} -justify center -fill $S(ColorAnnotation,$a1)
			} else {
				if {$BB($a1,$a2) == 0.0} {
					#$w create oval $x $y $x $y -fill black -outline black
					$w create text $x $y -text x -fill black -font {Arial 6 normal}
				} else {
					set v [expr $F * ($BB($a1,$a2) / $somme)]
					$w create oval [expr $x - $v] [expr $y - $v] \
						[expr $x + $v] [expr $y + $v] \
						-fill #AAA -outline #AAA -tags "$a1 $a2 $somme $somme2 FluxBubble"
					$w create text $x $y -text $BB($a1,$a2) -font {Arial 6 normal} -fill white
				}
			}
			incr x $delta
		}
		$w create text $x $y -text $somme -font {Arial 6 normal} -fill $S(ColorAnnotation,$a1)
		incr y $delta
	}
	$S(canvas2) configure -scrollregion [$S(canvas2) bbox all]
}
proc FluxBubbleEnter {w} {
	global S IDT BB 
	set i [$w find withtag current]
	set tags [$w gettags $i]
	set from [lindex $tags 0]
	set to [lindex $tags 1]
	set somme [lindex $tags 2]

	$S(infopath) insert end "> *** $BB($from,$to) transition(s) FROM $from TO $to "
	$S(infopath) insert end " Total number of transition(s) TO $to: $somme\n"
	$S(infopath) see current
	focus $S(infopath) 
}
# version 2
proc BubbleMapFill {node} {
	global IDT T A S BB TansitionMatrix
	set a $TansitionMatrix($IDT,$node) 
	if {$T($IDT,typ,$node) != "f"} {
		set afils1 $TansitionMatrix($IDT,$T($IDT,fils1,$node))
		set afils2 $TansitionMatrix($IDT,$T($IDT,fils2,$node))
		incr BB($a,$afils1)
		incr BB($a,$afils2)
		BubbleMapFill $T($IDT,fils1,$node)
		BubbleMapFill $T($IDT,fils2,$node)
	} else {
		return
	}
}
########################
# INDICE DE DISPERSION
########################
proc TansitionMatrixDispersion {} {
	global IDT T A S BB TansitionMatrix
	catch {unset BB}
	# liste annotation triee
	set LA [lsort -dictionary $A($IDT,AnnotationList)] 
	# majoritaire, tous les nodes
	foreach n $T($IDT,all_cod) {
		set max 0
		# l'ID referancant les annotations est dbv $T($IDT,dbv,$n)
		set node $T($IDT,dbv,$n)
		foreach {k v} [array get A $IDT,$node,*] {
			set a [lindex [split $k ,] end]
			if {$v > $max} {
				set max $v
				set A($IDT,majoritaire,$n) $a
			}
		}
	}
	# Initialisation de la matrice BB
	foreach a1 $LA {
		foreach a2 $LA {
			# represente la transition a1 vers a2 ou inverse
			set BB($a1,$a2) 0
		}
	}
	# Filling
	# decompte de toute les transitions BB(a1,a2)
	BubbleMapFill 0
	# 
	# N($a1,x) a vers x (donneur)
	# N(x,$a1) x vers a (receveur)
	# N($a1,$a1) a vers a (intra)
	foreach a1 $LA {
		set N($a1,x) 0.0
		set N($a1,$a1) 0.0
		foreach a2 $LA {
			if {$a1 != $a2} {
				set N($a1,x) [expr $N($a1,x) + $BB($a1,$a2)]
			} else {
				set N($a1,$a1) [expr $N($a1,$a1) + $BB($a1,$a2)]
			}
		}
	}
	foreach a1 $LA {
		set N(x,$a1) 0.0
		foreach a2 $LA {
			if {$a1 != $a2} {
				set N(x,$a1) [expr $N(x,$a1) + $BB($a2,$a1)]
			} else {
				set N($a1,$a1) [expr $N($a1,$a1) + $BB($a2,$a1)]
			}
		}
	}
	# indice de dispersion
	global D 
	foreach a $LA {
		set denominateur [expr $N($a,x) + $N(x,$a) + $N($a,$a) - 1]
		# ajout test denominateur = 0 (une seule séquence a l'annotation)
		set D($a,x) [expr $N($a,x) / $denominateur]
		set D(x,$a) [expr $N(x,$a) / $denominateur]
	}
	# histogramme
	$S(canvas2) delete all
	# ici Vmax est 1.0
	set Smax 1000.0
	set F $Smax
	set w $S(canvas2)
	set y 15
	$w create text 15 -5 -text "Dispersion" -font {Arial 12 normal} -fill #CCC -anchor e

	foreach a $LA {
		set x 15 
		$w create text $x [expr $y +6] -text $a -font {Arial 8 normal} -fill $S(ColorAnnotation,$a) -anchor e -tags "L"
		$w create rectangle $x $y [expr $x + $F * $D($a,x)] [expr $y +10] -fill #CC0000 -outline #CC0000 -tag "out$a"
		$w create rectangle $x $y [expr $x + $F * $D(x,$a)] [expr $y +10] -fill #99CC66 -outline #99CC66 -tag "in$a"
		if {$D($a,x) <= $D(x,$a)} {
			$w raise out$a
		} else {
			$w raise in$a
		}
		incr y 15 
	}
	$S(canvas2) configure -scrollregion [$S(canvas2) bbox all]
}
########################
# INDICE DE SYMETRY
########################
proc TansitionMatrixSymetry {} {
	global IDT T A S BB TansitionMatrix
	catch {unset BB}
	$S(canvas2) delete all
	# liste annotation triee
	set LA [lsort -dictionary $A($IDT,AnnotationList)] 
	# majoritaire, tous les nodes
	foreach n $T($IDT,all_cod) {
		set max 0
		# l'ID referancant les annotations est dbv $T($IDT,dbv,$n)
		set node $T($IDT,dbv,$n)
		foreach {k v} [array get A $IDT,$node,*] {
			set a [lindex [split $k ,] end]
			if {$v > $max} {
				set max $v
				set A($IDT,majoritaire,$n) $a
			}
		}
	}
	# Initialisation de la matrice BB
	foreach a1 $LA {
		foreach a2 $LA {
			# represente la transition a1 vers a2 ou inverse
			set BB($a1,$a2) 0
		}
	}
	# Filling
	BubbleMapFill 0
	# max
	set synergyMax 0
	foreach a1 $LA {
		foreach a2 $LA {
			set symetry [expr $BB($a1,$a2) - $BB($a2,$a1)]
			if {$symetry > $synergyMax} {set synergyMax $symetry}
		}
	}
	#
	set deltadelta 20
	set Smax [expr $deltadelta / 2.0]
	set F [expr $Smax / $synergyMax]
	set w $S(canvas2)
	set y 15
	$w create text 15 -5 -text "Symetry matrix" -font {Arial 12 normal} -fill #CCC -anchor e

	foreach a1 $LA {
		set x 15 
		# a1 : l'annotation d'origine
		$w create text $x $y -text $a1 -font {Arial 8 normal} -fill $S(ColorAnnotation,$a1) -anchor e -tags "L"
		incr x 30
		## calculer le nombre total de transitions sur lui même
		set luimeme 0.0
		foreach a2 $LA {
			if {$a1 == $a2} {
				set luimeme [expr $luimeme + $BB($a1,$a2)]
			}
		}
		# 
		foreach a2 $LA {
			if {$a1 == $a2} {
				# diagonale
				$w create text $x $y -text "$luimeme" -font {Arial 8 normal} -justify center -fill $S(ColorAnnotation,$a1)
			} else {
				# indice de symetry de A vers B
				# indice de symetry de a vers b = Nombre de transition de a vers b - Nombre transtions b vers a
				set symetry [expr $BB($a1,$a2) - $BB($a2,$a1)]
				if {$symetry == 1 || $symetry == -1} {
					set delta [expr $F * abs($symetry)]
					$w create oval [expr $x - $delta] [expr $y - $delta] \
						[expr $x + $delta] [expr $y + $delta] \
						-fill #AAA -outline #AAA -tags "$a1 $a2 SymetryBubble"
					$w create text $x $y -text $symetry -font {Arial 6 normal} -fill black
				} elseif {$symetry == 0} {
					# faire la difference entre un echange parfaitement symétrique et pas d'échange du tout
					if {$BB($a1,$a2) == 0 && $BB($a2,$a1) ==0} {
						# pas d'échange
						$w create text $x $y -text . -font {Arial 6 normal} -fill black
					} else {
						# echange symetrique
						#$w create text $x $y -text $BB($a1,$a2) -font {Arial 10 bold} -fill orange
						$w create rectangle [expr $x-3] [expr $y-3] [expr $x+3] [expr $y+3] -fill orange -outline orange
					}
				} elseif { $symetry < 0 } {
					set delta [expr $F * abs($symetry)]
					$w create oval [expr $x - $delta] [expr $y - $delta] \
						[expr $x + $delta] [expr $y + $delta] \
						-fill #CC0000 -outline #CC0000 -tags "$a1 $a2 SymetryBubble"
					$w create text $x $y -text $symetry -font {Arial 6 normal} -fill black
				} else {
					set delta [expr $F * $symetry]
					$w create oval [expr $x - $delta] [expr $y - $delta] \
						[expr $x + $delta] [expr $y + $delta] \
						-fill #99CCFF -outline #99CCFF -tags "$a1 $a2 SymetryBubble"
					$w create text $x $y -text $symetry -font {Arial 6 normal} -fill black
				}
			}
			incr x $deltadelta
		}
		incr y $deltadelta
	}
	$S(canvas2) configure -scrollregion [$S(canvas2) bbox all]
}
########################
# TOOLBOX WHAT
########################
proc infowhatMake {w x y} {
	global IDT A T S AA
	set id [$w find withtag current]
	set tags [$w gettags $id]
	set n [string range [lindex $tags [lsearch -glob $tags N*]] 1 end]
	#puts >>>>>>>>$n
	set nodea $T($IDT,dbv,$n)
	# INIT 
	set apc "NODE $n ($T($IDT,dbv,$n)): \n\n"
	# AA
	foreach {k v} [array get AA $IDT,distribution,$n,*] {
		set a [lindex [split $k ,] end]
		set apc [format "%s%s%s%s" $apc "$a " "$v" "\n"]
	}
	# T
	set apc [format "%s%s" $apc "\n"]
	set apc [format "%s%s" $apc "Branch length: $T($IDT,dbl,$n)"]
	# DISPLAY
	$S(canvas2) delete all
	$S(canvas2) create text 20 20 -text $apc -font {Helvetica 10 normal} -anchor nw
	$S(canvas2) configure -scrollregion [$S(canvas2) bbox all]
}
### similaire a BLLmake juste le connecteur differe un peu
proc BLLmakeS {w x y txt id tagbranch} {
	global IDT B S
	set S(mox) $x
	set S(moy) $y
	set idtext [format "%s%s%s" BLL ¬∂ $id]
	set idlink [format "%s%s%s" LIN ¬∂ $id]
	$w delete $idtext
	$w delete $idlink
	set co [$w coords $tagbranch]
	set wi [lindex [$w itemconfigure $tagbranch -width ] end]
	set yi [expr [lindex $co 1] + ( $wi / 2.0)]
	set xi [expr ([lindex $co 0] + [lindex $co 2]) / 2.0]
	$w create text [$w canvasx [expr $x + 30 ] ] [$w canvasy [expr $y + 30 ]]  \
			-text $txt -font {Arial 10 normal} -fill black -anchor nw \
			-tags "BLL BLLS sbulle $idtext" -anchor w
	$w create line  $xi $yi [$w canvasx [expr $x + 30 ] ] [$w canvasy [expr $y + 30 ]]  \
			-width 1 -fill black -tags "BLL BLLlink $idlink"
	# MEM
	set B(BLLtxt,$id) $txt
	set B(BLLnod,$id) $tagbranch
	set B(BLLidt,$id) $idtext
	set B(BLLidl,$id) $idlink
	set B(BLLcol,$id) black
	set B(BLLgfo,$id) {Arial 10 normal}
	set B(BLLxxx,$id) [expr $x + 30 ]
	set B(BLLyyy,$id) [expr $y + 30 ]
	# Liste
	lappend B($w,bll) $id
}
########################
# ROOT
########################

proc MultiRoot {} {
	global S T IDT G
	foreach id $T($IDT,all_cod) {
		set s [Root&RebuildNewickString TEST $id]
		set node $T($IDT,dbv,$id)
		set nameout [format "%s%s%s%s" $S($IDT,filetree) -ROOT $node .txt]
		set okout [file join $S(pathuser) $nameout]
		set fid [open $okout w]
		puts $fid $s
		close $fid 
	}
}
#
proc RootCap {w} {
	global S T IDT G
	catch {unset G}
	set tags [$S(canvas) gettags current]
	#puts $tags
	set n [string range [lindex $tags end-1] 1 end]
	set s [Root&RebuildNewickString TEST $n]
	#puts $s
	set node $T($IDT,dbv,$n)
	set nameout [format "%s%s%s%s" $S($IDT,filetree) -ROOT $node .txt]
	set okout [file join $S(pathuser) $nameout]
	set fid [open $okout w]
	puts $fid $s
	close $fid 
	set file $S($IDT,filetree) 
	# attention ici changement de IDT courant
	MakeMap .pan.g.g.g
	#$S(canvas) delete all
	set S($IDT,filetree) $file
	# NEWICK cleaning, checking
	regsub -all "\n| " $s "" s
	regsub -all {\[} $s "" s
	regsub -all {:-} $s ":" s
	regsub -all {\]} $s "" s
	regsub -all {;} $s "" s
	# si il n'y a pas de valeurs de longueurs de branches
	# on met des longueurs de branches à 1 partout
	if {[string match *:* $s] == 0} {
		regsub -all "," $s {:1.0,} s
		regsub -all {\)} $s {:1.0)} s
	}
	# data structure
	NewickToDataStructure $s
	#set archiveT [array get T]
	$S(infopath) insert end "> [llength $T($IDT,ue_lab)] strains\n"
	$S(infopath) insert end "> Root $file (node $n)\n"
	$S(infopath) see current
	Drawtree
}
proc Root&RebuildNewickString {id n} {
	global IDT T N S
	if {$T($IDT,typ,$n) == "r" || $T($IDT,typ,$T($IDT,pere,$n)) == "r" } {return $S(NewickOriginal)} else {
		# attention le code n est interne, l'ID referancant les annotations est $T($IDT,dbv,$n)
		set nNewick $T($IDT,dbv,$n)
		# découpage aleatoire des longueurs de branche sous root
		#set rootl1 [::math::random 0 $T($IDT,dbl,$n)]
		set rootl1 [expr $T($IDT,dbl,$n) /2.0]
		set rootl2 [expr $T($IDT,dbl,$n) - $rootl1]
		# pere
		set nodepere $T($IDT,pere,$n)
		set nodepereNewick $T($IDT,dbv,$nodepere)
		# frere
		if {$T($IDT,fils1,$nodepere) == $n} {set nodefrere $T($IDT,fils2,$nodepere)} else {set nodefrere $T($IDT,fils1,$nodepere)}
		# depart 
		set tp [string last ")" $S(NewickOriginal)]
		set dx [string range $S(NewickOriginal) [expr $tp + 1] end]
		set idroot [string range $dx 0 [expr [string last ":" $dx] - 1]]
		
		set N($id) [format "%s%s%s%s%s%s%s%s%s%s%s" ooo $nodepereNewick : $rootl2 , $nNewick : $rootl1 ccc $idroot:0 \;]
		# remplissage descendant pour $n ($nNewick)
		RebuildNewickStringDown $id $n
		# remplissage ascendant pour $nodepere
		RebuildNewickStringUp $id $nodepere $nodefrere
		# insertion parentheses
		regsub -all ooo $N($id) ( N($id)
		regsub -all ccc $N($id) ) N($id)
		return $N($id)
	}
}
# on remplace le pere (np) de (np,n) par (npp,nf)np
# npp etant le pere de np et nf le frere de n (cad l'autre fils de np)
proc RebuildNewickStringUp {id np nf} {
	global IDT T N
	set npp $T($IDT,pere,$np)
	set nfNewick $T($IDT,dbv,$nf)
	set npNewick $T($IDT,dbv,$np)
	set nppNewick $T($IDT,dbv,$npp)
	if {$T($IDT,typ,$npp) == "r"} {
		if {$T($IDT,fils1,$npp) == $np} {set nodefrere $T($IDT,fils2,$npp)} else {set nodefrere $T($IDT,fils1,$npp)}
		set nodefrereNewick $T($IDT,dbv,$nodefrere)
		set p [format "%s%s%s" ooo $npNewick :]
		set lb [expr $T($IDT,dbl,$nodefrere) + $T($IDT,dbl,$np)]
		set add [format "%s%s%s%s%s%s%s%s%s%s%s%s" ooo ooo $nodefrereNewick : $lb , $nfNewick : $T($IDT,dbl,$nf) ccc $npNewick :  ]
		regsub $p $N($id) $add N($id)
		RebuildNewickStringDown $id $nodefrere
		RebuildNewickStringDown $id $nf
		return
	} else {
		set p [format "%s%s%s" ooo $npNewick :]
		set add [format "%s%s%s%s%s%s%s%s%s%s%s%s" ooo ooo $nppNewick : $T($IDT,dbl,$np) , $nfNewick : $T($IDT,dbl,$nf) ccc $npNewick :  ]
		regsub $p $N($id) $add N($id)
		# Descente sur nf
		RebuildNewickStringDown $id $nf
		# Relance
		if {$T($IDT,fils1,$npp) == $np} {set nodefrere $T($IDT,fils2,$npp)} else {set nodefrere $T($IDT,fils1,$npp)}
		RebuildNewickStringUp $id $npp $nodefrere
	}
}
#
proc RebuildNewickStringDown {id n } {
	global IDT T N
	if {$T($IDT,typ,$n) == f} {
		return
	} else {
		set nNewick $T($IDT,dbv,$n)
		set pattern [format "%s%s%s%s%s" * ooo $nNewick : *]
		if {[string match $pattern $N($id) ]} {
			# GAUCHE
			set p1 [format "%s%s%s" ooo $nNewick :]
			set code1 $T($IDT,dbv,$T($IDT,fils1,$n))
			set code2 $T($IDT,dbv,$T($IDT,fils2,$n))
			set adds1 [format "%s%s%s%s%s%s%s%s%s%s%s%s" ooo ooo $code1 : $T($IDT,dbl,$T($IDT,fils1,$n)) , $code2 : $T($IDT,dbl,$T($IDT,fils2,$n)) ccc $nNewick :  ]
			regsub $p1 $N($id) $adds1 N($id)
			RebuildNewickStringDown $id $T($IDT,fils1,$n)
			RebuildNewickStringDown $id $T($IDT,fils2,$n)
			return
		} else {
			set pattern [format "%s%s%s%s%s" * , $nNewick : *]
			if {[string match $pattern $N($id) ]} {
				# DROIT
				set p2 [format "%s%s%s" , $nNewick :]
				set code1 $T($IDT,dbv,$T($IDT,fils1,$n))
				set code2 $T($IDT,dbv,$T($IDT,fils2,$n))
				set adds2 [format "%s%s%s%s%s%s%s%s%s%s%s%s" , ooo $code1 : $T($IDT,dbl,$T($IDT,fils1,$n)) , $code2 : $T($IDT,dbl,$T($IDT,fils2,$n)) ccc $nNewick :  ]
				regsub $p2 $N($id) $adds2 N($id)
				RebuildNewickStringDown $id $T($IDT,fils1,$n)
				RebuildNewickStringDown $id $T($IDT,fils2,$n)
				return
			}
		}
	}
}

########################
# LADDER
########################
# Up or Down
proc LaderizeSwap {d} {
	global IDT T S X
	# topology
	switch -- $d {
		up {
			LaderizeSwapGoUp 0
		}
		down {
			LaderizeSwapGoDown 0
		}
	}
	set X(0) ""
	set T($IDT,ue_cod) [Node2LeafsCode2 0 0]
	# graphic
	$S(canvas) delete all
	$S(canvas2) delete all
	PhyNJ $S(canvas)
	$S(canvas) configure -scrollregion [$S(canvas) bbox all]
	$S(canvas2) configure -scrollregion [$S(canvas2) bbox all]
	unset X
}
###
proc LaderizeSwapGoUp {n} {
	global IDT T S  
	if {$T($IDT,typ,$n) == f} {
		return
	} else {
		set nb1 [llength [Node2LeafsCode2 $T($IDT,fils1,$n) $T($IDT,fils1,$n)]]
		set nb2 [llength [Node2LeafsCode2 $T($IDT,fils2,$n) $T($IDT,fils2,$n)]]
		if {$nb1 >= $nb2} {
			set trans $T($IDT,fils1,$n)
			set T($IDT,fils1,$n) $T($IDT,fils2,$n)
			set T($IDT,fils2,$n) $trans
		} 
		LaderizeSwapGoUp $T($IDT,fils1,$n)
		LaderizeSwapGoUp $T($IDT,fils2,$n)
	} 
}
###
proc LaderizeSwapGoDown {n} {
	global IDT T S 
	if {$T($IDT,typ,$n) == f} {
		return
	} else {
		set nb1 [llength [Node2LeafsCode2 $T($IDT,fils1,$n) $T($IDT,fils1,$n)]]
		set nb2 [llength [Node2LeafsCode2 $T($IDT,fils2,$n) $T($IDT,fils2,$n)]]
		if {$nb2 >= $nb1} {
			set trans $T($IDT,fils1,$n)
			set T($IDT,fils1,$n) $T($IDT,fils2,$n)
			set T($IDT,fils2,$n) $trans
		} 
		LaderizeSwapGoDown $T($IDT,fils1,$n)
		LaderizeSwapGoDown $T($IDT,fils2,$n)
	} 
}
###
proc Swap {w} {
	global IDT T S 
	set tags [$w gettags current]
	set n [string range [lindex $tags [lsearch -glob $tags N*]] 1 end]
	if {$n != ""} {
		set trans $T($IDT,fils1,$n)
		set T($IDT,fils1,$n) $T($IDT,fils2,$n)
		set T($IDT,fils2,$n) $trans
		set X(0) ""
		set T($IDT,ue_cod) [Node2LeafsCode2 0 0]
		# graphic
		$S(canvas) delete all
		$S(canvas2) delete all
		PhyNJ $S(canvas)
		$S(canvas) configure -scrollregion [$S(canvas) bbox all]
		$S(canvas2) configure -scrollregion [$S(canvas2) bbox all]
		unset X
	}
}
###############################################################################


###############################################################################
# Clusters 100% background color
###############################################################################
#
proc Cluster100ColorCAP {} {
	global IDT T S A TC SApost
	# annotation de la feuille 
	foreach f $T($IDT,ue_cod) {
		#set f2 $T($IDT,dbv,$f)
		set f2 $T($IDT,ctl,$f)
		set Vmax 0.0
		foreach {k v} [array get A $IDT,$f2,*] {
			#puts "$f2 $k $v"
			if {$v >= $Vmax} {
				set Vmax $v
				set SApost($f) [lindex [split $k ,] end]
			} 
		}
	}
	#
	Cluster100ColorBgGO $S(canvas) 0
}
#
proc Cluster100ColorBgGO {w n} {
	global IDT T S G Bracket SApost
	if {$T($IDT,typ,$n) == "f"} {
		return 
	} else {
		set lf [Node2Leafs2 $n]
		set la {}
		foreach f $lf {
			if {[lsearch $la $SApost($T($IDT,ltc,$f))] == -1} {
				lappend la $SApost($T($IDT,ltc,$f))
			}
		}
		if {[llength $la] == 1} {
			set a [lindex $la 0]
			TreeBackgroundColorMake  $w $n $a
			return
		} else  {
			Cluster100ColorBgGO $w $T($IDT,fils1,$n) 
			Cluster100ColorBgGO $w $T($IDT,fils2,$n) 
			return
		}
	}
}
###############################################################################


###############################################################################
# TreeColor (FG & BG) = f(annotation, seuils)
###############################################################################
#
proc AAtreeColorCAP {{seuil 1}} {
	global IDT T S A AA TC
	catch {unset TC}
	# TC noeud internes
	switch -exact $S(AAtreeColorWhat) {
		majority {AAtreeColorMajoritaire}
		pupko {AAtreeColorPupko}
		parsimony {AAtreeColorparsimony}
	}
	# TC feuilles 
	foreach n $T($IDT,ue_cod) {
		foreach var $A($IDT,AnnotationList) {
			if {$AA($IDT,distribution,$n,$var) == 1} {
				set TC($IDT,annotation,$n) $var
				set TC($IDT,proba,$n) 1.00
			}
		}
	}
	# Hi
	AAtreeColor
}
# identification majoritaire et transfert
proc AAtreeColorMajoritaire {} {
	global IDT T S A AA P Z  TC
	foreach n $T($IDT,allint_cod) {
		set max 0
		foreach {k v} [array get AA $IDT,distribution,$n,*] {
			set a [lindex [split $k ,] end]
			if {$v > $max} {
				set max $v
				set AA($IDT,majority,$n) $a
			}
		}
	}
	foreach n $T($IDT,allint_cod) {
		set annotation $AA($IDT,majority,$n)
		set TC($IDT,annotation,$n) $annotation
		set TC($IDT,proba,$n) $AA($IDT,distribution,$n,$annotation)
	}
}
# transfert pupko
proc AAtreeColorPupko {} {
	global IDT T AA TC
	foreach n $T($IDT,allint_cod) {
		set TC($IDT,annotation,$n) $AA($IDT,pupko,$n)
		set TC($IDT,proba,$n) 0.99
	}
}
# transfert parsimony
proc AAtreeColorparsimony {} {
	global IDT AA T TC
	foreach n $T($IDT,allint_cod) {
		set TC($IDT,annotation,$n) $AA($IDT,parsimony,$n)
		set TC($IDT,proba,$n) 0.99
	}
}
#
proc AAtreeColor {{seuil 1}} {
	global IDT T S A TC
	# MODE
	switch -exact $S(AAtreeColorOperation) {
		fg {
			ResetTreeForegroundColor
			TreeForegroundColor
		}
		bg {
			ResetTreeBackgroundColor
			TreeBackgroundColor $S(canvas) 0
		}
	}
	update idletasks
}
#
proc TreeBackgroundColorCAP {} {
	global IDT T S A TC
	TreeBackgroundColor $S(canvas) 0

}
# 
proc TreeForegroundColor {{value 0.5}} {
	global IDT A T S TC
	set w $S(canvas)
	foreach id $T($IDT,allint_cod) {
		set a $TC($IDT,annotation,$id)
		if {[info exist S(ColorAnnotation,$a)]} {
		# possibilité d'AA non résolue (eg parsimony)
			set color $S(ColorAnnotation,$a)
		} else {
			set color black
		}
		if {$TC($IDT,proba,$id) >= $S(aatreecolorscale1)} {
			$w itemconfigure H$id -fill $color
			$w itemconfigure V$id -fill $color
			$w addtag FGtree=$a withtag H$id
			$w addtag FGtree=$a withtag V$id
		} else {
			$w itemconfigure H$id -fill black
			$w itemconfigure V$id -fill black
		}
	}
	# feuilles
	foreach id $T($IDT,ue_cod) {
		set a $TC($IDT,annotation,$id)
		$w itemconfigure H$id -fill $S(ColorAnnotation,$a)
		$w addtag FGtree=$a withtag H$id
	}
}
#
proc TreeBackgroundColor {w n } {
	global IDT T S A AA TC
	set a $TC($IDT,annotation,$n) 
	set p $TC($IDT,proba,$n)
	if {$T($IDT,typ,$n) != "f"} { 
		# proba >= seuil utilisateur ?
		if {$p >= $S(aatreecolorscale1)} {
			if {[NodeAnnot100 $n $a]} {
				# cluster 100% identifié
				# Flooding A REFAIRE 
				TreeBackgroundColorMake $w $n $a 
				return
			} else {
				# relance sur les fils
				TreeBackgroundColor $w $T($IDT,fils1,$n) 
				TreeBackgroundColor $w $T($IDT,fils2,$n) 
			}
		} else {
			# NON
			# relance sur les fils
			TreeBackgroundColor $w $T($IDT,fils1,$n) 
			TreeBackgroundColor $w $T($IDT,fils2,$n) 
			return
		}
	} else {
		$w itemconfigure H$n -fill $S(ColorAnnotation,$a)
		$w addtag FGtree=$a withtag H$n
		return
	}
}
#
proc TreeBackgroundColorMake {w n a} {
	global IDT S T B
	set B($IDT,1,$n) {}; set B($IDT,2,$n) {}; set B($IDT,3,$n) {}
	TreeBackgroundMakeColorOutlineD $w $n $n $a
	TreeBackgroundMakeColorOutlineG $w $n $n $a
	TreeBackgroundMakeColorOutlineE $w $n $a $n 
	lassign [rgbToHsv {*}[scan $S(ColorAnnotation,$a) "#%2x%2x%2x"]] h s v
	set NEWrgbBGtree [hsvToRgb $h $S(ColorSaturation,BGtree) $S(ColorValue,BGtree)]
	set r [lindex $NEWrgbBGtree 0] 
	set g [lindex $NEWrgbBGtree 1]
	set b [lindex $NEWrgbBGtree 2]
	set color [format "#%2X%2X%2X" $r $g $b] ; # passage en hexadecimal
	$w create polygon "$B($IDT,1,$n) $B($IDT,3,$n) $B($IDT,2,$n)"	\
		-fill $color -outline $color -tags "BGtree=$a SHRINK TAGBGSUBTREE" 
	$w create polygon "$B($IDT,1,$n) $B($IDT,3,$n) $B($IDT,2,$n)"	\
		-fill $color -outline $color -tags "BGtree=$a SHRINK TAGBGSUBTREE" -smooth 1 -splinesteps 100
	$w lower TAGBGSUBTREE
}
# 
proc TreeBackgroundMakeColorOutlineD {w p n a} {
	global B T IDT
	if {$T($IDT,typ,$n) == "f"} {
		if {[NodeAnnotFlood $n $a] } { 
			set c [$w coords H$n]
			set x1 [lindex $c 0]
			set y1 [lindex $c 1]
			set x2 [lindex $c 2]
			set y2 [lindex $c 3]
			set B($IDT,2,$p) [concat " $x2 $y2 " $B($IDT,2,$p)]
			return
		} else {
			return
		}
	} else {
		set c [$w coords H$n]
		set x1 [lindex $c 0]
		set y1 [lindex $c 1]
		set x2 [lindex $c 2]
		set y2 [lindex $c 3]
		set B($IDT,2,$p) [concat " $x1 $y1 $x2 $y2 " $B($IDT,2,$p)]
		set n1 $T($IDT,fils1,$n)
		set n2 $T($IDT,fils2,$n)
		set c1 [$w coords H$n1]
		set c2 [$w coords H$n2]
		set y1 [lindex $c1 1]
		set y2 [lindex $c2 1]
		if {$y1 > $y2} {
			TreeBackgroundMakeColorOutlineD $w $p $T($IDT,fils2,$n) $a
		} else {
			TreeBackgroundMakeColorOutlineD $w $p $T($IDT,fils1,$n) $a
		}
	}
}
# 
proc TreeBackgroundMakeColorOutlineG {w p n a} {
	global B T IDT
	if {$T($IDT,typ,$n) == "f"} {
		set c [$w coords H$n]
		set x1 [lindex $c 0]
		set y1 [lindex $c 1]
		set x2 [lindex $c 2]
		set y2 [lindex $c 3]
		lappend B($IDT,1,$p) $x2 $y2  
		return
	} else {
		set c [$w coords H$n]
		set x1 [lindex $c 0]
		set y1 [lindex $c 1]
		set x2 [lindex $c 2]
		set y2 [lindex $c 3]
		lappend B($IDT,1,$p) $x2 $y2 $x1 $y1
		set n1 $T($IDT,fils1,$n)
		set n2 $T($IDT,fils2,$n)
		set c1 [$w coords H$n1]
		set c2 [$w coords H$n2]
		set y1 [lindex $c1 1]
		set y2 [lindex $c2 1]
		if {$y1 > $y2} {
			TreeBackgroundMakeColorOutlineG $w $p $T($IDT,fils1,$n) $a
		} else {
			TreeBackgroundMakeColorOutlineG $w $p $T($IDT,fils2,$n) $a
		}
	}
}
# 
proc TreeBackgroundMakeColorOutlineE {w np annotation n} {
	global T B IDT
	if {$T($IDT,typ,$n) == f} {
		# on prend le plus grand x et on arrete
		set c [$w coords H$n]
		set x1 [lindex $c 0]
		set y1 [lindex $c 1]
		set x2 [lindex $c 2]
		set y2 [lindex $c 3]
		set y [expr ($y1 + $y2) / 2.0]
		if {$x1 > $x2} {
			lappend B($IDT,3,$np) $x1 $y 
		} else {
			lappend B($IDT,3,$np) $x2 $y
		}
		return
	} else {
		# on relance mais vers le bas en premier
		set n1 $T($IDT,fils1,$n)
		set n2 $T($IDT,fils2,$n)
		set c1 [$w coords H$n1]
		set c2 [$w coords H$n2]
		set y1 [lindex $c1 1]
		set y2 [lindex $c2 1]
		if {$y1 > $y2} {
			# n1 est en bas
			TreeBackgroundMakeColorOutlineE $w $np $annotation $T($IDT,fils1,$n)
			TreeBackgroundMakeColorOutlineE $w $np $annotation $T($IDT,fils2,$n)
		} else {
			# n2 est en bas
			TreeBackgroundMakeColorOutlineE $w $np $annotation $T($IDT,fils2,$n)
			TreeBackgroundMakeColorOutlineE $w $np $annotation $T($IDT,fils1,$n)
		}
	}
}
#
proc NodeAnnot100 {n a } {
	global IDT T S A AA TC
	foreach ni [Node2SubTreeNodes $n] {
		if {$TC($IDT,annotation,$ni) != $a} {return 0} 
	}
	return 1
}
#
proc NodeAnnotFlood {internalnode annotation} {
	global S A T IDT
	set node $T($IDT,dbv,$internalnode)
	set apc {}
	foreach {k v} [array get A $IDT,$node,*] {
		set a [lindex [split $k ,] end]
		if { $a == $annotation} {
			return 1
		}
	}
	return 0
}
#
proc FgColorOntheFly {} {
	global IDT S T TC A AA
	# TC noeud internes
	switch -exact $S(AAtreeColorWhat) {
		majority {AAtreeColorMajoritaire}
		pupko {AAtreeColorPupko}
		parsimony {AAtreeColorparsimony}
	}
	# TC feuilles 
	foreach n $T($IDT,ue_cod) {
		foreach var $A($IDT,AnnotationList) {
			if {$AA($IDT,distribution,$n,$var) == 1} {
				set TC($IDT,annotation,$n) $var
				set TC($IDT,proba,$n) 1.00
			}
		}
	}
}
# toolbox sachant un node user hi path en amont et aval avec la même annotation (majo/pupk/par)
proc FgTreeColorPath {w x y} {
	global IDT TC S
	FgColorOntheFly
	set tags [$w gettags current]
	set n [string range [lindex $tags [lsearch -glob $tags N*]] 1 end]
	if {$n != ""} {
		ResetTreeForegroundColor
		FgTreeColorPathGoAVAL $n $TC($IDT,annotation,$n) $S(ColorAnnotation,$TC($IDT,annotation,$n))
		FgTreeColorPathGoAMONT $n $TC($IDT,annotation,$n) $S(ColorAnnotation,$TC($IDT,annotation,$n))
		set S(fgcolornodetransient) $S(ColorAnnotation,$TC($IDT,annotation,$n)) ;# leaveevent du node
	}
}
# additif (pas de reset fg)
proc FgTreeColorPathShift {w x y} {
	global IDT TC S
	set tags [$w gettags current]
	set n [string range [lindex $tags [lsearch -glob $tags N*]] 1 end]
	if {$n != ""} {
		FgTreeColorPathGoAVAL $n $TC($IDT,annotation,$n) $S(ColorAnnotation,$TC($IDT,annotation,$n))
		FgTreeColorPathGoAMONT $n $TC($IDT,annotation,$n) $S(ColorAnnotation,$TC($IDT,annotation,$n))
		set S(fgcolornodetransient) $S(ColorAnnotation,$TC($IDT,annotation,$n)) ;# leaveevent du node

	}
}
#
proc FgTreeColorPathGoAVAL {n ap color} {
	global S T TC IDT
	if {$TC($IDT,annotation,$n) == $ap} {
		# Hi
		$S(canvas) itemconfigure H$n -fill $color
		$S(canvas) itemconfigure V$n -fill $color
		if {$T($IDT,typ,$n) != "f"} {
			FgTreeColorPathGoAVAL $T($IDT,fils1,$n) $ap $color
			FgTreeColorPathGoAVAL $T($IDT,fils2,$n) $ap $color
		} else {
			return
		}
	}
}
#
proc FgTreeColorPathGoAMONT {n ap color} {
	global S T TC IDT
	if {$TC($IDT,annotation,$n) == $ap} {
		# Hi
		$S(canvas) itemconfigure H$n -fill $color
		$S(canvas) itemconfigure V$n -fill $color
		if {$T($IDT,typ,$n) != "r"} {
			set pere $T($IDT,pere,$n)
			FgTreeColorPathGoAMONT $pere $ap $color
			# relancer AVAL sur le frere
			set n1 $T($IDT,fils1,$pere)
			set n2 $T($IDT,fils2,$pere)
			if {$n1 != $n} {FgTreeColorPathGoAVAL $n1 $ap $color}
			if {$n2 != $n} {FgTreeColorPathGoAVAL $n2 $ap $color}
		} else {
			return
		}
	}
}

###############################################################################


###############################################################################
# ANNOTATION DES FEUILLES
###############################################################################
# LEAF LABELS ON/OFF
proc leafOnOff {} {
	global IDT S
	set id [lindex [$S(canvas) find withtag L] 0]
	set state [lindex [$S(canvas) itemconfigure $id -state] end]
	if {$state == "hidden"} {
		$S(canvas) itemconfigure L -state normal
	} else {
		$S(canvas) itemconfigure L -state hidden
	}
}

# 
proc LeafAnnotationColorLabels {} {
	global IDT T S A SApost
	set w $S(canvas)

	# quelle l'annotation de la feuille ?
	foreach f $T($IDT,ue_cod) {
		set f2 $T($IDT,dbv,$f)
		foreach {k v} [array get A $IDT,$f2,*] {
			if {$v >= 0.99} {
				set a [lindex [split $k ,] end]
				set SApost($f) $a
			} 
		}
		# coloration label feuille
		lassign [rgbToHsv {*}[scan $S(ColorAnnotation,$SApost($f)) "#%2x%2x%2x"]] h s v
		set NEWrgbPupkoAnnot [hsvToRgb $h $S(ColorSaturation,LeafLabel) $S(ColorValue,LeafLabel)]
		set r [lindex $NEWrgbPupkoAnnot 0] 
		set g [lindex $NEWrgbPupkoAnnot 1]
		set b [lindex $NEWrgbPupkoAnnot 2]
		set color [format "#%2X%2X%2X" $r $g $b] ; # passage en hexadecimal
		$w itemconfigure EUL$T($IDT,ctl,$f) -fill $color
		$w addtag leaflabel=$SApost($f) withtag EUL$T($IDT,ctl,$f) ;# pour les mises à jour couleur
	}
}
#
proc LeafAnnotationColorBrackets {} {
	global IDT T S A SApost AA
	set w $S(canvas)
	$w delete AnnotMatrix
	# annotation de la feuille 
	foreach f $T($IDT,ue_cod) {
		#set f2 $T($IDT,dbv,$f)
		set f2 $T($IDT,ctl,$f)
		set Vmax 0.0
		
		foreach {k v} [array get A $IDT,$f2,*] {
			#puts "$f2 $k $v"
			if {$v >= $Vmax} {
				set Vmax $v
				set SApost($f) [lindex [split $k ,] end]
			} 
		}
	}
	# BRACKET TAB
	set XMAX1 [lindex [$w bbox Z] 2]
	set XMAX2 [lindex [$w bbox L] 2]
	set XMAX3 [lindex [$w bbox AnnotMatrix] 2]
	if {$XMAX1 < $XMAX2} {set XMAX $XMAX2 } {set XMAX $XMAX1}
	if {$XMAX < $XMAX3} {set XMAX $XMAX3 }
	set tabulation $XMAX
	l_bracket_annotation $w 0 $tabulation
	$w configure -scrollregion [$w bbox all]
}
###
proc l_bracket_annotation {w n tabulation} {
	global IDT T S G Bracket SApost
	if {$T($IDT,typ,$n) == "f"} {
		BracketDrawLeafs $w $T($IDT,ctl,$n) $SApost($n) black black {Arial 8 bold} $tabulation 3 
	} else {
		set G($n) {}
		set lf [Node2Leafs2 $n]
		set la {}
		foreach f $lf {
			if {[info exists SApost($T($IDT,ltc,$f))] == 0} {
				puts "PB sur $f pour node $n"
			} else {
				if {[lsearch $la $SApost($T($IDT,ltc,$f))] == -1} {
					lappend la $SApost($T($IDT,ltc,$f))
				}
			}
		}
		if {[llength $la] == 1} {
			BracketDrawLeafs $w $lf $la  black  black {Arial 8 bold} $tabulation 3 
			return
		} else  {
			l_bracket_annotation $w $T($IDT,fils1,$n) $tabulation
			l_bracket_annotation $w $T($IDT,fils2,$n) $tabulation
			return
		}
	}
}
###
proc BracketDrawLeafs {w leafs text outline colortext fo x dx } {
	global IDT A T S
	set ltag {}
	foreach l $leafs {
		lappend ltag "H$T($IDT,ltc,$l)"
	}
	set couleurRGB $S(ColorAnnotation,$text)
	lassign [rgbToHsv {*}[scan $couleurRGB "#%2x%2x%2x"]] h s v
	# on garde la teinte de l'annotation , et saturation / valur (luminosité) selon user
	set NEWrgb [hsvToRgb $h $S(ColorSaturation,$S(UpdateColorCible)) $S(ColorValue,$S(UpdateColorCible))]
	# passage en hexadecimal
	set r [lindex $NEWrgb 0] 
	set g [lindex $NEWrgb 1]
	set b [lindex $NEWrgb 2]
	set color [format "#%2X%2X%2X" $r $g $b]
	set co [eval $w bbox $ltag]
	set x1 [expr [lindex $co 0] + 1]
	set x2 [expr [lindex $co 2] - 1]
	set y1 [expr [lindex $co 1] + 1]
	set y2 [expr [lindex $co 3] - 1]
	if {$S($IDT,rotation) == 0} {
		$w create rectangle $x $y1 [expr $x + $dx] $y2 \
			-fill $color -outline $color -width 1 -tags "AnnotMatrix AnnotMatrixSquare=$text"
		set ym [expr ($y1 + $y2) / 2.0]
		if {$S(LeafAnnotationColorBracketsText) == 1 } {
			$w create text [expr $x + $dx + 5] $ym -text $text \
				-anchor w -font $fo -fill $colortext -tags "AnnotMatrix AnnotMatrixText AnnotMatrixText=$text"
		}
	}
}
# visulaiser les identifiants noeuds
# attention on affiche les ID du fichier annotation et non pas les codes internes
proc AnnotateNodeID {} {
	global IDT A T S
	set w $S(canvas)
	set presence [$w find withtag NodeAnnotID]
	if {$presence != ""} {
		$w delete NodeAnnotID
	} else {
		foreach n $T($IDT,allint_cod) {
			set co [$w coords H$n ]
			#set x [expr ([lindex $co 0] + [lindex $co 2]) / 2.0]
			set x [expr [lindex $co 0] +5]
			set y [expr (([lindex $co 1] + [lindex $co 3]) / 2.0) ]
			# attention le code n est interne, l'ID referancant les annotations est dbv
			set text $T($IDT,dbv,$n)
			$w create text $x $y -text $text -font $S(font) -anchor w -tags "NodeAnnot NodeAnnotID" -fill $S(IDColor)
		}
	}
}
#
proc IDColor {} {
	global S
	set color [tk_chooseColor]
	if {$color != ""} {
		set S(IDColor) $color
	}
}
########################
# AFFICHAGE SUPPORTS
########################
# visulaiser les supports
proc AnnotateNodeSupport {{scale 0}} {
	global IDT A T S
	if {$S(checksupportonoff) == 1 } {
		set w $S(canvas)
		$w delete NodeAnnotSP
		switch -exact $S(SupportHightligth) {
			fgtree {
				ResetTreeForegroundColor
				foreach n $T($IDT,allint_cod) {
					if {$n != 0} {
						set text [rnd $T($IDT,$n,support) 2]
						if {$text <= $S(SupportTresholdUp)} {
							if {$text > $S(SupportTresholdDown)} {
								$w itemconfigure H$n -fill $S(SupportColor)
								$w itemconfigure V$n -fill $S(SupportColor)
							}
						}
					}
				}
			}
			antree {
				foreach n $T($IDT,allint_cod) {
					if {$n != 0} {
						set text [rnd $T($IDT,$n,support) 2]
						if {$text <= $S(SupportTresholdUp)} {
							if {$text > $S(SupportTresholdDown)} {
								set co [$w coords H$n ]
								set x [expr ([lindex $co 0] + [lindex $co 2]) / 2.0]
								set co [$w coords H$n ]
								set y [expr (([lindex $co 1] + [lindex $co 3]) / 2.0) +5]
								$w create text $x $y -text $text -font $S(font) -fill $S(SupportColor) -tags "NodeAnnot NodeAnnotSP"
							}
						}
					}
				}
			}
		}
	}
}
proc SupportColor {} {
	global S
	set color [tk_chooseColor]
	if {$color != ""} {
		set S(SupportColor) $color
	}
}
proc SupportOnOff {} {
	global S
	if {$S(checksupportonoff) == 0} {
		set S(checksupportonoff) 1
		AnnotateNodeSupport
	} else {
		set S(checksupportonoff) 0
		#ResetTreeForegroundColor
		$S(canvas) delete NodeAnnotSP
	}
}

########################
# AFFICHAGE LONGUEURS DE BRANCHE
########################
# visulaiser les longueur de branche
proc AnnotateNodeBranchLength {} {
	global IDT A T S
	set w $S(canvas)
	set presence [$w find withtag NodeAnnotBL]
	if {$presence != ""} {
		$w delete NodeAnnotBL
	} else {
		foreach n $T($IDT,all_cod) {
			set co [$w coords H$n ]
			set x [expr ([lindex $co 0] + [lindex $co 2]) / 2.0]
			set co [$w coords H$n ]
			set y [expr (([lindex $co 1] + [lindex $co 3]) / 2.0) -5]
			$w create text $x $y -text [rnd $T($IDT,dbl,$n) 2] -font $S(font) -tags "NodeAnnot NodeAnnotBL" -fill $S(BrLenColor)
		}
	}
}
proc BrLenColor {} {
	global S
	set color [tk_chooseColor]
	if {$color != ""} {
		set S(BrLenColor) $color
	}
}
########################
# COLLAPSE
########################
proc CollapseGo {} {
	global IDT A T S
	set S(lnodessansmap) {}
	foreach node $T($IDT,allint_cod) {
		if {$T($IDT,$node,support) < $S(supportthreshold)} {
			set T($IDT,dbl,$node) 0
			lappend S(lnodessansmap) $node
		}
	}
	# recalculer $T($IDT,xmax)
	set T($IDT,xmax) 0
	TMAXupdate 0 0
	# DESSIN
	$S(canvas) delete all
	TreeInit
	PhyNJ $S(canvas)
	$S(canvas) configure -scrollregion [$S(canvas) bbox all]
}
#
proc TMAXupdate {node sx} {
	global IDT A T S
	if {$T($IDT,typ,$node) == "f"} {
		set sx [expr $sx + $T($IDT,dbl,$node)]
		set T($IDT,sox,$node) $sx
		if {$sx >= $T($IDT,xmax)} {set T($IDT,xmax) $sx}
		return 
	} else  {
		set ss [expr $sx + $T($IDT,dbl,$node)]
		TMAXupdate $T($IDT,fils1,$node) $ss
		TMAXupdate $T($IDT,fils2,$node) $ss
		return
	}
}

########################
# ZOOM
########################

#  Mark the first (x,y) coordinate for zooming.
proc zoomMark {c x y} {
	global IDT zoomArea
	set zoomArea(x0) [$c canvasx $x]
	set zoomArea(y0) [$c canvasy $y]
	$c create rectangle $x $y $x $y -outline black -tag zoomArea
}

#  Zoom in to the area selected by itemMark and
#  itemStroke.
proc zoomStroke {c x y} {
	global IDT zoomArea
	set zoomArea(x1) [$c canvasx $x]
	set zoomArea(y1) [$c canvasy $y]
	$c coords zoomArea $zoomArea(x0) $zoomArea(y0) $zoomArea(x1) $zoomArea(y1)
}
#  Zoom in to the area selected by itemMark and
#  itemStroke.
proc zoomArea {c x y} {
	global IDT zoomArea
	#  Get the final coordinates.
	#  Remove area selection rectangle
	set zoomArea(x1) [$c canvasx $x]
	set zoomArea(y1) [$c canvasy $y]
	$c delete zoomArea
	#  Check for zero-size area
	if {($zoomArea(x0)==$zoomArea(x1)) || ($zoomArea(y0)==$zoomArea(y1))} {
		return
	}
	#  Determine size and center of selected area
	set areaxlength [expr {abs($zoomArea(x1)-$zoomArea(x0))}]
	set areaylength [expr {abs($zoomArea(y1)-$zoomArea(y0))}]
	set xcenter [expr {($zoomArea(x0)+$zoomArea(x1))/2.0}]
	set ycenter [expr {($zoomArea(y0)+$zoomArea(y1))/2.0}]
	set winxlength [winfo width $c]
	set winylength [winfo height $c]
	set xscale [expr {$winxlength/$areaxlength}]
	set yscale [expr {$winylength/$areaylength}]
	if { $xscale > $yscale } {
		set factor $yscale
	} else {
		set factor $xscale
	}
	#  Perform zoom operation
	zoom $c $factor $xcenter $ycenter $winxlength $winylength
}

proc zoom { canvas factor \
			{xcenter ""} {ycenter ""} \
			{winxlength ""} {winylength ""} } {
	set winxlength [winfo width $canvas]; # Always calculate [ljl]
	set winylength [winfo height $canvas]
	if { [string equal $xcenter ""] } {
		set xcenter [$canvas canvasx [expr {$winxlength/2.0}]]
		set ycenter [$canvas canvasy [expr {$winylength/2.0}]]
	}
	$canvas scale all 0 0 $factor $factor
	set xcenter [expr {$xcenter * $factor}]
	set ycenter [expr {$ycenter * $factor}]
	set x0 1.0e30; set x1 -1.0e30 ;
	set y0 1.0e30; set y1 -1.0e30 ;
	foreach item [$canvas find all] {
		switch -exact [$canvas type $item] {
			"arc" -
			"line" -
			"oval" -
			"polygon" -
			"rectangle" {
				set coords [$canvas coords $item]
				foreach {x y} $coords {
					if { $x < $x0 } {set x0 $x}
					if { $x > $x1 } {set x1 $x}
					if { $y < $y0 } {set y0 $y}
					if { $y > $y0 } {set y1 $y}
				}
			}
		}
	}
	set xlength [expr {$x1-$x0}]
	set ylength [expr {$y1-$y0}]
	foreach {ax0 ay0 ax1 ay1} [$canvas bbox all] {break}
	while { ($ax0<$x0) || ($ay0<$y0) || ($ax1>$x1) || ($ay1>$y1) } {
		set x0 [expr {$x0-$xlength}]
		set x1 [expr {$x1+$xlength}]
		set y0 [expr {$y0-$ylength}]
		set y1 [expr {$y1+$ylength}]
		set xlength [expr {$xlength*3.0}]
		set ylength [expr {$ylength*3.0}]
	}
	set newxleft [expr {($xcenter-$x0-($winxlength/2.0))/$xlength}]
	set newytop  [expr {($ycenter-$y0-($winylength/2.0))/$ylength}]
	$canvas configure -scrollregion [list $x0 $y0 $x1 $y1]
	$canvas xview moveto $newxleft
	$canvas yview moveto $newytop
	$canvas configure -scrollregion [$canvas bbox all]
}
proc zoom2 { canvas Xfactor Yfactor \
			{xcenter ""} {ycenter ""} \
			{winxlength ""} {winylength ""} } {
	set winxlength [winfo width $canvas]
	set winylength [winfo height $canvas]
	if { [string equal $xcenter ""] } {
		set xcenter [$canvas canvasx [expr {$winxlength/2.0}]]
		set ycenter [$canvas canvasy [expr {$winylength/2.0}]]
	}
	$canvas scale all 0 0 $Xfactor $Yfactor
	set xcenter [expr {$xcenter * $Xfactor}]
	set ycenter [expr {$ycenter * $Yfactor}]
	set x0 1.0e30; set x1 -1.0e30 ;
	set y0 1.0e30; set y1 -1.0e30 ;
	foreach item [$canvas find all] {
		switch -exact [$canvas type $item] {
			"arc" -
			"line" -
			"oval" -
			"polygon" -
			"rectangle" {
				set coords [$canvas coords $item]
				foreach {x y} $coords {
					if { $x < $x0 } {set x0 $x}
					if { $x > $x1 } {set x1 $x}
					if { $y < $y0 } {set y0 $y}
					if { $y > $y0 } {set y1 $y}
				}
			}
		}
	}
	set xlength [expr {$x1-$x0}]
	set ylength [expr {$y1-$y0}]
	foreach {ax0 ay0 ax1 ay1} [$canvas bbox all] {break}
	
	while { ($ax0<$x0) || ($ay0<$y0) || ($ax1>$x1) || ($ay1>$y1) } {
		set x0 [expr {$x0-$xlength}]
		set x1 [expr {$x1+$xlength}]
		set y0 [expr {$y0-$ylength}]
		set y1 [expr {$y1+$ylength}]
		set xlength [expr {$xlength*3.0}]
		set ylength [expr {$ylength*3.0}]
	}
	set newxleft [expr {($xcenter-$x0-($winxlength/2.0))/$xlength}]
	set newytop  [expr {($ycenter-$y0-($winylength/2.0))/$ylength}]
	$canvas configure -scrollregion [list $x0 $y0 $x1 $y1]
	$canvas xview moveto $newxleft
	$canvas yview moveto $newytop
	$canvas configure -scrollregion [$canvas bbox all]
}

#
proc Sfittocontent {w} {
	global IDT S duplication
	set co [$w bbox all]
	set xi1 [lindex $co 0]
	set yi1 [lindex $co 1]
	set xi2 [lindex $co 2]
	set yi2 [lindex $co 3]
	set wx [winfo width $w]
	set wy [winfo height $w]
	set dx [expr $xi2 - $xi1]
	set dy [expr $yi2 - $yi1]
	set fx  [expr double($wx) / double($dx)]
	set fy  [expr double($wy) / double($dy)]
	zoom2 $w $fx $fy
}

proc SfittocontentX {w} {
	global IDT S duplication
	set S($w,deltaX) [expr [winfo width  $w] -100]
	PhyNJ $w
	$w delete gmap
	foreach key [array names duplication $w,*] {
		unset duplication($key)
	}
	foreach sce $S($w,map) {
		DrawMapping $w $sce
		$w xview moveto 0
		$w yview moveto 0
		BLLupdate $w
	}
}
#
proc SfittocontentY {w} {
	global IDT S
	set S($w,deltaY) [expr [winfo height  $w] -50]
	PhyNJ $w
	$w delete gmap
	foreach sce $S($w,map) {
		DrawMapping $w $sce
	}
}
#
proc ZoomX+ {w} {
	global IDT S
	set S($w,deltaX) [expr $S($w,deltaX) +1000]
	PhyNJ $w
	$w delete gmap
	foreach sce $S($w,map) {
		DrawMapping $w $sce
	}
}
#
proc ZoomX- {w} {
	global IDT S
	set S($w,deltaX) [expr $S($w,deltaX) -1000]
	PhyNJ $w
	$w delete gmap
	foreach sce $S($w,map) {
		DrawMapping $w $sce
	}
}
#
proc ZoomY+ {w} {
	global IDT S
	set S($w,deltaY) [expr $S($w,deltaY) +3000]
	PhyNJ $w
	$w delete gmap
	foreach sce $S($w,map) {
		DrawMapping $w $sce
	}
}
#
proc ZoomY- {w} {
	global IDT S
	set S($w,deltaY) [expr $S($w,deltaY) -3000]
	PhyNJ $w
	$w delete gmap
	foreach sce $S($w,map) {
		DrawMapping $w $sce
	}
}
#
proc Zoom- {w} {
	global IDT S
	set S($w,deltaX) [expr $S($w,deltaX) -100]
	set S($w,deltaY) [expr $S($w,deltaY) -1000]
	PhyNJ $w
	$w delete gmap
	foreach sce $S($w,map) {
		DrawMapping $w $sce
	}
}
#
proc Zoom+ {w} {
	global IDT S
	set S($w,deltaX) [expr $S($w,deltaX) +100]
	set S($w,deltaY) [expr $S($w,deltaY) +1000]
	PhyNJ $w
	$w delete gmap
	foreach sce $S($w,map) {
		#puts "DrawMapping $w $sce"
		DrawMapping $w $sce
	}
}

################################################################################


################################################################################
# INFERENCE ANNOTATION ANCESTRALES PAR PARCIMONIE
###############################################################################
#
proc AAcomputeparsimony {} {
	global IDT S T A SA AA
	catch {unset SA}
	set S(includingoutroupcheck) 1 ;# prise en compte root
	switch $S(parsimonyMode) {
		1 {
			# UPPASS + ACCTRAN 
			### Initialisation feuille
			foreach feuille $T($IDT,ue_lab) { 
				foreach annotation $A($IDT,AnnotationList)  {
					if { $A($IDT,$feuille,$annotation) == 1} {
						set SA($T($IDT,ltc,$feuille)) $annotation
					}
				}
			}
			# UPPASS + ACCTRAN
			UPPASS 0
			ACCTRAN 0
		}
		2 {
			# UPPASS + DOWNPASS + DELTRAN 
			### Initialisation feuille
			foreach feuille $T($IDT,ue_lab) { 
				foreach annotation $A($IDT,AnnotationList)  {
					if { $A($IDT,$feuille,$annotation) == 1} {
						set SA($T($IDT,ltc,$feuille)) $annotation
					}
				}
			}
			UPPASS 0
			DOWNPASS 0
			DELTRAN 0
			
		}
		3 {
			# parsimony old
			# DELTRAN
			set S(aainference) DELTRAN
			FitchUp 
			FitchDown
		}
		4 {
			# parsimony old
			# ACCTRAN 
			set S(aainference) ACCTRAN
			FitchUp 
			FitchDown 
		}
	}
	# DATA trans
	foreach n $T($IDT,all_cod)  {
		set AA($IDT,parsimony,$n) $SA($n)
	}
}
#
proc UPPASS {n} {
	global IDT T S A SA
	### si N feuille alors  S(N) = état associé à N
	if {$T($IDT,typ,$n) == f } {
		return $SA($n)
	} else {
		set A1 [UPPASS $T($IDT,fils1,$n)]
		set A2 [UPPASS $T($IDT,fils2,$n)]
		set intersection [operatorAND2 $A1 $A2]
		if {$intersection != {}} {
			set SA($n) $intersection
		} else {
			set SA($n) [concat $A1 $A2]
		}
	}
}
#
proc DOWNPASS {n} {
	global IDT T S A SA
	if {$T($IDT,typ,$n) != "f" } {
		if {$T($IDT,typ,$n) != "r" } {
			set ap $SA($T($IDT,pere,$n))
			set ag $SA($T($IDT,fils1,$n))
			set ad $SA($T($IDT,fils2,$n))
			set AA [operatorAND2 [operatorAND2 $ag $ad] $ap]
			if {$AA == {}} {
				# si intersection globale vide alors union de toutes les intersections deux à deux
				set AA [concat [operatorAND2 $ag $ad] " " [operatorAND2 $ap $ag] " " [operatorAND2 $ap $ad]]
			}
			if {$AA == {}} {
				# si encore vide alors union de tout
				set AA [concat $ap " " $ag " " $ad ]
			}
			# AA2 est l'intersection deAA avec le pere (nb si AA est unique on la retrouve dans l'intersection)
			set AA2 [operatorAND2 $ap $AA]
			if {$AA2 == ""} {set SA($n) $AA} else {set SA($n) $AA2}
		}
		DOWNPASS $T($IDT,fils1,$n)
		DOWNPASS $T($IDT,fils2,$n)
	}
}
#
proc ACCTRAN {n} {
	global IDT T S A SA
	if {$T($IDT,typ,$n) != "f" } {
		set ag $SA($T($IDT,fils1,$n))
		set ad $SA($T($IDT,fils2,$n))
		set a $SA($n)
		# G
		set IG [operatorAND2 $a $ag]
		if {$IG != {}} {
			set SA($T($IDT,fils1,$n)) $IG
		}
		# D
		set ID [operatorAND2 $a $ad]
		if {$ID != {}} {
			set SA($T($IDT,fils2,$n)) $ID
		}
		ACCTRAN $T($IDT,fils1,$n)
		ACCTRAN $T($IDT,fils2,$n)
	}
}
#
proc DELTRAN {n} {
	global IDT T S A SA
	if {$T($IDT,typ,$n) != "f" && $T($IDT,typ,$n) != "r" } {
		set ap $SA($T($IDT,pere,$n))
		set a $SA($n)
		set I [operatorAND2 $a $ap] 
		if {$I != {}} {
			set SA($n) $I
		}
		DELTRAN $T($IDT,fils1,$n)
		DELTRAN $T($IDT,fils2,$n)
	}
}
### SA table des annotations
proc FitchUp {} {
	global IDT T S A SA
	### Initialisation Annotation
	foreach feuille $T($IDT,ue_lab) { 
		foreach annotation $A($IDT,AnnotationList)  {
			if { $A($IDT,$feuille,$annotation) == 1} {
				set SA($T($IDT,ltc,$feuille)) $annotation
			}
		}
	}
	### NODES
	for {set i [expr $T($IDT,tot) - 1 ]} {$i >= 1} {incr i -1} {
		foreach b $T($IDT,cbg,$i) {
			set intersection [operatorAND $SA($T($IDT,fils1,$b)) $SA($T($IDT,fils2,$b))]
			if {$intersection != {}} {
				set SA($b) $intersection
			} else {
				set SA($b) [concat $SA($T($IDT,fils1,$b)) $SA($T($IDT,fils2,$b))]
			}
		}
	}
	### ROOT
	set intersection [operatorAND $SA(1) $SA(2)]
	if {$intersection != {}} {
		set SA(0) $intersection
	} else {
		set SA(0) [concat $SA(1) $SA(2)]
	}
}
### ACCTRANS VS DELTRANS
# root a deux fils : outgroup et n, et n a deux fils n1 et n2
# si on ne considere pas l'outgroup, lancer FitchDown sur n1 et sur n2
# -> lancer FitchDown sur les noeuds de generation 2
# les annotations ancestrales sont dans SA($codenode)
proc FitchDown {} {
	global IDT T S A SA
	if {$S(includingoutroupcheck)} {
		# switch selon mode
		if {$S(aainference) == "ACCTRAN"} {
			# ACCTRANS
			foreach b $T($IDT,cbg,1) {
				FitchDownAccTrans $b
			}
		} elseif {$S(aainference) == "DELTRAN"}  {
			# DELTRANS
			foreach b $T($IDT,cbg,1) {
				FitchDownDelTrans $b
			}
		}
	} else {
		# switch selon mode
		if {$S(aainference) == "ACCTRAN"} {
			# ACCTRANS
			foreach b $T($IDT,cbg,2) {
				FitchDownAccTrans $b
			}
		} elseif {$S(aainference) == "DELTRAN"}  {
			# DELTRANS
			foreach b $T($IDT,cbg,2) {
				FitchDownDelTrans $b
			}
		}
	}
	$S(infopath) insert end "> Ancestral annotations computed with parsimony $S(aainference)\n"
	$S(infopath) see current
}
#
proc FitchDownAccTrans {n} {
	global IDT T S A SA
	if {$T($IDT,typ,$n) == f || [lsearch -exact $SA($n) *] != -1 } {
		# n est feuille ou n est annote *
		# rien a faire et stop
		return
	} else  {
		# intersection annotations de n et son pere
		set intersection  [operatorAND2 $SA($T($IDT,pere,$n)) $SA($n)]
		if {$intersection != {}} {
			set SA($n)  $intersection
		}
		# si intersection vide on laisse tel quel
		# ... et on continue
		FitchDownAccTrans $T($IDT,fils1,$n)
		FitchDownAccTrans $T($IDT,fils2,$n)
	}
}
#
proc FitchDownDelTrans {n} {
	global IDT T S A SA
	if {$T($IDT,typ,$n) == f || [lsearch -exact $SA($n) *] != -1} {
		# n est feuille ou n est annote *
		# rien a faire et stop
		return
	} else {
		if {[lsearch -exact $SA($T($IDT,fils1,$n)) *] != -1 || [lsearch -exact $SA($T($IDT,fils2,$n)) *] != -1}  {
			# si un des fils est annote * intersection annotation de n avec son pere
			set AA [operatorAND2 $SA($n) $SA($T($IDT,pere,$n))]
			# si intersection non vide alors c'est l'annotation de n, sinon on ne fait rien
			if {$AA != {}} {set SA($n) $AA}
		} else  {
			# ici forcement, annotation de n et de ses fils est differente de *
			# cas standard
			# pour lisibilite, les annotations de n p g et d sont respectivement
			set an $SA($n)
			set ap $SA($T($IDT,pere,$n))
			set ag $SA($T($IDT,fils1,$n))
			set ad $SA($T($IDT,fils2,$n))
			# intersection globale ap ag ad   
			set AA [operatorAND2 [operatorAND2 $ag $ad] $ap]
			if {$AA == {}} {
				# si intersection globale vide alors union de toutes les intersections deux à deux
				set AA [concat [operatorAND2 $ag $ad] " " [operatorAND2 $ap $ag] " " [operatorAND2 $ap $ad]]
				if {$AA == {}} {
					# si encore vide alors union de tout
					set AA [concat $ap " " $ag " " $ad ]
				}
			}
			# AA2 est l'intersection de AA avec le pere (nb si AA est unique on la retrouve dans l'intersection)
			set AA2 [operatorAND2 $ap $AA]
			if {$AA2 == ""} {set SA($n) $AA} else {set SA($n) $AA2}
		}
		# ... et on continue
		FitchDownDelTrans $T($IDT,fils1,$n)
		FitchDownDelTrans $T($IDT,fils2,$n)
	}
}


################################################################################


################################################################################
# MAP SCENARIO OUTPUT PUPKO
###############################################################################
proc MapPupko {n a} {
	global IDT T A AA P PupkoTree
	set PupkoTree [format "%s%s%s" () $a ";"]
	if {$a != $P($IDT,$n,$a,fils1)} {
		# une transition
	}
	if {$a != $P($IDT,$n,$a,fils2)} {
		# transition
	}
	MapPukoRecur $a fils1 $T($IDT,fils1,$n) $P($IDT,$n,$a,fils1) 
	MapPukoRecur $a fils2 $T($IDT,fils2,$n) $P($IDT,$n,$a,fils2) 
}
proc MapPupkoRecur {ap way n a} {
	global IDT T A AA P PupkoTree
	if {$a == $ap} {
		MapPupkoRecur $a $T($IDT,fils1,$n) $P($IDT,$n,$a,fils1)
		MapPupkoRecur $a $T($IDT,fils2,$n) $P($IDT,$n,$a,fils2)
	} else {
	}
}
################################################################################

################################################################################
# FREQUENCE ANNOTATION
################################################################################
# A($IDT,f,annotation) = frequence
# A($IDT,a,feuille) = annotation qui a la prob 1
# CAS frequences des annotations au sein de l'arbre
proc FrequenceAnnot01 {} {
	global IDT T S A AA
	foreach annotation $A($IDT,AnnotationList) {
		set A($IDT,f,$annotation) 0.0
		foreach id $T($IDT,ue_cod) {
			set feuille $T($IDT,dbv,$id)
			set A($IDT,f,$annotation) [expr $A($IDT,f,$annotation) + $A($IDT,$feuille,$annotation)]
		}
		set A($IDT,f,$annotation) [expr double($A($IDT,f,$annotation) / $T($IDT,nbue))]
	}
	foreach id $T($IDT,ue_cod) {
		set feuille $T($IDT,dbv,$id)
		foreach annotation $A($IDT,AnnotationList) {
			if {$A($IDT,$feuille,$annotation) == 1.0} {set A($IDT,a,$id) $annotation} 
		}
	}
}
# cas annotations equiprobables
proc FrequenceAnnot02 {} {
	global IDT T S A AA MX
	set f [expr 1.0 / [llength $A($IDT,AnnotationList)]] 
	foreach annotation $A($IDT,AnnotationList) {
		set A($IDT,f,$annotation) $f
	}
	foreach id $T($IDT,ue_cod) {
		set feuille $T($IDT,dbv,$id)
		foreach annotation $A($IDT,AnnotationList) {
			if {$A($IDT,$feuille,$annotation) == 1.0} {set A($IDT,a,$id) $annotation} 
		}
	}
}
################################################################################


################################################################################
# GOLDEN SEARCH
# VERIFICATION OK 
################################################################################
proc NombreOrCAP {b1 b2 to} {
	global IDT phi resphi
	set phi [expr double((1 + sqrt(5)) / 2 )]
	set resphi [expr 2 - $phi]
	set sonde [expr $b1 + ($b2 - $b1) / (1 + $phi)]
	set factor [NombreOrGo $b1 $sonde $b2 $to]
	return $factor
}
# VERIFICATION OK
proc NombreOrGo {x1 x2 x3 tau} {
	global IDT phi resphi
	set d1 [expr $x3 - $x2]
	set d2 [expr $x2 - $x1]
	if {$d1 > $d2} {
		set x [expr $x2 + $resphi * $d1]
	} else {
		set x [expr $x2 - $resphi * $d2]
	}
	if {[expr abs($d1)] < [expr $tau * (abs($x2) + abs($x))] } { 
		return [expr ($x3 + $x1) / 2.0 ]
	}
	# assert(f(x) != f(x2));
	# [Pruning $x] < [Pruning $x2]
	# > si on cherche un maximum
	# < si on chercher un minimum
	if {[Pruning $x] > [Pruning $x2]} {
		if {$d1 > $d2} {
			return [NombreOrGo $x2 $x $x3 $tau] 
		} else {
			return [NombreOrGo $x1 $x $x2 $tau] 
		}
	} else {
		if {$d1 > $d2} {
			return [NombreOrGo $x1 $x2 $x $tau]
		} else {
			return [NombreOrGo $x $x2 $x3 $tau]
		}
	}
}
################################################################################


################################################################################
# PRUNING
################################################################################
# f : facteur d'echelle 
# ::tcl::mathfunc::pow x y Computes the value of x raised to the power y. 
proc Pruning {f} {
	global IDT T S A MX AA SCALINGsum  SCALINGfactor
	# Init
	set SCALINGfactor 2.0 
	set SCALINGsum 0.0 
	set mu $MX($IDT,mu)
	catch {unset MX}
	set MX($IDT,mu) $mu
	# feuille
	foreach id $T($IDT,ue_cod) {
		set MX($IDT,a,$id) $A($IDT,a,$id)
	}
	# 
	set sum 0.0
	foreach a $A($IDT,AnnotationList) {
		set sum [expr $sum + [like-down2 0 $a $f] * $A($IDT,f,$a)]
	}
	# OUT
	# AVEC SCALING
	#set nba [llength $A($IDT,AnnotationList)]
	#set sumOut [expr double(log(double($sum))) - (($SCALINGsum / $nba) * log(2))]
	#puts "L avec scaling: $sum "
	#puts "log(L sans scaling) = $sumOut => L sans scaling = exp($sumOut)"
	#puts "L sans scaling = [expr exp($sumOut)]"
	
	# SANS SCALING (nouvelle méthode catch sur produit mise à 0 si impossible)
	set sumOut $sum
	#puts $sumOut
	return $sumOut
}
# 
proc FacteurNormalisation {} {
	global IDT S A MX FACTEURNORMALISATION
	if {[info exists MX($IDT,mu)]} {
		return $MX($IDT,mu)
	} else {
		# somme des carrés des fréquences des annotations
		set sum2 0.0
		foreach annotation $A($IDT,AnnotationList) {
			set sum2 [expr double($sum2 + $A($IDT,f,$annotation) * $A($IDT,f,$annotation))]
		}
		# complément de 1 et inverse
		set MX($IDT,mu) [expr double(1.0 / (1.0 - $sum2))]
		return $MX($IDT,mu)
	}
}
# POSTORDER
proc like-down2 {n a1 f} {
	global IDT T A MX SCALINGsum SCALINGfactor
	if {[info exists MX($IDT,probaUP,$n,$a1)]} {return $MX($IDT,probaUP,$n,$a1)}
	if {$T($IDT,typ,$n) == "f"} {
		# feuille
		if {$MX($IDT,a,$n) == $a1} {set MX($IDT,probaUP,$n,$a1) 1 ; return 1} else {set MX($IDT,probaUP,$n,$a1) 0 ; return 0}
	} else {
		set lbG [expr double($T($IDT,dbl,$T($IDT,fils1,$n))*$f)]
		set lbD [expr double($T($IDT,dbl,$T($IDT,fils2,$n))*$f)]
		set sumG 0 ; set sumD 0
		foreach a2 $A($IDT,AnnotationList) {
			set sumG [expr $sumG + [probchange $a1 $a2 $lbG] * [like-down2 $T($IDT,fils1,$n) $a2 $f]]
			set sumD [expr $sumD + [probchange $a1 $a2 $lbD] * [like-down2 $T($IDT,fils2,$n) $a2 $f]]
		}
		# scaling OK
		#set sumG [Scaling $sumG]
		#set sumD [Scaling $sumD]
		#set sumout [expr double($sumG * $sumD)]
		#set f [expr 2 * $SCALINGfactor]
		#set MX($IDT,probaUP,$n,$a1) [expr double ($sumout / [::tcl::mathfunc::pow 2 $f])]
		# autre methode mise à zero
		if [catch {set sumout [expr double($sumG * $sumD)]} result] {
			set sumout 0.0
		} 
		set MX($IDT,probaUP,$n,$a1) $sumout
		return $sumout
	}
}
# 
proc probchange {a1 a2 lb} {
	global IDT T A MX
	set mu $MX($IDT,mu)
	if {$a1 == $a2} {
		# δab = 1
		set PC [expr double(exp(-$lb*$mu) + (1 - exp(-$lb*$mu)) * $A($IDT,f,$a2))]
	} else {
		# δab = 0
		set PC [expr double((1 - exp(-$lb*$mu)) * $A($IDT,f,$a2))] 
	}
	return $PC
}
# 
proc top-down {n} {
	global IDT T A S MX
	if {$n == 0} {
		foreach a $A($IDT,AnnotationList) {
			set MX($IDT,probaDOWN,$n,$a) 1.0
		}
	}
	set G $T($IDT,fils1,$n)
	set D $T($IDT,fils2,$n)
	set lg $T($IDT,dbl,$G) 
	set ld $T($IDT,dbl,$D) 
	calcul-up $G $lg $n $D $ld
	calcul-up $D $ld $n $G $lg
	if {$T($IDT,typ,$G) != "f"} {top-down $G}
	if {$T($IDT,typ,$D) != "f"} {top-down $D}
}
#
proc calcul-up {X lx P N ln} {
	global IDT T A S MX
	foreach a1 $A($IDT,AnnotationList) {
		if {$T($IDT,typ,$N) == "f"} {
			set tab($a1) [probchange $a1 $MX($IDT,a,$N) $ln] 
		} else {
			set aux 0.0
			foreach a2 $A($IDT,AnnotationList) {
				set aux [expr double($aux + ([probchange $a1 $a2 $ln] * $MX($IDT,probaUP,$N,$a2)))]
			}
			set tab($a1) $aux
		}
	}
	foreach a1 $A($IDT,AnnotationList) {
			set aux 0.0
			foreach a2 $A($IDT,AnnotationList) {
				set aux [expr double($aux + ([probchange $a1 $a2 $lx] * $MX($IDT,probaDOWN,$P,$a2) * $tab($a2)))]
			}
			set MX($IDT,probaDOWN,$X,$a1) $aux
	}
}
#
proc posteriors {n} {
	global IDT A T S MX SCALINGfactor
	set tot 0.0
	foreach a $A($IDT,AnnotationList) {
		#set probaUP [Scaling $MX($IDT,probaUP,$n,$a)]
		#set probaDOWN [Scaling $MX($IDT,probaDOWN,$n,$a)]
		
		if [catch {set aux [expr double($A($IDT,f,$a) * $MX($IDT,probaUP,$n,$a) * $MX($IDT,probaDOWN,$n,$a))]} result] {
			set aux 0.0
		}
		#set aux [expr double($A($IDT,f,$a) * $MX($IDT,probaUP,$n,$a) * $MX($IDT,probaDOWN,$n,$a))]
		#set aux [expr double($A($IDT,f,$a) * $probaUP * $probaDOWN)]
		#set f [expr 2 * $SCALINGfactor]
		#set aux [expr double ($aux / [::tcl::mathfunc::pow 2 $f])]
		set MX($IDT,posterior,$n,$a) $aux
		set tot [expr $tot + $aux]
	}
	foreach a $A($IDT,AnnotationList) {
		if [catch {set post [expr double($MX($IDT,posterior,$n,$a) / $tot)]} result] {
			set post 0.0
		}
		set MX($IDT,posterior,$n,$a) $post
	}
	if {$T($IDT,typ,$n) != "f"} {
		posteriors $T($IDT,fils1,$n)
		posteriors $T($IDT,fils2,$n)
	}
}
# like-down2 (au dessus) fusionne like-down et calcul
# n : id node ; a : annotation ; f : facteur d'echelle
proc like-down {n a f} {
	global IDT T A AA MX SCALINGseuil SCALINGSUM SCALINGfactor
	if {[info exists MX($IDT,probaUP,$n,$a)]} {return $MX($IDT,probaUP,$n,$a)}
	if {$T($IDT,typ,$n) == "f"} {
		# feuille
		if {$MX($IDT,a,$n) == $a} {set MX($IDT,probaUP,$n,$a) 1 ; return 1} else {set MX($IDT,probaUP,$n,$a) 0 ; return 0}
	} else {
		set Lg [Calcul $T($IDT,fils1,$n) $a $f]
		set Ld [Calcul $T($IDT,fils2,$n) $a $f]
		# scaling
		set Lg [Scaling $Lg]
		set Ld [Scaling $Ld]
		set MX($IDT,probaUP,$n,$a) [expr double($Lg * $Ld)]
		return $MX($IDT,probaUP,$n,$a)
	}
}
# n : id node ; a1 : annotation ; f : facteur echelle
proc Calcul {n a1 f} {
	global IDT T S A SCALINGSUM
	set lb [expr double($T($IDT,dbl,$n)*$f)]
	set sum 0
	foreach a2 $A($IDT,AnnotationList) {
		set sum [expr $sum + [probchange $a1 $a2 $lb] * [like-down $n $a2 $f]]
	}
	return $sum
}
################################################################################


################################################################################
# SCALING
################################################################################
proc Scaling {L} {
	global IDT T A AA SCALINGsum SCALINGfactor 
	set SCALINGsum [expr $SCALINGsum + $SCALINGfactor]
	set newL [expr double($L * [::tcl::mathfunc::pow 2 $SCALINGfactor])]
	return $newL
}
#
proc zTOlz {z lz} {
	set z [string range $z [expr [string first + $z 0] + 1] end ]
	if {$z > 63} {
		lappend lz 63
		zTOlz [expr $z -63] $lz
	} else {
		lappend lz $z
		return $lz
	}
}
################################################################################


################################################################################
# INFERENCE ANNOTATION ANCESTRALES PAR MAXIMUM DE VRAISSEMBLANCE MARGINAL F81-like
# structure de calcul MX 
# structure output AA($IDT,maklik,$id,$a)
###############################################################################
#
proc PaelaResetAA {what} {
	global IDT AA
	set pattern [format "%s%s%s%s" $IDT , $what ,*]
	foreach {k v} [array get AA $pattern] {
		catch {unset AA($k)}
	}
}
#
proc ALGO4 {} {
	global IDT S T A AA MX
	### INIT 
	catch {unset MX} ; # MX
	PaelaResetAA maxlik ; # reset AA sur IDT,distribution,*
	### ANNOTATION FREQUENCY, stockage A 
	if {$S(F81frequenceannotMODE) == 1 } {
		FrequenceAnnot01
	} elseif {$S(F81frequenceannotMODE) == 2 } {
		FrequenceAnnot02
	}
	### Facteur Normalisation
	FacteurNormalisation
	### Facteur d'echelle PRUNING and branch length standardisation
	set factor [NombreOrCAP 0.000001 1.0 0.0001] ; # Iteration Pruning
	set SUM [Pruning $factor]
	foreach {k v } [array get T $IDT,dbl,*] {
		set T($k) [expr double($v * $factor)]
	}
	### TOP DOWN
	top-down 0
	### POSTERIOR
	posteriors 0
	### DATA STRUCTURE TRANSFERT MX vers AA
	# OLD $T($IDT,all_cod)
	foreach id $T($IDT,allint_cod) {
		foreach a $A($IDT,AnnotationList) {
			set AA($IDT,distribution,$id,$a) $MX($IDT,posterior,$id,$a) 
		}
	}
	### INTERFACE, GRAPHICS & INFO
	#$S(infopath) insert end "> Ancestral annotations computed with MARGINAL maximum likelihood\n"
	#$S(infopath) insert end "> Priors type : $S(F81frequenceannotMODE)\n"
	#$S(infopath) insert end "> Normalization factor: $factor\n"
	#$S(infopath) insert end "> Maximum likelihood: $SUM\n"
	#$S(infopath) see current
}
################################################################################


################################################################################
# INFERENCE ANNOTATION ANCESTRALES PAR MAXIMUM DE VRAISSEMBLANCE JOINT - PUPKO
################################################################################
# ALGO5 applique un facteur d'échelle avant de lancer pupko : A VERIFIER si c'est OK
proc ALGO5 {} {
	global IDT S T PU A AA 
	### INIT
	catch {unset PU}
	PaelaResetAA pupko ; # reset AA sur IDT,pupko,*
	### ANNOTATION FREQUENCY
	if {$S(F81frequenceannotMODE) == 1 } {
		FrequenceAnnot01
	} elseif {$S(F81frequenceannotMODE) == 2 } {
		FrequenceAnnot02
	}
	### Facteur Normalisation
	FacteurNormalisation
	### PRUNING, scale factor and branch length standardisation
	set factor [NombreOrCAP 0.000001 1.0 0.0001]
	set SUM [Pruning $factor]
	foreach {k v } [array get T $IDT,dbl,*] {
		set T($k) [expr double($v * $factor)]
	}
	### PUPKO 
	PupkoUp 0
	### ROOT & GO
	set max 0.0
	foreach annotation $A($IDT,AnnotationList) {
		set val [expr double($PU($IDT,0,$annotation) * $A($IDT,f,$annotation))]
		if {$val > $max} {set max $val ; set annotationroot $annotation}
	}
	set AA($IDT,pupko,0) $annotationroot
	set PU($IDT,AnnotationRoot) $annotationroot
	PupkoDown 0 $annotationroot
	### DATA STRUCTURE TRANSFERT PU vers AA
	foreach id $T($IDT,allint_cod) {
		set AA($IDT,pupko,$id) $PU($IDT,$id) 
	}
	### INTERFACE, GRAPHICS & INFO
	#$S(infopath) insert end "> Ancestral annotations computed with JOINT maximum likelihood (PUPKO)\n"
	#$S(infopath) insert end "> Priors: $S(F81frequenceannotMODE)\n"
	#$S(infopath) insert end "> Normalization factor: $factor\n"
	#$S(infopath) see current
}
#
proc PupkoUp {n} {
	global IDT T A PU
	if {$T($IDT,typ,$n) == "f"} {
		foreach annotation $A($IDT,AnnotationList) {
			if {$annotation == $A($IDT,a,$n)} {
				set PU($IDT,$n,$annotation) 1.0
			} else {
				set PU($IDT,$n,$annotation) 0.0
			}
		}
	} else {
		PupkoUp $T($IDT,fils1,$n)
		PupkoUp $T($IDT,fils2,$n)
		foreach annotation $A($IDT,AnnotationList) {
			set probMaxfils1 0.0
			set probMaxfils2 0.0
			foreach annotationFils $A($IDT,AnnotationList) {
				set changefils1 [probchange $annotation $annotationFils $T($IDT,dbl,$T($IDT,fils1,$n))]
				set changefils2 [probchange $annotation $annotationFils $T($IDT,dbl,$T($IDT,fils2,$n))]
				set probfils1 [expr double($PU($IDT,$T($IDT,fils1,$n),$annotationFils) * $changefils1)]
				set probfils2 [expr double($PU($IDT,$T($IDT,fils2,$n),$annotationFils) * $changefils2)]
				if {$probfils1 >= $probMaxfils1} {set probMaxfils1 $probfils1 ; set Afils1Max $annotationFils} 
				if {$probfils2 >= $probMaxfils2} {set probMaxfils2 $probfils2 ; set Afils2Max $annotationFils} 
			}	
			if [catch {set probaUp [expr double($probMaxfils1 * $probMaxfils2)]}] {
				set probaUp 0.0
			}
			set PU($IDT,$n,$annotation) $probaUp
			set PU($IDT,$n,$annotation,fils1) $Afils1Max
			set PU($IDT,$n,$annotation,fils2) $Afils2Max
		}
	}
}
#
proc PupkoDown {n a} {
	global IDT S T A PU 
	if {$T($IDT,typ,$n) != "f"} {
		set PU($IDT,$n) $a
		PupkoDown $T($IDT,fils1,$n) $PU($IDT,$n,$a,fils1) 
		PupkoDown $T($IDT,fils2,$n) $PU($IDT,$n,$a,fils2) 
	}
}
###############################################################################


###############################################################################
# TRANSITION QUERY
###############################################################################
# 
proc TransitionQueryData {} {
	global IDT T S TQ A AA
	catch {unset TQ}
	# migration data
	switch -exact $S(TransitionQueryWhat) {
		majority {TransitionQueryMajoritaire}
		pupko {TransitionQueryPupko}
		parsimony {TransitionQueryparsimony}
	}
	# OK feuilles 
	foreach n $T($IDT,ue_cod) {
		#set feuille $T($IDT,dbv,$n)
		foreach var $A($IDT,AnnotationList) {
			# ok 
			if {$AA($IDT,distribution,$n,$var) == 1} {
				set TQ($IDT,annotation,$n) $var
				set TQ($IDT,proba,$n) 1.00
			}
		}
	}
}
# transfert MAJORITAIRE
proc TransitionQueryMajoritaire {} {
	global IDT T S A AA P Z TQ
	# identification majoritaire
	foreach n $T($IDT,allint_cod) {
		set max 0
		foreach {k v} [array get AA $IDT,distribution,$n,*] {
			set a [lindex [split $k ,] end]
			if {$v > $max} {
				set max $v
				set AA($IDT,majority,$n) $a
			}
		}
	}
	# transfert
	foreach n $T($IDT,allint_cod) {
		set annotation $AA($IDT,majority,$n)
		set TQ($IDT,annotation,$n) $annotation
		set TQ($IDT,proba,$n) $AA($IDT,distribution,$n,$annotation)
	}
}
# transfert PUPKO
proc TransitionQueryPupko {} {
	global IDT T AA TQ
	foreach n $T($IDT,allint_cod) {
		set TQ($IDT,annotation,$n) $AA($IDT,pupko,$n)
		set TQ($IDT,proba,$n) 0.99
	}
}
# transfert parsimony
proc TransitionQueryparsimony {} {
	global IDT AA T TQ
	foreach n $T($IDT,allint_cod) {
		set TQ($IDT,annotation,$n) $AA($IDT,parsimony,$n)
		set TQ($IDT,proba,$n) 0.99
	}
}
#
proc TransitionQueryRun {} {
	global IDT T S A AA P Z 
	TransitionQueryData
	set scenario $S(TransitionQueryString)
	TransitionQueryHistory
	ResetLineWidth
	ResetTreeForegroundColor
	$S(canvas) delete TRANSITIONPATH
	$S(canvas) delete TRANSITIONCURVEPATH
	TransitionQueryBottomUp
	$S(canvas) raise TRANSITIONPATH
}
#
proc TransitionQueryHistory {{history 1}} {
	global S
	if {$history == 1} {
		set S(TransitionHistory) [DelRep [lappend S(TransitionHistory) $S(TransitionQueryString)]]
	}
}
#
proc TransitionQueryRunPostCommand {query} {
	global IDT T S A AA P Z 
	set S(TransitionQueryString) $query
	TransitionQueryRun
}
#
proc TransitionQueryRunAlternate {} {
	global IDT T S A AA P Z 
	TransitionQueryResetHi
	update idletasks
	for {set i 0} {$i <= 3} {incr i} {
		foreach s $S(TransitionHistory) {
			after 1000
			TransitionQueryRunPostCommand $s
			update idletasks
		}
	}
}
#
proc TransitionQueryFirst {} {
	global IDT T S A AA P Z 
	set s [lindex $S(TransitionHistory) 0]
	TransitionQueryRunPostCommand $s
	update idletasks
}
#
proc TransitionQueryPrevious {} {
	global IDT T S A AA P Z 
	regsub -all " " $S(TransitionQueryString) "" current
	set index 0 
	set long [llength $S(TransitionHistory)]
	foreach s $S(TransitionHistory) {
		regsub -all " " $s "" candidate 
		if {$current == $candidate} {
			set go [expr $index -1]
			if {$go == -1} {
				# de la premiere à la derniere
				TransitionQueryRunPostCommand  [lindex $S(TransitionHistory) end]
			} else {
				TransitionQueryRunPostCommand  [lindex $S(TransitionHistory) $go]
			}
			update idletasks
		}
		incr index 1
	}
}
#
proc TransitionQueryPlay {} {
	TransitionQueryRunAlternate
}
#
proc TransitionQueryNext {} {
	global IDT T S A AA P Z 
	regsub -all " " $S(TransitionQueryString) "" current
	set index 0 
	set long [llength $S(TransitionHistory)]
	foreach s $S(TransitionHistory) {
		regsub -all " " $s "" candidate 
		incr index 1
		if {$current == $candidate} {
			if {$index == $long} {
				# de la derniere à la premiere
				TransitionQueryRunPostCommand  [lindex $S(TransitionHistory) 0]
			} else {
				TransitionQueryRunPostCommand  [lindex $S(TransitionHistory) $index]
			}
			update idletasks
		}
	}
}
#
proc TransitionQueryLast {} {
	global IDT T S A AA P Z 
	set s [lindex $S(TransitionHistory) end]
	TransitionQueryRunPostCommand $s
	update idletasks
}
#
proc TransitionQueryResetHi {} {
	global IDT T S A AA P Z 
	ResetLineWidth
	ResetTreeForegroundColor
	$S(canvas) delete TRANSITIONPATH
	$S(canvas) delete TRANSITIONCURVEPATH

}
#
proc TransitionHistory {m} {
	global S
	$m delete 0 end
	foreach h $S(TransitionHistory) {
		$m add command -label "$h" -command "TransitionQueryRunPostCommand [list $h]"
	}
	$m add separator
	$m add command -label "Reset history" -command "set S(TransitionHistory) {}"
	$m add command -label "Reset highlight" -command "TransitionQueryResetHi"
	$m add command -label "Load transitions queries..." -command "TranstionQueryLoad"
	$m add command -label "Save history..." -command "TranstionQuerySave"
	#$m add separator
	#$m add command -label "Alternate" -command TransitionQueryRunAlternate
}
#
proc TranstionQuerySave {} {
	global S 
	set file [tk_getSaveFile -initialdir $S(pathuser) -defaultextension ".txt" \
			-title "Save transitions queries..."]
	if {$file != "" } {
		set S(pathuser) [file dirname $file]
		set fid [open $file w]
		foreach s $S(TransitionHistory) {
			regsub -all "  " $s " " row
			puts $fid $s
		}
		close $fid
	}
}
#
proc TranstionQueryLoad {} {
	global S 
	set file [tk_getOpenFile -initialdir $S(pathuser) -title "Load transitions queries... "]
	if {$file != "" } {
		if [catch {open $file r} fid] {
			return 0
		} else {
			set S(TransitionHistory) {}
			while {[eof $fid] != 1} {
				set row  [gets $fid]
				regsub -all "  " $row " " row
				lappend S(TransitionHistory) $row
			}
		}
	}
}

# pour chaque feuille etablir son profil AA de la feuille à la racine (sans les repetitions )
# conserver les nodes
# ensuite match de ces profils avec query, incluant *
# attention query peut correspondre a des nodes internes uniquement 
proc TransitionQueryBottomUp {} {
	global IDT T S A AA P Z TQ TQP
	catch {unset TQP}
	# inference des profils AA par feuille
	# VOIR si extensions a tous les codes
	foreach f $T($IDT,ue_cod) {
		# liste des nodes de la feuille à la racine et liste  AA sans repetition
		TransitionQueryBottomUpNodeListANDprofil $f $f
	}
	# Matching des profils avec QUERY et allumage des profils retenus
	set query ""
	foreach element $S(TransitionQueryString) {
		set query [format "%s%s" $query $element]
	}
	# chaque profil liste vers chaine
	foreach f $T($IDT,ue_cod) {
		set profil "" 
		foreach element $TQP($f,profil) {
			set profil [format "%s%s" $profil $element]
		}
		# comparaison
		#puts "QUERY: $query PROFIL: $profil"
		if {[string match $query $profil]} {
			foreach n $TQP($f,nodes) {
				TransitionQueryNodeHi $n
			}
		}
	}
}
#
proc TransitionQueryBottomUpNodeListANDprofil {f node} {
	global IDT T S A AA P Z TQ TQP
	lappend TQP($f,nodes) $node
	lappend TQP($f,profil) $TQ($IDT,annotation,$node)
	if {$T($IDT,typ,$node) != "r"} {
		TransitionQueryBottomUpNodeListANDprofil $f $T($IDT,pere,$node)
	} else {
		set TQP($f,profil) [lreverse [DelRepConsecutive $TQP($f,profil)]]
		return
	}
}
### A B B A B B -> A B A B
proc DelRepConsecutive {lin}   {
	set lou {}
	foreach i $lin {
		if {[lindex $lou end] != $i} {lappend lou $i}
	}
	return $lou
}

#
proc TransitionQueryNodeHi {n} {
	global IDT T S A AA P Z TQ
	set color $S(ColorAnnotation,$TQ($IDT,annotation,$n))
	if {$T($IDT,typ,$n) == "r"} {
		# si root 
		set co [$S(canvas) coords V$n]
		$S(canvas) create line $co -width 2 -fill $color -tag TRANSITIONPATH
	} elseif {$T($IDT,typ,$n) == "f"} {
		# si feuille
		set co [$S(canvas) coords H$n]
		$S(canvas) create line $co -width 2 -fill $color -tag TRANSITIONPATH
	} else {
		# noeud interne
		set co [$S(canvas) coords H$n]
		$S(canvas) create line $co -width 2 -fill $color -tag TRANSITIONPATH
		set coV [$S(canvas) coords V$n]
		$S(canvas) create line $coV -width 2 -fill $color -tag TRANSITIONPATH

		#set Anode $TQ($IDT,annotation,$n)
		#set AnodeF1 $TQ($IDT,annotation,$T($IDT,fils1,$n))
		#set AnodeF2 $TQ($IDT,annotation,$T($IDT,fils2,$n))
		#if {$Anode == $AnodeF1} {
		#	$S(canvas) create line [lindex $co 0] [lindex $co 1] [lindex $coV 0] [lindex $coV 1] -width 2 -fill $color -tag TRANSITIONPATH
		#}
		#if {$Anode == $AnodeF2} {
		#	$S(canvas) create line [lindex $co 0] [lindex $co 1] [lindex $coV 2] [lindex $coV 3]  -width 2 -fill $color -tag TRANSITIONPATH
		#}
		
	}
	$S(canvas) lower TRANSITIONPATH
}
#
proc TransitionQueryHi {solution} {
	global IDT T S A AA P Z TQ
	set nSTART [lindex $solution end]
	set nSTOP [lindex $solution 0]
	if {$nSTART == "*"} {
		# NEW
		set lnodes [lreverse [TransitionQueryHiLnode $nSTART $nSTOP {}]]
	} else {
		set lnodes [lreverse [TransitionQueryHiLnode $nSTART $nSTOP {}]]
	}
	set lxy {}
	foreach n $lnodes {
		set color $S(ColorAnnotation,$TQ($IDT,annotation,$n))
		set co [$S(canvas) coords H$T($IDT,pere,$n)]
		lappend lxy [lindex $co 0] [lindex $co 1]
		set co [$S(canvas) coords H$n]
		lappend lxy [lindex $co 2] [lindex $co 3] [lindex $co 2] [lindex $co 3] [lindex $co 0] [lindex $co 1] 
		$S(canvas) create line $lxy -width 3 -fill $color -tag TRANSITIONPATH
		set lxy {}
	}
	$S(canvas) raise TRANSITIONPATH
}
#
proc TransitionQueryInsert {s} {
	global IDT T S A AA P Z 
	if {$S(TransitionQueryString) == "?"} {
		set S(TransitionQueryString) $s
	} else {
		# insertion
		$S(TransitionQueryStringPath) insert insert " $s "
	}
}
#
proc TransitionQueryInsertList {m} {
	global IDT S A
	$m delete 0 end
	$m add command -label "Clear" -command "set S(TransitionQueryString) ?" -background black
	$m add separator
	$m add command -label "*" -command "TransitionQueryInsert *" -background black
	$m add separator
	foreach v [lsort -dictionary $A($IDT,AnnotationList)] {
		$m add command -label "$v" -command "TransitionQueryInsert $v" -background $S(ColorAnnotation,$v)
	}
}
###############################################################################

###############################################################################
# BUBBLE
###############################################################################
	
#
proc BubbleCAP {} {
	global IDT T S Bubble
	# init
	catch {unset Bubble}
	#
	switch -exact $S(AAbubbleType) {
		majority {BubbleMajoritaire}
		pupko {BubblePupko}
		parsimony {Bubbleparsimony}
	}
	# presence selection ?
	#set selection {}
	#foreach v $A($IDT,AnnotationList) {
	#	lappend selection $S($IDT,BubbleOnly,$v) 
	#}
	#if {[lsearch $selection 1] == -1} {
	#	tk_messageBox -type ok  -message "Please, select at least one annotation" -icon info
	#} else {
		# display
		#BubbleDisplayCap 0 $Bubble($IDT,0) 
	#}
	# display
	BubbleDisplayCap 0 $Bubble($IDT,0) 

}
# majoritaire d'une distribution (max. lik. ou baeysian)
proc BubbleMajoritaire {} {
	global IDT T S A AA P Z Bubble
	# recherche de la majoritaire
	# A VENIR : test si calcul deja fait
	foreach n $T($IDT,allint_cod) {
		set max 0
		foreach {k v} [array get AA $IDT,distribution,$n,*] {
			set a [lindex [split $k ,] end]
			if {$v > $max} {
				set max $v
				set AA($IDT,majority,$n) $a
			}
		}
	}
	foreach n $T($IDT,allint_cod) {
			set Bubble($IDT,$n) $AA($IDT,majority,$n)
	}
}
# pupko
proc BubblePupko {} {
	global IDT T AA Bubble
	# transition AA vers Bubble
	foreach n $T($IDT,allint_cod) {
		set Bubble($IDT,$n) $AA($IDT,pupko,$n)
	}
}
# apres parsimony
proc Bubbleparsimony {} {
	global IDT AA T Bubble S
	# transition AA vers Bubble
	foreach n $T($IDT,allint_cod) {
		if {[llength $AA($IDT,parsimony,$n)] == 1} {
			set Bubble($IDT,$n) $AA($IDT,parsimony,$n)
		} else {
			set Bubble($IDT,$n) white
			set S($IDT,BubbleOnly,white) 1
		}
	}
}
#
proc BubbleOnlyCommand {m} {
	global  A S IDT
	$m delete 0 end 
	$m add command -label "Select all" -command "BubbleOnlyCommandSelectAll"
	$m add command -label "Unselect all" -command "BubbleOnlyCommandUnselectAll"
	$m add separator
	foreach v [lsort -dictionary $A($IDT,AnnotationList)] {
		$m add checkbutton -label "$v" -variable S($IDT,BubbleOnly,$v)  -background $S(ColorAnnotation,$v) -command BubbleCAP
	}
}
#
proc BubbleOnlyCommandSelectAll {} {
	global  A S IDT
	foreach v $A($IDT,AnnotationList) {
		set S($IDT,BubbleOnly,$v) 1
	}
	BubbleCAP
}
proc BubbleOnlyCommandUnselectAll {} {
	global  A S IDT
	foreach v $A($IDT,AnnotationList) {
		set S($IDT,BubbleOnly,$v) 0
	}
	BubbleCAP
}
#
proc BubbleDisplayCap {n a} {
	global S Bubble
	$S(canvas) delete bubblecircle
	set delta [expr $S(SurfaceMaxBubble) / 2.0]
	BubbleDisplay $n $delta
	BubbleSimplifyVisuCAP
}
#
proc BubbleDisplay {n d} {
	global IDT S T A P Bubble
	if {$T($IDT,typ,$n) != "f"} {
		set a $Bubble($IDT,$n)
		if {$S($IDT,BubbleOnly,$a)} {
			set co [$S(canvas) coords H$n]
			set x [lindex $co 0]
			set y [expr ([lindex $co 1] + [lindex $co 3]) / 2.0 ]
			# pb des noeuds non résolu (eg parsimony)
			if [catch {set color $S(ColorAnnotation,$a)} result] {
				set color white
			} 
			$S(canvas) create oval [expr $x - $d] [expr $y - $d] [expr $x + $d] [expr $y + $d] \
				-fill $color -outline black -tags "bubblecircle bubblecirclenode=$n bubbleannotation=$a"
		} else {
			$S(canvas) delete bubbleannotation=$a
		}
		BubbleDisplay $T($IDT,fils1,$n) $d
		BubbleDisplay $T($IDT,fils2,$n) $d
	}
}
#
proc BubbleSimplifyVisuCAP {} {
	global IDT T S A P  Bubble 
	if {$S(AAbubbleSimplify)} {
		set Bubble(liste) {} 
		BubbleSimplifyVisu 0 $Bubble($IDT,0)
	} else {
		$S(canvas) itemconfigure bubblecircle -state normal
	}
}
#
proc BubbleSimplifyVisu {n a} {
	global IDT T S A P  Bubble
	set w $S(canvas)
	if {[NodeAnnotTest $n $a]} {
		# un cluster est identifié
		# les nodes sous-jacents ont la même annotation
		BubbleHidden $w $n  
		set Bubble(liste) [concat $Bubble(liste) " $n"]
		return
	} else {
		# relance sur les fils
		if {$T($IDT,typ,$T($IDT,fils1,$n)) != "f"} {
			BubbleSimplifyVisu $T($IDT,fils1,$n) $Bubble($IDT,$T($IDT,fils1,$n))
		} 
		if {$T($IDT,typ,$T($IDT,fils2,$n)) != "f"} {
			BubbleSimplifyVisu $T($IDT,fils2,$n) $Bubble($IDT,$T($IDT,fils2,$n)) 
		}
	}
}
#
proc NodeAnnotTest {n a } {
	global IDT T S A P Bubble
	set t 1
	foreach ni [Node2SubTreeNodes $n] {
		if { $T($IDT,typ,$ni) != "f"} {
			if {$Bubble($IDT,$ni) != $a} {set t 0 }
		}
	}
	return $t
}
# utilise dans bubble simplify
proc BubbleHidden {w n} {
	global IDT T Bubble
	set n1 $T($IDT,fils1,$n)
	set n2 $T($IDT,fils2,$n)
	$w itemconfigure bubblecirclenode=$n1 -state hidden
	$w itemconfigure bubblecirclenode=$n2 -state hidden
	if {$T($IDT,typ,$n1) != "f"} {
		BubbleHidden $w $n1
	} 
	if {$T($IDT,typ,$n2) != "f"} {
		BubbleHidden $w $n2
	} 
	return
}
#
proc BubbleOnOff {} {
	global IDT S Bubble
	set w $S(canvas)
	set item [lindex [$w find withtag bubblecircle] 0]
	if {$item != ""} {
		set state [lindex [$w itemconfigure $item -state] end]
		if {$state == "hidden"} {
			$w itemconfigure bubblecircle -state normal
			BubbleSimplifyVisuCAP
		} else {
			$w itemconfigure bubblecircle -state hidden
		}
	}
}
#
proc BubbleZoom {s} {
	global IDT S T P A Bubble
	set delta 1.0
	switch $s {
		-	{
			set S(SurfaceMaxBubble) [expr $S(SurfaceMaxBubble) -$delta]
		}
		+	{
			set S(SurfaceMaxBubble) [expr $S(SurfaceMaxBubble) +$delta]
		}
	}
	if {$S(SurfaceMaxBubble) < 0.0} {
		set S(SurfaceMaxBubble) 0.0
	}
	
	
	BubbleDisplayCap 0 $Bubble($IDT,0) 
}
#
proc TransitionMapZoom {s} {
	global IDT S T P A Bubble
	switch $s {
		-	{
			set S(SurfaceBubbleTransitionMap) [expr $S(SurfaceBubbleTransitionMap) -1]
		}
		+	{
			set S(SurfaceBubbleTransitionMap) [expr $S(SurfaceBubbleTransitionMap) +1]

		}
	}
	if {$S(SurfaceBubbleTransitionMap) < 0.0} {
		set S(SurfaceBubbleTransitionMap) 0.0
	}
	# RESTORE, tout sauf S(SurfaceBubbleTransitionMap) 
	switch -exact $S($S(canvas2),mode) { 
		mono {
			set S(TransitionMapMonoLayout) $S($S(canvas2),layout) 
			set S(TransitionMapLayoutView) $S($S(canvas2),view) 
			set S(TransitionMapLayoutViewCluster) $S($S(canvas2),cluster) 
			set S(TransitionMapMonoNode) $S($S(canvas2),node) 
			set S(TransitionMapMonoAnnotation) $S($S(canvas2),annotation) 
			set S(TransitionMapMonoArrow) $S($S(canvas2),arrow) 
		}
		multi {
			set S(TransitionMapMultiLayout) $S($S(canvas2),layout) 
			set S(TransitionMapMultiSize) $S($S(canvas2),size) 
			set S(TransitionMapMultiAnnotation) $S($S(canvas2),annotation) 
			set S(TransitionMapMultiArrow) $S($S(canvas2),arrow) 
		}
	}
	set S(TransitionMapWhat) $S($S(canvas2),what)
	set S(TransitionMapFactorXuser) $S($S(canvas2),TransitionMapFactorXuser)
	set S(TransitionMapFontSize) $S($S(canvas2),TransitionMapFontSize)
	set S(TransitionMapFactorYuser) $S($S(canvas2),TransitionMapFactorYuser)
	
	TransitionMap $S($S(canvas2),mode)
}
proc TransitionMapZoomText {s} {
	global IDT S T P A Bubble
	switch $s {
		-	{
			set S(TransitionMapFontSize) [expr $S(TransitionMapFontSize) -1]
		}
		+	{
			set S(TransitionMapFontSize) [expr $S(TransitionMapFontSize) +1]

		}
	}
	if {$S(TransitionMapFontSize) < 1} {
		set S(TransitionMapFontSize) 1
	}
	# RESTORE, tout sauf  S(TransitionMapFontSize) 
	switch -exact $S($S(canvas2),mode) { 
		mono {
			set S(TransitionMapMonoLayout) $S($S(canvas2),layout) 
			set S(TransitionMapLayoutView) $S($S(canvas2),view) 
			set S(TransitionMapLayoutViewCluster) $S($S(canvas2),cluster) 
			set S(TransitionMapMonoNode) $S($S(canvas2),node) 
			set S(TransitionMapMonoAnnotation) $S($S(canvas2),annotation) 
			set S(TransitionMapMonoArrow) $S($S(canvas2),arrow) 
		}
		multi {
			set S(TransitionMapMultiLayout) $S($S(canvas2),layout) 
			set S(TransitionMapMultiSize) $S($S(canvas2),size) 
			set S(TransitionMapMultiAnnotation) $S($S(canvas2),annotation) 
			set S(TransitionMapMultiArrow) $S($S(canvas2),arrow) 
		}
	}
	set S(TransitionMapWhat) $S($S(canvas2),what)
	set S(TransitionMapFactorXuser) $S($S(canvas2),TransitionMapFactorXuser)
	set S(TransitionMapFactorYuser) $S($S(canvas2),TransitionMapFactorYuser)
	set S(SurfaceBubbleTransitionMap) $S($S(canvas2),SurfaceBubbleTransitionMap)

	
	TransitionMap $S($S(canvas2),mode)
}
proc TransitionMapFactorXuser {s} {
	global IDT S T P A Bubble
	switch $s {
		+	{
			set S(TransitionMapFactorXuser) [expr $S(TransitionMapFactorXuser) *1.25]
		}
		-	{
			set S(TransitionMapFactorXuser) [expr $S(TransitionMapFactorXuser) * 0.75]
		}
	}
	if {$S(TransitionMapFactorXuser) <= 0} {
		set S(TransitionMapFactorXuser) 1
	}
	
	# RESTORE, tout sauf  S(TransitionMapFactorXuser) 
	switch -exact $S($S(canvas2),mode) { 
		mono {
			set S(TransitionMapMonoLayout) $S($S(canvas2),layout) 
			set S(TransitionMapLayoutView) $S($S(canvas2),view) 
			set S(TransitionMapLayoutViewCluster) $S($S(canvas2),cluster) 
			set S(TransitionMapMonoNode) $S($S(canvas2),node) 
			set S(TransitionMapMonoAnnotation) $S($S(canvas2),annotation) 
			set S(TransitionMapMonoArrow) $S($S(canvas2),arrow) 
		}
		multi {
			set S(TransitionMapMultiLayout) $S($S(canvas2),layout) 
			set S(TransitionMapMultiSize) $S($S(canvas2),size) 
			set S(TransitionMapMultiAnnotation) $S($S(canvas2),annotation) 
			set S(TransitionMapMultiArrow) $S($S(canvas2),arrow) 
		}
	}
	set S(TransitionMapWhat) $S($S(canvas2),what)
	set S(TransitionMapFontSize) $S($S(canvas2),TransitionMapFontSize)
	set S(TransitionMapFactorYuser) $S($S(canvas2),TransitionMapFactorYuser)
	set S(SurfaceBubbleTransitionMap) $S($S(canvas2),SurfaceBubbleTransitionMap)	
	
	
	
	
	TransitionMap $S($S(canvas2),mode)
}
#
proc TransitionMapFactorYuser {s} {
	global IDT S T P A Bubble
	switch $s {
		+	{
			set S(TransitionMapFactorYuser) [expr $S(TransitionMapFactorYuser) *1.25]
		}
		-	{
			set S(TransitionMapFactorYuser) [expr $S(TransitionMapFactorYuser) * 0.75]
		}
	}
	if {$S(TransitionMapFactorYuser) <= 0} {
		set S(TransitionMapFactorYuser) 1
	}
	# RESTORE, tout sauf S(TransitionMapFactorYuser) 
	switch -exact $S($S(canvas2),mode) { 
		mono {
			set S(TransitionMapMonoLayout) $S($S(canvas2),layout) 
			set S(TransitionMapLayoutView) $S($S(canvas2),view) 
			set S(TransitionMapLayoutViewCluster) $S($S(canvas2),cluster) 
			set S(TransitionMapMonoNode) $S($S(canvas2),node) 
			set S(TransitionMapMonoAnnotation) $S($S(canvas2),annotation) 
			set S(TransitionMapMonoArrow) $S($S(canvas2),arrow) 
		}
		multi {
			set S(TransitionMapMultiLayout) $S($S(canvas2),layout) 
			set S(TransitionMapMultiSize) $S($S(canvas2),size) 
			set S(TransitionMapMultiAnnotation) $S($S(canvas2),annotation) 
			set S(TransitionMapMultiArrow) $S($S(canvas2),arrow) 
		}
	}
	set S(TransitionMapWhat) $S($S(canvas2),what)
	set S(TransitionMapFactorXuser) $S($S(canvas2),TransitionMapFactorXuser)
	set S(TransitionMapFontSize) $S($S(canvas2),TransitionMapFontSize)
	set S(SurfaceBubbleTransitionMap) $S($S(canvas2),SurfaceBubbleTransitionMap)

	
	TransitionMap $S($S(canvas2),mode) 
	
}

###############################################################################


################################################################################
### TRANSITION MAP DYNAMIC
proc TransitionMapDynamic {} {
	global IDT S T TM A AA
	set w $S(canvas)
	if {[$w find withtag tresholdbar] != ""} {
		$w delete tresholdbar
		set S(tool) moveall
	} else {
		# curseur vertical sur arbre init
		set co [$w bbox Z]
		set x1 [lindex $co 0]
		set y1 [lindex $co 1]
		set y2 [lindex $co 3]
		set deltay [expr $y2 - $y1]
		$w create line [expr $x1 + 0 ] [expr $y1 - 10 ] [expr $x1 + 0 ] [expr $y1 + $deltay + 10 ] -fill magenta -width 2 -tag "tresholdbar"
		set S(tool) tresholdbar
		set S(TransitionMapDynamicMoveY1) [expr $y1 - 10 ]
		set S(TransitionMapDynamicMoveY2) [expr $y1 + $deltay + 10 ]
		# DATA
		TransitionMapData
	}
}
#
proc TransitionMapDynamicMove {W x y} {
	global IDT S T TM A AA
	set w $S(canvas)
	set xok [$w canvasx $x]
	set S(TransitionMapDynamicMoveTreshol) $xok
	$w delete tresholdbar
	$w create line $xok $S(TransitionMapDynamicMoveY1) $xok $S(TransitionMapDynamicMoveY2) -fill magenta -width 2 -tag "tresholdbar"
	$S(canvas2) delete all
	TransitionMapDynamicNewick
	update idletasks
}
#
proc TransitionMapDynamicNewick {} {
	global A AA Z IDT T TT S pupkonewickstring TM
	catch {unset Z}
	# on reprend les variables graphiques
	TransitionMapSetRestore
	# inference du graphe d'enfants/annotations
	set Z($IDT,enfants,0) {}
	TransitionMapDynamicNewickGo 0 0 0 x $TM($IDT,annotation,0)
	if {$Z($IDT,enfants,0) != {}} {
		set pupkonewickstring [format "%s%s%s%s%s%s%s" "(" NODE0 ")" $TM($IDT,annotation,0) =0 :0 ";" ]
		if {$S($S(canvas2),mode) == "mono"} {
			TransitionMapNewickRecurVERSIONmono 0
			TransitionMapParser
			TransitionMapLayoutMono
		} else {
			TransitionMapNewickRecurVERSIONmulti 0 
			TransitionMapParser
			TransitionMapLayoutMulti
		}
	} else {
		# si que root
		set w $S(canvas2)
		set TAGOBJECT [format "%s%s" OBJECT= [randomColor]]
		set node 0
		set treenode 0
		set annotation $TM($IDT,annotation,$node)
		set annotation3 [string range $TM($IDT,annotation,$node) 0 2]
		set s $S(SurfaceBubbleTransitionMap) ;# taile pastille
		set fo [list Helvetica $S(TransitionMapFontSize) normal]
		set x 10
		set y 10
		foreach {x1 y1 x2 y2} [$w cget -scrollregion] {break}
		set y [expr ($y2 + $y1) / 2.0]
		# attention ambiguité AA (parcimonie)
		if {[info exist S(ColorAnnotation,$annotation)]} {
			# pas d'ambiguité
			set color $S(ColorAnnotation,$annotation)
			set annotation3 [string range $annotation 0 2]
		} else {
			# annotation ancestrales multiple (non resolue en parsimony)
			set color lightgrey
			set annotation3 *
		}
		set treefeuilles [Node2Leafs2 $treenode]
		set nbfeuilles [llength $treefeuilles]
		# pourcentage de feuilles de l'arbre phylogenetique issues de node ayant la meme annotation que node
		set nbfeuillesmemeannotation 0.0
		foreach fid $treefeuilles {
			set f $T($IDT,vbd,$fid)
			if {$TM($IDT,annotation,$f) == $annotation} {set nbfeuillesmemeannotation [expr $nbfeuillesmemeannotation + 1.0]}
		}
		set percent [expr $nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0]
		set complement [expr 1 - $percent ]
		set apc [list [list "?" [rnd $complement 2] "#DDD"] [list $annotation [rnd $percent 2] $color]]
		# draw
		if {$S(TransitionMapLayoutViewCluster) == "piechart"} {
			piechart2 $w [expr $x - $s ] [expr $y- $s] [expr $s + $s] [expr $s + $s] $treenode $apc "TMCIRCLE NODE$node TREENODE$treenode ANNOTATION=$annotation $TAGOBJECT"
		} else {
			$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
			-fill $color -outline white -tags "TMCIRCLE NODE$node  TREENODE$treenode ANNOTATION=$annotation  $TAGOBJECT" 
		}
		set what ""
		set percentText [format "%s%s" [expr round(($nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0) *100.00)] "%"]
		if {$S(TransitionMapMonoNode)} {append what $percentText}
		if {$S(TransitionMapMonoAnnotation)} {append what \n$annotation3}
		$w create text $x $y -text $what -fill black -font $fo -justify center -tags "TMTEXT NODE$node ANNOTATION=$annotation " 
		$w configure -scrollregion [$w bbox all]
		update idletasks
	}
}
#
proc TransitionMapDynamicNewickGo {np n g ap a} {
	global IDT T S A P Z TM
	set co [$S(canvas) bbox H$n]
	set xn [lindex $co 0]
	if {$xn <= $S(TransitionMapDynamicMoveTreshol)} {
		if {$ap == $a} {
			if {$T($IDT,typ,$n) != "f"} {
				TransitionMapDynamicNewickGo $np $T($IDT,fils1,$n) $g $a $TM($IDT,annotation,$T($IDT,fils1,$n))
				TransitionMapDynamicNewickGo $np $T($IDT,fils2,$n) $g $a $TM($IDT,annotation,$T($IDT,fils2,$n))
			} 
		} else {
			# n fait partit du scenario 
			if {$np != $n} {
				if {$T($IDT,typ,$n) != "f"} {
					lappend Z($IDT,enfants,$np) $n
					set Z($IDT,enfants,$n) {}
				}
			} 
			#
			if {[AncetralAnnotTest $n $a]} {
				# si tous les nodes internes enfants de n ont la même annotation
				set Z($IDT,enfants,$n) {}
				return
			} else {
				# on continue si n n'est pas une feuille
				if {$T($IDT,typ,$n) != "f"} {
					TransitionMapDynamicNewickGo $n $T($IDT,fils1,$n) [expr $g +1 ] $a $TM($IDT,annotation,$T($IDT,fils1,$n))
					TransitionMapDynamicNewickGo $n $T($IDT,fils2,$n) [expr $g +1 ] $a $TM($IDT,annotation,$T($IDT,fils2,$n))
				}
			}
		}
	} else {
		set Z($IDT,enfants,$n) {}
		return
	}
}
################################################################################


################################################################################
### TRANSITION MAP
### data structure TM
################################################################################
#
proc TransitionMapData {} {
	global IDT T S TM A AA
	#catch {unset TM}
	# remplissage TM
	# OK noeuds internes TM noeud internes
	switch -exact $S(TransitionMapWhat) {
		majority {TransitionMapMajoritaire}
		pupko {TransitionMapPupko}
		parsimony {TransitionMapparsimony}
	}
	# OK feuilles 
	foreach n $T($IDT,ue_cod) {
		#set feuille $T($IDT,dbv,$n)
		foreach var $A($IDT,AnnotationList) {
			# ok 
			if {$AA($IDT,distribution,$n,$var) == 1} {
				set TM($IDT,annotation,$n) $var
				set TM($IDT,proba,$n) 1.00
			}
		}
	}
}
# transfert MAJORITAIRE
proc TransitionMapMajoritaire {} {
	global IDT T S A AA P Z TM
	# identification majoritaire
	foreach n $T($IDT,allint_cod) {
		set max 0
		foreach {k v} [array get AA $IDT,distribution,$n,*] {
			set a [lindex [split $k ,] end]
			if {$v > $max} {
				set max $v
				set AA($IDT,majority,$n) $a
			}
		}
	}
	# transfert
	foreach n $T($IDT,allint_cod) {
		set annotation $AA($IDT,majority,$n)
		set TM($IDT,annotation,$n) $annotation
		set TM($IDT,proba,$n) $AA($IDT,distribution,$n,$annotation)
	}
}
# transfert PUPKO
proc TransitionMapPupko {} {
	global IDT T AA TM
	foreach n $T($IDT,allint_cod) {
		set TM($IDT,annotation,$n) $AA($IDT,pupko,$n)
		set TM($IDT,proba,$n) 0.99
	}
}
# transfert parsimony
proc TransitionMapparsimony {} {
	global IDT AA T TM
	foreach n $T($IDT,allint_cod) {
		set TM($IDT,annotation,$n) $AA($IDT,parsimony,$n)
		set TM($IDT,proba,$n) 0.99
	}
}
###########################################################################


################################################################################
# TRANSITION MAP NEWICK Versions mono et multi
################################################################################
# 
proc TransitionMapNewickMono {} {
	global A AA Z IDT T S pupkonewickstring TM
	catch {unset Z}
	# inference du graphe d'enfants/annotations
	TransitionMapNewickGo 0 0 0 x $TM($IDT,annotation,0)
	set pupkonewickstring [format "%s%s%s%s%s%s%s" "(" NODE0 ")" $TM($IDT,annotation,0) =0 :0 ";" ]
	TransitionMapNewickRecurVERSIONmono 0
}
#
proc TransitionMapNewickMulti {} {
	global A AA Z ZZ IDT T S pupkonewickstring TM
	catch {unset Z}
	# inference du graphe
	TransitionMapNewickGo 0 0 0 x $TM($IDT,annotation,0)
	# MULTI 
	set pupkonewickstring [format "%s%s%s%s%s%s%s" "(" NODE0 ")" $TM($IDT,annotation,0) =0 :0 ";" ]
	switch -exact $S(TransitionMapMultiLevel) {
		1 {
			TransitionMapNewickRecurVERSIONmulti 0 ; # OK
		}
		2 {
			catch {unset ZZ}
			TransitionMapNewickGoLevel2 
			TransitionMapNewickRecurVERSIONmulti2 0 ; # OK
		}
	}
	
}
# 
proc TransitionMapNewickGo {np n g ap a} {
	global IDT T S A P Z TM
	if {$ap == $a} {
		if {$T($IDT,typ,$n) != "f"} {
			TransitionMapNewickGo $np $T($IDT,fils1,$n) $g $a $TM($IDT,annotation,$T($IDT,fils1,$n))
			TransitionMapNewickGo $np $T($IDT,fils2,$n) $g $a $TM($IDT,annotation,$T($IDT,fils2,$n))
		} 
	} else {
		# n fait partit du scenario 
		if {$np != $n} {
			if {$T($IDT,typ,$n) != "f"} {
				lappend Z($IDT,enfants,$np) $n
				set Z($IDT,enfants,$n) {}
			}
		}
		#
		if {[AncetralAnnotTest $n $a]} {
			# si tous les nodes internes enfants de n ont la même annotation
			set Z($IDT,enfants,$n) {}
			return
		} else {
			# on continue si n n'est pas une feuille
			if {$T($IDT,typ,$n) != "f"} {
				TransitionMapNewickGo $n $T($IDT,fils1,$n) [expr $g +1 ] $a $TM($IDT,annotation,$T($IDT,fils1,$n))
				TransitionMapNewickGo $n $T($IDT,fils2,$n) [expr $g +1 ] $a $TM($IDT,annotation,$T($IDT,fils2,$n))
			}
		}
	}
}
#
proc TransitionMapNewickGoLevel2 {} {
	global IDT T S A P ZZ TM Z ZZcompteur ZZtrans ALLZZcompteur
	# RESET
	catch {unset ZZ}
	catch {unset ZZtrans}
	set ZZcompteur 0
	set ALLZZcompteur {}
	# ROOT
	set ZZ($IDT,annotation,$ZZcompteur) $TM($IDT,annotation,0)
	set ZZ($IDT,enfants,$ZZcompteur) {}
	# Go
	TransitionMapNewickGoLevel2Go $ZZcompteur 0
	# completion 
	foreach zzi $ALLZZcompteur {
	 if [info exist ZZ($IDT,enfants,$zzi)] {} else {set ZZ($IDT,enfants,$zzi) ""}
	}
}
proc TransitionMapNewickGoLevel2Go {pzz n} {
	global IDT T S A P ZZ TM Z ZZcompteur ZZtrans ALLZZcompteur
	set lgroupes [TransitionMapNewickGoLevel2Regroupement $n]
	foreach {a lnodes} $lgroupes {
		incr ZZcompteur
		set ZZtrans($IDT,ZZtonodes,$ZZcompteur) $lnodes ;# pour le hi transition map vers tree
		lappend ALLZZcompteur $ZZcompteur
		set ZZ($IDT,annotation,$ZZcompteur) $a
		lappend ZZ($IDT,enfants,$pzz) $ZZcompteur
		foreach node $lnodes {set ZZtrans($IDT,nodetoZZ,$node) $ZZcompteur}
	}
	foreach enfant $Z($IDT,enfants,$n) {
		TransitionMapNewickGoLevel2Go $ZZtrans($IDT,nodetoZZ,$enfant) $enfant
	}
}
# 
proc TransitionMapNewickGoLevel2Regroupement {n} {
	global IDT T S A P ZZ TM Z ZZcompteur ZZtrans
	set lannotation {}
	foreach enfant $Z($IDT,enfants,$n) {
		lappend lannotation $TM($IDT,annotation,$enfant)
	}
	set lannotation [DelRep $lannotation]
	set lgroupes {}
	foreach a $lannotation {
		set lenfants {}
		foreach enfant $Z($IDT,enfants,$n) {
			if {$TM($IDT,annotation,$enfant) == $a } { 
				lappend lenfants $enfant
			}
		}
		lappend lgroupes $a $lenfants
	}
	return $lgroupes
}
#
proc TransitionMapNewickRecurVERSIONmono {n} {
	global A Z IDT T pupkonewickstring TM
	# n est le node de la generation supérieur
	if {[llength $Z($IDT,enfants,$n)] == 1} {
		# le node a un enfant
		# le code node est remplace par ($codenodeenfant)annotationden=$codenodeenfant:1 et relance sur enfant
		set enfant [lindex $Z($IDT,enfants,$n) 0]
		if {$Z($IDT,enfants,$enfant) == {} } {
			# A VOIR N (lui meme ) en code enfant
			set insert [format "%s%s%s" $TM($IDT,annotation,$enfant) =$enfant :1 ]
		} else {
			set insert [format "%s%s%s%s%s%s" "(" NODE$enfant ")" $TM($IDT,annotation,$enfant) =$enfant :1 ]
		}
		regsub NODE$n $pupkonewickstring $insert pupkonewickstring
		# RELANCE
		if {$Z($IDT,enfants,$enfant) != {} } {
			TransitionMapNewickRecurVERSIONmono $enfant
		}
	} else {
		# le node a plusieurs enfants
		set insert ""
		foreach enfant $Z($IDT,enfants,$n) {
			if {$insert == ""} {
				if {$Z($IDT,enfants,$enfant) == {} } {
					set insert [format "%s%s%s" $TM($IDT,annotation,$enfant) "=$enfant" ":1"   ]
				} else {
					set insert [format "%s%s%s%s%s%s"  "(" NODE$enfant ")" $TM($IDT,annotation,$enfant) "=$enfant" ":1"   ]
				}
			} else {
				if {$Z($IDT,enfants,$enfant) == {} } {
					set insert [format "%s%s%s%s%s" $insert "," $TM($IDT,annotation,$enfant) "=$enfant" ":1" ]
				} else {
					set insert [format "%s%s%s%s%s%s%s%s" $insert "," "(" NODE$enfant ")" $TM($IDT,annotation,$enfant) "=$enfant" ":1" ]
				}
			}
		}
		# insertion
		regsub NODE$n $pupkonewickstring $insert pupkonewickstring
		# relance sur chaque enfant
		foreach enfant $Z($IDT,enfants,$n) {
			if {$Z($IDT,enfants,$enfant) != {} } {
				TransitionMapNewickRecurVERSIONmono $enfant
			}
		}
	}
}
# n est "(NODE$n)$annotation-de-n:nombre-repetitions"
proc TransitionMapNewickRecurVERSIONmulti {n} {
	global A Z IDT T pupkonewickstring TM
	if {[llength $Z($IDT,enfants,$n)] == 1} {
		# le node a un seul enfant
		set enfant [lindex $Z($IDT,enfants,$n) 0]
		if {$Z($IDT,enfants,$enfant) == {} } {
			set insert [format "%s%s%s" $TM($IDT,annotation,$enfant) =$enfant :1]
		} else {
			set insert [format "%s%s%s%s%s%s" "(" NODE$enfant ")" $TM($IDT,annotation,$enfant) =$enfant :1]
		}
		regsub NODE$n $pupkonewickstring $insert pupkonewickstring
		# RELANCE
		if {$Z($IDT,enfants,$enfant) != {} } {
			TransitionMapNewickRecurVERSIONmulti $enfant
		} else {
			return
		}
	} else {
		# le node a plusieurs enfants
		set LA {}
		foreach enfant $Z($IDT,enfants,$n) {
			if {[lsearch -exact $LA $TM($IDT,annotation,$enfant)] == -1} {
				#lappend LA $A($IDT,pupko,$enfant)
				lappend LA $TM($IDT,annotation,$enfant)
			}
		}
		set MEGAinsert ""
		foreach annotation $LA {
			set insertAnnotation "" ; set nb 0 ; set lcodtreenode ""
			foreach enfant $Z($IDT,enfants,$n) {
				if {$TM($IDT,annotation,$enfant) ==  $annotation} {
					if {$Z($IDT,enfants,$enfant) == {} } {
						set insert [format "%s%s%s" $TM($IDT,annotation,$enfant) "=$enfant" ":1"]
					} else {
						set insert  NODE$enfant 
					}
					# si enfant a un ou des enfants sinon pas la peine de les remettre
					if {$Z($IDT,enfants,$enfant) != {}} {
						if {$insertAnnotation == ""} {
							set insertAnnotation $insert
						} else {
							set insertAnnotation [format "%s%s%s" $insertAnnotation "," $insert ]
						}
					}
					incr nb
					set lcodtreenode [format "%s%s" $lcodtreenode =$enfant]
				}
			}
			# insert d'annotation 
			if {$insertAnnotation != ""} {
				set insertAnnotation [format "%s%s%s%s%s%s%s" "(" $insertAnnotation ")" $annotation $lcodtreenode ":" $nb ]
			} else {
				set insertAnnotation [format "%s%s%s%s"  $annotation $lcodtreenode ":" $nb ]
			}
			if {$MEGAinsert == ""} {
				set MEGAinsert $insertAnnotation
			} else {
				set MEGAinsert [format "%s%s%s" $MEGAinsert "," $insertAnnotation ]
			}
		}
		# INSERTION
		regsub NODE$n $pupkonewickstring $MEGAinsert pupkonewickstring
		# RELANCE
		foreach enfant $Z($IDT,enfants,$n) {
			if {$Z($IDT,enfants,$enfant) != {} } {
				TransitionMapNewickRecurVERSIONmulti $enfant
			}
		}
	}
}
#
proc TransitionMapNewickRecurVERSIONmulti2 {n} {
	global A ZZ ZZtrans IDT T pupkonewickstring TM
	if {[llength $ZZ($IDT,enfants,$n)] == 1} {
		# le node a un seul enfant
		set enfant [lindex $ZZ($IDT,enfants,$n) 0]
		regsub -all " " $ZZtrans($IDT,ZZtonodes,$enfant) "=" eee

		if {$ZZ($IDT,enfants,$enfant) == {} } {
			#set insert [format "%s%s%s" $ZZ($IDT,annotation,$enfant) =$enfant :1]
			set insert [format "%s%s%s" $ZZ($IDT,annotation,$enfant) =$eee :1]
		} else {
			#set insert [format "%s%s%s%s%s%s" "(" NODE$enfant ")" $ZZ($IDT,annotation,$enfant) =$enfant :1]
			set insert [format "%s%s%s%s%s%s" "(" NODE$enfant ")" $ZZ($IDT,annotation,$enfant) =$eee :1]
		}
		regsub NODE$n $pupkonewickstring $insert pupkonewickstring
		# RELANCE
		if {$ZZ($IDT,enfants,$enfant) != {} } {
			TransitionMapNewickRecurVERSIONmulti2 $enfant
		} else {
			return
		}
	} else {
		# le node a plusieurs enfants
		set LA {}
		foreach enfant $ZZ($IDT,enfants,$n) {
			if {[lsearch -exact $LA $ZZ($IDT,annotation,$enfant)] == -1} {
				lappend LA $ZZ($IDT,annotation,$enfant)
			}
		}
		set MEGAinsert ""
		foreach annotation $LA {
			set insertAnnotation "" ; set nb 0 ; set lcodtreenode ""
			foreach enfant $ZZ($IDT,enfants,$n) {
				regsub -all " " $ZZtrans($IDT,ZZtonodes,$enfant) "=" eee

				if {$ZZ($IDT,annotation,$enfant) ==  $annotation} {
					if {$ZZ($IDT,enfants,$enfant) == {} } {
						#set insert [format "%s%s%s" $ZZ($IDT,annotation,$enfant) "=$enfant" ":1"]
						set insert [format "%s%s%s" $ZZ($IDT,annotation,$enfant) "=$eee" ":1"]
					} else {
						set insert  NODE$enfant 
					}
					# si enfant a un ou des enfants sinon pas la peine de les remettre
					if {$ZZ($IDT,enfants,$enfant) != {}} {
						if {$insertAnnotation == ""} {
							set insertAnnotation $insert
						} else {
							set insertAnnotation [format "%s%s%s" $insertAnnotation "," $insert ]
						}
					}
					incr nb
					#set lcodtreenode [format "%s%s" $lcodtreenode =$enfant]
					set lcodtreenode [format "%s%s" $lcodtreenode =$eee]
				}
			}
			# insert d'annotation 
			if {$insertAnnotation != ""} {
				set insertAnnotation [format "%s%s%s%s%s%s%s" "(" $insertAnnotation ")" $annotation $lcodtreenode ":" $nb ]
			} else {
				set insertAnnotation [format "%s%s%s%s"  $annotation $lcodtreenode ":" $nb ]
			}
			if {$MEGAinsert == ""} {
				set MEGAinsert $insertAnnotation
			} else {
				set MEGAinsert [format "%s%s%s" $MEGAinsert "," $insertAnnotation ]
			}
		}
		# INSERTION
		regsub NODE$n $pupkonewickstring $MEGAinsert pupkonewickstring
		# RELANCE
		foreach enfant $ZZ($IDT,enfants,$n) {
			if {$ZZ($IDT,enfants,$enfant) != {} } {
				TransitionMapNewickRecurVERSIONmulti2 $enfant
			}
		}
	}
}
#
proc PupkoSaveNewick {} {
	global S pupkonewickstring
	set file [tk_getSaveFile -initialdir $S(pathuser) -defaultextension ".txt" \
			-title "Save Joint scenario map as a newick string..."]
	if {$file != "" } {
		set S(pathuser) [file dirname $file]
		set fid [open $file w]
		puts -nonewline $fid $pupkonewickstring
		close $fid
	}
}
#
proc AncetralAnnotTest {n a } {
	global IDT T S A P TM
	set t 1
	foreach ni [Node2SubTreeNodes $n] {
		if { $T($IDT,typ,$ni) != "f"} {
			if {$TM($IDT,annotation,$ni) != $a} {set t 0 }
		}
	}
	return $t
}
################################################################################


################################################################################
# TRANSITION MAP PARSER 
# PARSER NEWICK SCENARIO PUPKO
################################################################################
# reset TT et renvoie un TT avec
# TT(IDT,all) tous les codes nodes
# TT(IDT,$codenode,enfants) liste des enfants, liste vide si pas d'enfant
# TT(IDT,$codenode,label) annotation=nodetreeoriginal:nombretransitions
# TT(IDT,generation,$codenode) $generation de node
# TT(IDT,generation,$generation) liste des nodes de la generation $generation
# TT(IDT,generation,$generation) liste des nodes de la generation $generation
proc TransitionMapParser {} {
	global IDT TT pupkonewickstring CompteurIDpupko generationMax
	#set pupkonewickstring "(USA:1,East_Asia:3,Middle_East:1,(West_Africa:1,Southeast_Asia:2,East_Asia:1,East_Africa:1,(India:1,(Central_America:10,Caribbean_Islands:2)South_America:3,Central_America:2,(South_America:1)USA:1)Caribbean_Islands:1)India:1,East_Africa:1,Oceania:3,Central_America:1)Southeast_Asia:0;"
	set CompteurIDpupko 0
	set generationMax 0
	catch {unset TT}
	set id [expr [MultiFork $pupkonewickstring] -1]
	set lenfants [MultiForkToList [string range $pupkonewickstring 1 $id]]
	set node [string range $pupkonewickstring [expr 2 + $id] end-1] 
	lappend TT($IDT,all) 0
	set TT($IDT,codetogeneration,0) 0
	lappend TT($IDT,generationtocode,0) 0
	set TT($IDT,0,label) $node ;# annotation:nombretransitions
	foreach enfant $lenfants {
		TransitionMapParserGo $enfant 0 1
	}
}
#
proc TransitionMapParserGo {s nodepere generation} {
	global IDT TT CompteurIDpupko generationMax
	incr CompteurIDpupko
	set codenode $CompteurIDpupko
	lappend TT($IDT,all) $codenode
	lappend TT($IDT,$nodepere,enfants) $codenode
	if {$generation >= $generationMax} {set generationMax $generation}
	set TT($IDT,$codenode,enfants) {}
	set id [MultiFork $s]
	set nodelabel [string range $s $id  end]
	if {[string range $nodelabel 0 0] == ")"} {set nodelabel [string range $nodelabel 1 end]}
	set TT($IDT,$codenode,label) $nodelabel ;# annotation:nombretransitions
	set TT($IDT,codetogeneration,$codenode) $generation ;# annotation:nombretransitions
	lappend TT($IDT,generationtocode,$generation) $codenode
	set lenfants [MultiForkToList [string range $s 1 [expr $id -1]]]
	foreach enfant $lenfants {
		TransitionMapParserGo $enfant $codenode [expr $generation + 1]
	}
	return
}
#
proc MultiFork {s} {
	set i 0
	set id 0
	foreach c [split $s {}] {
		switch -exact -- $c {
			( {incr i}
			) {incr i -1}
		}
		if {$i == 0} {return $id}
		incr id
	}
	return -1
}
#
proc MultiForkToList {s} {
	set i 0
	set out ""
	foreach c [split $s {}] {
		switch -exact -- $c {
			( {incr i}
			) {incr i -1}
			, { if {$i == 0} {set c "  "}}
		}
		if {$c != " "} {set out [format "%s%s" $out $c]}
	}
	return $out
}
###########################################################################


###########################################################################
### TRANSITION MAP MULTI
###########################################################################

#
proc TransitionMapLayoutMulti {} {
	global S T IDT
	set w $S(canvas2)
	# scale factors
	#foreach {x1 y1 x2 y2} [$w cget -scrollregion] {break}
	#set canvasW [expr double($x2 - $x1)]
	#set canvasH [expr double($y2 - $y1)]
	set canvasW [winfo width $S(canvas2)]
	set canvasH [winfo height $S(canvas2)]
	# Y factor
	set s $S(SurfaceBubbleTransitionMap)
	set nb [llength [PupkoListFeuille]]  ; # attention en Y on regarde l'arbre des transitions
	#set S(TransitionMapYinter) [expr ($canvasH - (($s + $s) * $nb)) / ($nb +1) * $S(TransitionMapFactorYuser)] 
	set S(TransitionMapYinter) [expr $s * $S(TransitionMapFactorYuser)] 
	# X factor
	set T($IDT,xmax) 0 ; TMAXupdate 0 0
	set S(TransitionMapXfactor) [expr double($canvasW / $T($IDT,xmax)) * $S(TransitionMapFactorXuser)]
	# Font
	set S(TransitionMapFont) [list Helvetica $S(TransitionMapFontSize) normal]
	switch -exact $S(TransitionMapMultiLayout) {
		radial {
			TransitionMapLayoutMultiRadialGN
		}
		slanted {
			TransitionMapLayoutMultiSlantedGN
		}
		rectangular {
			TransitionMapLayoutMultiRectangleGN
		}
	}
}
#
proc TransitionMapLayoutMultiRadialGN {} {
	global IDT TT A S  generationMax
	set w $S(canvas2)
	$w delete all
	set lfeuilles [PupkoListFeuille]
	set NB [llength $lfeuilles]
	set alphaunit [expr 6.28319 / $NB]
	set angle 0.0
	set rayonbase [expr 20.0 * $S(TransitionMapXfactor)]
	set minus [expr $rayonbase / $generationMax]
	set s $S(SurfaceBubbleTransitionMap) ;# taile pastille
	set f [list Helvetica $S(TransitionMapFontSize) normal]
	foreach node $lfeuilles {
		set angle [expr $angle + $alphaunit]
		set rayon $rayonbase 
		set x [expr sin($angle) * $rayon]
		set y [expr cos($angle) * $rayon]
		set anglemean($node) $angle
		set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
		set annotation [lindex [split $annotationetnode =] 0 ]
		set transition [lindex [split $TT($IDT,$node,label) :] end ]
		# attention possible plusieurs treenode =node1=node2=node3
		set idx [string first = $annotationetnode]
		set treenode [string range $annotationetnode $idx end ]
		# attention ambiguité AA (parcimonie)
		if {[info exist S(ColorAnnotation,$annotation)]} {
			# pas d'ambiguité
			set color $S(ColorAnnotation,$annotation)
			set annotation3 [string range $annotation 0 2]
		} else {
			# annotation ancestrales multiple (non resolue en parsimony)
			set color lightgrey
			set annotation3 *
		}
		$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
			-fill $color -outline white -tags "MaxLikpiechart=$annotation TMCIRCLE  TREENODE$treenode NODE$node  ANNOTATION=$annotation TRANSITION=$transition " 
		set what ""
		if {$S(TransitionMapMultiAnnotation)} {set what $annotation3}
		if {$S(TransitionMapMultiAnnotationTransition)} {set what $what\n$transition}
		$w create text $x $y -text $what -fill black -font $f -justify center -tags "TMTEXT  TREENODE$treenode NODE$node  ANNOTATION=$annotation TRANSITION=$transition " 
	}
	for {set generation $generationMax} {$generation >= 0} {incr generation -1} {
		foreach node $TT($IDT,generationtocode,$generation) {
			if {$TT($IDT,$node,enfants) != {}} {
				set anglesum 0.0
				foreach enfant $TT($IDT,$node,enfants) {
					set anglesum [expr $anglesum + $anglemean($enfant)]
				}
				set angle [expr $anglesum / [llength $TT($IDT,$node,enfants)]] 
				set anglemean($node) $angle
				set rayon [expr $rayonbase - ($generationMax - $TT($IDT,codetogeneration,$node)) * $minus  ]
				set x [expr sin($angle) * $rayon]
				set y [expr cos($angle) * $rayon]
				set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
				set annotation [lindex [split $annotationetnode =] 0 ]
				set transition [lindex [split $TT($IDT,$node,label) :] end ]
				# attention possible plusieurs treenode =node1=node2=node3
				set idx [string first = $annotationetnode]
				set treenode [string range $annotationetnode $idx end ]
				# attention ambiguité AA (parcimonie)
				if {[lsearch $A($IDT,AnnotationList) $annotation] != -1} {
					# pas d'ambiguité
					set color $S(ColorAnnotation,$annotation)
					set annotation3 [string range $annotation 0 2]
				} else {
					# annotation ancestrales multiple (non resolue en parsimony)
					set color lightgrey
					set annotation3 *
				}
				$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
					-fill $color -outline white -tags "MaxLikpiechart=$annotation TMCIRCLE TREENODE$treenode NODE$node ANNOTATION=$annotation TRANSITION=$transition " 
				set what ""
				if {$S(TransitionMapMultiAnnotation)} {set what $annotation3}
				if {$S(TransitionMapMultiAnnotationTransition)} {set what $what\n$transition}
				$w create text $x $y -text $what -fill black -font $f -justify center -tags "TMTEXT  TREENODE$treenode NODE$node  ANNOTATION=$annotation TRANSITION=$transition " 
				# relier pere/enfants
				foreach enfant $TT($IDT,$node,enfants) {
					set co [$w coords NODE$enfant]
					set lt [$w gettags NODE$enfant]
					#set aid [lindex [split [lindex [lsearch $lt TRANSITION=*] $lt] =] end]
					set x1 [expr ([lindex $co 0]+[lindex $co 2]) / 2.0]
					set y1 [expr ([lindex $co 1]+[lindex $co 3]) / 2.0]
					# pour que les fleches arrivent en bord des pastilles annotations
					set delta [expr sqrt($x1 * $x1 + $y1 * $y1) - $s] ;# norme vecteur
					set x1 [expr sin($anglemean($enfant)) * $delta]
					set y1 [expr cos($anglemean($enfant)) * $delta]
					set annotation [lindex [split $TT($IDT,$enfant,label) :] 0 ]
					set transition [lindex [split $TT($IDT,$enfant,label) :] end ]
					set ft 1
					if {$S(TransitionMapMultiSize)} {
						if {$S(TransitionMapMultiArrow)} {
							$w create line $x $y $x1 $y1 -width [expr $transition * $ft] -arrow last -arrowshape {15 20 5} -tags "EDGEPUPKO NODE$enfant ANNOTATION=$annotation TRANSITION=$transition " 
						} else {
							$w create line $x $y $x1 $y1 -width [expr $transition * $ft]  -tags "EDGEPUPKO NODE$enfant ANNOTATION=$annotation TRANSITION=$transition " 
						}
					} else {
						if {$S(TransitionMapMultiArrow)} {
							$w create line $x $y $x1 $y1 -width 1 -arrow last -arrowshape {15 20 5} -tags "EDGEPUPKO NODE$enfant ANNOTATION=$annotation TRANSITION=$transition " 
						} else {
							$w create line $x $y $x1 $y1 -width 1  -tags "EDGEPUPKO NODE$enfant ANNOTATION=$annotation TRANSITION=$transition " 
						}
					}
				}
			} 
		}
	}
	$w lower EDGEPUPKO
	$w configure -scrollregion [$w bbox all]
}
proc TransitionMapLayoutMultiSlantedGN {} {
	global IDT TT A S  generationMax
	set w $S(canvas2)
	$w delete all
	set s $S(SurfaceBubbleTransitionMap) ;# taile pastille
	set f [list Helvetica $S(TransitionMapFontSize) normal]
	# X 
	set xMax [expr 10* $generationMax * $S(TransitionMapFactorXuser)]
	set dx [expr 20 * $S(TransitionMapFactorXuser)]
	# Y 
	set y 0.0
	set dy [expr $s * $S(TransitionMapFactorYuser)] 
	#
	foreach node [PupkoListFeuille] {
		set y [expr $y + $dy]
		set x $xMax
		set yarray($node) $y
		set xarray($node) $x
		set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
		set annotation [lindex [split $annotationetnode =] 0 ]
		set transition [lindex [split $TT($IDT,$node,label) :] end ]
		# attention possible plusieurs treenode =node1=node2=node3
		set idx [string first = $annotationetnode]
		set treenode [string range $annotationetnode $idx end ]
		# attention ambiguité AA (parcimonie)
		if {[info exist S(ColorAnnotation,$annotation)]} {
			# pas d'ambiguité
			set color $S(ColorAnnotation,$annotation)
			set annotation3 [string range $annotation 0 2]
		} else {
			# annotation ancestrales multiple (non resolue en parsimony)
			set color lightgrey
			set annotation3 *
		}
		$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
			-fill $color -outline white -tags "MaxLikpiechart=$annotation TMCIRCLE TREENODE$treenode NODE$node ANNOTATION=$annotation TRANSITION=$transition " 
		set what ""
		if {$S(TransitionMapMultiAnnotation)} {set what $annotation3}
		if {$S(TransitionMapMultiAnnotationTransition)} {set what $what\n$transition}
		$w create text $x $y -text $what -fill black -font $f -justify center -tags "TMTEXT  TREENODE$treenode NODE$node  ANNOTATION=$annotation TRANSITION=$transition " 
				
	}
	for {set generation $generationMax} {$generation >= 0} {incr generation -1} {
		foreach node $TT($IDT,generationtocode,$generation) {
			if {$TT($IDT,$node,enfants) != {}} {
				set ymean 0.0
				foreach enfant $TT($IDT,$node,enfants) {
					set ymean [expr $ymean + $yarray($enfant)]
				}
				set y [expr $ymean / [llength $TT($IDT,$node,enfants)]] 
				set yarray($node) $y
				set x [expr $xMax - ($dx * ($generationMax - $generation))]
				set xarray($node) $x
				set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
				set annotation [lindex [split $annotationetnode =] 0 ]
				set transition [lindex [split $TT($IDT,$node,label) :] end ]
				# attention ambiguité AA (parcimonie)
				if {[info exist S(ColorAnnotation,$annotation)]} {
					# pas d'ambiguité
					set color $S(ColorAnnotation,$annotation)
					set annotation3 [string range $annotation 0 2]
				} else { 
					# annotation ancestrales multiple (non resolue en parsimony)
					set color lightgrey
					set annotation3 *
				}
				# attention possible plusieurs treenode =node1=node2=node3
				set idx [string first = $annotationetnode]
				set treenode [string range $annotationetnode $idx end ]
		
				#set annotation3 [string range $annotation 0 2]
				$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
					-fill $color -outline white -tags "MaxLikpiechart=$annotation  TMCIRCLE  TREENODE$treenode  NODE$node ANNOTATION=$annotation TRANSITION=$transition " 
				set what ""
				if {$S(TransitionMapMultiAnnotation)} {set what $annotation3}
				if {$S(TransitionMapMultiAnnotationTransition)} {set what $what\n$transition}
				$w create text $x $y -text $what -fill black -font $f -justify center -tags "TMTEXT  TREENODE$treenode NODE$node  ANNOTATION=$annotation TRANSITION=$transition " 
				
				# relier pere/enfants
				foreach enfant $TT($IDT,$node,enfants) {
					set x $xarray($node)
					set y $yarray($node)
					set x1 $xarray($enfant)
					set y1 $yarray($enfant)
					set annotation [lindex [split $TT($IDT,$enfant,label) :] 0 ]
					set transition [lindex [split $TT($IDT,$enfant,label) :] end ]
					set ft 1
					if {$S(TransitionMapMultiSize)} {
						if {$S(TransitionMapMultiArrow)} {
							$w create line $x $y $x $y1 $x1 $y1 -width [expr $transition * $ft] -smooth true -splinesteps 100 -arrow last -arrowshape {15 20 5} -tags "EDGEPUPKO NODE$enfant ANNOTATION=$annotation TRANSITION=$transition " 
						} else {
							$w create line $x $y $x $y1 $x1 $y1 -width [expr $transition * $ft] -smooth true -splinesteps 100  -tags "EDGEPUPKO NODE$enfant ANNOTATION=$annotation TRANSITION=$transition " 
						}
					} else {
						if {$S(TransitionMapMultiArrow)} {
							$w create line $x $y $x $y1 $x1 $y1 -width 1 -arrow last -arrowshape {15 20 5} -smooth true -splinesteps 100 -tags "EDGEPUPKO NODE$enfant ANNOTATION=$annotation TRANSITION=$transition " 
						} else {
							$w create line $x $y $x $y1 $x1 $y1 -width 1 -smooth true -splinesteps 100 -tags "EDGEPUPKO NODE$enfant ANNOTATION=$annotation TRANSITION=$transition " 
						}
					}
				}
			} 
		}
	}
	$w lower EDGEPUPKO
	#$w xview moveto 0
	$w configure -scrollregion [$w bbox all]
}
#
proc TransitionMapLayoutMultiRectangleGN {} {
	global IDT TT A S  generationMax
	set w $S(canvas2)
	$w delete all
	set s $S(SurfaceBubbleTransitionMap) ;# taile pastille
	set f [list Helvetica $S(TransitionMapFontSize) normal]
	# X 
	set xMax [expr 20* $generationMax * $S(TransitionMapFactorXuser)]
	set dx [expr 20 * $S(TransitionMapFactorXuser)]
	# Y 
	set y 0.0
	set dy  [expr $s * $S(TransitionMapFactorYuser)] 
	foreach node [PupkoListFeuille] {
		set y [expr $y + $dy]
		set x $xMax
		set yarray($node) $y
		set xarray($node) $x
		set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
		set annotation [lindex [split $annotationetnode =] 0 ]
		set transition [lindex [split $TT($IDT,$node,label) :] end ]
		# attention possible plusieurs treenode =node1=node2=node3
		set idx [string first = $annotationetnode]
		set treenode [string range $annotationetnode $idx end ]
		# attention ambiguité AA (parcimonie)
		if {[info exist S(ColorAnnotation,$annotation)]} {
			# pas d'ambiguité
			set color $S(ColorAnnotation,$annotation)
			set annotation3 [string range $annotation 0 2]
		} else {
			# annotation ancestrales multiple (non resolue en parsimony)
			set color lightgrey
			set annotation3 *
		}
		$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
			-fill $color -outline white -tags "MaxLikpiechart=$annotation TMCIRCLE  TREENODE$treenode NODE$node ANNOTATION=$annotation TRANSITION=$transition " 
		set what ""
		if {$S(TransitionMapMultiAnnotation)} {set what $annotation3}
		if {$S(TransitionMapMultiAnnotationTransition)} {set what $what\n$transition}
		$w create text $x $y -text $what -fill black -font $f -justify center -tags "TMTEXT  TREENODE$treenode NODE$node  ANNOTATION=$annotation TRANSITION=$transition " 
	}
	for {set generation [expr $generationMax -1]} {$generation >= 0} {incr generation -1} {
		foreach node $TT($IDT,generationtocode,$generation) {
			if {$TT($IDT,$node,enfants) != {}} {
				set ymean 0.0
				foreach enfant $TT($IDT,$node,enfants) {
					set ymean [expr $ymean + $yarray($enfant)]
				}
				set y [expr $ymean / [llength $TT($IDT,$node,enfants)]] 
				set yarray($node) $y
				set x [expr $xMax - ($dx * ($generationMax - $generation))]
				set xarray($node) $x
				set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
				set annotation [lindex [split $annotationetnode =] 0 ]
				set transition [lindex [split $TT($IDT,$node,label) :] end ]
				# attention possible plusieurs treenode =node1=node2=node3
				set idx [string first = $annotationetnode]
				set treenode [string range $annotationetnode $idx end ]
				# attention ambiguité AA (parcimonie)
				if {[info exist S(ColorAnnotation,$annotation)]} {
					# pas d'ambiguité
					set color $S(ColorAnnotation,$annotation)
					set annotation3 [string range $annotation 0 2]
				} else {
					# annotation ancestrales multiple (non resolue en parsimony)
					set color lightgrey
					set annotation3 *
				}
				$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
					-fill $color -outline white -tags "MaxLikpiechart=$annotation TMCIRCLE TREENODE$treenode NODE$node ANNOTATION=$annotation TRANSITION=$transition " 
				set what ""
				if {$S(TransitionMapMultiAnnotation)} {set what $annotation3}
				if {$S(TransitionMapMultiAnnotationTransition)} {set what $what\n$transition}
				$w create text $x $y -text $what -fill black -font $f -justify center -tags "TMTEXT  TREENODE$treenode NODE$node  ANNOTATION=$annotation TRANSITION=$transition " 
				# relier pere/enfants
				foreach enfant $TT($IDT,$node,enfants) {
					set annotation [lindex [split $TT($IDT,$enfant,label) :] 0 ]
					set transition [lindex [split $TT($IDT,$enfant,label) :] end ]
					set ft 1
					set x $xarray($node)
					set y $yarray($node)
					set x1 $xarray($enfant)
					set y1 $yarray($enfant)
					if {$S(TransitionMapMultiSize)} {
						if {$S(TransitionMapMultiArrow)} {
							$w create line $x $y $x $y1 -width 1  -tags "EDGEPUPKO NODE$enfant " 
							$w create line $x $y1 [expr $x1 - $s] $y1 -width [expr $transition * $ft]  -arrow last -arrowshape {15 20 5} -tags "EDGEPUPKO NODE$enfant " 
						} else {
							$w create line $x $y $x $y1 -width 1 -tags "EDGEPUPKO NODE$enfant " 
							$w create line $x $y1 [expr $x1 - $s] $y1 -width [expr $transition * $ft] -tags "EDGEPUPKO NODE$enfant " 
						}
					} else {
						if {$S(TransitionMapMultiArrow)} {
							$w create line $x $y $x $y1 -width 1 -tags "EDGEPUPKO NODE$enfant " 
							$w create line $x $y1 [expr $x1 - $s] $y1 -width 1 -arrow last -arrowshape {15 20 5} -tags "EDGEPUPKO NODE$enfant " 
						} else {
							$w create line $x $y $x $y1 -width 1 -tags "EDGEPUPKO NODE$enfant " 
							$w create line $x $y1 [expr $x1 - $s] $y1 -width 1 -tags "EDGEPUPKO NODE$enfant " 
						}
					}
				}
			} 
		}
	}
	$w lower EDGEPUPKO
	#$w xview moveto 0
	$w configure -scrollregion [$w bbox all]
}
###########################################################################

###########################################################################
### TRANSITION MAP MONO
###########################################################################
#
proc TransitionMap {mode} {
	global S
		TransitionMapData ; # OK
		switch -exact $mode {
			mono {
				TransitionMapNewickMono ; # OK
				TransitionMapParser ; # OK
				TransitionMapLayoutMono
			}
			multi {
				TransitionMapNewickMulti ; # OK
				TransitionMapParser ; # OK
				TransitionMapLayoutMulti 
			}
		}
		# memoire pour TransitionMapDynamic
		TransitionMapSet $mode
}
#
proc TransitionMapSet {mode} {
	global S
	switch -exact $mode {
		mono {
			set S($S(canvas2),layout) $S(TransitionMapMonoLayout)
			set S($S(canvas2),view) $S(TransitionMapLayoutView)
			set S($S(canvas2),cluster) $S(TransitionMapLayoutViewCluster)
			set S($S(canvas2),node) $S(TransitionMapMonoNode)
			set S($S(canvas2),annotation) $S(TransitionMapMonoAnnotation)
			set S($S(canvas2),arrow) $S(TransitionMapMonoArrow)
		}
		multi {
			set S($S(canvas2),layout) $S(TransitionMapMultiLayout)
			set S($S(canvas2),size) $S(TransitionMapMultiSize)
			set S($S(canvas2),annotation) $S(TransitionMapMultiAnnotation)
			set S($S(canvas2),arrow) $S(TransitionMapMultiArrow)
		}
	}
	# 
	set S($S(canvas2),mode) $mode
	set S($S(canvas2),what) $S(TransitionMapWhat)
	set S($S(canvas2),TransitionMapFactorXuser) $S(TransitionMapFactorXuser)
	set S($S(canvas2),TransitionMapFactorYuser) $S(TransitionMapFactorYuser)
	set S($S(canvas2),TransitionMapFontSize) $S(TransitionMapFontSize)
	set S($S(canvas2),SurfaceBubbleTransitionMap) $S(SurfaceBubbleTransitionMap)
}
#
proc TransitionMapSetRestore {} {
	global S
	switch -exact $S($S(canvas2),mode) { 
		mono {
			set S(TransitionMapMonoLayout) $S($S(canvas2),layout) 
			set S(TransitionMapLayoutView) $S($S(canvas2),view) 
			set S(TransitionMapLayoutViewCluster) $S($S(canvas2),cluster) 
			set S(TransitionMapMonoNode) $S($S(canvas2),node) 
			set S(TransitionMapMonoAnnotation) $S($S(canvas2),annotation) 
			set S(TransitionMapMonoArrow) $S($S(canvas2),arrow) 
		}
		multi {
			set S(TransitionMapMultiLayout) $S($S(canvas2),layout) 
			set S(TransitionMapMultiSize) $S($S(canvas2),size) 
			set S(TransitionMapMultiAnnotation) $S($S(canvas2),annotation) 
			set S(TransitionMapMultiArrow) $S($S(canvas2),arrow) 
		}
	}
	set S(TransitionMapWhat) $S($S(canvas2),what)
	set S(TransitionMapFactorXuser) $S($S(canvas2),TransitionMapFactorXuser)
	set S(TransitionMapFontSize) $S($S(canvas2),TransitionMapFontSize)
	set S(TransitionMapFactorYuser) $S($S(canvas2),TransitionMapFactorYuser)
	set S(SurfaceBubbleTransitionMap) $S($S(canvas2),SurfaceBubbleTransitionMap)
}
#
proc TransitionMapSetReset {} {
	global S
	foreach {k v} [array get S $S(canvas2),*] {
		unset S($k)
	}
	$S(canvas2) delete all
}
#
proc TransitionMapLayoutMono {} {
	global S T IDT
	set w $S(canvas2)
	set canvasW [winfo width $w]
	set canvasH [winfo height $w]
	# Y factor
	set s $S(SurfaceBubbleTransitionMap)
	set nb [llength [PupkoListFeuille]]  ; # attention en Y on regarde l'arbre des transitions
	set S(TransitionMapYinter) [expr $s * $S(TransitionMapFactorYuser)] 
	# X factor
	set T($IDT,xmax) 0 ; TMAXupdate 0 0
	set S(TransitionMapXfactor) [expr double($canvasW / $T($IDT,xmax)) * $S(TransitionMapFactorXuser)]
	# Font
	set S(TransitionMapFont) [list Helvetica $S(TransitionMapFontSize) normal]
	switch -exact $S(TransitionMapMonoLayout) {
		radial {
			switch -exact $S(TransitionMapLayoutView) {
				gen {TransitionMapLayoutMonoRadialGN}
				bra {TransitionMapLayoutMonoRadialBL}
			}
		}
		slanted {
			switch -exact $S(TransitionMapLayoutView) {
				gen {TransitionMapLayoutMonoSlantedGN}
				bra {TransitionMapLayoutMonoSlantedBL}
			}	
		}
		rectangular {
			switch -exact $S(TransitionMapLayoutView) {
				gen {TransitionMapLayoutMonoRectangleGN}
				bra {TransitionMapLayoutMonoRectangleBL}
			}	
		}
	}
}
#
proc TransitionMapLayoutMonoRadialGN {} {
	global IDT T TT TM A S  generationMax
	set w $S(canvas2)
	$w delete all
	set lfeuilles [PupkoListFeuille]
	set s $S(SurfaceBubbleTransitionMap) ;# taile pastille
	set fo [list Helvetica $S(TransitionMapFontSize) normal]
	set NB [llength $lfeuilles]
	set alphaunit [expr 6.28319 / $NB]
	set angle 0.0
	set rayonbase [expr 10.0 * $S(TransitionMapXfactor)]
	set minus [expr $rayonbase / $generationMax]
	foreach node $lfeuilles {
		set angle [expr $angle + $alphaunit]
		set rayon $rayonbase 
		set x [expr sin($angle) * $rayon]
		set y [expr cos($angle) * $rayon]
		set anglemean($node) $angle
		set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
		set annotation [lindex [split $annotationetnode =] 0 ]
		set treenode [lindex [split $annotationetnode =] 1 ]
		# attention ambiguité AA (parcimonie)
		if {[info exist S(ColorAnnotation,$annotation)]} {
			# pas d'ambiguité
			set color $S(ColorAnnotation,$annotation)
			set annotation3 [string range $annotation 0 2]
		} else {
			# annotation ancestrales multiple (non resolue en parsimony)
			set color lightgrey
			set annotation3 *
		}
		set treefeuilles [Node2Leafs2 $treenode]
		set nbfeuilles [llength $treefeuilles]
		# pourcentage de feuilles de l'arbre phylogenetique issues de node ayant la meme annotation que node
		set nbfeuillesmemeannotation 0.0
		foreach fid $treefeuilles {
			set f $T($IDT,vbd,$fid)
			if {$TM($IDT,annotation,$f) == $annotation} {set nbfeuillesmemeannotation [expr $nbfeuillesmemeannotation + 1.0]}
		}
		set percent [expr $nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0]
		
		# draw
		if {$S(TransitionMapLayoutViewCluster) == "piechart"} {
			set complement [expr 1 - $percent ]
			set apc [list [list "?" [rnd $complement 2] "#DDD"] [list $annotation [rnd $percent 2] $color]]
			piechart2 $w [expr $x - $s ] [expr $y- $s] [expr $s + $s] [expr $s + $s] $treenode $apc "TMCIRCLE NODE$node TREENODE$treenode ANNOTATION=$annotation"
		} else {
			$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
			-fill $color -outline white -tags "TMCIRCLE NODE$node  TREENODE$treenode ANNOTATION=$annotation" 
		}	
		set what ""
		set percentText [format "%s%s" [expr round(($nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0) *100.00)] "%"]
		if {$S(TransitionMapMonoNode)} {append what $percentText}
		if {$S(TransitionMapMonoAnnotation)} {append what \n$annotation3}
		$w create text $x $y -text $what -fill black -font $fo -justify center -tags "TMTEXT NODE$node ANNOTATION=$annotation " 		
		#PathSameAnnotationWithTipsDifferent $w $treenode $annotation $x $y $s
	}
	for {set generation $generationMax} {$generation >= 0} {incr generation -1} {
		foreach node $TT($IDT,generationtocode,$generation) {
			if {$TT($IDT,$node,enfants) != {}} {
				set anglesum 0.0
				foreach enfant $TT($IDT,$node,enfants) {
					set anglesum [expr $anglesum + $anglemean($enfant)]
				}
				set angle [expr $anglesum / [llength $TT($IDT,$node,enfants)]] 
				set anglemean($node) $angle
				set rayon [expr $rayonbase - ($generationMax - $TT($IDT,codetogeneration,$node)) * $minus  ]
				set x [expr sin($angle) * $rayon]
				set y [expr cos($angle) * $rayon]
				set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
				set annotation [lindex [split $annotationetnode =] 0 ]
				set treenode [lindex [split $annotationetnode =] 1 ]
				# attention ambiguité AA (parcimonie)
				if {[lsearch $A($IDT,AnnotationList) $annotation] != -1} {
					# pas d'ambiguité
					set color $S(ColorAnnotation,$annotation)
					set annotation3 [string range $annotation 0 2]
				} else {
					# annotation ancestrales multiple (non resolue en parsimony)
					set color lightgrey
					set annotation3 *
				}
				set treefeuilles [Node2Leafs2 $treenode]
				set nbfeuilles [llength $treefeuilles]
				# pourcentage de feuilles de l'arbre phylogenetique issues de node ayant la meme annotation que node
				set nbfeuillesmemeannotation 0.0
				foreach fid $treefeuilles {
					set f $T($IDT,vbd,$fid)
					if {$TM($IDT,annotation,$f) == $annotation} {set nbfeuillesmemeannotation [expr $nbfeuillesmemeannotation + 1.0]}
				}
				set percent [expr $nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0]
				set complement [expr 1 - $percent ]
				set apc [list [list "?" [rnd $complement 2] "#DDD"] [list $annotation [rnd $percent 2] $color]]
				# draw
				if {$S(TransitionMapLayoutViewCluster) == "piechart"} {
					piechart2 $w [expr $x - $s ] [expr $y- $s] [expr $s + $s] [expr $s + $s] $treenode $apc "TMCIRCLE NODE$node TREENODE$treenode ANNOTATION=$annotation"
				} else {
					$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
					-fill $color -outline white -tags "TMCIRCLE NODE$node  TREENODE$treenode ANNOTATION=$annotation" 
				}
				set what ""
				set percentText [format "%s%s" [expr round(($nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0) *100.00)] "%"]
				if {$S(TransitionMapMonoNode)} {append what $percentText}
				if {$S(TransitionMapMonoAnnotation)} {append what \n$annotation3}
				$w create text $x $y -text $what -fill black -font $fo -justify center -tags "TMTEXT NODE$node ANNOTATION=$annotation" 
				#PathSameAnnotationWithTipsDifferent $w $treenode $annotation $x $y $s
				# relier pere/enfants
				foreach enfant $TT($IDT,$node,enfants) {
					set co [$w coords NODE$enfant]
					set lt [$w gettags NODE$enfant]
					#set aid [lindex [split [lindex [lsearch $lt TRANSITION=*] $lt] =] end]
					set x1 [expr ([lindex $co 0]+[lindex $co 2]) / 2.0]
					set y1 [expr ([lindex $co 1]+[lindex $co 3]) / 2.0]
					# pour que les fleches arrivent en bord des pastilles annotations
					set delta [expr sqrt($x1 * $x1 + $y1 * $y1) - $s] ;# norme vecteur
					set x1 [expr sin($anglemean($enfant)) * $delta]
					set y1 [expr cos($anglemean($enfant)) * $delta]
					set annotation [lindex [split $TT($IDT,$enfant,label) :] 0 ]
					set transition [lindex [split $TT($IDT,$enfant,label) :] end ]
					if {$S(TransitionMapMonoArrow)} {
						#$w create line $x $y $x1 $y1 -width 1 -arrow last -arrowshape {15 20 5} -tags "EDGEPUPKO NODE$enfant" 
						$w create line $x $y $x1 $y1 -width 1 -arrow last -arrowshape {15 20 5} -tags "EDGEPUPKO NODE$enfant" 
					} else {
						#$w create line $x $y $x1 $y1 -width 1 -tags "EDGEPUPKO NODE$enfant" 
						$w create line $x $y $x1 $y1 -width 1 -tags "EDGEPUPKO NODE$enfant" 
					}
				}
			} 
		}
	}
	$w lower EDGEPUPKO
	$w configure -scrollregion [$w bbox all]
}
#
proc TransitionMapLayoutMonoRadialBL {} {
	global IDT T TT TM A S  generationMax
	set w $S(canvas2)
	$w delete all
	set lfeuilles [PupkoListFeuille]
	set s $S(SurfaceBubbleTransitionMap) ;# taile pastille
	set fo [list Helvetica $S(TransitionMapFontSize) normal]
	set NB [llength $lfeuilles]
	set alphaunit [expr 6.28319 / $NB]
	set factorX $S(TransitionMapXfactor)
	set angle 0.0
	set rayonbase 140.0
	set minus [expr $rayonbase / $generationMax]
	foreach node $lfeuilles {
		set angle [expr $angle + $alphaunit]
		#set rayon $rayonbase 
		set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
		set annotation [lindex [split $annotationetnode =] 0 ]
		set treenode [lindex [split $annotationetnode =] 1 ]
		set rayon [expr $factorX * [DistanceToRoot $treenode $T($IDT,dbl,$treenode)]]
		set x [expr sin($angle) * $rayon]
		set y [expr cos($angle) * $rayon]
		set anglemean($node) $angle
		# attention ambiguité AA (parcimonie)
		if {[info exist S(ColorAnnotation,$annotation)]} {
			# pas d'ambiguité
			set color $S(ColorAnnotation,$annotation)
			set annotation3 [string range $annotation 0 2]
		} else {
			# annotation ancestrales multiple (non resolue en parsimony)
			set color lightgrey
			set annotation3 *
		}
		set treefeuilles [Node2Leafs2 $treenode]
		set nbfeuilles [llength $treefeuilles]
		# pourcentage de feuilles de l'arbre phylogenetique issues de node ayant la meme annotation que node
		set nbfeuillesmemeannotation 0.0
		foreach fid $treefeuilles {
			set f $T($IDT,vbd,$fid)
			if {$TM($IDT,annotation,$f) == $annotation} {set nbfeuillesmemeannotation [expr $nbfeuillesmemeannotation + 1.0]}
		}
		set percent [expr $nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0]
		# draw
		if {$S(TransitionMapLayoutViewCluster) == "piechart"} {
			set complement [expr 1 - $percent ]
			set apc [list [list "?" [rnd $complement 2] "#DDD"] [list $annotation [rnd $percent 2] $color]]
			piechart2 $w [expr $x - $s ] [expr $y- $s] [expr $s + $s] [expr $s + $s] $treenode $apc "TMCIRCLE NODE$node TREENODE$treenode ANNOTATION=$annotation"
		} else {
			$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
			-fill $color -outline white -tags "TMCIRCLE NODE$node  TREENODE$treenode ANNOTATION=$annotation" 
		}	
		set what ""
		set percentText [format "%s%s" [expr round(($nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0) *100.00)] "%"]
		if {$S(TransitionMapMonoNode)} {append what $percentText}
		if {$S(TransitionMapMonoAnnotation)} {append what \n$annotation3}
		$w create text $x $y -text $what -fill black -font $fo -justify center -tags "TMTEXT NODE$node ANNOTATION=$annotation " 
		#PathSameAnnotationWithTipsDifferent $w $treenode $annotation $x $y $s
	}
	for {set generation $generationMax} {$generation >= 0} {incr generation -1} {
		foreach node $TT($IDT,generationtocode,$generation) {
			if {$TT($IDT,$node,enfants) != {}} {
				set anglesum 0.0
				foreach enfant $TT($IDT,$node,enfants) {
					set anglesum [expr $anglesum + $anglemean($enfant)]
				}
				set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
				set annotation [lindex [split $annotationetnode =] 0 ]
				set treenode [lindex [split $annotationetnode =] 1 ]
				set angle [expr $anglesum / [llength $TT($IDT,$node,enfants)]] 
				set anglemean($node) $angle
				set rayon [expr $factorX * [DistanceToRoot $treenode $T($IDT,dbl,$treenode)]]
				#set rayon [expr $rayonbase - ($generationMax - $TT($IDT,codetogeneration,$node)) * $minus  ]
				set x [expr sin($angle) * $rayon]
				set y [expr cos($angle) * $rayon]
				# attention ambiguité AA (parcimonie)
				if {[lsearch $A($IDT,AnnotationList) $annotation] != -1} {
					# pas d'ambiguité
					set color $S(ColorAnnotation,$annotation)
					set annotation3 [string range $annotation 0 2]
				} else {
					# annotation ancestrales multiple (non resolue en parsimony)
					set color lightgrey
					set annotation3 *
				}
				set treefeuilles [Node2Leafs2 $treenode]
				set nbfeuilles [llength $treefeuilles]
				# pourcentage de feuilles de l'arbre phylogenetique issues de node ayant la meme annotation que node
				set nbfeuillesmemeannotation 0.0
				foreach fid $treefeuilles {
					set f $T($IDT,vbd,$fid)
					if {$TM($IDT,annotation,$f) == $annotation} {set nbfeuillesmemeannotation [expr $nbfeuillesmemeannotation + 1.0]}
				}
				set percent [expr $nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0]
				set complement [expr 1 - $percent ]
				set apc [list [list "?" [rnd $complement 2] "#DDD"] [list $annotation [rnd $percent 2] $color]]
				# draw
				if {$S(TransitionMapLayoutViewCluster) == "piechart"} {
					piechart2 $w [expr $x - $s ] [expr $y- $s] [expr $s + $s] [expr $s + $s] $treenode $apc "TMCIRCLE NODE$node TREENODE$treenode ANNOTATION=$annotation"
				} else {
					$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
					-fill $color -outline white -tags "TMCIRCLE NODE$node  TREENODE$treenode ANNOTATION=$annotation" 
				}
				set what ""
				set percentText [format "%s%s" [expr round(($nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0) *100.00)] "%"]
				if {$S(TransitionMapMonoNode)} {append what $percentText}
				if {$S(TransitionMapMonoAnnotation)} {append what \n$annotation3}
				$w create text $x $y -text $what -fill black -font $fo -justify center -tags "TMTEXT NODE$node ANNOTATION=$annotation" 
				#PathSameAnnotationWithTipsDifferent $w $treenode $annotation $x $y $s
				# relier pere/enfants
				foreach enfant $TT($IDT,$node,enfants) {
					set co [$w coords NODE$enfant]
					set lt [$w gettags NODE$enfant]
					set x1 [expr ([lindex $co 0]+[lindex $co 2]) / 2.0]
					set y1 [expr ([lindex $co 1]+[lindex $co 3]) / 2.0]
					set delta [expr sqrt($x1 * $x1 + $y1 * $y1) - $s] ;# norme vecteur
					set x1 [expr sin($anglemean($enfant)) * $delta]
					set y1 [expr cos($anglemean($enfant)) * $delta]
					if {$S(TransitionMapMonoArrow)} {
						$w create line $x $y $x1 $y1 -width 1 -arrow last -arrowshape {15 20 5} -tags "EDGEPUPKO NODE$enfant" 
					} else {
						$w create line $x $y $x1 $y1 -width 1 -tags "EDGEPUPKO NODE$enfant" 
					}
				}
			} 
		}
	}
	$w lower EDGEPUPKO
	$w configure -scrollregion [$w bbox all]
}
# 
proc TransitionMapLayoutMonoSlantedBL {} {
	global IDT T TT TM A S  generationMax
	set w $S(canvas2)
	$w delete all
	set lfeuilles [PupkoListFeuille]
	# affectation
	set s $S(SurfaceBubbleTransitionMap) ; # taille pastille
	set dy $S(TransitionMapYinter)
	set factorX $S(TransitionMapXfactor)
	set fo $S(TransitionMapFont) 
	#
	set y 0.0
	foreach node $lfeuilles {
		# annotation=nodetreeorigial:nbtransition
		set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
		set annotation [lindex [split $annotationetnode =] 0 ]
		set treenode [lindex [split $annotationetnode =] 1 ]
		# X & Y
		set y [expr $y + $dy]
		set x [expr $factorX * [DistanceToRoot $treenode $T($IDT,dbl,$treenode)]]
		set yarray($node) $y
		set xarray($node) $x
		# attention ambiguité AA (parcimonie)
		if {[info exist S(ColorAnnotation,$annotation)]} {
			# pas d'ambiguité
			set color $S(ColorAnnotation,$annotation)
			set annotation3 [string range $annotation 0 2]
		} else {
			# annotation ancestrales multiple (non resolue en parsimony)
			set color lightgrey
			set annotation3 *
		}
		#
		set treefeuilles [Node2Leafs2 $treenode]
		set nbfeuilles [llength $treefeuilles]
		# pourcentage de feuilles de l'arbre phylogenetique issues de node ayant la meme annotation que node
		set nbfeuillesmemeannotation 0.0
		foreach fid $treefeuilles {
			set f $T($IDT,vbd,$fid)
			if {$TM($IDT,annotation,$f) == $annotation} {set nbfeuillesmemeannotation [expr $nbfeuillesmemeannotation + 1.0]}
		}
		set percent [expr $nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0]
		# draw
		if {$S(TransitionMapLayoutViewCluster) == "piechart"} {
				set complement [expr 1 - $percent ]
				set apc [list [list "?" [rnd $complement 2] "#DDD"] [list $annotation [rnd $percent 2] $color]]
			piechart2 $w [expr $x - $s ] [expr $y- $s] [expr $s + $s] [expr $s + $s] $treenode $apc "TMCIRCLE NODE$node TREENODE$treenode ANNOTATION=$annotation"
		} else {
			$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
			-fill $color -outline white -tags "TMCIRCLE NODE$node  TREENODE$treenode ANNOTATION=$annotation" 
		}
		set what ""
		set percentText [format "%s%s" [expr round(($nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0) *100.00)] "%"]
		if {$S(TransitionMapMonoNode)} {append what $percentText}
		if {$S(TransitionMapMonoAnnotation)} {append what \n$annotation3}
		$w create text $x $y -text $what -fill black -font $fo -justify center -tags "TMTEXT NODE$node ANNOTATION=$annotation " 
		# voir feuille rattaché au cluster et annote differement
		#PathSameAnnotationWithTipsDifferent $w $treenode $annotation $x $y $s
	}
	for {set generation $generationMax} {$generation >= 0} {incr generation -1} {
		foreach node $TT($IDT,generationtocode,$generation) {
			if {$TT($IDT,$node,enfants) != {}} {
				set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
				set annotation [lindex [split $annotationetnode =] 0 ]
				set treenode [lindex [split $annotationetnode =] 1 ]
				# Y, y moyen des enfants
				set ymean 0.0
				foreach enfant $TT($IDT,$node,enfants) {
					set ymean [expr $ymean + $yarray($enfant)]
				}
				set y [expr $ymean / [llength $TT($IDT,$node,enfants)]]
				set x [expr $factorX * [DistanceToRoot $treenode $T($IDT,dbl,$treenode)]]
				set yarray($node) $y
				set xarray($node) $x
				# attention ambiguité AA (parcimonie)
				if {[info exist S(ColorAnnotation,$annotation)]} {
					# pas d'ambiguité
					set color $S(ColorAnnotation,$annotation)
					set annotation3 [string range $annotation 0 2]
				} else {
					# annotation ancestrales multiple (non resolue en parsimony)
					set color lightgrey
					set annotation3 *
				}
				#
				set treefeuilles [Node2Leafs2 $treenode]
				set nbfeuilles [llength $treefeuilles]
				# pourcentage de feuilles de l'arbre phylogenetique issues de node ayant la meme annotation que node
				set nbfeuillesmemeannotation 0.0
				foreach fid $treefeuilles {
					set f $T($IDT,vbd,$fid)
					if {$TM($IDT,annotation,$f) == $annotation} {set nbfeuillesmemeannotation [expr $nbfeuillesmemeannotation + 1.0]}
				}
				set percent [expr $nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0]
				set complement [expr 1 - $percent ]
				set apc [list [list "?" [rnd $complement 2] "#DDD"] [list $annotation [rnd $percent 2] $color]]
				# draw
				if {$S(TransitionMapLayoutViewCluster) == "piechart"} {
					piechart2 $w [expr $x - $s ] [expr $y- $s] [expr $s + $s] [expr $s + $s] $treenode $apc "TMCIRCLE NODE$node TREENODE$treenode ANNOTATION=$annotation"
				} else {
					$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
					-fill $color -outline white -tags "TMCIRCLE NODE$node  TREENODE$treenode ANNOTATION=$annotation" 
				}
				set what ""
				set percentText [format "%s%s" [expr round(($nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0) *100.00)] "%"]
				if {$S(TransitionMapMonoNode)} {append what $percentText}
				if {$S(TransitionMapMonoAnnotation)} {append what \n$annotation3}
				$w create text $x $y -text $what -fill black -font $fo -justify center -tags "TMTEXT NODE$node ANNOTATION=$annotation" 
				# voir feuille rattaché au cluster et annote differement
				#PathSameAnnotationWithTipsDifferent $w $treenode $annotation $x $y $s
				# relier pere/enfants
				foreach enfant $TT($IDT,$node,enfants) {
					set x $xarray($node)
					set y $yarray($node)
					set x1 $xarray($enfant)
					set y1 $yarray($enfant)
					if {$S(TransitionMapMonoArrow)} {
						$w create line $x $y $x $y1  [expr $x1 - $s] $y1 -width 1 -smooth true -splinesteps 100 -arrow last -arrowshape {15 20 5} -tags "EDGEPUPKO NODE$enfant" 
					} else {
						$w create line $x $y $x $y1  [expr $x1 - $s] $y1 -width 1 -smooth true -splinesteps 100 -tags "EDGEPUPKO NODE$enfant" 
					}
				}
			} 
		}
	}
	$w lower EDGEPUPKO
	$w configure -scrollregion [$w bbox all]
}
#
proc TransitionMapLayoutMonoSlantedGN {} {
	global IDT T TT TM A S  generationMax
	set w $S(canvas2)
	$w delete all
	set lfeuilles [PupkoListFeuille]
	set s $S(SurfaceBubbleTransitionMap) ;# taile pastille
	set dy $S(TransitionMapYinter)
	set dx $S(TransitionMapXfactor)
	set fo [list Helvetica $S(TransitionMapFontSize) normal]
	set y 0.0
	set xMax 500.0
	foreach node $lfeuilles {
		set y [expr $y + $dy]
		set x $xMax
		set yarray($node) $y
		set xarray($node) $x
		# annotation=nodetreeorigial:nbtransition
		set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
		set annotation [lindex [split $annotationetnode =] 0 ]
		set treenode [lindex [split $annotationetnode =] 1 ]
		# attention ambiguité AA (parcimonie)
		if {[info exist S(ColorAnnotation,$annotation)]} {
			# pas d'ambiguité
			set color $S(ColorAnnotation,$annotation)
			set annotation3 [string range $annotation 0 2]
		} else {
			# annotation ancestrales multiple (non resolue en parsimony)
			set color lightgrey
			set annotation3 *
		}
		set treefeuilles [Node2Leafs2 $treenode]
		set nbfeuilles [llength $treefeuilles]
		# pourcentage de feuilles de l'arbre phylogenetique issues de node ayant la meme annotation que node
		set nbfeuillesmemeannotation 0.0
		foreach fid $treefeuilles {
			set f $T($IDT,vbd,$fid)
			if {$TM($IDT,annotation,$f) == $annotation} {set nbfeuillesmemeannotation [expr $nbfeuillesmemeannotation + 1.0]}
		}
		set percent [expr $nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0]
		
		# draw
		if {$S(TransitionMapLayoutViewCluster) == "piechart"} {
			set complement [expr 1 - $percent ]
			set apc [list [list "?" [rnd $complement 2] "#DDD"] [list $annotation [rnd $percent 2] $color]]
			piechart2 $w [expr $x - $s ] [expr $y- $s] [expr $s + $s] [expr $s + $s] $treenode $apc "TMCIRCLE NODE$node TREENODE$treenode ANNOTATION=$annotation"
		} else {
			$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
			-fill $color -outline white -tags "TMCIRCLE NODE$node  TREENODE$treenode ANNOTATION=$annotation" 
		}		
		set what ""
		set percentText [format "%s%s" [expr round(($nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0) *100.00)] "%"]
		if {$S(TransitionMapMonoNode)} {append what $percentText}
		if {$S(TransitionMapMonoAnnotation)} {append what \n$annotation3}
		$w create text $x $y -text $what -fill black -font $fo -justify center -tags "TMTEXT NODE$node ANNOTATION=$annotation " 		
		#PathSameAnnotationWithTipsDifferent $w $treenode $annotation $x $y $s
	}
	for {set generation $generationMax} {$generation >= 0} {incr generation -1} {
		foreach node $TT($IDT,generationtocode,$generation) {
			if {$TT($IDT,$node,enfants) != {}} {
				set ymean 0.0
				foreach enfant $TT($IDT,$node,enfants) {
					set ymean [expr $ymean + $yarray($enfant)]
				}
				set y [expr $ymean / [llength $TT($IDT,$node,enfants)]] 
				set yarray($node) $y
				set x [expr $xMax - ($dx * ($generationMax - $generation))]
				set xarray($node) $x
				set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
				set annotation [lindex [split $annotationetnode =] 0 ]
				set treenode [lindex [split $annotationetnode =] 1 ]
				#  ambiguité AA (parcimonie)
				if {[info exist S(ColorAnnotation,$annotation)]} {
					# pas d'ambiguité
					set color $S(ColorAnnotation,$annotation)
					set annotation3 [string range $annotation 0 2]
				} else {
					# annotation ancestrales multiple (non resolue en parsimony)
					set color lightgrey
					set annotation3 *
				}
				set treefeuilles [Node2Leafs2 $treenode]
				set nbfeuilles [llength $treefeuilles]
				# pourcentage de feuilles de l'arbre phylogenetique issues de node ayant la meme annotation que node
				set nbfeuillesmemeannotation 0.0
				foreach fid $treefeuilles {
					set f $T($IDT,vbd,$fid)
					if {$TM($IDT,annotation,$f) == $annotation} {set nbfeuillesmemeannotation [expr $nbfeuillesmemeannotation + 1.0]}
				}
				set percent [expr $nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0]
				set complement [expr 1 - $percent ]
				set apc [list [list "?" [rnd $complement 2] "#DDD"] [list $annotation [rnd $percent 2] $color]]
				# draw
				if {$S(TransitionMapLayoutViewCluster) == "piechart"} {
					piechart2 $w [expr $x - $s ] [expr $y- $s] [expr $s + $s] [expr $s + $s] $treenode $apc "TMCIRCLE NODE$node TREENODE$treenode ANNOTATION=$annotation"
				} else {
					$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
					-fill $color -outline white -tags "TMCIRCLE NODE$node  TREENODE$treenode ANNOTATION=$annotation" 
				}
				set what ""
				set percentText [format "%s%s" [expr round(($nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0) *100.00)] "%"]
				if {$S(TransitionMapMonoNode)} {append what $percentText}
				if {$S(TransitionMapMonoAnnotation)} {append what \n$annotation3}
				$w create text $x $y -text $what -fill black -font $fo -justify center -tags "TMTEXT NODE$node ANNOTATION=$annotation" 
				#PathSameAnnotationWithTipsDifferent $w $treenode $annotation $x $y $s
				# relier pere/enfants
				foreach enfant $TT($IDT,$node,enfants) {
					set x $xarray($node)
					set y $yarray($node)
					set x1 $xarray($enfant)
					set y1 $yarray($enfant)
					if {$S(TransitionMapMonoArrow)} {
						$w create line $x $y $x $y1  [expr $x1 - $s] $y1 -width 1 -smooth true -splinesteps 100 -arrow last -arrowshape {15 20 5} -tags "EDGEPUPKO NODE$enfant" 
					} else {
						$w create line $x $y $x $y1  [expr $x1 - $s] $y1 -width 1 -smooth true -splinesteps 100 -tags "EDGEPUPKO NODE$enfant" 
					}
				}
			} 
		}
	}
	$w lower EDGEPUPKO
	#$w xview moveto 0
	$w configure -scrollregion [$w bbox all]
}
# 
proc TransitionMapLayoutMonoRectangleBL {} {
	global IDT T TT TM A S  generationMax
	set w $S(canvas2)
	$w delete all
	set lfeuilles [PupkoListFeuille]
	# affectation
	set s $S(SurfaceBubbleTransitionMap) ;# taile pastille
	set dy $S(TransitionMapYinter)
	set factorX $S(TransitionMapXfactor)
	set fo $S(TransitionMapFont) 
	#
	set y 0.0
	foreach node $lfeuilles {
		# annotation=nodetreeorigial:nbtransition
		set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
		set annotation [lindex [split $annotationetnode =] 0 ]
		set treenode [lindex [split $annotationetnode =] 1 ]
		# X & Y
		set y [expr $y + $dy]
		set x [expr $factorX * [DistanceToRoot $treenode $T($IDT,dbl,$treenode)]]
		set yarray($node) $y
		set xarray($node) $x
		# attention ambiguité AA (parcimonie)
		if {[info exist S(ColorAnnotation,$annotation)]} {
			# pas d'ambiguité
			set color $S(ColorAnnotation,$annotation)
			set annotation3 [string range $annotation 0 2]
		} else {
			# annotation ancestrales multiple (non resolue en parsimony)
			set color lightgrey
			set annotation3 *
		}
		set treefeuilles [Node2Leafs2 $treenode]
		set nbfeuilles [llength $treefeuilles]
		# pourcentage de feuilles de l'arbre phylogenetique issues de node ayant la meme annotation que node
		set nbfeuillesmemeannotation 0.0
		foreach fid $treefeuilles {
			set f $T($IDT,vbd,$fid)
			if {$TM($IDT,annotation,$f) == $annotation} {set nbfeuillesmemeannotation [expr $nbfeuillesmemeannotation + 1.0]}
		}
		set percent [expr $nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0]
		# draw		
		if {$S(TransitionMapLayoutViewCluster) == "piechart"} {
				set complement [expr 1 - $percent ]
				set apc [list [list "?" [rnd $complement 2] "#DDD"] [list $annotation [rnd $percent 2] $color]]
			piechart2 $w [expr $x - $s ] [expr $y- $s] [expr $s + $s] [expr $s + $s] $treenode $apc "TMCIRCLE NODE$node TREENODE$treenode ANNOTATION=$annotation"
		} else {
			$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
			-fill $color -outline white -tags "TMCIRCLE NODE$node  TREENODE$treenode ANNOTATION=$annotation" 
		}
		set what ""
		set percentText [format "%s%s" [expr round(($nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0) *100.00)] "%"]
		if {$S(TransitionMapMonoNode)} {append what $percentText}
		if {$S(TransitionMapMonoAnnotation)} {append what \n$annotation3}
		$w create text $x $y -text $what -fill black -font $fo -justify center -tags "TMTEXT NODE$node ANNOTATION=$annotation " 
		#PathSameAnnotationWithTipsDifferent $w $treenode $annotation $x $y $s
	}
	for {set generation $generationMax} {$generation >= 0} {incr generation -1} {
		foreach node $TT($IDT,generationtocode,$generation) {
			if {$TT($IDT,$node,enfants) != {}} {
				set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
				set annotation [lindex [split $annotationetnode =] 0 ]
				set treenode [lindex [split $annotationetnode =] 1 ]
				# X & Y
				set ymean 0.0
				foreach enfant $TT($IDT,$node,enfants) {
					set ymean [expr $ymean + $yarray($enfant)]
				}
				set y [expr $ymean / [llength $TT($IDT,$node,enfants)]] 
				set x [expr $factorX * [DistanceToRoot $treenode $T($IDT,dbl,$treenode)]]
				set yarray($node) $y
				set xarray($node) $x
				# attention ambiguité AA (parcimonie)
				if {[info exist S(ColorAnnotation,$annotation)]} {
					# pas d'ambiguité
					set color $S(ColorAnnotation,$annotation)
					set annotation3 [string range $annotation 0 2]
				} else {
					# annotation ancestrales multiple (non resolue en parsimony)
					set color lightgrey
					set annotation3 *
				}
				#
				set treefeuilles [Node2Leafs2 $treenode]
				set nbfeuilles [llength $treefeuilles]
				# pourcentage de feuilles de l'arbre phylogenetique issues de node ayant la meme annotation que node
				set nbfeuillesmemeannotation 0.0
				foreach fid $treefeuilles {
					set f $T($IDT,vbd,$fid)
					if {$TM($IDT,annotation,$f) == $annotation} {set nbfeuillesmemeannotation [expr $nbfeuillesmemeannotation + 1.0]}
				}
				set percent [expr $nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0]
				set complement [expr 1 - $percent ]
				set apc [list [list "?" [rnd $complement 2] "#DDD"] [list $annotation [rnd $percent 2] $color]]
				# draw
				if {$S(TransitionMapLayoutViewCluster) == "piechart"} {
					piechart2 $w [expr $x - $s ] [expr $y- $s] [expr $s + $s] [expr $s + $s] $treenode $apc "TMCIRCLE NODE$node TREENODE$treenode ANNOTATION=$annotation"
				} else {
					$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
					-fill $color -outline white -tags "TMCIRCLE NODE$node  TREENODE$treenode ANNOTATION=$annotation" 
				}
				set what ""
				set percentText [format "%s%s" [expr round(($nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0) *100.00)] "%"]
				if {$S(TransitionMapMonoNode)} {append what $percentText}
				if {$S(TransitionMapMonoAnnotation)} {append what \n$annotation3}
				$w create text $x $y -text $what -fill black -font $fo -justify center -tags "TMTEXT NODE$node ANNOTATION=$annotation" 
				#PathSameAnnotationWithTipsDifferent $w $treenode $annotation $x $y $s
				# relier pere/enfants
				foreach enfant $TT($IDT,$node,enfants) {
					set x $xarray($node)
					set y $yarray($node)
					set x1 $xarray($enfant)
					set y1 $yarray($enfant)
					if {$S(TransitionMapMonoArrow)} {
						$w create line $x $y $x $y1 -width 1 -tags "EDGEPUPKO NODE$enfant " 
						$w create line $x $y1 [expr $x1 - $s] $y1 -width 1 -arrow last -arrowshape {15 20 5} -tags "EDGEPUPKO NODE$enfant " 
					} else {
						$w create line $x $y $x $y1 -width 1 -tags "EDGEPUPKO NODE$enfant " 
						$w create line $x $y1 $x1 $y1 -width 1 -tags "EDGEPUPKO NODE$enfant " 
					}
				}
			} 
		}
	}
	$w lower EDGEPUPKO
	#$w xview moveto 0
	$w configure -scrollregion [$w bbox all]
}
#
proc TransitionMapLayoutMonoRectangleGN {} {
	global IDT T TT TM A S  generationMax
	set w $S(canvas2)
	$w delete all
	set lfeuilles [PupkoListFeuille]
	set s $S(SurfaceBubbleTransitionMap) ;# taile pastille
	set dy $S(TransitionMapYinter)
	set dx $S(TransitionMapXfactor)
	set fo [list Helvetica $S(TransitionMapFontSize) normal]
	set y 0.0
	set xMax 500.0
	foreach node $lfeuilles {
		set y [expr $y + $dy]
		set x $xMax
		set yarray($node) $y
		set xarray($node) $x
		# annotation=nodetreeorigial:nbtransition
		set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
		set annotation [lindex [split $annotationetnode =] 0 ]
		set treenode [lindex [split $annotationetnode =] 1 ]
		# attention ambiguité AA (parcimonie)
		if {[info exist S(ColorAnnotation,$annotation)]} {
			# pas d'ambiguité
			set color $S(ColorAnnotation,$annotation)
			set annotation3 [string range $annotation 0 2]
		} else {
			# annotation ancestrales multiple (non resolue en parsimony)
			set color lightgrey
			set annotation3 *
		}
		set treefeuilles [Node2Leafs2 $treenode]
		set nbfeuilles [llength $treefeuilles]
		# pourcentage de feuilles de l'arbre phylogenetique issues de node ayant la meme annotation que node
		set nbfeuillesmemeannotation 0.0
		foreach fid $treefeuilles {
			set f $T($IDT,vbd,$fid)
			if {$TM($IDT,annotation,$f) == $annotation} {set nbfeuillesmemeannotation [expr $nbfeuillesmemeannotation + 1.0]}
		}
		set percent [expr $nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0]
		# draw
		if {$S(TransitionMapLayoutViewCluster) == "piechart"} {
			set complement [expr 1 - $percent ]
			set apc [list [list "?" [rnd $complement 2] "#DDD"] [list $annotation [rnd $percent 2] $color]]
			piechart2 $w [expr $x - $s ] [expr $y- $s] [expr $s + $s] [expr $s + $s] $treenode $apc "TMCIRCLE NODE$node TREENODE$treenode ANNOTATION=$annotation"
		} else {
			$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
			-fill $color -outline white -tags "TMCIRCLE NODE$node  TREENODE$treenode ANNOTATION=$annotation" 
		}		
		set what ""
		set percentText [format "%s%s" [expr round(($nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0) *100.00)] "%"]
		if {$S(TransitionMapMonoNode)} {append what $percentText}
		if {$S(TransitionMapMonoAnnotation)} {append what \n$annotation3}
		$w create text $x $y -text $what -fill black -font $fo -justify center -tags "TMTEXT NODE$node ANNOTATION=$annotation " 		
		#PathSameAnnotationWithTipsDifferent $w $treenode $annotation $x $y $s
	}
	for {set generation $generationMax} {$generation >= 0} {incr generation -1} {
		foreach node $TT($IDT,generationtocode,$generation) {
			if {$TT($IDT,$node,enfants) != {}} {
				set ymean 0.0
				foreach enfant $TT($IDT,$node,enfants) {
					set ymean [expr $ymean + $yarray($enfant)]
				}
				set y [expr $ymean / [llength $TT($IDT,$node,enfants)]] 
				set yarray($node) $y
				set x [expr $xMax - ($dx * ($generationMax - $generation))]
				set xarray($node) $x
				set annotationetnode [lindex [split $TT($IDT,$node,label) :] 0 ]
				set annotation [lindex [split $annotationetnode =] 0 ]
				set treenode [lindex [split $annotationetnode =] 1 ]
				#  ambiguité AA (parcimonie)
				if {[info exist S(ColorAnnotation,$annotation)]} {
					# pas d'ambiguité
					set color $S(ColorAnnotation,$annotation)
					set annotation3 [string range $annotation 0 2]
				} else {
					# annotation ancestrales multiple (non resolue en parsimony)
					set color lightgrey
					set annotation3 *
				}
				set treefeuilles [Node2Leafs2 $treenode]
				set nbfeuilles [llength $treefeuilles]
				# pourcentage de feuilles de l'arbre phylogenetique issues de node ayant la meme annotation que node
				set nbfeuillesmemeannotation 0.0
				foreach fid $treefeuilles {
					set f $T($IDT,vbd,$fid)
					if {$TM($IDT,annotation,$f) == $annotation} {set nbfeuillesmemeannotation [expr $nbfeuillesmemeannotation + 1.0]}
				}
				set percent [expr $nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0]
				set complement [expr 1 - $percent ]
				set apc [list [list "?" [rnd $complement 2] "#DDD"] [list $annotation [rnd $percent 2] $color]]
				# draw
				if {$S(TransitionMapLayoutViewCluster) == "piechart"} {
					piechart2 $w [expr $x - $s ] [expr $y- $s] [expr $s + $s] [expr $s + $s] $treenode $apc "TMCIRCLE NODE$node TREENODE$treenode ANNOTATION=$annotation"
				} else {
					$w create oval [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
					-fill $color -outline white -tags "TMCIRCLE NODE$node  TREENODE$treenode ANNOTATION=$annotation" 
				}
				set what ""
				set percentText [format "%s%s" [expr round(($nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0) *100.00)] "%"]
				if {$S(TransitionMapMonoNode)} {append what $percentText}
				if {$S(TransitionMapMonoAnnotation)} {append what \n$annotation3}
				$w create text $x $y -text $what -fill black -font $fo -justify center -tags "TMTEXT NODE$node ANNOTATION=$annotation" 
				#PathSameAnnotationWithTipsDifferent $w $treenode $annotation $x $y $s
				# relier pere/enfants
				foreach enfant $TT($IDT,$node,enfants) {
					set x $xarray($node)
					set y $yarray($node)
					set x1 $xarray($enfant)
					set y1 $yarray($enfant)
					if {$S(TransitionMapMonoArrow)} {
						$w create line $x $y $x $y1 -width 1 -tags "EDGEPUPKO NODE$enfant " 
						$w create line $x $y1 [expr $x1 - $s] $y1 -width 1 -arrow last -arrowshape {15 20 5} -tags "EDGEPUPKO NODE$enfant " 
					} else {
						$w create line $x $y $x $y1 -width 1 -tags "EDGEPUPKO NODE$enfant " 
						$w create line $x $y1 $x1 $y1 -width 1 -tags "EDGEPUPKO NODE$enfant " 
					}
				}
			} 
		}
	}
	$w lower EDGEPUPKO
	#$w xview moveto 0
	$w configure -scrollregion [$w bbox all]
}
#
proc DistanceToRoot {n l} {
	global T IDT
	if {$n != 0} {
		set p $T($IDT,pere,$n)
		set l [expr $l + $T($IDT,dbl,$p)]
		DistanceToRoot $p $l
	} else {
		return $l
	}
}


# sachant n, je parcours l'arbre down tant que l'annotation ne change pas
# pour chaque node traversé, test si des feuilles rattaché directement et si annotée differement
proc PathSameAnnotationWithTipsDifferent {w n ap x y s } {
	global S T TM IDT
	set S(TransitionMapPathTipDifferent) {}
	set fo [list Helvetica $S(TransitionMapFontSize) normal]
	PathSameAnnotationWithTipsDifferentGo $n $ap
	if {$S(TransitionMapPathTipDifferent) != {} } {
		# x y x x z y -> x 3 y 2 z
		catch {unset latmp}
		set total 0
		foreach a $S(TransitionMapPathTipDifferent) {
			if [info exist latmp($a)] {incr latmp($a)} else {set latmp($a) 1 ; incr total}
		}
		# 90° = 1.5708rad
		set angleinter [expr 1.5708 / $total] 
		set angle [expr 1.5708 *2];# depart
		set rayon [expr  $s + 5]
		# pour chaque annotation differente tracé en x y avec nb de fois
		foreach {a nb} [array get latmp *] {
			set angle [expr $angle - $angleinter]
			set x1 [expr cos($angle) * $rayon ]
			set y1 [expr sin($angle) * $rayon ]
			set color $S(ColorAnnotation,$a)
			#$w create line $x $y [expr $x + $x1] [expr $y + $y1] -fill $color -width 1 -tags transitionmaptips
			$w create oval [expr $x + $x1 -$latmp($a)-1] [expr $y + $y1-$latmp($a)-1] [expr $x + $x1+$latmp($a)+1] [expr $y + $y1+$latmp($a)+1] -fill $color -outline $color -tags transitionmaptips
			#$w create text [expr $x + $x1] [expr $y + $y1 -3] -text $latmp($a) -anchor center -fill black -font $fo -tags transitionmaptips
		}
		$w lower transitionmaptips
	}
}
proc PathSameAnnotationWithTipsDifferentGo {n ap} {
	global S T TM IDT
	if {$TM($IDT,annotation,$n) == $ap} {
		# node dans la path, test de feuille
		# feuille 1
		if {$T($IDT,typ,$T($IDT,fils1,$n)) == "f"} {
			if {$TM($IDT,annotation,$T($IDT,fils1,$n)) != $ap} {
				# retenir annotation
				lappend S(TransitionMapPathTipDifferent) $TM($IDT,annotation,$T($IDT,fils1,$n))
			}
		} else {
			PathSameAnnotationWithTipsDifferentGo $T($IDT,fils1,$n) $ap 
		}
		# feuille 2
		if {$T($IDT,typ,$T($IDT,fils2,$n)) == "f"} {
			if {$TM($IDT,annotation,$T($IDT,fils2,$n)) != $ap} {
				# retenir annotation
				lappend S(TransitionMapPathTipDifferent) $TM($IDT,annotation,$T($IDT,fils2,$n))
			}
		} else {
			PathSameAnnotationWithTipsDifferentGo $T($IDT,fils2,$n) $ap  
		}
	} else {
		return
	}
}
###########################################################################


###########################################################################
### TRANSITION MAP LAYOUT
###########################################################################


# Graphe RADIAL scenario pupko
proc PupkoListFeuille {} {
	global IDT LISTFEUILLE TT
	set LISTFEUILLE {}
	PupkoListFeuilleGo 0
	# classer la liste LISTEFEUILLE par profondeur (generation)
	#set newl {}
	#foreach n $LISTFEUILLE { lappend newl [format "%s%s%s" $TT($IDT,codetogeneration,$n) = $n]}
	#set liste [lsort -increasing $newl]
	#set final {}
	#foreach i $liste {
	#	lappend final [lindex [split $i =] end]
	#}
	#return $final
	return $LISTFEUILLE
}
#
proc PupkoListFeuilleGo {n} {
	global LISTFEUILLE IDT TT
	foreach node $TT($IDT,$n,enfants) {
		if {$TT($IDT,$node,enfants) != {}} {
			PupkoListFeuilleGo $node
		} else {
			lappend LISTFEUILLE $node
		}
	}
}
proc PupkoListFeuilleGoTEST {n} {
	global LISTFEUILLE IDT TT
	set listok1 {}
	set listok2 {}
	foreach node $TT($IDT,$n,enfants) {
		if {$TT($IDT,$node,enfants) != {}} {
			lappend listok2 $node
		} else {
			lappend listok1 $node
		}
	}
	foreach i $listok1 {lappend LISTFEUILLE $i}
	foreach i $listok2 {PupkoListFeuilleGo $i}
}
#
proc PupkoMapZoom {f} {
	global S
	zoom $S(canvas2) $f
}
# 
proc TransitionMapHiTreeNodeEnter {w x y} {
	global S TM IDT
	set i [$w find withtag current]
	# extraire IDT de $w
	set iiddtt [string range [lindex [split $w .] 5] 1 end]
	set S(canvastransitionmaptransit) [format "%s%s%s" .pan.g.g.g.m $iiddtt .p.main.canvas.c]
	set tags [$w gettags $i]
	set indexnode [lsearch -glob $tags TREENODE*] 
	set TREENODE [lindex $tags $indexnode]
	set ln [string trimleft $TREENODE TREENODE]
	regsub -all = $ln " " ltreenode
	foreach n $ltreenode {
		#HiSameAnnotationPath $n $TM($IDT,annotation,$n)
		HiSameAnnotationPath $n $TM($iiddtt,annotation,$n) $iiddtt
	}
}
#
proc HiSameAnnotationPath {n ap iiddtt} {
	global S T TM IDT
	#  $TM($IDT,annotation,$n) == $ap
	if {$TM($iiddtt,annotation,$n) == $ap} {
		#set originalcolor [lindex [$S(canvas) itemconfigure H$n -fill] end]
		#set originalsize [lindex [$S(canvas) itemconfigure H$n -width] end]
		set originalcolor [lindex [$S(canvastransitionmaptransit) itemconfigure H$n -fill] end]
		set originalsize [lindex [$S(canvastransitionmaptransit) itemconfigure H$n -width] end]
		lappend S(TransitionMapHiNodeNode) $n
		set S(TransitionMapHiNodeColor,$n) $originalcolor
		set S(TransitionMapHiNodeSize,$n) $originalsize
		$S(canvastransitionmaptransit) itemconfigure H$n -width 2 -fill #FAFAD2
		$S(canvastransitionmaptransit) itemconfigure V$n -width 2 -fill #FAFAD2
		# $T($IDT,typ,$n) != "f"
		if {$T($iiddtt,typ,$n) != "f"} {
			#HiSameAnnotationPath $T($IDT,fils1,$n) $ap 
			#HiSameAnnotationPath $T($IDT,fils2,$n) $ap 
			HiSameAnnotationPath $T($iiddtt,fils1,$n) $ap $iiddtt
			HiSameAnnotationPath $T($iiddtt,fils2,$n) $ap $iiddtt
		} else {
			return
		}
	}
}
#
proc TransitionMapHiTreeNodeLeave {w x y} {
	global S
	foreach n $S(TransitionMapHiNodeNode) {
		#$S(canvas) itemconfigure H$n -width $S(TransitionMapHiNodeSize,$n) -fill $S(TransitionMapHiNodeColor,$n)
		#$S(canvas) itemconfigure V$n -width $S(TransitionMapHiNodeSize,$n) -fill $S(TransitionMapHiNodeColor,$n)
		$S(canvastransitionmaptransit) itemconfigure H$n -width $S(TransitionMapHiNodeSize,$n) -fill $S(TransitionMapHiNodeColor,$n)
		$S(canvastransitionmaptransit) itemconfigure V$n -width $S(TransitionMapHiNodeSize,$n) -fill $S(TransitionMapHiNodeColor,$n)
	}
	unset S(TransitionMapHiNodeNode)
	foreach {k v} [array get S TransitionMapHiNodeSize,*] {unset k}
	foreach {k v} [array get S TransitionMapHiNodeColor,*] {unset k}
}
#
proc TransitionMapHiTreeNodeRelease {w x y} {
	global S
	set tag [$w gettags current]	
}
################################################################################

################################################################################
### MUlTICOLLAPSE cluster 100
################################################################################
proc TreeCompress {} {
	global S
	TreeCompressGo
	switch -exact $S(CompressMode) {
		map {
			MakeMap .pan.g.g.g
			LoadTreeFile testTREE.txt
			LoadAnnotationFile testANNOTATION.txt
		}
		fil {
			#info
		}
	}
}
# set S(CompressFreqCluster) identification cluster frequence de l'annotation au sein du cluster
# set S(CompressTaux) taux de feuilles (random selection) du cluster original retrouvées dans le cluster symbolique 
proc TreeCompressGo {} {
	global IDT T S A AA TC list100 lnodes AList100TipsAnnotation Feuille
	# list100 contient la liste des nodes à remplacer 
	# par une feuille artificielle unique 
	set list100 {}
	catch {unset Feuille}
	
	# identification des annotations des feuilles
	foreach f $T($IDT,ue_cod) {
		foreach {k v} [array get AA $IDT,distribution,$f,*] {
			if {$v >= 0.99} {
				set a [lindex [split $k ,] end]
				set Feuille($f) $a
			} 
		}
	}
	
	# identification n 100%
	set AList100TipsAnnotation {}
	CollapseClusterGo 0
	
	# reconstruction Newick sans les clusters100
	set s [BuiltNewickWithoutCluster 0]
	
	# save newick
	set file testTREE.txt
	if {$file != "" } {
		set S(pathuser) [file dirname $file]
		set fid [open $file w]
		puts -nonewline $fid $s
		close $fid
		
	}
	# fichier annotation
	set file testANNOTATION.txt
	if {$file != "" } {
		set S(pathuser) [file dirname $file]
		set fid [open $file w]
		puts -nonewline $fid "ID"
		foreach a $A($IDT,AnnotationList) {
			puts -nonewline $fid ",$a" 
		}
		puts $fid "\n"
		foreach {n a} $AList100TipsAnnotation {
			if {$n != ""} {
				puts -nonewline $fid "$n"
				foreach v $A($IDT,AnnotationList) {
					if {$v == $a} {
						puts -nonewline $fid ",1" 
					} else {
						puts -nonewline $fid ",0" 
					}
				}
				puts $fid "\n"
			}
		}
		close $fid
	}
}
#
# set S(CompressFreqCluster) taux de l'annotation pour identifier cluster
# set S(CompressTaux) taux d'insertion de feuilles dans le cluster symbolique, min 2
proc CollapseClusterGo {n} {
	global IDT T S A AA TC list100 AList100TipsAnnotation Feuille
	if {$T($IDT,typ,$n) == "f"} { 
		lappend AList100TipsAnnotation $n $Feuille($n)
		return
	} else {
		set lf [Node2LeafsCode3 $n]
		#set nbfeuilles [expr [llength $lf] * 1.0] ; # nb feuilles = nombre total d'annotation
		foreach f $lf {lappend la $Feuille($f)}
		set annotationPack [DelRep $la]
		set nbannotationsdifferente [expr [llength $annotationPack] * 1.0]
		#set frequency [expr [llength $annotationPack] / $nbfeuilles]
		#puts "FREQUENCY $frequency vs $S(CompressFreqCluster)"
		
		#$frequency <= $S(CompressFreqCluster)
		if {$nbannotationsdifferente <= $S(CompressFreqCluster)} {
			# cluster 100% identifié
			#puts CLUSTER!
			#puts "LA $annotationPack"

			lappend list100 $n
			lappend AList100TipsAnnotation $n [lindex $annotationPack 0]
			return
		} else {
			# relance sur les fils
			CollapseClusterGo $T($IDT,fils1,$n) 
			CollapseClusterGo $T($IDT,fils2,$n) 
		}
	}
}
#
proc Node2LeafsCode3 {n} {
	global IDT T 
	if {$T($IDT,typ,$n) == f} {
		return $n
	} else {
		set l1 [Node2LeafsCode3 $T($IDT,fils1,$n)]
		set l2 [Node2LeafsCode3 $T($IDT,fils2,$n)]
		set l [concat $l1 " $l2"]
		return $l
	}
}
#
proc BuiltNewickWithoutCluster {n} {
	global IDT T list100 NewNewick
	set fils1 $T($IDT,fils1,$n)
	set fils2 $T($IDT,fils2,$n)
	set dbl $T($IDT,dbl,$n)
	set NewNewick [format "%s%s%s%s%s%s%s%s%s" ooo $fils1 , $fils2 ccc $n : $dbl ";" ]
	BuiltNewickWithoutClusterForFils1 $fils1
	BuiltNewickWithoutClusterForFils2 $fils2
	regsub -all ooo $NewNewick ( NewNewick
	regsub -all ccc $NewNewick ) NewNewick
	return $NewNewick
}
#
proc BuiltNewickWithoutClusterForFils1 {n} {
	global IDT T list100 NewNewick 
	set typ $T($IDT,typ,$n)
	set dbl $T($IDT,dbl,$n)
	if {$typ == "f"} {
		set adds1 [format "%s%s%s%s%s" ooo $n : $dbl ,]
		set p1 [format "%s%s%s" ooo $n ,]
		regsub -all  $p1 $NewNewick $adds1 NewNewick
		return
	} else {
		if {[lsearch $list100 $n] != -1} {
			# n est un cluster 100
			set adds1 [format "%s%s%s%s%s" ooo $n : $dbl , ]
			set p1 [format "%s%s%s" ooo $n ,]
			regsub -all  $p1 $NewNewick $adds1 NewNewick
			return
		} else {
			set fils1 $T($IDT,fils1,$n)
			set fils2 $T($IDT,fils2,$n)
			set adds1 [format "%s%s%s%s%s%s%s%s%s%s" ooo ooo $fils1 , $fils2 ccc $n : $dbl , ]
			set p1 [format "%s%s%s" ooo $n ,]
			regsub -all  $p1 $NewNewick $adds1 NewNewick
			BuiltNewickWithoutClusterForFils1 $fils1
			BuiltNewickWithoutClusterForFils2 $fils2
			return 
		}
	}
}
#
proc BuiltNewickWithoutClusterForFils2 {n} {
	global IDT T list100 NewNewick 
	set typ $T($IDT,typ,$n)
	set dbl $T($IDT,dbl,$n)
	if {$typ == "f"} {
		set adds2 [format "%s%s%s%s%s" , $n : $dbl ccc]
		set p2 [format "%s%s%s" , $n ccc]
		regsub -all $p2 $NewNewick $adds2 NewNewick
		return
	} else {
		if {[lsearch $list100 $n] != -1} {
			# n est un cluster 100
			set adds2 [format "%s%s%s%s%s" , $n : $dbl ccc ]
			set p2 [format "%s%s%s" , $n ccc]
			regsub -all $p2 $NewNewick $adds2 NewNewick
			return 
		} else {
			set fils1 $T($IDT,fils1,$n)
			set fils2 $T($IDT,fils2,$n)
			set adds2 [format "%s%s%s%s%s%s%s%s%s%s" , ooo $fils1 , $fils2 ccc $n : $dbl ccc ]
			set p2 [format "%s%s%s" , $n ccc]
			regsub -all $p2 $NewNewick $adds2 NewNewick
			BuiltNewickWithoutClusterForFils1 $fils1
			BuiltNewickWithoutClusterForFils2 $fils2
			return 
		}
	}
}
proc NODEIDincr {} {
	global NODEID
	set NODEID [expr $NODEID + 1]
	return $NODEID
}
################################################################################



################################################################################
# ROTATION
################################################################################
proc RotateItem {w tagOrId Ox Oy angle} {
	set angle [expr {$angle * atan(1) * 4 / 180.0}] ;# Radians
	foreach id [$w find withtag "$tagOrId" ] {	 ;# Do each component separately
		set xy {}
		foreach {x y} [$w coords $id] {
			# rotates vector (Ox,Oy)->(x,y) by angle clockwise
			set x [expr {$x - $Ox}]			 ;# Shift to origin
			set y [expr {$y - $Oy}]
			set xx [expr {$x * cos($angle) - $y * sin($angle)}] ;# Rotate
			set yy [expr {$x * sin($angle) + $y * cos($angle)}]
			set xx [expr {$xx + $Ox}]		   ;# Shift back
			set yy [expr {$yy + $Oy}]
			lappend xy $xx $yy
		}
		$w coords $id $xy
	}
	$w configure -scrollregion [$w bbox all]
}
#
proc RotateTree {} {
	global S IDT
	set w $S(canvas)
	if {$S($IDT,rotation) == 0} {
		set S($IDT,rotation) 90
		RotateItem $w all 0 0 90
		$w itemconfigure L -angle 270
		$w itemconfigure AnnotMatrixText -angle 270
		$w itemconfigure L -anchor w
		$w itemconfigure AnnotMatrixText -anchor w
	} else {
		set S($IDT,rotation) 0
		RotateItem $w all 0 0 -90
		$w itemconfigure L -angle 0
		$w itemconfigure AnnotMatrixText -angle 0
		$w itemconfigure L -anchor w
		$w itemconfigure AnnotMatrixText -anchor w
	}
	$w configure -scrollregion [$w bbox all]

}

# Tree with phylotype color-encoded and
proc RotateBoard {} {
	global S
	set w $S(canvas2)
	RotateItem $w all 0 0 90
	$w itemconfigure L -anchor e
}

################################################################################


################################################################################
# EDITION 
################################################################################
proc leafFont {} {
	global S
	set S(fontanswer) 0
	FontPanel
	vwait S(fontanswer)
	if {$S(fontanswer) == 1} {
		set S(Leaffont) $S(gfo)
		$S(canvas) itemconfigure L -font $S(gfo)
	}
}
proc FontPanel {} {
	global S
	destroy .pamelafont
	set wp [toplevel .pamelafont]
	wm title $wp "Font selection"

	set wmain [ttk::frame $wp.controls]
	# family
	ttk::menubutton $wmain.f -text "Family" -menu $wmain.f.m
	set m  [menu  $wmain.f.m -tearoff 0]
	foreach ff [font families] {
		$m add radiobutton -label $ff -variable S(-family) -value $ff -command  {FontPanel_FontSwitch}
	}
	# weight
	ttk::menubutton $wmain.w -text "Weight" -menu $wmain.w.m
	set m  [menu  $wmain.w.m -tearoff 0]
	$m add radiobutton -variable S(-weight) -value normal -label "Normal" -command  {FontPanel_FontSwitch}
	$m add radiobutton -variable S(-weight) -value bold -label "Bold" -command  {FontPanel_FontSwitch}
	# size
	ttk::menubutton $wmain.s -text "Size" -menu $wmain.s.m
	set m  [menu  $wmain.s.m -tearoff 0]
	foreach size {4 5 6 7 8 9 10 11 12 14 16} {
		$m add radiobutton -variable S(-size) -value $size -label $size -command  {FontPanel_FontSwitch}
	}
	# slant
	ttk::menubutton $wmain.a -text "Slant" -menu $wmain.a.m
	set m  [menu  $wmain.a.m -tearoff 0]
	$m add radiobutton -variable S(-slant) -value roman -label "Roman" -command  {FontPanel_FontSwitch}
	$m add radiobutton -variable S(-slant) -value italic -label "Italic" -command  {FontPanel_FontSwitch}
	# underline
	ttk::menubutton $wmain.u -text "Underline" -menu $wmain.u.m
	set m  [menu  $wmain.u.m -tearoff 0]
	$m add radiobutton -variable S(-underline) -value 1 -label "Yes" -command  {FontPanel_FontSwitch}
	$m add radiobutton -variable S(-underline) -value 0 -label "No" -command  {FontPanel_FontSwitch}
	# overstrike
	ttk::menubutton $wmain.o -text "Overstrike" -menu $wmain.o.m
	set m  [menu  $wmain.o.m -tearoff 0]
	$m add radiobutton -variable S(-overstrike) -value 1 -label "Yes" -command  {FontPanel_FontSwitch}
	$m add radiobutton -variable S(-overstrike) -value 0 -label "No" -command  {FontPanel_FontSwitch}


	# sample
	set S(-family) Arial
	set S(-weight) normal
	set S(-size) 10
	set S(-slant) roman
	set S(-underline) 0
	set S(-overstrike) 0
	set S(gfo) [list -family $S(-family) \
			-weight $S(-weight) \
			-size $S(-size) \
			-slant $S(-slant) \
			-underline $S(-underline) \
			-overstrike $S(-overstrike)]
	message $wp.msg -aspect 1000 -font $S(gfo)  \
			-text "ABCDEFGHIJKLMNOPQRSTUVWXYZ\
			\nabcdefghijklmnopqrstuvwxyz\
			\n0123456789\
			\n!@#$%^&*()_+-=[]{};\
			\n:\"'` ~,.<>/?\\|"
	set S(pathFontMSG) $wp.msg
	set okcancel [ttk::frame $wp.okcancel]
	button $okcancel.ok -text ok -command {FontPanel_ok}
	button $okcancel.cancel -text Cancel -command {FontPanel_cancel}
	# PACK
	pack $wmain.f $wmain.w $wmain.s $wmain.a $wmain.u $wmain.o -side top -expand yes -fill x
	pack $okcancel.ok $okcancel.cancel -side left
	pack $wmain $wp.msg $wp.okcancel -side top -expand yes -fill x
}
proc FontPanel_ok {} {
	global S
	set S(fontanswer) 1
	destroy .pamelafont
}
proc FontPanel_cancel {} {
	global S
	set S(fontanswer) 2
	destroy .pamelafont
}
proc FontPanel_FontSwitch {} {
	global S
	set S(gfo) [list -family $S(-family) \
			-weight $S(-weight) \
			-size $S(-size) \
			-slant $S(-slant) \
			-underline $S(-underline) \
			-overstrike $S(-overstrike)]
	$S(pathFontMSG) configure  -font $S(gfo)
}
################################################################################


################################################################################
# PHYLOTYPE MAP clusters
################################################################################
proc makeMapCluster {w lp} {
	global IDT S A SA T VA  Pdirect G Pere Y trans PhylotypeDATA
	$w delete all
	catch {unset Pdirect}
	catch {unset Pere}
	catch {unset Y}
	catch {unset trans}
	catch {unset G}
	set lp [lsort -increasing $lp]
	# pour chaque phylotype rechercher les phylotype directs
	# quelle que soit la generation 
	foreach p $lp {
		set Pdirect($p) ""
		append Pdirect($p) [SearchDirectPhylotype $T($IDT,fils1,$p) $lp]
		append Pdirect($p) [SearchDirectPhylotype $T($IDT,fils2,$p) $lp]
	}
	# les PERE
	foreach {k v} [array get Pdirect *] {
		foreach n $v {
			set Pere($n) $k
		}
	}
	# generation sur les phylotypes = nb de parents jusqu'a la racine
	foreach p $lp {
		set gg [GenerationPhylotype  $p 1]
		set GenPhy($p) $gg
		if {[info exists GenPhy(gen,$gg)]} {
			lappend GenPhy(gen,$gg) $p
		} else  {
			set GenPhy(gen,$gg) $p
			lappend GenPhy(all) $gg
		}
	}
	set GenPhy(all) [lsort -increasing $GenPhy(all)]
	set GenPhyMax [lindex $GenPhy(all) end]
	# on ordonne les phylotype d'une generation selon l'ordre des phylotype de la generation precedente
	# a l'interieur d'une generation on peut ordonner les phylotypes selon un critere comme l'annotation ou sz
	foreach g $GenPhy(all) {
		if {$g != $GenPhyMax} {
			set gsuivante [expr $g + 1]
			set lpsuivante {}
			foreach pg $GenPhy(gen,$g) {
				if {[info exists Pdirect($pg)]} {
					foreach pdirect  $Pdirect($pg) {
						lappend lpsuivante $pdirect
					}
				}
			}
			set GenPhy(gen,$gsuivante) $lpsuivante
		}
	}
	# les phylotypes representes par des cercles de surface
	# proportionellle au nombre de strains, quelle que soit les generations
	set SurfaceMax 0
	set SurfaceMin 80
	foreach p $lp  {
		#set s [expr sqrt($VA($trans($p,id),sz))]
		set a $S(annotationphylotype,$p)
		set SZi $T($IDT,sz,$a,$p)
		set s [expr sqrt($SZi)]
		if  {$s >= $SurfaceMax} {set SurfaceMax $s}
		if  {$s < $SurfaceMin} {set SurfaceMin $s}
	}
	# GRAPHIQUE
	set xcenter [expr 200.0 / 2]
	set ycenter [expr 200.0 / 2]
	set inter 50 ;# nb pixels separant deux generations
	set Yinter 50
	# facteur d'Ã©chelle le rayon de cercle doit etre inferieur ou egal a Yinter / 2
	set Fsurface [expr ($Yinter / 2.0) / $SurfaceMax]
	set Fsurface [expr $Yinter / $SurfaceMax]   
	### organisation des phylotypes
	### organisation de la generation 1
	set lphylotypesavecenfants {}
	set lphylotypessansenfants {}
	foreach p $GenPhy(gen,1) {
		if {[llength $Pdirect($p)] == 0} {
			lappend lphylotypessansenfants $p
		} else  {
			lappend lphylotypesavecenfants $p
		}
	}
	# classement alpha
	set lpaa {}
	foreach p $lphylotypessansenfants {
		#set id $trans($p,id)
		set id $p
		lappend lpaa [format  "%s%s%s" $S(annotationphylotype,$p) ~ $p]
	}
	set lpaa [lsort -dictionary $lpaa]
	set lpfinal {}
	foreach pai $lpaa {
		lappend lpfinal [lindex [split $pai ~] end]
	}
	set lphylotypessansenfants $lpfinal
	# classement alpha
	set lpaa {}
	foreach p $lphylotypesavecenfants {
		#set id $trans($p,id)
		lappend lpaa [format  "%s%s%s" $S(annotationphylotype,$p) ~ $p]
	}
	set lpaa [lsort -dictionary $lpaa]
	set lpfinal {}
	foreach pai $lpaa {
		lappend lpfinal [lindex [split $pai ~] end]
	}
	set lphylotypesavecenfants $lpfinal
	set $GenPhy(gen,1) "  $lphylotypessansenfants $lphylotypesavecenfants "
	### organisation recurssive maintenant
	foreach p $GenPhy(gen,1) {
		PhylotypeOrganisation $p
	}
	foreach g $GenPhy(all) {
		if {$g != $GenPhyMax} {
			set gsuivante [expr $g + 1]
			set lpsuivante {}
			foreach pg $GenPhy(gen,$g) {
				if {[info exists Pdirect($pg)]} {
					foreach pdirect  $Pdirect($pg) {
						lappend lpsuivante $pdirect
					}
				}
			}
			set GenPhy(gen,$gsuivante) $lpsuivante
		}
	}
	# i = generation
	foreach i $GenPhy(all) {
		set x [expr $i * $inter]
		set n 0
		set listephylotype $GenPhy(gen,$i)
		foreach p  $listephylotype {
			set annotation $S(annotationphylotype,$p)
			set color $S(ColorAnnotation,$annotation) 
			set y [expr $n * $Yinter]
			#set id $trans($p,id)
			set id $p
			set a $S(annotationphylotype,$p)
			set delta [expr $Fsurface * sqrt($T($IDT,sz,$a,$p))]
			set treefeuilles [Node2Leafs2 $p]
			set nbfeuilles [llength $treefeuilles]
			# pourcentage de feuilles de l'arbre phylogenetique issues de node ayant la meme annotation que node
			set nbfeuillesmemeannotation 0.0
			foreach fid $treefeuilles {
				set f $T($IDT,vbd,$fid)
				if {$PhylotypeDATA($IDT,annotation,$f) == $annotation} {set nbfeuillesmemeannotation [expr $nbfeuillesmemeannotation + 1.0]}
			}
			set percent [expr $nbfeuillesmemeannotation*1.0 / $nbfeuilles*1.0]
			set complement [expr 1 - $percent ]
			set apc [list [list "?" [rnd $complement 2] "#DDD"] [list $annotation [rnd $percent 2] $color]]
			piechart2 $w [expr $x - $delta/2.0 ] [expr $y- $delta/2.0 -100] $delta $delta $p $apc "dot G$i Pi$p o"
			set co [$w bbox Pi$p]
			set y [lindex $co 3]
			set tteexxtt [format "%s%s%s" $id  : $S(annotationphylotype,$p)]
			set idi [$w create text $x [expr $y +2] -text $tteexxtt -tags "dot G$i a tteexxtt$p L" -anchor n -font {Arial 8 bold} -fill black]
			incr n
		}
	}
	# justification inter generation
	set deltaYmax 0
	foreach i [lsort -increasing $GenPhy(all)] {
		set co [$w bbox G$i]
		set deltaY($i) [expr [lindex $co 3] - [lindex $co 1]]
		if {$deltaY($i) >= $deltaYmax} {set deltaYmax $deltaY($i)}
	}
	foreach i [lsort -increasing $GenPhy(all)] {
		set delta [expr ($deltaYmax - $deltaY($i)) / 2.0]
		$w move G$i 0 $delta
	}
	set co [$w bbox all]
	set yroot [expr ([lindex $co 1] + [lindex $co 3]) / 2.0]
	#set yroot	[expr $deltaYmax / 2.0]
	# dessiner la racine et relier la racine au phylotypes sans phylotype pere
	#$w create oval -10 [expr $yroot -10] 10 [expr $yroot +10] -tag "dot" -fill white -outline black
	$w create oval -5 [expr $yroot -5] 5 [expr $yroot +5] -tag "dot ROOT" -fill black -outline black
	foreach p $lp {
		if {[info exists Pere($p)] == 0} {
			# ici $p n'a pas de phylotype pere il est reliÃ© directement a la racine
			set co [$w coords ROOT]
			set x1 [expr [lindex $co 0] + (([lindex $co 2] - [lindex $co 0]) / 2.0)]
			set y1 [expr [lindex $co 1] + (([lindex $co 3] - [lindex $co 1]) / 2.0)]
			set co [$w bbox Pi$p]
			#set x2 [lindex $co 0]
			#set y2 [lindex $co 1]
			set x2 [expr [lindex $co 0] + (([lindex $co 2] - [lindex $co 0]) / 2.0)]
			set y2 [expr [lindex $co 1] + (([lindex $co 3] - [lindex $co 1]) / 2.0)]
			set couleur [lindex [$w itemconfigure Pi$p -fill] end]
			set quartinter [expr $inter / 4.0]
			set xmid1 [expr $x1 + $quartinter ]
			set ymid1 $y1
			set xmid2 [expr $x1 + ($inter / 2.0) ]
			set ymid2 [expr $y1 + (($y2 - $y1) / 2.0)]
			set xmid3 [expr $x1 + ($quartinter *3) ]
			set ymid3 $y2
			$w create line [expr $x1 +3] [expr $y1 +3 ] $xmid1 $ymid1 $xmid2 $ymid2 $xmid3 $ymid3 [expr $x2 -3] [expr $y2-3] \
					-tag "dot line" -fill black -smooth true -splinesteps 10
		}
	}
	# line relier chaque noeud a ses fils directs $P($p)
	### Sep 2012 listes des AA entre phylotypes pere et fils, list lAApf
	set lAApf {}
	set lbreak {} ; set compteurbreak 0
	foreach p $lp {
		if {$Pdirect($p) != {}} {
			set co [$w coords Pi$p]
			set x1 [expr [lindex $co 0] + (([lindex $co 2] - [lindex $co 0]) / 2.0)]
			set y1 [expr [lindex $co 1] + (([lindex $co 3] - [lindex $co 1]) / 2.0)]
			foreach pf $Pdirect($p) {
				set couleur [lindex [$w itemconfigure Pi$pf -fill] end]
				set co [$w bbox Pi$pf]
				set xipf [lindex $co 0]
				set x2 [expr [lindex $co 0] + (([lindex $co 2] - [lindex $co 0]) / 2.0)]
				set y2 [expr [lindex $co 1] + (([lindex $co 3] - [lindex $co 1]) / 2.0)]
				set quartinter [expr $inter / 4.0]
				set xmid1 [expr $x1 + $quartinter ]
				set ymid1 $y1
				set xmid2 [expr $x1 + ($inter / 2.0) ]
				set ymid2 [expr $y1 + (($y2 - $y1) / 2.0)]
				set xmid3 [expr $x1 + ($quartinter *3) ]
				set ymid3 $y2
				set llaa [lreverse [recursaa  $T($IDT,pere,$pf) $p ]]
				set test 0
				set laabreak {}
				foreach aai $llaa {
					if {[llength $aai] == 0} {
						# cas phylotype imbriquÃ©s sans intermÃ©diaire
					} elseif {[llength $aai] == 1}  {
						# cas standard
						if {$aai != $PhylotypeDATA($IDT,annotation,$pf) && $aai != $PhylotypeDATA($IDT,annotation,$p) } {
							lappend laabreak [lsort -dictionary $aai]
							set test 1
						}
					} else {
						# cas AA non unique
						if {[lsearch $aai $PhylotypeDATA($IDT,annotation,$pf)] == -1 && [lsearch $aai $PhylotypeDATA($IDT,annotation,$p)] == -1 } {
							set test 1
							lappend laabreak [lsort -dictionary $aai]
						}
					}
				}
				if {$test} {
					$w itemconfigure tteexxtt$pf -fill red
					# attention une rupture d'annotation peut etre une liste d'annotation
					set annotationbreak [DelRep $laabreak]
					lappend lbreak $pf $annotationbreak
				}
				$w create line [expr $x1 +3] [expr $y1 +3 ] $xmid1 $ymid1 $xmid2 $ymid2 $xmid3 $ymid3 [expr $x2 -3] [expr $y2-3] \
					-tag "dot line" -fill black -smooth true -splinesteps 10
			}
		}
	}
	#close $fd
	$w move dot $xcenter $ycenter
	$w move r $xcenter $ycenter
	$w raise ROOT
	$w raise o
	$w raise a
	$w raise reddot
	$w lower line
	# LEGEND des annotations break
	set co [$w bbox all]
	set xbreak [expr [lindex $co 2] + 5]
	set ybreak [lindex $co 1]
	foreach {compteur annotation} $lbreak {
		set id1 [$w create text $xbreak $ybreak -text $compteur -anchor e -font {Arial 8 bold} -fill white  -tag LEGEND]
		set co [$w bbox $id1]
		set xt1 [lindex $co 0]
		set yt1 [lindex $co 1]
		set xt2 [lindex $co 2]
		set yt2 [lindex $co 3]
		set id2 [$w create oval [expr $xt1 +2] $yt1 [expr $xt2 +2] $yt2 -fill red -outline red  -tag LEGEND]
		$w lower $id2 $id1
		$w create text [expr $xt2 + 10] [expr ($yt1 + $yt2) / 2.0] -text $annotation -anchor w -font {Arial 8 normal} -fill black   -tag LEGEND
		incr ybreak 30
		$w configure -scrollregion [$w bbox all]
		update idletasks
	}
	$w configure -scrollregion [$w bbox all]

}
proc piechart2 {w x y width height node data tags} {
	global IDT S
	set coords [list $x $y [expr {$x+$width}] [expr {$y+$height}]]
	set xm  [expr {$x+$width/2.}]
	set ym  [expr {$y+$height/2.}]
	set rad [expr {$width/2.+20}]
	set sum 0
	foreach item $data {set sum [expr {$sum + [lindex $item 1]}]}
	set start 270
	foreach item $data {
		foreach {name n color} $item break
		set extent [expr {$n*359.9/$sum}]
		$w create arc $coords -start $start -extent $extent -fill $color -outline white -tags "Cammenb CammenbCIR Cammenb$node MaxLikpiechart=$name $tags"
		set angle [expr {($start-90+$extent/2)/180.*acos(-1)}]
		set tx [expr $xm-$rad*sin($angle)]
		set ty [expr $ym-$rad*cos($angle)]
		set n [rnd $n 2]
		if {$n != 0.0} {
			#$w create text $tx $ty -text $n -state $S(piechartdisplaynumber) -tag txt -font $S(annotation-font) -tags "Cammenb CammenbTXT Cammenb$node" -fill $color
			set start [expr $start+$extent]
		}
	}
}
proc GenerationPhylotype  {p n} {
	global IDT Pere
	if {[info exists Pere($p)] == 0} {
		# on est a la racine
		return $n
	} else {
		incr n
		GenerationPhylotype $Pere($p) $n
	}
}


proc PhylotypeOrganisation {p} {
	global IDT Pdirect trans VA S
	if {[info exist Pdirect($p)]} {
		
		if {[llength $Pdirect($p)] == 0} {
			return
		} else  {
			set lphylotypesavecenfants {}
			set lphylotypessansenfants {}
			foreach pi $Pdirect($p) {
				if {[info exist Pdirect($pi)]} {
					if {[llength $Pdirect($pi)] == 0} {
						lappend lphylotypessansenfants $pi
					} else  {
						lappend lphylotypesavecenfants $pi
					}
					# classement alpha
					set lpaa {}
					foreach po $lphylotypessansenfants {
						#set id $trans($po,id)
						set id $po
						lappend lpaa [format  "%s%s%s" $S(annotationphylotype,$po) ~ $po]
					}
					set lpaa [lsort -dictionary $lpaa]
					set lpfinal {}
					foreach pai $lpaa {
						lappend lpfinal [lindex [split $pai ~] end]
					}
					set lphylotypessansenfants $lpfinal
					# classement alpha
					set lpaa {}
					foreach po $lphylotypesavecenfants {
						#set id $trans($po,id)
						set id $po
						lappend lpaa [format  "%s%s%s" $S(annotationphylotype,$po) ~ $po]
					}
					set lpaa [lsort -dictionary $lpaa]
					set lpfinal {}
					foreach pai $lpaa {
						lappend lpfinal [lindex [split $pai ~] end]
					}
					set lphylotypesavecenfants $lpfinal
					#
					set Pdirect($p) "  $lphylotypessansenfants $lphylotypesavecenfants "
					#
				} else  {
					return
				}
			}
			
			foreach p $Pdirect($p) {
				PhylotypeOrganisation $p
			}
			
		}
	} else  {
		return
	}
}
#
proc recursaa {n npere {laa {}}} {
	global T A IDT PhylotypeDATA
	if {$n == $npere} {
		return $laa
	} else {
		#lappend laa $A($IDT,majoritaire,$n) 
		lappend laa $PhylotypeDATA($IDT,annotation,$n) 
		if {[info exists T(pere,$n)]} {
			recursaa $T(pere,$n) $npere $laa
		} else {
				return $laa
		}
	}
}

################################################################################



################################################################################
# RESET GRAPHIQUES
################################################################################
#
proc ResetAll {} {
	global IDT S
	ResetTreeForegroundColor
	ResetTreeBackgroundColor
	ResetLineWidth
	ResetLeaveForegroundColor
	ResetLeaveAnnotation
	ResetNodeAnnotation
	TransitionQueryResetHi
}
#
proc ResetTreeForegroundColor {} {
	global IDT S
	$S(canvas) itemconfigure Z -fill black 
}
#
proc ResetTreeBackgroundColor {} {
	global IDT S
	$S(canvas) delete TAGBGSUBTREE
}
#
proc ResetLineWidth {} {
	global IDT S
	$S(canvas) itemconfigure Z -width 1 
}
#
proc ResetLeaveForegroundColor {} {
	global IDT S
	$S(canvas) itemconfigure L -fill black 
}
#
proc ResetLeaveAnnotation {} {
	global IDT S
	$S(canvas) delete AnnotMatrix

}
#
proc ResetNodeAnnotation {} {
	global IDT S
	$S(canvas) delete PhylotypeROOT
}
#
proc ResetMapCanvas {} {
	global IDT S
	$S(canvas) delete all
	$S(canvas) configure -scrollregion [$S(canvas) bbox all]
}
proc ResetMapBoards {} {
	global IDT S
	foreach id $S(BoardList,$IDT) {
		set board [format "%s%s" $S($id,path) .canvas.c]
		$board delete all
		$board configure -scrollregion [$board bbox all]
	}
}
################################################################################

################################################################################
# MINI CANVAS
################################################################################
proc PhyNJmini {w} {
	global IDT T S B G
	$w delete S
	$w delete M
	$w delete Slabel
	set deltaX [expr [lindex [$w configure -width] end] -30]
	set fx [expr $deltaX / $T($IDT,xmax)]
	set fy [expr [expr [lindex [$w configure -width] end] -30] / ($T($IDT,nbue)+ 0.00)]
	set S($w,fy) $fy
	set S($w,fx) $fx
	set n 0
	### LEAVES
	foreach i $T($IDT,ue_cod) {
		incr n
		set y [expr $n * $fy ]
		set G($i,x) [expr ($T($IDT,sox,$i) - $T($IDT,dbl,$i)) * $fx]
		set G($i,y) $y
		$w create line [expr $T($IDT,sox,$i) * $fx] $G($i,y) $G($i,x) $G($i,y) \
				-tags "Z $i H H$i S NODE$T($IDT,ctl,$i)  S$T($IDT,dbv,$i) " \
				-fill  black -width 1
	}
	### EDGES
	for {set i [expr $T($IDT,tot) - 1 ]} {$i >= 1} {incr i -1} {
		foreach b $T($IDT,cbg,$i) {
			set G($b,y) [expr ($G($T($IDT,fils1,$b),y) + $G($T($IDT,fils2,$b),y)) / 2.0]
			set G($b,x) [expr $G($T($IDT,fils1,$b),x) - ($T($IDT,dbl,$b) * $fx)]
			# Vertical
			$w create line $G($T($IDT,fils1,$b),x) $G($T($IDT,fils1,$b),y) $G($T($IDT,fils1,$b),x) $G($T($IDT,fils2,$b),y) \
					-tags "Z S V V$T($IDT,dbv,$b) NODE$T($IDT,dbv,$b) S$T($IDT,dbv,$b) CODEV~$b" -fill black  -width 1 \
					-cap round -join round
			# Horizontal
			$w create line $G($T($IDT,fils1,$b),x) $G($b,y) $G($b,x) $G($b,y)  \
					-tags "Z V H$b H$T($IDT,dbv,$b) NODE$T($IDT,dbv,$b) S CODEH~$b" \
					-fill black  -width 1
		}
	}
	### ROOT
	set ch [$w coords H$T($IDT,fils1,0)]
	set cb [$w coords H$T($IDT,fils2,0)]
	set yh [lindex $ch 1]
	set yb [lindex $cb 1]
	set x [lindex $ch 2]
	$w create line $x $yh $x $yb -tags "V0 NODE0 Z S S1000 V" -fill black -width 1 \
			-cap round -join round
	$w lower V
	$w move all 20 20 
}
proc zoomwindow {w color} {
	global IDT S
	set co [$w bbox Z]
	set x1 [lindex $co 0]
	set y1 [lindex $co 1]
	set x2 [lindex $co 2]
	set y2 [lindex $co 3]
	$w delete COLOR$color
	$w create rectangle $x1 $y1 $x2 $y2  -outline $color -tags "brush brushw COLOR$color"
	$w create oval $x2 $y2 [expr $x2 + 10] [expr $y2 + 10] -outline $color -fill $color -tags "brush corner COLOR$color"
	$w bind brushw <1> "MoveBrushStart %W %x %y"
	$w bind corner <1> "MoveCornerStart %W %x %y"
}
# TRANSLATION
proc MoveBrushStart {w x y} {
	global IDT S
	set tags [$w gettags current]
	set S(zoomwindowtag) [lindex $tags [lsearch -glob $tags COLOR*]]
	set S(zoomwindowx) $x
	set S(zoomwindowy) $y
	bind $w <B1-Motion> "MoveBrushTrans %W %x %y"
	bind $w <ButtonRelease-1> "MoveBrushEnd %W %x %y"
}
proc MoveBrushTrans {w x y} {
	global IDT S
	set xx [expr $x - $S(zoomwindowx)]
	set yy [expr $y - $S(zoomwindowy)]
	$w move $S(zoomwindowtag) $xx $yy
	set S(zoomwindowx) $x
	set S(zoomwindowy) $y   
}
proc MoveBrushEnd {w x y} {
	global IDT S
	set color [string range $S(zoomwindowtag) 5 end]
	moveminiversmap $w $S(canvas) $color
	bind $w <B1-Motion> ""
	bind $w <ButtonRelease-1> ""
}
#
proc moveminiversmap {mini c color} {
	global IDT zoomArea
	# mini canvas
	set w1 [winfo width $mini]
	set h1 [winfo height $mini]
	# canvas
	foreach {x1 y1 x2 y2} [$c cget -scrollregion] {break}
	set w2 [expr ($x2 - $x1) * 1.0]
	set h2 [expr ($y2 - $y1) * 1.0]
	#fx
	set fx [expr $w2 / (1.00 * $w1)]
	set fy [expr $h2 / (1.00 * $h1)]
	# brush  
	set co [$mini bbox "COLOR$color && brushw"]  
	set newxcanvas [expr $fx * [lindex $co 0] ]
	set newycanvas [expr $fy * [lindex $co 1] ]
	# le xview moveto travaille sur une proportion
	set xxx [expr $newxcanvas / $w2]
	set yyy [expr $newycanvas / $h2]
	$c xview moveto 0
	$c yview moveto 0
	$c xview moveto $xxx
	$c yview moveto $yyy
}
# ZOOM
proc MoveCornerStart {w x y} {
	global IDT S
	set tags [$w gettags current]
	set S(zoomwindowtag) [lindex $tags [lsearch -glob $tags COLOR*]]
	foreach {x1 y1 x2 y2} [$w coords $S(zoomwindowtag)] break
	set S(zoombrushx) $x1
	set S(zoombrushy) $y1
	bind $w <B1-Motion> "MoveCornerTrans %W %x %y"
	bind $w <ButtonRelease-1> "MoveCornerEnd %W %x %y"
}
proc MoveCornerTrans {w x y} {
	global IDT S
	$w delete $S(zoomwindowtag)
	set color [string range $S(zoomwindowtag) 5 end]
	$w create rectangle $S(zoombrushx) $S(zoombrushy) $x $y -outline $color -tags "brush brushw $S(zoomwindowtag)"
	$w create oval $x $y [expr $x +10] [expr $y+10] -outline $color -fill $color -tags "brush corner $S(zoomwindowtag)"
}

proc MoveCornerEnd {w x y} {
	global IDT S
	# MAP ZOOM
	set color [string range $S(zoomwindowtag) 5 end]
	zoomminiversmap $w $S(canvas) $color
	#
	bind $w <B1-Motion> ""
	bind $w <ButtonRelease-1> ""
}
# garder les memes (x et y ) rapport de tailles entre d'une part
# brush/minicanvas et d'autre part canvasview/canvas
proc zoomminiversmap {mini map color} {
	# brush
	set co [$mini coords "COLOR$color && brushw"]
	set w1 [expr ([lindex $co 2] - [lindex $co 0]) + 40.0]
	set h1 [expr ([lindex $co 3] - [lindex $co 1]) + 40.0]
	# mini canvas
	set w2 [winfo width $mini]
	set h2 [winfo height $mini]
	# canvas
	foreach {x1 y1 x2 y2} [$map cget -scrollregion] {break}
	set w4 [expr $x2 - $x1]
	set h4 [expr $y2 - $y1]
	# view 
	set lxprop [$map xview]
	set w3 [expr ([lindex $lxprop 1] - [lindex $lxprop 0]) * $w4]
	set lyprop [$map yview]
	set h3 [expr ([lindex $lyprop 1] - [lindex $lyprop 0]) * $h4]
	# target
	set w3target [expr ($w1 * $w4) / $w2]
	set h3target [expr ($h1 * $h4) / $h2]
	#
	set fx [expr $w3  / $w3target ]
	set fy [expr $h3  / $h3target ]
	$map scale all 0 0 $fx $fy
	$map configure -scrollregion [$map bbox all]
}  
################################################################################

################################################################################
# TREE NEWICK PARSING
################################################################################
#
proc CreateTree {file} {
	global IDT S T 
	set fid [open $file r]
	set s [read -nonewline $fid]
	set S(NewickOriginal) $s
	close $fid
	# NEWICK cleaning, checking
	regsub -all "\n| " $s "" s
	regsub -all {\[} $s "" s
	regsub -all {:-} $s ":" s
	regsub -all {\]} $s "" s
	regsub -all {;} $s "" s
	# si il n'y a pas de valeurs de longueurs de branches
	# on met des longueurs de branches à 1 partout
	if {[string match *:* $s] == 0} {
		regsub -all "," $s {:1.0,} s
		regsub -all {\)} $s {:1.0)} s
	}
	# data structure
	NewickToDataStructure $s
	#set archiveT [array get T]
	$S(infopath) insert end "> [llength $T($IDT,ue_lab)] strains\n"
	$S(infopath) see current
	Drawtree
}
#
proc NewickToDataStructure {s} {
	global IDT T IDnode
	# INITIALISATION variables
	set T($IDT,xmax) 0 ;# branch length max
	set T($IDT,tot) 0 ;# level max
	set T($IDT,all_cod) {0} ;# codes nodes
	set T($IDT,ue_cod) {} ;# UE - codes
	set T($IDT,ue_lab) {} ;# UE - labels
	
	##### ATTENTION AJOUT
	if {[Dicho $s] == 1} {
			set s [format "%s%s%s" ( $s ):0.0]
	}
		
	### les deux premiers membres et lancement reccurssion
	set tp [string last ")" $s]
	set dt [string range $s 0 $tp]
	# ROOT
	set IDnode 0
	set T($IDT,dbl,$IDnode) 0
	set T($IDT,typ,$IDnode) r
	set T($IDT,allint_cod) $IDnode
	#set T($IDT,vbd,0) $IDnode
	# il faut alle chercher le vrai code 
	set tp [string last ")" $s]
	set dx [string range $s [expr $tp + 1] end]	
	set idroot [string range $dx 0 [expr [string last ":" $dx] - 1]]
	set T($IDT,vbd,$idroot) $IDnode
	set T($IDT,dbv,$IDnode) $idroot
	# FILS1 et FILS2 de ROOT
	set id [BgBdx $dt] ;# id est l'index de la virgule separant deux fils
	set bg [string range $dt [expr $id + 1] [expr [string length $dt] - 2]]
	set bd [string range $dt 1 [expr $id - 1]]
	set IDfils1 [incr IDnode]
	set IDfils2 [incr IDnode]
	set T($IDT,pere,$IDfils1) 0
	set T($IDT,pere,$IDfils2) 0
	set T($IDT,fils1,0) $IDfils1
	set T($IDT,fils2,0) $IDfils2
	### RECCURSSION
	NewickParser $bg $IDfils1 1 0
	NewickParser $bd $IDfils2 1 0
	set T($IDT,nbue) [llength $T($IDT,ue_cod)]
	return
}
# s : newick string ; code : IDnode ; n : node generation , sx : branch length sum
proc NewickParser {s code n sx} {
	global IDT T IDnode
	lappend T($IDT,all_cod) $code
	if {[string match *,* $s]} {
		if {[Dicho $s] == 1} {
			set s [format "%s%s%s" ( $s ):0]
		}
		set tp [string last ")" $s]
		set dt [string range $s 0 $tp]
		set dx [string range $s [expr $tp + 1] end]
		set T($IDT,dbl,$code) [expr abs([string range $dx [expr [string last ":" $dx] + 1] end])]
		set codefromnewick [string range $dx 0 [expr [string last ":" $dx] - 1]]
		if {$codefromnewick == "" } {
			set codefromnewick [incr IDnode]
		}
		set T($IDT,dbv,$code) $codefromnewick
		set T($IDT,vbd,$codefromnewick) $code
		set id [BgBdx $dt]
		set bg [string range $dt [expr $id + 1] [expr [string length $dt] - 2]]
		set bd [string range $dt 1 [expr $id - 1]]
		# ajout de la liste des codes internes
		lappend T($IDT,allint_cod) $code
		lappend T($IDT,cbg,$n) $code
		set T($IDT,typ,$code) n
		set nn [expr $n + 1]
		set ss [expr $sx + $T($IDT,dbl,$code)]
		set IDfils1 [incr IDnode]
		set T($IDT,pere,$IDfils1) $code
		set T($IDT,fils1,$code) $IDfils1
		NewickParser $bg $IDfils1 $nn $ss
		set IDfils2 [incr IDnode]
		set T($IDT,pere,$IDfils2) $code
		set T($IDT,fils2,$code) $IDfils2
		NewickParser $bd $IDfils2 $nn $ss
	} {
		set tp [string last ":" $s]
		set dt [string range $s 0 [expr $tp - 1]]
		set dx [string range $s [expr $tp + 1] end]
		set T($IDT,dbl,$code) [expr abs([string range $dx [expr [string last ":" $dx] + 1] end])]		
		set T($IDT,vbd,$dt) $code
		set T($IDT,dbv,$code) $dt
		lappend T($IDT,ue_lab) $dt
		set T($IDT,ctl,$code) $dt
		set T($IDT,ltc,$dt) $code
		set T($IDT,typ,$code) f
		lappend T($IDT,ue_cod) $code
		set sx [expr $sx + $dx]
		set T($IDT,sox,$code) $sx
		if {$sx >= $T($IDT,xmax)} {set T($IDT,xmax) $sx}
		if {$n >= $T($IDT,tot)} {set T($IDT,tot) $n}
		return
	}
}
# si s est (a,b,c) le parser renvoie bg = a et bd = b,c
# ici peu importe le regroupement il s'agit d'un plateau
# ou seront inseres des longueur de branche de 0 selon le resultat de Dicho
proc BgBdx {s} {
	set i -1
	set id -1
	foreach c [split $s {}] {
		incr id
		switch -exact -- $c {
			( {incr i}
			) {incr i -1}
			, {if {$i == 0} {return $id}}
		}
	}
	return ""
}
# permet la gestion des noeud
proc Dicho {s} {
	set i 0 ; set id 0 ; set r 1
	foreach c [split $s {}] {
		switch -exact -- $c {
			( {incr i}
			) {incr i -1}
		}
		if {$i == 0} {
			set r [string match *,* [string range $s [expr $id + 1] end]]
			break
		}
		incr id
	}
	return $r
}
################################################################################

################################################################################
# CURVES PROBA
################################################################################
#
proc CurvesProbaOptions {m} {
	global S IDT
	# selection map pour mettre à jour la liste des feuilles
	$m delete 0 end
	$m add command -label "Reset path"  -command "CurveProbaResetPath"
}
# sachant une map ID mettre a jour la liste des feuilles
proc ScenarioMakeListBoxSelectMapGo {idmap} {
	global S IDT T
	#$S(strainlistpath) delete 0 end
	#foreach l [lsort -dictionary -increasing $T($idmap,ue_lab)] {
	#	$S(strainlistpath) insert end $l
	#}
}
proc CurveProbaResetPath {} {
	global S IDT T
	$S(canvas) delete TRANSITIONCURVEPATH
}

#
proc ScenarioMakeGraphic {w x y} {
	set tags [$w gettags current]
	set n [string range [lindex $tags [lsearch -glob $tags N*]] 1 end]
	ScenarioMake $n
}
#
proc ScenarioMakeListBox {x y} {
	global IDT T S
	#set n [$S(strainlistpath) get @$x,$y]
	ScenarioMake $T($IDT,ltc,$n)
}
#
proc ScenarioMake {node} {
	global IDT S T A AA
	if {$node != ""} {
		$S(canvas2) delete all
		set S(scenariopere) $node
		ScenarioPere $node
		# CANVAS 1
		TransitionCurveHiTree $node
		#set H [lindex [$S(canvas2) configure -height] end]
		set H 200
		# CANVAS 2
		foreach a [lsort -dictionary $A($IDT,AnnotationList)] {
			set lxy {}
			foreach n [lsort -dictionary -increasing $S(scenariopere)] {
				set co [$S(canvas) coords H$n]
				set x [lindex $co 0]
				#set nodea $T($IDT,dbv,$n)
				#set nodea $n
				#set y [expr $H - ($A($IDT,$nodea,$a) * $H)]
				#set y [expr $H - ($AA($IDT,distribution,$nodea,$a) * $H)]
				set y [expr $H - ($AA($IDT,distribution,$n,$a) * $H)]
				lappend lxy $x $y
			}
			# 
			$S(canvas2) create line $lxy -tags " " -fill $S(ColorAnnotation,$a) -smooth 1 -splinesteps 100
		}
		# lignes verticales marquant les noeuds internes
		foreach n $S(scenariopere) {
			set co [$S(canvas) coords H$n]
			set x1 [lindex $co 0]
			set y1 0
			set y2 $H 
			$S(canvas2) create line $x1 $y1 $x1 $y2 -tags "V$n" -fill grey -dash {2 2}
		}
		# camemberts
		makeMapScenario $S(canvas2) $S(scenariopere)
		# TEST 
		# construction d'un faisceaux de courbres de proba partant du noeud selectionné
		# à chaque feuilles issu de ce noeud en teinte des couleurs d'annotation désaturées
		if {$S(ProbaCurveMode) == 2} {
			if {$T($IDT,typ,$node) != "f"} {
				set feuilles [lsort -dictionary -increasing [Node2Leafs2 $node]]
				foreach f $feuilles {
					MultiScenarioMake $S(canvas2) $node $T($IDT,ltc,$f)
				}
			}
		}
		# scenario majoritaire
		# ScenarioBarMajoritaireEtPupko $node $H
		# alignement
		set x0 [lindex [$S(canvas) bbox Z] 0]
		$S(canvas2) move all $x0 1
		$S(canvas2) configure -scrollregion [$S(canvas2) bbox all]
	}
}
#
proc TransitionCurveHiTree {n} {
	global S T lntransitioncurvepost
	$S(canvas) delete TRANSITIONCURVEPATH
	# de n a root
	TransitionCurveHiTreePreHi $S(scenariopere)
	# de n aux feuilles
	set lntransitioncurvepost {}
	TransitionCurveHiTreePostNode $n
	TransitionCurveHiTreePostHi $lntransitioncurvepost
	$S(canvas) lower TRANSITIONCURVEPATH
	unset lntransitioncurvepost
}
# 
proc TransitionCurveHiTreePreHi {ln} {
	global IDT T S A AA P Z 
	foreach n $ln {
		if {$T($IDT,typ,$n) != "r"} {
			set lxy {}
			set co [$S(canvas) coords H$T($IDT,pere,$n)]
			lappend lxy [lindex $co 0] [lindex $co 1]
			set co [$S(canvas) coords H$n]
			lappend lxy [lindex $co 2] [lindex $co 3] [lindex $co 0] [lindex $co 1]
			$S(canvas) create line $lxy  -width 5 -fill [::tk::Darken #FAFAD2 85] -tag TRANSITIONCURVEPATH
		}
	}
}
#
proc TransitionCurveHiTreePostNode {n} {
	global IDT T S lntransitioncurvepost 
	if {$T($IDT,typ,$n) != "f" } {
		lappend lntransitioncurvepost $n
		TransitionCurveHiTreePostNode $T($IDT,fils1,$n)
		TransitionCurveHiTreePostNode $T($IDT,fils2,$n)
	} else {
		lappend lntransitioncurvepost $n
		return $lntransitioncurvepost
	}
}
#
proc TransitionCurveHiTreePostHi {ln} {
	global IDT T S A AA P Z 
	foreach n $ln {
		set lxy {}
		set co [$S(canvas) coords H$T($IDT,pere,$n)]
		lappend lxy [lindex $co 0] [lindex $co 1]
		set co [$S(canvas) coords H$n]
		lappend lxy [lindex $co 2] [lindex $co 3] [lindex $co 0] [lindex $co 1]
		$S(canvas) create line $lxy  -width 5 -fill [::tk::Darken #FAFAD2 85] -tag TRANSITIONCURVEPATH
	}
}
#
proc MultiScenarioMake {w node f} {
	global IDT S T A AA
	set S(scenariopere) $f
	ScenarioPere2 $node $f
	#puts >>>$S(scenariopere)
	#set H [lindex [$S(canvas2) configure -width] end]
	set H 200
	foreach a [lsort -dictionary $A($IDT,AnnotationList)] {
		set lxy {}
		foreach n [lsort -dictionary -increasing $S(scenariopere)] {
			set co [$S(canvas) coords H$n]
			set x [lindex $co 0]
			#set y [expr $H - ($A($IDT,probapost,$n,$a) * $H)]
			#set y [expr $H - ($A($IDT,$n,$a) * $H)]
			#set nodea $n
			#set nodea $T($IDT,dbv,$n)
			#set y [expr $H - ($A($IDT,$nodea,$a) * $H)]
			#set y [expr $H - ($AA($IDT,distribution,$nodea,$a) * $H)]
			set y [expr $H - ($AA($IDT,distribution,$n,$a) * $H)]
			lappend lxy $x $y
		}
		$w create line $lxy -tags " " -fill $S(ColorAnnotation,$a) -smooth 1 -splinesteps 10
	}
}
#
proc makeMapScenario {w ln} {
	global IDT S T A AA
	$w delete Cammenb
	set delta [expr $S(SurfaceMax) +10]
	foreach n $ln {
		set co [$w coords V$n ]
		set x [lindex $co 0]
		set y [expr ([lindex $co 1]+[lindex $co 3])/2.0]
		# liste annotation proba couleur
		set apc {}
		# attention le code n est interne, l'ID referancant les annotatios est dbv
		#set nodea $T($IDT,dbv,$n)
		set nodea $n
		set sommeproba 0
		set sommeprobaINF 0
		set probaMax 0.0
		foreach {k v} [array get AA $IDT,distribution,$nodea,*] {
			set a [lindex [split $k ,] end]
			#set a [string range $k [expr [string first , $k] +1] end]
			lappend apc [list $a $v $S(ColorAnnotation,$a)]
			set sommeproba [expr $sommeproba + $v]
		}
		lappend apc [list "*" $sommeprobaINF black]
		##### AJOUTER a la liste APC un element AUTRE si la somme n'est pas 1
		#set somme [expr 1 - $sommeproba - $sommeprobaINF]
		set somme [expr 1 - $sommeproba ]
		if {$somme > 0.001} {
			lappend apc [list "?" [rnd $somme 2] grey]
		}
		# dessin
		piechart $w [expr $x - $delta/2.0 ] [expr $y- $delta/2.0 -100] $delta $delta $n $apc
	}
	#
	$w raise Cammenb
	$w itemconfigure CammenbTXT -state hidden
	# affichage des ID nodes en dessous des camemberts
	set bashaut b
	foreach node $ln {
		set co [$w bbox Cammenb$node]
		set x [expr ([lindex $co 0] + [lindex $co 2]) / 2.0]
		if {$bashaut == "b"} {
			set y [expr [lindex $co 3] + 10.0]
			set bashaut h
		} else {
			set y [expr [lindex $co 1] - 10.0]
			set bashaut b
		}
		#$w create text $x $y -text $T($IDT,dbv,$node) -font {Helvetica 10 bold}
		$w create text $x $y -text $node -font {Helvetica 10 bold}

	}
	$w configure -scrollregion [$w bbox all]
	update idletasks
}
#
proc ScenarioBarMajoritaire {w x y} {
	global IDT S T A AA SC
	set tags [$w gettags current]
	set node [string range [lindex $tags [lsearch -glob $tags N*]] 1 end]
	if {$node != ""} {
		catch {unset SC}
		# identification majoritaire
		foreach n $T($IDT,allint_cod) {
			set max 0
			foreach {k v} [array get AA $IDT,distribution,$n,*] {
				set a [lindex [split $k ,] end]
				if {$v > $max} {
					set max $v
					set AA($IDT,majority,$n) $a
				}
			}
		}
		# transfert
		foreach n $T($IDT,allint_cod) {
			set annotation $AA($IDT,majority,$n)
			set SC($IDT,annotation,$n) $annotation
			set SC($IDT,proba,$n) $AA($IDT,distribution,$n,$annotation)
		}
		# feuilles 
		foreach n $T($IDT,ue_cod) {
			foreach var $A($IDT,AnnotationList) {
				# ok 
				if {$AA($IDT,distribution,$n,$var) == 1} {
					set SC($IDT,annotation,$n) $var
					set SC($IDT,proba,$n) 1.00
				}
			}
		}
		ScenarioBarMajoritaireGo $node
	}
}
#
proc ScenarioBarMajoritaireGo {node} {
	global IDT S T SC
	set S(scenariopere) $node ;# init
	ScenarioPere $node ;# S(scenariopere)
	#
	set co [$S(canvas2) bbox all]
	set x [lindex $co 2]
	set y [lindex $co 3]
	set ybar [expr $y + 20]
	# du node à la racine
	foreach n [lsort -dictionary -increasing $S(scenariopere)] {
		set co [$S(canvas) coords H$n]
		set x1 [lindex $co 0]
		set x2 [lindex $co 2]
		set a $SC($IDT,annotation,$n)
		$S(canvas2) create rectangle $x1 $ybar $x2 [expr $ybar + 20] \
			-fill $S(ColorAnnotation,$a) -outline $S(ColorAnnotation,$a) \
			-tags "ScenarioMajority$node"
	}
	#
	set co [$S(canvas2) bbox ScenarioMajority$node]
	set x [lindex $co 2]
	set y [lindex $co 3]
	$S(canvas2) create text [expr $x + 20 ] [expr ([lindex $co 1] + [lindex $co 3]) / 2.0] \
		-text "Majority ($node)" -font {Arial 10 normal} -anchor w
	$S(canvas2) configure -scrollregion [$S(canvas2) bbox all]
}
#
proc ScenarioBarPupko {w x y} {
	global IDT S T A AA SC
	set tags [$w gettags current]
	set node [string range [lindex $tags [lsearch -glob $tags N*]] 1 end]
	if {$node != ""} {
		catch {unset SC}
		# transfert
		foreach n $T($IDT,allint_cod) {
			set SC($IDT,annotation,$n) $AA($IDT,pupko,$n)
			set SC($IDT,proba,$n) 0.99
		}
		# feuilles 
		foreach n $T($IDT,ue_cod) {
			foreach var $A($IDT,AnnotationList) {
				# ok 
				if {$AA($IDT,distribution,$n,$var) == 1} {
					set SC($IDT,annotation,$n) $var
					set SC($IDT,proba,$n) 1.00
				}
			}
		}
		ScenarioBarPupkoGo $node
	}
}
#
proc ScenarioBarPupkoGo {node} {
	global IDT S T A  SC
	### PUPKO
	set S(scenariopere) $node ;# init
	ScenarioPere $node ;# S(scenariopere)
	
	#
	set co [$S(canvas2) bbox all]
	set x [lindex $co 2]
	set y [lindex $co 3]
	set ybar [expr $y + 20]
	#
	foreach n [lsort -dictionary -increasing $S(scenariopere)] {
		set co [$S(canvas) coords H$n]
		set x1 [lindex $co 0]
		set x2 [lindex $co 2]
		set y 300
		set a $SC($IDT,annotation,$n)
		$S(canvas2) create rectangle $x1 $ybar $x2 [expr $ybar + 20] \
			-fill $S(ColorAnnotation,$a) -outline $S(ColorAnnotation,$a) \
			-tags "ScenarioPupko$node"
	}
	#
	set co [$S(canvas2) bbox ScenarioPupko$node]
	set x [lindex $co 2]
	set y [lindex $co 3]
	$S(canvas2) create text [expr $x + 20 ] [expr ([lindex $co 1] + [lindex $co 3]) / 2.0] \
		-text "Joint scenario ($node)" -font {Arial 10 normal} -anchor w
}
# du noeud pointé à la racine
proc ScenarioPere {n} {
	global IDT T S
	set p $T($IDT,pere,$n)
	lappend S(scenariopere) $p
	if {$T($IDT,typ,$p) == "r"} {
		return $S(scenariopere)
	} else {
		ScenarioPere $p
	}
}
# de la feuille au noeud pointé
# n2 : la feuille au départ
proc ScenarioPere2 {n n2} {
	global IDT T S
	set p $T($IDT,pere,$n2)
	if { $p == $n} {
		lappend S(scenariopere) $p
		return $S(scenariopere)
	} else {
		lappend S(scenariopere) $p
		if {$T($IDT,typ,$p) == "r"} {
			return $S(scenariopere)
		} else {
			ScenarioPere2 $n $p
		}
	}
}
################################################################################


################################################################################
# FILTERS 
################################################################################
# selection des annotations de node ayant une proba > s
# ou s = M - ∂/2 avec
# M proba de l'annotation majoritaire 
# ∂ écart type des proba des annotations de node
proc Filter01 {} {
	global IDT S T A AA 
	set nb [llength $A($IDT,AnnotationList)]
	set lni {}
	foreach n $T($IDT,allint_cod) {
		# attention le code n est interne, l'ID referancant les annotatios est dbv
		#set nodea $T($IDT,dbv,$n)
		set somX 0.0
		set somX2 0.0
		set MAX 0.0
		set nbx 0.0
		set la {}
		# [array get A $IDT,$nodea,*]
		foreach {k v} [array get AA $IDT,distribution,$n,*] {
			set somX [expr $somX + $v]
			set somX2 [expr $somX2 + $v * $v]
			if {$v >= $MAX} {set MAX $v}
			set nbx [expr $nbx +1.0]
		}
		if {$nbx != 0} {
			set ET [expr sqrt(abs($somX2 - ($somX * $somX)) / $nbx)]
			set ET2 [expr $ET / 2.0]
			set seuil [expr $MAX - $ET2]
			set seuil [expr $MAX - $ET]
			# [array get A $IDT,$nodea,*]
			foreach {k v} [array get AA $IDT,distribution,$n,*] {
				if {$v >= $seuil} {
					lappend la $k
				}
			}
			if {[llength $la] >= 2} {
				lappend lni $n
			}
		}
	}
	# GRAPHIC
	$S(canvas) delete filtercircleFilter01
	foreach ni $lni {
		set co [$S(canvas) coords H$ni]
		set x [lindex $co 0]
		set y [expr ([lindex $co 1] + [lindex $co 3]) / 2.0 ]
		SymbolDraw $x $y filtercircleFilter01 $ni
	}
	FilterStack
}
# Surligner les nodes ou l'annotation majoritaire 
# et l'annotation Pupko sont différentes 
proc Filter02 {} {
	global IDT S T A AA
	set nb [llength $A($IDT,AnnotationList)]
	set lni {}
	foreach n $T($IDT,allint_cod) {
		# attention le code n est interne, l'ID referancant les annotatios est dbv
		#set nodea $T($IDT,dbv,$n)
		set MAX 0.0
		set MaxA ?
		# majoritaire
		foreach {k v} [array get AA $IDT,distribution,$n,*] {
			if {$v >= $MAX} {
				set MAX $v
				set MaxA [lindex [split $k ,] end]
			}
		}
		# identique à l'annotation pupko ?
		if {$MaxA != $AA($IDT,pupko,$n)} {
			lappend lni $n
		}
	}
	# GRAPHIC
	$S(canvas) delete filtercircleFilter02
	foreach ni $lni {
		set co [$S(canvas) coords H$ni]
		set x [lindex $co 0]
		set y [expr ([lindex $co 1] + [lindex $co 3]) / 2.0 ]
		SymbolDraw $x $y filtercircleFilter02 $ni
	}
	FilterStack
}
# Surligner les nodes ou l'annotation pupko de n 
# est differente de l'annotation pupko du noeud pere de n 
proc Filter03 {} {
	global IDT S T A AA
	set nb [llength $A($IDT,AnnotationList)]
	set lni {}
	foreach n $T($IDT,allint_cod) {
		if {$T($IDT,typ,$n) != "r"} {
			if {$AA($IDT,pupko,$T($IDT,pere,$n)) != $AA($IDT,pupko,$n)} {
				lappend lni $n
			}
		}
	}
	# GRAPHIC
	$S(canvas) delete filtercircleFilter03
	foreach ni $lni {
		set co [$S(canvas) coords H$ni]
		set x [lindex $co 0]
		set y [expr ([lindex $co 1] + [lindex $co 3]) / 2.0 ]
		SymbolDraw $x $y filtercircleFilter03 $ni
	}
	FilterStack
}
# Surligner les nodes ou l'annotation majoritaire de n 
# est differente de l'annotation majoritaire du noeud pere de n 
proc Filter04 {} {
	global IDT S T A AA
	set nb [llength $A($IDT,AnnotationList)]
	set lni {}
	foreach n $T($IDT,allint_cod) {
		# majoritaire
		set MAX 0.0
		set MaxA ?
		foreach {k v} [array get AA $IDT,distribution,$n,*] {
			if {$v >= $MAX} {
				set MAX $v
				set MaxA [lindex [split $k ,] end]
			}
		}
		set X(majo,$n) $MaxA
	}
	foreach n $T($IDT,allint_cod) {
		if {$T($IDT,typ,$n) != "r"} {
			if {$X(majo,$T($IDT,pere,$n)) != $X(majo,$n)} {
				lappend lni $n
			}
		}
	}
	# GRAPHIC
	$S(canvas) delete filtercircleFilter04
	foreach ni $lni {
		set co [$S(canvas) coords H$ni]
		set x [lindex $co 0]
		set y [expr ([lindex $co 1] + [lindex $co 3]) / 2.0 ]
		#puts "NODE $ni $co X $x Y $y"
		SymbolDraw $x $y filtercircleFilter04 $ni
	}
	FilterStack
}
proc SymbolDraw {x y filter ni} {
	global IDT S T A 
	switch -exact $S(filtersymboltype) {
		rectangle {
			$S(canvas) create rectangle \
			[expr $x - $S(filtersymboltypewidth)] [expr $y - $S(filtersymboltypeheight)] \
			[expr $x + $S(filtersymboltypewidth)] [expr $y + $S(filtersymboltypeheight)] \
			 -outline $S(filtersymbolcoloroutline) -tags "filtercircle $filter filtercirclenode=$ni "
		}
		oval {
			$S(canvas) create oval \
			[expr $x - $S(filtersymboltypewidth)] [expr $y - $S(filtersymboltypeheight)] \
			[expr $x + $S(filtersymboltypewidth)] [expr $y + $S(filtersymboltypeheight)] \
			-outline $S(filtersymbolcoloroutline) -tags "filtercircle $filter filtercirclenode=$ni "
		}
		annotxt {
			$S(canvas) create text [expr $x - $S(filtersymboltypewidth)] [expr $y - $S(filtersymboltypeheight)] -text $S(filtersymboltypeannotation) \
			-font $S(tree-font) -fill $S(filtersymbolcoloroutline) -tags "filtercircle $filter filtercirclenode=$ni "
		}
		arrow {
			$S(canvas) create line \
			[expr $x - $S(filtersymboltypewidth)] [expr $y - $S(filtersymboltypeheight)] \
			$x $y \
			-width 2 -fill $S(filtersymbolcoloroutline) -smooth true -splinesteps 100 -arrow last -arrowshape {5 5 3} \
			-tags "filtercircle $filter filtercirclenode=$n "
			}
		}
}

# reset de tous les filtres
proc FilterReset {} {
	global IDT S T A 
	$S(canvas) delete filtercircleFilter01
	$S(canvas) delete filtercircleFilter02
	$S(canvas) delete filtercircleFilter03
	$S(canvas) delete filtercircleFilter04
}
# gestion des empilements des symboles filtre
proc FilterStack {} {
	global IDT S T A
	$S(canvas) lower filtercircleFilter02
}
# panel de definition du symbole de filtre
proc FilterSymbolPanel {} {
	global IDT S T A
	destroy .filtersym
	toplevel .filtersym 
	wm title .filtersym "Filter symbol" 
	
	# TYPE
	set cadre1 [labelframe .filtersym.cadre1 -text "Type"  ]
	$cadre1 configure -labelwidget [MakeFlagLabelFrame $cadre1 "Type" ]
	ttk::radiobutton $cadre1.rec -text "Rectangle" -variable S(filtersymboltype) -value rectangle
	ttk::radiobutton $cadre1.ova -text "Oval" -variable S(filtersymboltype) -value oval
	ttk::radiobutton $cadre1.txt -text "Text" -variable S(filtersymboltype) -value annotxt
	ttk::radiobutton $cadre1.arr -text "Arrow" -variable S(filtersymboltype) -value arrow
	ttk::entry $cadre1.what -width 10 -font {Helvetica 12 normal} -textvariable S(filtersymboltypeannotation)
	grid $cadre1.rec -row 0 -column 0 -sticky news
	grid $cadre1.ova -row 1 -column 0 -sticky news
	grid $cadre1.txt -row 2 -column 0 -sticky news
	grid $cadre1.arr -row 3 -column 0 -sticky news
	grid $cadre1.what -row 2 -column 1 -sticky news
	grid columnconfigure $cadre1 1 -weight 1
	grid rowconfigure $cadre1 1 -weight 1
	
	# SIZE/POSITION & COLOR
	set cadre2 [labelframe .filtersym.cadre2 -text "Size/Position & Color"  ]
	$cadre2 configure -labelwidget [MakeFlagLabelFrame $cadre2 "Size/Position & Color" ]
	ttk::entry $cadre2.width -width 10  -font {Helvetica 12 normal} -textvariable S(filtersymboltypewidth)
	ttk::entry $cadre2.height -width 10  -font {Helvetica 12 normal} -textvariable S(filtersymboltypeheight)
	button $cadre2.color -text "Color" -background $S(filtersymbolcoloroutline) -command filtersymboloutline 
	set S(pathbuttonfiltersymbol) $cadre2.color
	grid $cadre2.width -row 0 -column 1 -sticky news
	grid $cadre2.height -row 1 -column 1 -sticky news
	grid $cadre2.color -row 2 -column 0 -columnspan 2 -sticky news
	grid columnconfigure $cadre2 1 -weight 1
	grid rowconfigure $cadre2 1 -weight 1
	$S(pathbuttonfiltersymbol) configure -highlightbackground $S(filtersymbolcoloroutline)

	# PACK
	pack $cadre1 $cadre2 -side top -expand yes -fill x
}
#
proc filtersymboloutline {} {
	global IDT S T A
	set color [tk_chooseColor]
	if {$color != ""} {
		set S(filtersymbolcoloroutline) $color
		$S(pathbuttonfiltersymbol) configure -highlightbackground $color
	}
}
#

################################################################################

################################################################################
# PIECHART 
################################################################################
#
proc PREmakeMap {{v 1}} {
	global IDT S T A AA
	# on teste sur la racine
	if {[info exists A($IDT,AnnotationList)]} {
		set l [array get AA $IDT,distribution,0,*]
		if {$l != {}} {
			makeMap $S(canvas) $T($IDT,allint_cod) 
			set S(CamembType) p
		}
	}
}
#
proc makeMap {w ln} {
	global IDT S T AA
	$w delete Cammenb
	set lcamemb2Hide {}
	set delta $S(SurfaceMax) 
	foreach n $ln {
		# test des polytomies
		if {[lsearch -exact  $S(lnodessansmap) $n] == -1} {
			#nb de sequences sous-jacentes
			NodeNoToLeEffectif $n
			# center cercle 
			set co [$w coords V$n ]
			set x [lindex $co 0]
			set y [expr ([lindex $co 1]+[lindex $co 3])/2.0]
		# liste annotation proba couleur
		set apc {}
		# attention le code n est interne, l'ID referancant les annotatios est dbv
		#set nodea $T($IDT,dbv,$n)
		set sommeproba 0
		set sommeprobaINF 0
		set probaMax 0.0
		foreach {k v} [array get AA $IDT,distribution,$n,*] {
			set a [lindex [split $k ,] end]
			if {$v >= $probaMax} {set probaMax $v} 
			if {$v >= $S(probathreshold)} {
				set sommeproba [expr $sommeproba + $v]
				lappend apc [list $a $v $S(ColorAnnotation,$a)]
			} else {
				set sommeprobaINF [expr $sommeprobaINF + $v]
			}
		}
		#
		lappend apc [list "*" $sommeprobaINF black]
		if {$probaMax >= $S(camenbdisplayreshold)} {
			lappend lcamemb2Hide $n
		}
		
		# MODE FILTRE SUPPORT
		#if {$S(piechartFilterSupport)} {
		#	if {$T($IDT,$n,support) < $S(piechartFilterSupportTresh)} {
		#		lappend lcamemb2Hide $n
		#	}
		#}
		# MODE FILTRE M-%(user)M
		# on veut voir les piecharts pour les noeud ayant une majo discutable
		# cad ayant plus de 1 annotation dont la proba est > $probaMax - ($probaMax * $S(piechartFilterPercentTresh))
		if {$S(piechartFilterPercent)} {
			set seuil [expr $probaMax - ($probaMax * $S(piechartFilterPercentTresh))]
			set NumberA 0
			foreach {k v} [array get AA $IDT,distribution,$n,*] {
				if {$v >= $seuil} {incr NumberA}
			}
			if {$NumberA == 1} {lappend lcamemb2Hide $n}
		}
		# idem mais cette fois ci avec standard deviation
		#if {$S(piechartFilterDeviation)} {
		#	set somX 0.0
		#	set somX2 0.0
		#	set nbx 0.0
		#	foreach {k v} [array get AA $IDT,distribution,$n,*] {
		#		set somX [expr $somX + $v]
		#		set somX2 [expr $somX2 + $v * $v]
		#		set nbx [expr $nbx +1.0]
		#	}
		#	set ET [expr sqrt(abs($somX2 - ($somX * $somX)) / $nbx)]
		#	set ET2 [expr $ET / 2.0]
		#	set seuil [expr $probaMax - $ET]
		#	set NumberA 0
		#	foreach {k v} [array get AA $IDT,distribution,$n,*] {
		#		if {$v >= $seuil} {incr NumberA}
		#	}
		#	if {$NumberA == 1} {lappend lcamemb2Hide $n}
		#}
		
		# Ajout a la liste APC un element AUTRE si la somme n'est pas 1
		set somme [expr 1 - $sommeproba - $sommeprobaINF]
		if {$somme > 0.001} {
			lappend apc [list "?" [rnd $somme 2] grey]
		}
		# dessin
		piechart $w [expr $x - $delta/2.0] [expr $y- $delta/2.0] $delta $delta $n $apc
		}
	}
	foreach c $lcamemb2Hide {
		$w itemconfigure Cammenb$c -state hidden
	}
	$S(canvas) raise bubblecircle
	$S(canvas) configure -scrollregion [$S(canvas) bbox all]
	update idletasks
}
# eg piechart .c 50 50 150 150 { {A1 0.5 red} {A2 0.5 purple}}
proc piechart {w x y width height node data } {
	global IDT S
	set coords [list $x $y [expr {$x+$width}] [expr {$y+$height}]]
	set xm  [expr {$x+$width/2.}]
	set ym  [expr {$y+$height/2.}]
	set rad [expr {$width/2.+20}]
	set sum 0
	foreach item $data {set sum [expr {$sum + [lindex $item 1]}]}
	set start 270
	foreach item $data {
		foreach {name n color} $item break
		set extent [expr {$n*359.9/$sum}]
		$w create arc $coords -start $start -extent $extent -fill $color -outline white -tags "Cammenb CammenbCIR Cammenb$node MaxLikpiechart=$name"
		set angle [expr {($start-90+$extent/2)/180.*acos(-1)}]
		set tx [expr $xm-$rad*sin($angle)]
		set ty [expr $ym-$rad*cos($angle)]
		set n [rnd $n 2]
		if {$n != 0.0} {
			$w create text $tx $ty -text $n -state $S(piechartdisplaynumber) -tag txt -font $S(annotation-font) -tags "Cammenb CammenbTXT Cammenb$node" -fill $color
			set start [expr $start+$extent]
		}
	}
}
#
proc PieValuesOnOff {} {
	global IDT S
	$S(canvas) itemconfigure CammenbTXT -state $S(piechartdisplaynumber)
} 
#
proc CammenbRemove {} {
	global IDT S
	$S(canvas) delete Cammenb 
} 
#
proc NodeNoToLeEffectif {n} {
	global IDT T SA
	if {[info exists T($IDT,tt,$n)]} {
		return $T($IDT,tt,$n)
	} else {
		if {$T($IDT,typ,$n) == f} {
				set T($IDT,tt,$n) 1
				return $T($IDT,tt,$n)
		} else  {
			set somme1 [NodeNoToLeEffectif $T($IDT,fils1,$n)]
			set somme2 [NodeNoToLeEffectif $T($IDT,fils2,$n)]
			set T($IDT,tt,$n) [expr $somme1 + $somme2]
			return $T($IDT,tt,$n)
		}
	}
}
#
proc ZoomCircle {s} {
	global IDT S T
	#set delta [expr $T($IDT,nbue) / 50.0]
	set delta 2.0
	switch $s {
		-	{
			set S(SurfaceMax) [expr $S(SurfaceMax) -$delta]
		}
		+	{
			set S(SurfaceMax) [expr $S(SurfaceMax) +$delta]
		}
	}
	if {$S(SurfaceMax) < 0.0} {
		set S(SurfaceMax) 0.0
	}
	makeMap $S(canvas) $T($IDT,allint_cod) 
}
################################################################################

################################################################################
# PHYLOTYPE 
################################################################################
#
proc PhylotypeMasterCAP {} {
	global IDT T S PhylotypeDATA A AA
	# init
	catch {unset PhylotypeDATA}
	#
	switch -exact $S(PhyloTypeWhat) {
		majority {PhylotypeDataMajoritaire}
		pupko {PhylotypeDataPupko}
		parsimony {PhylotypeDataparsimony}
	}
	#  feuilles 
	foreach n $T($IDT,ue_cod) {
		foreach var $A($IDT,AnnotationList) {
			if {$AA($IDT,distribution,$n,$var) == 1} {
				set PhylotypeDATA($IDT,annotation,$n) $var 
				set PhylotypeDATA($IDT,proba,$n) 1.00
			}
		}
	}
	# 
	PhylotypeMaster
}
# transition majoritaire vers structure d'analyse
proc PhylotypeDataMajoritaire {} {
	global IDT T S A AA PhylotypeDATA
	# majoritaire
	foreach n $T($IDT,allint_cod) {
		set max 0
		foreach {k v} [array get AA $IDT,distribution,$n,*] {
			set a [lindex [split $k ,] end]
			if {$v > $max} {
				set max $v
				set PhylotypeDATA($IDT,annotation,$n) $a
				set PhylotypeDATA($IDT,proba,$n) $v
			}
		}
	}
}
#
proc PhylotypeDataPupko {} {
	global IDT T S A AA PhylotypeDATA
	# transition pupko vers structure d'analyse
	foreach n $T($IDT,allint_cod) {
		set PhylotypeDATA($IDT,annotation,$n) $AA($IDT,pupko,$n)
		set PhylotypeDATA($IDT,proba,$n) 0.99
	}
}
#
proc PhylotypeDataparsimony {} {
	global IDT T S A AA PhylotypeDATA
	# transition parsimony vers structure d'analyse
	foreach n $T($IDT,allint_cod) {
		set PhylotypeDATA($IDT,annotation,$n) $AA($IDT,parsimony,$n)
		set PhylotypeDATA($IDT,proba,$n) 0.99
	}
}
#
proc PhylotypeMaster {} {
	global IDT T S A Phylotype
	set ALLPhylotypes {}
	# reset Hi
	switch -exact $S(PhyloTypeHi) {
		fg {
			ResetTreeForegroundColor
			$S(canvas) delete PhylotypeROOT
		}
		bg {
			$S(canvas) delete PhylotypeROOT
			$S(canvas) delete TAGBGSUBTREE
		}
	}
	#
	foreach annotation $A($IDT,AnnotationList) {
		set lp [Phylotype $annotation]
		if {$lp != {}} {
			# FILTRAGE
			set lpfiltre [PhylotypeFiltering $lp $annotation]
			# voir calcul pour tous les criteres de chaque phylotype
			foreach pi $lpfiltre {
				# pour la gestion des piles d'affichage
				lappend ALLPhylotypes $pi
				# memoire de l'annotation associée au phylotype
				set S(annotationphylotype,$pi) $annotation
			}
			DisplayPhylotype $S(canvas) $lpfiltre $annotation $S(ColorAnnotation,$annotation)
		} 
	}
	#
	DisplayPhylotypeSTACK $S(canvas) $ALLPhylotypes
	set S(AllPhylotypes) $ALLPhylotypes
	switch -exact $S(PhyloTypeDisplay) {
		map {
			makeMapCluster $S(canvas2) $ALLPhylotypes
		}
	}
	$S(canvas) lower PHYLOTYPE
	$S(canvas) raise bubblecircle
}
#
proc Phylotype {annotation} {
	global IDT T S A PhylotypeDATA
	# liste des nodes INTERNES ayant $annotation ET respectant seuil de proba
	set lnout {}
	foreach n $T($IDT,allint_cod) {
		set a $PhylotypeDATA($IDT,annotation,$n)
		if {$a == $annotation && $PhylotypeDATA($IDT,proba,$n) >= $S(criteria-proba-threshold)} {
			lappend lnout $n
		}
	}
	# IDENTIFICATION
	if {[llength $lnout] != {}} {
		set lp [PhylotypeIdentification $lnout $annotation]
		return $lp
	} else {
		return 0
	}
}
#
proc PhylotypeIdentification {lphylotype annotation} {
	global IDT S T
	# CRITERE DE RECHERCHE en selection
	foreach crit {pt sz df ps sl szdf dv sldv sp} {
		if {$S(criteriacheck,$crit)} {lappend lcriteria $crit}
	}
	#
	### VERIFIER Sl
    if {$lphylotype != {}} {
        if {[lsearch $lcriteria sl] != -1} {
            set lok {}
            foreach node $lphylotype  {
                if {$T($IDT,dbl,$node) >= $S(sl-threshold)} {
                    lappend lok $node
                }
            }
            set lphylotype $lok
        }
    }
	### VERIFIER Sp
    if {$lphylotype != {}} {
        if {[lsearch $lcriteria sp] != -1} {
            set lok {}
            foreach node $lphylotype  {
                # correctif si absence de support
                if {[info exists T($IDT,dbv,$node)]} {
                    if {$T($IDT,dbv,$node) >= $S(sp-threshold)} {
                        lappend lok $node
                    }
                }
            }
            set lphylotype $lok
        }
    }
	### OK Sz
	if {$lphylotype != {}} {
		if {[lsearch $lcriteria sz] != -1} {
			set lok {}
			foreach node $lphylotype  {
				set cs [Criteria-Sz $node $annotation]
				if {$cs >= $S(sz-threshold)} {
					lappend lok $node
				}
			}
			set lphylotype $lok
		}
	}
	### A VERIFIER Df
    if {$lphylotype != {}} {
        if {[lsearch $lcriteria df] != -1} {
            set lok {}
            foreach node $lphylotype  {
                set sc [Criteria-Df $node $annotation]
                if {$sc <= $S(df-threshold)} {
                    lappend lok $node
                }
            }
            set lphylotype $lok
        }
    }
	### A VERIFIER Sz / Df ratio
    if {$lphylotype != {}} {
        if {[lsearch $lcriteria szdf] != -1} {
            set lok {}
            foreach node $lphylotype  {
                set cs [Criteria-Sz $node $annotation]
                set sc [Criteria-Df $node $annotation]
                if {$sc > 0} {
                    set ratio [expr double ($cs) / double ($sc)]
                    if {$ratio >= $S(szdf-threshold)} {
                        lappend lok $node
                    }
                } elseif {$sc == 0}  {
                    lappend lok $node
                }
            }
            set lphylotype $lok
        }
    }
    ### A VERIFIER Ps
    if {$lphylotype != {}} {
        if {[lsearch $lcriteria ps] != -1} {
            set lok {}
            foreach node $lphylotype  {
                set ps [Criteria-Ps $node $annotation]
                if {$ps >= $S(ps-threshold)} {
                    lappend lok $node
                }
            }
            set lphylotype $lok
        }
    }
	### A VEIFIER Dv
    if {$lphylotype != {}} {
        if {[lsearch $lcriteria dv] != -1} {
            set lok {}
            foreach node $lphylotype  {
                set diversity [Criteria-Dv $node $annotation]
                if {$diversity <= $S(dv-threshold) } {
                    lappend lok $node
                }
            }
            set lphylotype $lok
        }
    }
	### A VERIFIER Sl / Dv ratio
    if {$lphylotype != {}} {
        if {[lsearch $lcriteria sldv] != -1} {
            set lok {}
            foreach node $lphylotype  {
                set sep $T($IDT,dbl,$node)
                set div [Criteria-Dv $node $annotation]
                if {$div > 0.0} {
                    set ratio  [expr double($sep) / double($div)]
                    if {$ratio >= $S(sldv-threshold) } {
                        lappend lok $node
                    }
                }
            }
            set lphylotype $lok
        }
    }
	return $lphylotype
}
# Sz
proc Criteria-Sz {node annotation} {
	global IDT T S A
	if {[info exists T($IDT,sz,$annotation,$node)]} {
		return $T($IDT,sz,$annotation,$node)
	} else {
		if {$T($IDT,typ,$node) == f} {
			# feuille
			if {[NodeAnnot $node $annotation]} {
				set T($IDT,sz,$annotation,$node) 1
				return 1
			} else {
				set T($IDT,sz,$annotation,$node) 0
				return 0
			}
		} elseif {[NodeAnnot $node $annotation]} {
			# node est annote avec l'annotation en cours d'analyse
			# on continue
			set e0 $T($IDT,fils1,$node)
			set e1 $T($IDT,fils2,$node)
			set T($IDT,sz,$annotation,$node) [expr [Criteria-Sz $e0 $annotation] + [Criteria-Sz $e1 $annotation]]
			return $T($IDT,sz,$annotation,$node)
		} else  {
			set T($IDT,sz,$annotation,$node) 0
			return 0
		}
	}
}
#
proc Criteria-Df {node annotation} {
	global IDT T S A 
	if {[info exists T($IDT,df,$annotation,$node)]} {
		return $T($IDT,df,$annotation,$node)
	} else  {
		# NodeAnnotDf renvoie 
		# 1 si annotation ambigue  
		# 2 si annotation * (donnee manquante) 
		# 3 si egalité annotation de node et annotation en cours d'analyse
		# 4 si annotation differente
		set a [NodeAnnotDf $node $annotation]
		if {$a == 2} {
			return 0
		} elseif {$a == 1 || $a == 4} {
			return 1
		} else {
			# annotation ok
			if {$T($IDT,typ,$node) == f} {
				return 0
			} else  {
				set T($IDT,df,$annotation,$node) [expr [Criteria-Df $T($IDT,fils1,$node) $annotation] + [Criteria-Df $T($IDT,fils2,$node) $annotation]]
				return $T($IDT,df,$annotation,$node)
			}
		}
	}
}
# Persistence
proc Criteria-Ps {node annotation} {
    global IDT T S A 
    if {[info exists T($IDT,ps,$annotation,$node)]} {
        return $T($IDT,ps,$annotation,$node)
    } else {
    	if {$T($IDT,typ,$node) == f} {
       	 	# on est sur une feuille
       	 	# avant return 1, modification avec l'article
     	 	return 0
    	} elseif  {[NodeAnnot $T($IDT,fils1,$node) $annotation] && [NodeAnnot $T($IDT,fils2,$node) $annotation]} {
        	# annotations ancestrales de fils1 et fils2 de node egalité stricte et on est sur un node interne , on continue
        	# recursion sur les enfants de node
        	set psG [Criteria-Ps $T($IDT,fils1,$node) $annotation]
        	set psD [Criteria-Ps $T($IDT,fils2,$node) $annotation]
        	set T($IDT,ps,$annotation,$node) [expr 1 + min($psG , $psD)]
        	return $T($IDT,ps,$annotation,$node)
    	} else {
        	# annotations ancestrales de fils1 ou fils2 (ou les deux) de node differente de annotation en cours d'analyse
        	return 0
    	}
    }
}
### Diversity
### Dv
proc Criteria-Dv {node annotation} {
    global IDT T S A 
    set sum [Sum $node $annotation]
    set cs [Criteria-Sz $node $annotation]
    if {$cs == 0} {
    } else  {
        return [expr double($sum) / double($cs)]
    }
}
proc Sum {node annotation} {
    global IDT T S A 
    if {[info exists T($IDT,sum,$annotation,$node)]} {
        return $T($IDT,sum,$annotation,$node)
    } else  {
        if {$T($IDT,typ,$node) == f} {
            # feuille
            return 0
        } elseif {[NodeAnnot $node $annotation]  == 0 } {
            # node n'est pas annote avec l'annotation en cours d'analyse
            return 0
        } else  {
            set bg $T($IDT,fils1,$node)
            set cl [Criteria-Sz $bg $annotation]
            set bd $T($IDT,fils2,$node)
            set cr [Criteria-Sz $bd $annotation]
            set T($IDT,sum,$annotation,$node) [expr ($cl *  $T($IDT,dbl,$bg)) + [Sum $bg $annotation] + ($cr * $T($IDT,dbl,$bd)) + [Sum $bd $annotation]]
            return $T($IDT,sum,$annotation,$node)
        }
    }
}
#
proc NodeAnnotDf {n annotation} {
	global IDT PhylotypeDATA
	if {[llength $PhylotypeDATA($IDT,annotation,$n)] != 1} {
		# annotation ambigue
		return 1
	} else  {
		# unique
		set a [lindex $PhylotypeDATA($IDT,annotation,$n) 0]
		if {$a == "*" } {
			return 2
		} elseif {$a == $annotation} {
			return 3
		} else {
			return 4
		}
	}
}
#
proc NodeAnnot {n annotation} {
	global IDT S A T PhylotypeDATA
	if {$S(criteriacheck,pt)} {
		if {$PhylotypeDATA($IDT,annotation,$n) == $annotation && $PhylotypeDATA($IDT,proba,$n) >= $S(criteria-proba-threshold)} {
			return 1
		} else {
			return 0
		}
	} else {
		if {$PhylotypeDATA($IDT,$n) == $annotation} {
			return 1
		} else {
			return 0
		}
	}
}
#
proc PhylotypeFiltering {lp annotation} {
	set lpfiltre {}
	foreach p $lp {
		if {[PhylotypeFiltrage $p $annotation $lp]} {
			lappend lpfiltre $p
		}
	}
	return $lpfiltre
}
# 
proc PhylotypeFiltrage {n a lp} {
	global IDT T
	if {$n == 0} {
		return 1
	} elseif {[Ascendant $T($IDT,pere,$n) $a $lp] == 0} {
		return 1
	} elseif {[Rupture $T($IDT,pere,$n) $a $lp] == 1} {
		return 1
	} else {
		return 0
	}
}
#
proc Ascendant {n a lp} {
	global IDT T
	if {[lsearch $lp $n] != -1 && [NodeAnnot $n $a] == 1} {
		return 1
	} elseif {$n == 0} {
		# racine (peut etre un phylotype annote differement ou pas un phylotype)
		# on conserve  le fils (qui est phylotype)
		return 0
	} else {
		# on continue en remontant
		Ascendant $T($IDT,pere,$n) $a $lp
	}
}
#
proc Rupture {n a lp} {
	global IDT T 
	if {[lsearch $lp $n] != -1 && [NodeAnnot $n $a] == 1} {
		return 0
	} elseif {[NodeAnnot $n $a] == 0} {
		return 1
	} else {
		Rupture $T($IDT,pere,$n) $a $lp
	}
}
#
proc DisplayPhylotype {w lp annotation color} {
	global IDT T S B
	# color
	switch -exact $S(PhyloTypeHi) {
		fg {
			set color  $S(ColorAnnotation,$annotation)
		}
		bg {
			lassign [rgbToHsv {*}[scan $S(ColorAnnotation,$annotation) "#%2x%2x%2x"]] h s v
			set NEWrgbBGtree [hsvToRgb $h $S(ColorSaturation,BGtree) $S(ColorValue,BGtree)]
			set r [lindex $NEWrgbBGtree 0] 
			set g [lindex $NEWrgbBGtree 1]
			set b [lindex $NEWrgbBGtree 2]
			set color [format "#%2X%2X%2X" $r $g $b] ; # passage en hexadecimal
		}
	}
	#
	set lnodes [lsort -increasing $lp]
	foreach n $lnodes {
		switch -exact $S(PhyloTypeHi) {
			fg {
				set c [$w coords H$n]
				set x1 [lindex $c 0] ; set y1 [lindex $c 1]
				set x2 [lindex $c 2] ; set y2 [lindex $c 3]
				set x [expr ($x1 + $x2) / 2.0]
				set y [expr ($y1 + $y2) / 2.0]
				$w create rectangle [expr $x -3] [expr $y -3] [expr $x +3] [expr $y +3] -outline $color -fill $color -tag "PhylotypeROOT"
				$w create text $x [expr $y +8] -text "$n : $annotation" -font {Arial 8 normal} -fill black -tag "PhylotypeROOT"
				FgPhylotype $w $n $annotation $color
			}
			bg {
			
				set c [$w coords H$n]
				set x1 [lindex $c 0] ; set y1 [lindex $c 1]
				set x2 [lindex $c 2] ; set y2 [lindex $c 3]
				set x [expr ($x1 + $x2) / 2.0]
				set y [expr ($y1 + $y2) / 2.0]
				$w create rectangle [expr $x -3] [expr $y -3] [expr $x +3] [expr $y +3] -outline black -fill $color -tag "PhylotypeROOT"
				$w create text $x [expr $y +8] -text "$n : $annotation" -font {Arial 8 normal} -fill black -tag "PhylotypeROOT"
				BgPhylotype $w $n $annotation $color
			}
		}
	}
}
#
proc DisplayPhylotypeSTACK {w lp } {
	set lnodes [lsort -increasing $lp]
	foreach p $lnodes {
		$w raise PHYLOTYPE$p
	}
	$w raise Z
	$w raise Cammenb
	$w raise PhylotypeROOT
	$w configure -scrollregion [$w bbox all]
	update idletasks
}
#
proc FgPhylotype {w n a color} {
	global S T TC IDT
	if {[NodeAnnot $n $a]} {
		# Hi
		$w itemconfigure H$n -fill $color
		$w itemconfigure V$n -fill $color
		if {$T($IDT,typ,$n) != "f"} {
			FgPhylotype $w $T($IDT,fils1,$n) $a $color
			FgPhylotype $w $T($IDT,fils2,$n) $a $color
		} else {
			return
		}
	}
}
#
proc BgPhylotype {w n a color} {
	global IDT S T B
	set B($IDT,1,$n) {}; set B($IDT,2,$n) {}; set B($IDT,3,$n) {}
	TreeBackgroundMakeColorOutlineDPhylotype $w $n $n $a
	TreeBackgroundMakeColorOutlineGPhylotype $w $n $n $a
	TreeBackgroundMakeColorOutlineEPhylotype $w $n $a $n 
	$w create polygon "$B($IDT,1,$n) $B($IDT,3,$n) $B($IDT,2,$n)"	\
		-fill $color -outline $color -tags "BGtree=$a SHRINK TAGBGSUBTREE PHYLOTYPE$n PHYLOTYPE BGtree=$a" 
	$w create polygon "$B($IDT,1,$n) $B($IDT,3,$n) $B($IDT,2,$n)"	\
		-fill $color -outline $color -tags "BGtree=$a SHRINK TAGBGSUBTREE" -smooth 1 -splinesteps 100
	$w lower TAGBGSUBTREE
}
# 
proc TreeBackgroundMakeColorOutlineDPhylotype {w p n a} {
	global B T IDT
	if {[NodeAnnot $n $a]} {
		if {$T($IDT,typ,$n) == "f"} {
			set c [$w coords H$n]
			set x1 [lindex $c 0]
			set y1 [lindex $c 1]
			set x2 [lindex $c 2]
			set y2 [lindex $c 3]
			set B($IDT,2,$p) [concat " $x2 $y2 " $B($IDT,2,$p)]
			return
		} else {
			set c [$w coords H$n]
			set x1 [lindex $c 0]
			set y1 [lindex $c 1]
			set x2 [lindex $c 2]
			set y2 [lindex $c 3]
			set B($IDT,2,$p) [concat " $x1 $y1 $x2 $y2 " $B($IDT,2,$p)]
			set n1 $T($IDT,fils1,$n)
			set n2 $T($IDT,fils2,$n)
			set c1 [$w coords H$n1]
			set c2 [$w coords H$n2]
			set y1 [lindex $c1 1]
			set y2 [lindex $c2 1]
			if {$y1 > $y2} {
				TreeBackgroundMakeColorOutlineDPhylotype $w $p $T($IDT,fils2,$n) $a
			} else {
				TreeBackgroundMakeColorOutlineDPhylotype $w $p $T($IDT,fils1,$n) $a
			}
		}
	} else {
		return
	}
}
# 
proc TreeBackgroundMakeColorOutlineGPhylotype {w p n a} {
	global B T IDT
	if {[NodeAnnot $n $a]} {
		if {$T($IDT,typ,$n) == "f"} {
			set c [$w coords H$n]
			set x1 [lindex $c 0]
			set y1 [lindex $c 1]
			set x2 [lindex $c 2]
			set y2 [lindex $c 3]
			lappend B($IDT,1,$p) $x2 $y2  
			return
		} else {
			set c [$w coords H$n]
			set x1 [lindex $c 0]
			set y1 [lindex $c 1]
			set x2 [lindex $c 2]
			set y2 [lindex $c 3]
			lappend B($IDT,1,$p) $x2 $y2 $x1 $y1
			set n1 $T($IDT,fils1,$n)
			set n2 $T($IDT,fils2,$n)
			set c1 [$w coords H$n1]
			set c2 [$w coords H$n2]
			set y1 [lindex $c1 1]
			set y2 [lindex $c2 1]
			if {$y1 > $y2} {
				TreeBackgroundMakeColorOutlineGPhylotype $w $p $T($IDT,fils1,$n) $a
			} else {
				TreeBackgroundMakeColorOutlineGPhylotype $w $p $T($IDT,fils2,$n) $a
			}
		}
	} else {
		return
	}
}
# 
proc TreeBackgroundMakeColorOutlineEPhylotype {w np annotation n} {
	global T B IDT PhylotypeDATA
	if {[NodeAnnot $n $annotation]} {
		if {$T($IDT,typ,$n) == f} {
			# on prend le plus grand x et on arrete
			set c [$w coords H$n]
			set x1 [lindex $c 0]
			set y1 [lindex $c 1]
			set x2 [lindex $c 2]
			set y2 [lindex $c 3]
			set y [expr ($y1 + $y2) / 2.0]
			if {$x1 > $x2} {
				lappend B($IDT,3,$np) $x1 $y 
			} else {
				lappend B($IDT,3,$np) $x2 $y
			}
			return
		} else {
			# on relance mais vers le bas en premier
			set n1 $T($IDT,fils1,$n)
			set n2 $T($IDT,fils2,$n)
			set c1 [$w coords H$n1]
			set c2 [$w coords H$n2]
			set y1 [lindex $c1 1]
			set y2 [lindex $c2 1]
			if {$y1 > $y2} {
				# n1 est en bas
					TreeBackgroundMakeColorOutlineEPhylotype $w $np $annotation $T($IDT,fils1,$n)
					TreeBackgroundMakeColorOutlineEPhylotype $w $np $annotation $T($IDT,fils2,$n)
			} else {
				# n2 est en bas
					TreeBackgroundMakeColorOutlineEPhylotype $w $np $annotation $T($IDT,fils2,$n)
					TreeBackgroundMakeColorOutlineEPhylotype $w $np $annotation $T($IDT,fils1,$n)
			}
		}
	} else {
		#set p [PhylotypeChecherPereAnnote $T($IDT,pere,$n) $annotation]
		set p $T($IDT,pere,$n)
		set c [$w coords H$p]
		set x1 [lindex $c 0]
		set y1 [lindex $c 1]
		set x2 [lindex $c 2]
		set y2 [lindex $c 3]
		set y [expr ($y1 + $y2) / 2.0]
		if {$x1 > $x2} {
			lappend B($IDT,3,$np) $x2 $y 
		} else {
			lappend B($IDT,3,$np) $x1 $y
		}
		return
	}
}
#
proc PhylotypeChecherPereAnnote {n annotation} {
	global T S IDT PhylotypeDATA
	#renvoyer p si ses deux fils sont annoté annotation
	if {$T($IDT,typ,$n) != "r"} {
		set p $T($IDT,pere,$n)
		set f1 $T($IDT,fils1,$p)
		set f2 $T($IDT,fils2,$p)
		if {$PhylotypeDATA($IDT,annotation,$f1) == $annotation && $PhylotypeDATA($IDT,annotation,$f2) == $annotation } {
			set node $p
		} else {
			set node [PhylotypeChecherPereAnnote $p $annotation]
		}
		return $node
	} else {
		return 0
	}
}
#
proc TreeBackgroundMakeColorOutlineEPhylotype {w np annotation n} {
	global T B IDT
		if {$T($IDT,typ,$n) == f} {
			if {[NodeAnnot $n $annotation]} {
				set c [$w coords H$n]
				set x1 [lindex $c 0]
				set y1 [lindex $c 1]
				set x2 [lindex $c 2]
				set y2 [lindex $c 3]
				set y [expr ($y1 + $y2) / 2.0]
				if {$x1 > $x2} {lappend B($IDT,3,$np) $x1 $y } else {lappend B($IDT,3,$np) $x2 $y}
			}
			return
		} else {
			# on relance mais vers le bas en premier
			if {[NodeAnnot $T($IDT,fils1,$n) $annotation] && [NodeAnnot $T($IDT,fils2,$n) $annotation]} {
				set n1 $T($IDT,fils1,$n)
				set n2 $T($IDT,fils2,$n)
				set c1 [$w coords H$n1]
				set c2 [$w coords H$n2]
				set y1 [lindex $c1 1]
				set y2 [lindex $c2 1]
				if {$y1 > $y2} {
					# n1 est en bas
					TreeBackgroundMakeColorOutlineEPhylotype $w $np $annotation $T($IDT,fils1,$n)
					TreeBackgroundMakeColorOutlineEPhylotype $w $np $annotation $T($IDT,fils2,$n)
				} else {
					# n2 est en bas
					TreeBackgroundMakeColorOutlineEPhylotype $w $np $annotation $T($IDT,fils2,$n)
					TreeBackgroundMakeColorOutlineEPhylotype $w $np $annotation $T($IDT,fils1,$n)
				}
			} elseif {[NodeAnnot $T($IDT,fils1,$n) $annotation]} {
					TreeBackgroundMakeColorOutlineEPhylotype $w $np $annotation $T($IDT,fils1,$n)
			} elseif {[NodeAnnot $T($IDT,fils2,$n) $annotation]} {
					TreeBackgroundMakeColorOutlineEPhylotype $w $np $annotation $T($IDT,fils2,$n)
			} else {
				return
			}
		}
}
#
proc DisplayPhylotypeBG {w n annotation color } {
	global IDT S T B
	set co [$w coords H$n]
	if {$co != ""} {
		set x1 [lindex $co 0]
		set y1 [lindex $co 1]
		set x2 [lindex $co 2]
		set y2 [lindex $co 3]
		set x [expr ($x1 + $x2) / 2.0]
		set y [expr ($y1 + $y2) / 2.0]
		set TAG [format "%s%s" PHYLOTYPE $n]
		set B(1,$n) {}; set B(2,$n) {}; set B(3,$n) {}
		GeneralSubTreeContourHaut $w $n $annotation $n
		GeneralSubTreeContourBas $w $n $annotation $n
		GeneralSubTreeContourExtrem $w $n $annotation $n
		$w create polygon "$x $y $B(1,$n) $B(3,$n) $B(2,$n)  $x $y" \
				-fill $color -outline $color -tags "$TAG PHYLOTYPE TAGBGSUBTREE BGtree=$annotation" \
				-smooth 1 -splinesteps 100 
	}
}
#
proc GeneralSubTreeContourHautORIGINAL {w p annotation n} {
	global IDT B T SA
	if {[NodeAnnot $n $annotation]} {
		if {$T($IDT,typ,$n) == f} {
			set c [$w coords H$n]
			set x1 [lindex $c 0]
			set y1 [lindex $c 1]
			set x2 [lindex $c 2]
			set y2 [lindex $c 3]
			if {$x1 < $x2} {
				set B(2,$p) [concat " $x1 $y1 " $B(2,$p)]
			} else  {
				set B(2,$p) [concat " $x2 $y2 " $B(2,$p)]
			}
			return
		} else  {
			set c [$w coords H$n]
			set x1 [lindex $c 0]
			set y1 [lindex $c 1]
			set x2 [lindex $c 2]
			set y2 [lindex $c 3]
			# attention orientation
			if {$x2 >= $x1} {
				set B(2,$p) [concat " $x2 $y2 $x1 $y1" $B(2,$p)]
			} else  {
				set B(2,$p) [concat " $x1 $y1 $x2 $y2" $B(2,$p)]
			}
			# attention on continue mais sur le bras haut seuleuement
			set n1 $T($IDT,fils1,$n)
			set n2 $T($IDT,fils2,$n)
			set c1 [$w coords H$n1]
			set c2 [$w coords H$n2]
			set y1 [lindex $c1 1]
			set y2 [lindex $c2 1]
			if {$y1 <= $y2} {
				# n1 est en haut
				GeneralSubTreeContourHaut $w $p $annotation $n1
				#return
			} else  {
				# n2 est en haut
				GeneralSubTreeContourHaut $w $p $annotation $n2
				#return
			}
		}
	} else {
		return
	}
}
#
proc GeneralSubTreeContourHaut {w p annotation n} {
	global IDT B T SA
	if {[NodeAnnot $n $annotation]} {
		if {$T($IDT,typ,$n) == f} {
			set c [$w coords H$n]
			set x1 [lindex $c 0]
			set y1 [lindex $c 1]
			set x2 [lindex $c 2]
			set y2 [lindex $c 3]
			if {$x1 < $x2} {
				set B(2,$p) [concat " $x1 $y1 " $B(2,$p)]
			} else  {
				set B(2,$p) [concat " $x2 $y2 " $B(2,$p)]
			}
			return
		} else  {
			# recherche nfils (en bas)
			set n1 $T($IDT,fils1,$n)
			set n2 $T($IDT,fils2,$n)
			set c1 [$w coords H$n1]
			set c2 [$w coords H$n2]
			set y1 [lindex $c1 1]
			set y2 [lindex $c2 1]
			if {$y1 <= $y2} {
				# n1 est fils
				set nfils $n1
			} else  {
				# n2 est fils
				set nfils $n2
			}
			set c [$w coords H$n]
			set x1 [lindex $c 0]
			set y1 [lindex $c 1]
			set x2 [lindex $c 2]
			set y2 [lindex $c 3]
			if {[NodeAnnot $nfils $annotation]} {
				# orientation
				if {$x2 >= $x1} {
					set B(2,$p) [concat " $x2 $y2 $x1 $y1" $B(2,$p)]
				} else  {
					set B(2,$p) [concat " $x1 $y1 $x2 $y2" $B(2,$p)]
				}
			} else {
				if {$x2 >= $x1} {
					set B(2,$p) [concat " $x2 $y2" $B(2,$p)]
				} else  {
					set B(2,$p) [concat " $x1 $y1" $B(2,$p)]
				}
			}
			GeneralSubTreeContourHaut $w $p $annotation $nfils
		}
	} else {
		return
	}
}
#
proc GeneralSubTreeContourBasORIGINAL {w p annotation n} {
	global IDT B T SA
	if {[NodeAnnot $n $annotation]} {
		if {$T($IDT,typ,$n) == f} {
			set c [$w coords H$n]
			set x1 [lindex $c 0]
			set y1 [lindex $c 1]
			set x2 [lindex $c 2]
			set y2 [lindex $c 3]
			if {$x1 < $x2} {
				lappend B(1,$p) $x1 $y1
			} else {
				lappend B(1,$p) $x2 $y2
			}
			return
		} else {
			set c [$w coords H$n]
			set x1 [lindex $c 0]
			set y1 [lindex $c 1]
			set x2 [lindex $c 2]
			set y2 [lindex $c 3]
			# orientation
			if {$x1 < $x2} {
				lappend B(1,$p) $x1 $y1 $x2 $y2 $x2 $y2
			} else {
				lappend B(1,$p) $x2 $y2 $x1 $y1 $x1 $y1
			}
			# attention on continue mais sur le bras bas seuleuement
			set n1 $T($IDT,fils1,$n)
			set n2 $T($IDT,fils2,$n)
			set c1 [$w coords H$n1]
			set c2 [$w coords H$n2]
			set y1 [lindex $c1 1]
			set y2 [lindex $c2 1]
			if {$y1 <= $y2} {
				# n2 est en bas
				GeneralSubTreeContourBas $w $p $annotation $n2
			} else  {
				# n1 est en bas
				GeneralSubTreeContourBas $w $p $annotation $n1
			}
		}
	} else {
		return
	}
}
#
proc GeneralSubTreeContourBas {w p annotation n} {
	global IDT B T SA
	if {[NodeAnnot $n $annotation]} {
		if {$T($IDT,typ,$n) == f} {
			return
		} else {
			# recherche nfils (en bas)
			set n1 $T($IDT,fils1,$n)
			set n2 $T($IDT,fils2,$n)
			set c1 [$w coords H$n1]
			set c2 [$w coords H$n2]
			set y1 [lindex $c1 1]
			set y2 [lindex $c2 1]
			if {$y1 <= $y2} {
				# n2 est fils
				set nfils $n2
			} else  {
				# n1 est fils
				set nfils $n1
			}
			set c [$w coords H$n]
			set x1 [lindex $c 0]
			set y1 [lindex $c 1]
			set x2 [lindex $c 2]
			set y2 [lindex $c 3]
			if {[NodeAnnot $nfils $annotation]} {
				# orientation
				if {$x1 < $x2} {
					lappend B(1,$p) $x1 $y1 $x2 $y2 
				} else {
					lappend B(1,$p) $x2 $y2 $x1 $y1 
				}
			} else {
				# orientation
				if {$x1 < $x2} {
					lappend B(1,$p) $x1 $y1  
				} else {
					lappend B(1,$p) $x2 $y2  
				}
			}
			# on continue
			GeneralSubTreeContourBas $w $p $annotation $nfils
		}
	} else {
		return
	}
}
# Contour extremites
proc GeneralSubTreeContourExtrem {w np annotation n} {
	global IDT T SA B C
	if {[NodeAnnot $n $annotation]} {
		if {$T($IDT,typ,$n) == f} {
			# on prend le plus grand x et on arrete
			set c [$w coords H$n]
			set x1 [lindex $c 0]
			set y1 [lindex $c 1]
			set x2 [lindex $c 2]
			set y2 [lindex $c 3]
			set y [expr ($y1 + $y2) / 2.0]
			set x [expr ($x1 + $x2) / 2.0]
			#lappend B(3,$np) $x $y $x $y
			lappend B(3,$np) $x $y 
			return
		} else {
			# on relance mais vers le bas en premier
			set n1 $T($IDT,fils1,$n)
			set n2 $T($IDT,fils2,$n)
			set c1 [$w coords H$n1]
			set c2 [$w coords H$n2]
			set y1 [lindex $c1 1]
			set y2 [lindex $c2 1]
			if {$y1 > $y2} {
				# n1 est en bas
				GeneralSubTreeContourExtrem $w $np $annotation $T($IDT,fils1,$n)
				GeneralSubTreeContourExtrem $w $np $annotation $T($IDT,fils2,$n)
			} else  {
				# n2 est en bas
				GeneralSubTreeContourExtrem $w $np $annotation $T($IDT,fils2,$n)
				GeneralSubTreeContourExtrem $w $np $annotation $T($IDT,fils1,$n)
			}
		}
	} else {
		#feuille ou interne pas on prend le plus petit x et on arrete
		#set c [$w coords H$n]
		#set x1 [lindex $c 0]
		#set y1 [lindex $c 1]
		#set x2 [lindex $c 2]
		#set y2 [lindex $c 3]
		#set y [expr ($y1 + $y2) / 2.0]
		#if {$x1 < $x2} {
		#	lappend B(3,$np) $x1 $y 
		#} else  {
		#	lappend B(3,$np) $x2 $y
		#}
		return
	}
}
# faire un algo qui remonte avec une seule liste de coords
proc PhylotypeContour {w ndepart annotation n } {
	global IDT
	if {[NodeAnnot $n $annotation]} {
		set c [$w coords H$n]
		set x [lindex $c 0]
		set y [lindex $c 1]
		lappend B($IDT,$np) $x $y 
	} else {
		PhylotypeContour $w  ndepart annotation n
	}
}
# exportation map phylotype en google map
# Voir version avec coordinates par seq (moyenne a faire pour les phylotypes)
# pour l'instant scenario majoritaire
# fleches selon hierarchie des phylotypes
# pb des fleches , epaisseur selon support, couleur selon annotation
# pb des scenarios multiples
			#<Placemark>
			#<styleUrl>#P1</styleUrl>
			#<MultiGeometry>
			#<LineString >
			#<tessellate>1</tessellate>
			#<altitudeMode>relativeToGround</altitudeMode>
			#<coordinates>
			#	3,50,40000 4,50,50000 5,50,40000 6,50,10000
			#</coordinates>
			#</LineString>
			#<Polygon>
			#	<altitudeMode>relativeToGround</altitudeMode>
			#	<outerBoundaryIs>
			#		<LinearRing>
			#			<coordinates>
			#			2,50,10000 3,50.5,40000 3,49.5,40000 2,50,10000
			#			</coordinates>
			#		</LinearRing>
			#	</outerBoundaryIs>
			#</Polygon>
			#</MultiGeometry>	
			#</Placemark>
proc PhylotypeMap {} {
	global IDT T S K
	# Load coordinates
	set file [tk_getOpenFile -initialdir $S(pathuser) -title "Load Coordinates... "]
	if {$file != "" } {
		if [catch {open $file r} fid] {
			return 0
		} else {
			while {[eof $fid] != 1} {
				set row  [gets $fid]
				regsub -all "," $row " " row
				set annotation [lindex $row 0]
				set longitude  [lindex $row 1]
				set latitude [lindex $row 2]
				set K($annotation,latitude) $latitude
				set K($annotation,longitude) $longitude
			}	
			#
			set lp [DelRep [lsort -increasing $S(AllPhylotypes)]]
			foreach p $lp {
				set K($p,annotation) $S(annotationphylotype,$p)
				set K($p,color) $S(ColorAnnotation,$K($p,annotation))
			}	
			# KML file
			set file 
			set fid [open "testPUU.kml" w ]
			# CAP
			puts $fid "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
			puts $fid "<kml xmlns=\"http://www.opengis.net/kml/2.2\" xmlns:gx=\"http://www.google.com/kml/ext/2.2\"  xmlns:kml=\"http://www.opengis.net/kml/2.2\"  xmlns:atom=\"http://www.w3.org/2005/Atom\">"
			puts $fid "<Document>"
			foreach p $lp {
				# STYLE
				set color [format "%s%s" ff [string range $K($p,color) 1 end]]
				#puts $color
				puts $fid "<Style id=\"P$p\">"
				# texte
				puts $fid "<BalloonStyle>"
				puts $fid "<bgColor>ffffffff</bgColor>"
				puts $fid "<text>"
				puts $fid "<!\[CDATA\["
				puts $fid "<font color=\"$color\" face=\"Courier\">P$p</font>"
				puts $fid "\]\]>"
				puts $fid $K($p,annotation)
				puts $fid [ListStrain $p $K($p,annotation)]
				puts $fid "</text>"
				puts $fid "</BalloonStyle>" 
				# line
				puts $fid "<LineStyle>"
				puts $fid "<color>$color</color>"
				puts $fid "<width>2</width>"
				puts $fid "</LineStyle>" 
				puts $fid "</Style>"
			}
			foreach p $lp {
				# MARK
				puts $fid "<Placemark>"
				puts $fid "<name>P$p</name>"
				puts $fid "<description>P$p</description>"
				puts $fid "<styleUrl>#P$p</styleUrl>"
				puts $fid "<Point>"
				puts $fid "<coordinates>"
				puts $fid [format "%s%s%s%s%s" $K($K($p,annotation),latitude) , $K($K($p,annotation),longitude) , 5000]
				puts $fid "</coordinates>"
				puts $fid "</Point>"
				puts $fid "</Placemark>"
			}
			# LINE reliant les phylotypes Pere / Fils
			global IDT P GenPhy
			PhylotypeHierarchy $lp
			# altitude selon generation
			set nb [llength $GenPhy(all)]
			set unitaltitutde [expr 1000000.0 / $nb ]
			foreach p $lp {
				if { $P($p) != ""} {
					foreach pfils $P($p) {
					
						puts $fid "<Placemark>"
						puts $fid "<styleUrl>#P$p</styleUrl>"
						puts $fid "<LineString>"
						puts $fid "<tessellate>1</tessellate>"
						puts $fid "<extrude>0</extrude>"
						puts $fid "<altitudeMode>relativeToGround</altitudeMode>"
						puts $fid "<coordinates>"
					
						set interLAT [expr ($K($K($p,annotation),latitude) + $K($K($pfils,annotation),latitude)) / 2.0]
						set interLON [expr ($K($K($p,annotation),longitude) + $K($K($pfils,annotation),longitude)) / 2.0]
						set a [expr ($nb - $GenPhy($p)+1) * $unitaltitutde]
						
						set interLAT [expr ($K($K($p,annotation),latitude) + $K($K($pfils,annotation),latitude)) / 2.0]
						set interLON [expr ($K($K($p,annotation),longitude) + $K($K($pfils,annotation),longitude)) / 2.0]
						set a [expr ($nb - $GenPhy($p)+1) * $unitaltitutde]
						
						# depart
						puts $fid [format "%s%s%s%s%s" $K($K($p,annotation),latitude) , $K($K($p,annotation),longitude) , 0 ]
						
						# preinter
						set PREinterLAT [expr ($K($K($p,annotation),latitude) + $interLAT) / 2.0]
						set PREinterLON [expr ($K($K($p,annotation),longitude) + $interLON) / 2.0]
						set PREa [expr $a - 100000]
						puts $fid [format "%s%s%s%s%s" $PREinterLAT , $PREinterLON , $PREa]
						
						#milieu (plus haute altitude relative a la generation du phylotype)
						puts $fid [format "%s%s%s%s%s" $interLAT , $interLON , $a]
						
						# postinter
						set POSTinterLAT [expr ($interLAT + $K($K($pfils,annotation),latitude)) / 2.0]
						set POSTinterLON [expr ($interLON + $K($K($pfils,annotation),longitude)) / 2.0]
						set POSTa [expr $a - 100000]
						puts $fid [format "%s%s%s%s%s" $POSTinterLAT , $POSTinterLON , $POSTa]

						# arrivee
						puts $fid [format "%s%s%s%s%s" $K($K($pfils,annotation),latitude) , $K($K($pfils,annotation),longitude) , 0]
						
						puts $fid "</coordinates>"
						puts $fid "</LineString>"
						puts $fid "</Placemark>"
					}
				}
			}
			# END
			puts $fid "</Document>"
			puts $fid "</kml>"
			close $fid
		}
	}
}
# List des feuilles d'un phylotype
proc ListStrain {node annotation} {
	global IDT T 
	if {$T($IDT,typ,$node) == f} {
		if {[NodeAnnot $node $annotation]} {
			return $T($IDT,ctl,$node)
			return
		} else  {
			return
		}
	} elseif {[NodeAnnot $node $annotation]} {
		set listefeuilles [ListStrain $T($IDT,fils1,$node) $annotation]
		foreach f [ListStrain $T($IDT,fils2,$node) $annotation] {
		   append listefeuilles " $f "
		}
		return $listefeuilles
	}
}
proc PhylotypeHierarchy {lp} {
	global IDT T Pere P GenPhy
	set lp [lsort -increasing $lp]
	foreach p $lp {
		set P($p) [SearchDirectPhylotype $T($IDT,fils1,$p) $lp]
		append P($p) [SearchDirectPhylotype $T($IDT,fils2,$p) $lp]
	}
	# les PERE
	foreach {k v} [array get P *] {
		foreach n $v {
			set Pere($n) $k
		}
	}
	# generation sur les phylotypes = nb de parents jusqu'a la racine
	foreach p $lp {
		set gg [GenerationPhylotype  $p 1]
		set GenPhy($p) $gg
		if {[info exists GenPhy(gen,$gg)]} {
			lappend GenPhy(gen,$gg) $p
		} else  {
			set GenPhy(gen,$gg) $p
			lappend GenPhy(all) $gg
		}
	}
}
proc SearchDirectPhylotype {node lp } {
	global IDT T
	if {$T($IDT,typ,$node) == f} {
		return
	} else {
		if {[lsearch -exact $lp $node] != -1} {
			return " $node "
		} else  {
			set a [SearchDirectPhylotype  $T($IDT,fils1,$node) $lp]
			append a [SearchDirectPhylotype  $T($IDT,fils2,$node) $lp]
			return $a
		}
		
	}
}
proc GenerationPhylotype  {p n} {
	global IDT Pere
	if {[info exists Pere($p)] == 0} {
		# on est a la racine
		return $n
	} else {
		incr n
		GenerationPhylotype $Pere($p) $n
	}
}
################################################################################



################################################################################
# COLOR
################################################################################
#
proc ColorAnnotation {m} {
	global IDT S A
	$m delete 0 end
	foreach v [lsort -dictionary $A($IDT,AnnotationList)] {
		$m add command -label "$v" -command "UpdateAnnotationColor $v" -background $S(ColorAnnotation,$v)
	}
	$m add separator
	$m add radiobutton -value linear -label "Mode linear" -variable S(ColorSetMode) -command ColorSetMode
	$m add radiobutton -value hierarchic -label "Mode hierarchic" -variable S(ColorSetMode) -command ColorSetMode
	$m add radiobutton -value loadcolor -label "Load..." -variable S(ColorSetMode) -command ColorSetMode
	$m add separator
	$m add command -label "Equalize" -command ColorUpdateEqualize
	$m add separator
	$m add command -label "Add Legend to current tree" -command {AddLegend 1}
	$m add command -label "Add Legend to current view" -command {AddLegend 2}
	$m add separator
	$m add command -label "Delete Legend from current tree" -command "$S(canvas) delete LEGEND"
	$m add command -label "Delete Legend current view" -command "$S(canvas2) delete LEGEND"

}
#
proc ColorSetMode {} {
	global S A IDT
	switch -exact $S(ColorSetMode) {
		linear {
			set annotations [lsort -dictionary $A($IDT,AnnotationList)]
			set lc [ColorProgressive [llength $annotations]]
			foreach valeur $annotations c $lc {
				set S(ColorAnnotation,$valeur) $c
			}
		}
		hierarchic {
			set lc [ColorProgressive2]
		}
		loadcolor {
			set colorfilename [tk_getOpenFile -initialdir $S(pathuser) -title "Select COLORS file... "]
			if {$colorfilename != ""} {
				if [catch {open $colorfilename r} fid] {
					return 0
				} else {
					while {[eof $fid] != 1} {
						set row  [gets $fid]
						regsub -all "," $row " " row
						set a [lindex $row 0]
						set c [lindex $row end]
						set S(ColorAnnotation,$a) $c
						puts "S(ColorAnnotation,$a) $c"
					}
					close $fid
				}
			}
		}
	}
	UpdateColorAll
}
#
proc UpdateAnnotationColor {v} {
	global IDT S
	set color [tk_chooseColor]
	if {$color != ""} {
		set S(ColorAnnotation,$v) $color
	} 
	ColorAnnotationUpdate $v $color
	return
}
#
proc ColorUpdateScale {} {
	global IDT S
	set S(ColorSaturation) $S(ColorSaturation,$S(UpdateColorCible))
	set S(ColorValue) $S(ColorValue,$S(UpdateColorCible))
}
# mise a la meme valeur S et V pour chaque H (annotation)
# et update graphique
proc ColorUpdateEqualize {} {
	global IDT S
	set S(ColorSaturation) 100
	set S(ColorValue) 250
	foreach cible {BGtree FGtree MaxLikpiechart PupkoAnnot TipsAnnot} {
		set S(UpdateColorCible) $cible 
		UpdateColor
	}
}
#
proc AddLegend {target} {
	global IDT TT A S 
	switch $target {
		1 {set w $S(canvas)}
		2 {set w $S(canvas2)}
	}
	$w delete LEGEND
	set co [$w bbox all]
	set x [expr [lindex $co 2] + 30]
	set y [lindex $co 1] 
	if {$x == ""} {set x 10}
	if {$y == ""} {set y 10}
	
	set s 3
	
	foreach annotation [lsort -dictionary $A($IDT,AnnotationList)] {
		$w create rectangle [expr $x - $s] [expr $y - $s] [expr $x + $s] [expr $y + $s] \
			-fill $S(ColorAnnotation,$annotation) -outline white -tags "LEGEND LEGEND=$annotation" 
		$w create text [expr $x + 10] $y -text $annotation -fill black -font {Helvetica 10 normal}  \
			-anchor w -tags "LEGEND LEGEND=$annotation" 
		incr y 10
	}
	$w configure -scrollregion [$w bbox all]
}

# nouvelle association couleur / annotation
# mise à jour des graphiques, avec respect S et V 
proc ColorAnnotationUpdate {a c} {
	global IDT S T SA A
	foreach id $S(MapList) {
		# pour chaque map
		set w1 [format "%s%s%s" .pan.g.g.g.m $id .p.main.canvas.c]
		lassign [rgbToHsv {*}[scan $c "#%2x%2x%2x"]] h s v
		
		# BGtree
		set NEWrgbBGtree [hsvToRgb $h $S(ColorSaturation,BGtree) $S(ColorValue,BGtree)]
		set r [lindex $NEWrgbBGtree 0] 
		set g [lindex $NEWrgbBGtree 1]
		set b [lindex $NEWrgbBGtree 2]
		set color [format "#%2X%2X%2X" $r $g $b] ; # passage en hexadecimal
		$w1 itemconfigure BGtree=$a -fill $color
		$w1 itemconfigure BGtree=$a -outline $color
		# chaque board idb  de map id
		foreach idb $S(BoardList,$id) {
			set idboard [lindex [split $idb "."] end]
			set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
			$w2 itemconfigure BGtree=$a -fill $color
			$w2 itemconfigure BGtree=$a -outline $color
		}
		# FGtree
		set NEWrgbFGtree [hsvToRgb $h $S(ColorSaturation,FGtree) $S(ColorValue,FGtree)]
		set r [lindex $NEWrgbFGtree 0] 
		set g [lindex $NEWrgbFGtree 1]
		set b [lindex $NEWrgbFGtree 2]
		set color [format "#%2X%2X%2X" $r $g $b] ; # passage en hexadecimal
		$w1 itemconfigure FGtree=$a -fill $color
		# chaque board idb  de map id
		foreach idb $S(BoardList,$id) {
			set idboard [lindex [split $idb "."] end]
			set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
			$w2 itemconfigure FGtree=$a -fill $color
		}
		# MaxLikpiechart 
		set NEWrgbBGpiechart [hsvToRgb $h $S(ColorSaturation,MaxLikpiechart) $S(ColorValue,MaxLikpiechart)]
		set r [lindex $NEWrgbBGpiechart 0] 
		set g [lindex $NEWrgbBGpiechart 1]
		set b [lindex $NEWrgbBGpiechart 2]
		set color [format "#%2X%2X%2X" $r $g $b] ; # passage en hexadecimal
		$w1 itemconfigure MaxLikpiechart=$a -fill $color
		foreach idb $S(BoardList,$id) {
			set idboard [lindex [split $idb "."] end]
			set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
			$w2 itemconfigure MaxLikpiechart=$a -fill $color
		}
		# PupkoAnnot 
		set NEWrgbPupkoAnnot [hsvToRgb $h $S(ColorSaturation,PupkoAnnot) $S(ColorValue,PupkoAnnot)]
		set r [lindex $NEWrgbPupkoAnnot 0] 
		set g [lindex $NEWrgbPupkoAnnot 1]
		set b [lindex $NEWrgbPupkoAnnot 2]
		set color [format "#%2X%2X%2X" $r $g $b] ; # passage en hexadecimal
		$w1 itemconfigure bubbleannotation=$a -fill $color
		foreach idb $S(BoardList,$id) {
			set idboard [lindex [split $idb "."] end]
			set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
			$w2 itemconfigure bubbleannotation=$a -fill $color
		}
		# leaf annotation
		set NEWrgbTipsAnnot [hsvToRgb $h $S(ColorSaturation,TipsAnnot) $S(ColorValue,TipsAnnot)]
		set r [lindex $NEWrgbTipsAnnot 0] 
		set g [lindex $NEWrgbTipsAnnot 1]
		set b [lindex $NEWrgbTipsAnnot 2]
		set color [format "#%2X%2X%2X" $r $g $b] ; # passage en hexadecimal
		$w1 itemconfigure AnnotMatrixSquare=$a -fill $color
		$w1 itemconfigure AnnotMatrixSquare=$a -outline $color
		$w1 itemconfigure AnnotMatrixText=$a -fill $color
		foreach idb $S(BoardList,$id) {
			set idboard [lindex [split $idb "."] end]
			set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
			$w2 itemconfigure AnnotMatrixSquare=$a -fill $color
			$w2 itemconfigure AnnotMatrixSquare=$a -outline $color
			$w2 itemconfigure AnnotMatrixText=$a -fill $color
		}
		# leaf labels
		$w1 itemconfigure leaflabel=$a -fill $color
	}
}
#
proc UpdateColor {{value bidon}} {
	global IDT S A
	set  S(ColorSaturation,$S(UpdateColorCible)) $S(ColorSaturation)
	set  S(ColorValue,$S(UpdateColorCible)) $S(ColorValue)
	if {[info exists A($IDT,AnnotationList)]} {
	foreach id $S(MapList) {
		# pour chaque map
		set w1 [format "%s%s%s" .pan.g.g.g.m $id .p.main.canvas.c]
		# pour chaque annotation
		foreach a [lsort -dictionary $A($IDT,AnnotationList)] {
			set couleurRGB $S(ColorAnnotation,$a)
			lassign [rgbToHsv {*}[scan $couleurRGB "#%2x%2x%2x"]] h s v
			# on garde la teinte de l'annotation , et saturation / valur (luminosité) selon user
			set NEWrgb [hsvToRgb $h $S(ColorSaturation,$S(UpdateColorCible)) $S(ColorValue,$S(UpdateColorCible))]
			# passage en hexadecimal
			set r [lindex $NEWrgb 0] 
			set g [lindex $NEWrgb 1]
			set b [lindex $NEWrgb 2]
			set color [format "#%2X%2X%2X" $r $g $b]
			switch -exact $S(UpdateColorCible) {
				BGtree {
					$w1 itemconfigure BGtree=$a -fill $color
					$w1 itemconfigure BGtree=$a -outline $color
					foreach idb $S(BoardList,$id) {
						set idboard [lindex [split $idb "."] end]
						set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
						$w2 itemconfigure BGtree=$a -fill $color
						$w2 itemconfigure BGtree=$a -outline $color
					}
				}
				FGtree {
					$w1 itemconfigure FGtree=$a -fill $color
					foreach idb $S(BoardList,$id) {
						set idboard [lindex [split $idb "."] end]
						set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
						$w2 itemconfigure FGtree=$a -fill $color
					}
				}
				MaxLikpiechart {
					$w1 itemconfigure MaxLikpiechart=$a -fill $color
					foreach idb $S(BoardList,$id) {
						set idboard [lindex [split $idb "."] end]
						set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
						$w2 itemconfigure MaxLikpiechart=$a -fill $color
					}
				}
				PupkoAnnot {
					$w1 itemconfigure bubbleannotation=$a -fill $color
					foreach idb $S(BoardList,$id) {
						set idboard [lindex [split $idb "."] end]
						set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
						$w2 itemconfigure bubbleannotation=$a -fill $color
					}
				}
				TipsAnnot {
					# LeafAnnotation : trop lent
					# les annotamatrix
					$w1 itemconfigure AnnotMatrixText=$a -fill $color
					$w1 itemconfigure AnnotMatrixSquare=$a -fill $color
					$w1 itemconfigure AnnotMatrixSquare=$a -outline $color
					foreach idb $S(BoardList,$id) {
						set idboard [lindex [split $idb "."] end]
						set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
						$w2 itemconfigure AnnotMatrixText=$a -fill $color
						$w2 itemconfigure AnnotMatrixSquare=$a -fill $color
						$w2 itemconfigure AnnotMatrixSquare=$a -outline $color
					}
					# leaf label
					$w1 itemconfigure leaflabel=$a -fill $color
				}
			}
		}
	}
	}
}

#
#
proc UpdateColorAll {} {
	global IDT S A
	set  S(ColorSaturation,$S(UpdateColorCible)) $S(ColorSaturation)
	set  S(ColorValue,$S(UpdateColorCible)) $S(ColorValue)
	if {[info exists A($IDT,AnnotationList)]} {
	foreach id $S(MapList) {
		# pour chaque map
		set w1 [format "%s%s%s" .pan.g.g.g.m $id .p.main.canvas.c]
		# pour chaque annotation
		foreach a [lsort -dictionary $A($IDT,AnnotationList)] {
			set couleurRGB $S(ColorAnnotation,$a)
			lassign [rgbToHsv {*}[scan $couleurRGB "#%2x%2x%2x"]] h s v
			# on garde la teinte de l'annotation , et saturation / valur (luminosité) selon user
			set NEWrgb [hsvToRgb $h $S(ColorSaturation,$S(UpdateColorCible)) $S(ColorValue,$S(UpdateColorCible))]
			# passage en hexadecimal
			set r [lindex $NEWrgb 0] 
			set g [lindex $NEWrgb 1]
			set b [lindex $NEWrgb 2]
			set color [format "#%2X%2X%2X" $r $g $b]

			#BGtree 
			$w1 itemconfigure BGtree=$a -fill $color
			$w1 itemconfigure BGtree=$a -outline $color
			foreach idb $S(BoardList,$id) {
				set idboard [lindex [split $idb "."] end]
				set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
				$w2 itemconfigure BGtree=$a -fill $color
				$w2 itemconfigure BGtree=$a -outline $color
			}
			#FGtree 
			$w1 itemconfigure FGtree=$a -fill $color
			foreach idb $S(BoardList,$id) {
				set idboard [lindex [split $idb "."] end]
				set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
				$w2 itemconfigure FGtree=$a -fill $color
			}
			#MaxLikpiechart 
			$w1 itemconfigure MaxLikpiechart=$a -fill $color
			foreach idb $S(BoardList,$id) {
				set idboard [lindex [split $idb "."] end]
				set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
				$w2 itemconfigure MaxLikpiechart=$a -fill $color
			}
			#PupkoAnnot 
			$w1 itemconfigure bubbleannotation=$a -fill $color
			foreach idb $S(BoardList,$id) {
				set idboard [lindex [split $idb "."] end]
				set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
				$w2 itemconfigure bubbleannotation=$a -fill $color
			}
			#TipsAnnot 
			# LeafAnnotation : trop lent
			# les annotamatrix
			$w1 itemconfigure AnnotMatrixText=$a -fill $color
			$w1 itemconfigure AnnotMatrixSquare=$a -fill $color
			$w1 itemconfigure AnnotMatrixSquare=$a -outline $color
			foreach idb $S(BoardList,$id) {
				set idboard [lindex [split $idb "."] end]
				set w2 [format "%s%s%s%s%s" .pan.g.g.g.m $id .p.board.p. $idboard .canvas.c]
				$w2 itemconfigure AnnotMatrixText=$a -fill $color
				$w2 itemconfigure AnnotMatrixSquare=$a -fill $color
				$w2 itemconfigure AnnotMatrixSquare=$a -outline $color
			}
			# leaf label
			$w1 itemconfigure leaflabel=$a -fill $color
		}
	}
	}
}
#
proc rgbToHsv {r g b} {
	set temp  [expr {min($r, $g, $b)}]
	set value [expr {max($r, $g, $b)}]
	set range [expr {$value-$temp}]
	if {$range == 0} {
		set hue 0
	} else {
		if {$value == $r} {
			set top [expr {$g-$b}]
			if {$g >= $b} {
				set angle 0
			} else {
				set angle 360
			}
		} elseif {$value == $g} {
			set top [expr {$b-$r}]
			set angle 120
		} elseif {$value == $b} {
			set top [expr {$r-$g}]
			set angle 240
		}
		set hue [expr { round( double($top) / $range * 60 + $angle ) }]
	}
	if {$value == 0} {
		set saturation 0
	} else {
		set saturation [expr { round( 255 - double($temp) / $value * 255 ) }]
	}
	return [list $hue $saturation $value]
}
 proc hsvToRgb {h s v} {
	set Hi [expr { int( double($h) / 60 ) % 6 }]
	set f [expr { double($h) / 60 - $Hi }]
	set s [expr { double($s)/255 }]
	set v [expr { double($v)/255 }]
	set p [expr { double($v) * (1 - $s) }]
	set q [expr { double($v) * (1 - $f * $s) }]
	set t [expr { double($v) * (1 - (1 - $f) * $s) }]
	switch -- $Hi {
		0 {
			set r $v
			set g $t
			set b $p
		}
		1 {
			set r $q
			set g $v
			set b $p
		}
		2 {
			set r $p
			set g $v
			set b $t
		}
		3 {
			set r $p
			set g $q
			set b $v
		}
		4 {
			set r $t
			set g $p
			set b $v
		}
		5 {
			set r $v
			set g $p
			set b $q
		}
		default {
			error "Wrong Hi value in hsvToRgb procedure! This should never happen!"
		}
	}
	set r [expr {round($r*255)}]
	set g [expr {round($g*255)}]
	set b [expr {round($b*255)}]
	return [list $r $g $b]
 }

# renvoie nb couleurs réparties sur le cercle chromatique
proc ColorProgressive {nb } {
	global IDT S
	set lc  {}
	set hue 0.5
	set s [expr $S(ColorSaturation) / 100] ; # plus c'est petit plus c'est éteint
	set v [expr $S(ColorValue) / 100] ; # plus c'est petit plus c'est sombre
	set incrementhue [expr 6.28 / $nb]
	for {set i 1} {$i <= $nb} {incr i} {
		append lc  " [ColorPanel_hsb2rgb $hue $s $v]"
		#append lc  " [::tk::Darken [ColorPanel_hsb2rgb $hue $s $v] 150]"
		set hue [expr $hue + $incrementhue]
	}
	return $lc
}

proc ColorPanel_hsb2rgb {h s v} {
	if {$h == 0} {
		set v [expr round(255*$v)]
		set r $v
		set g $v
		set b $v
	} else {
		if {$h >= 6.28318} {set h [expr $h-6.28318]}
		set h [expr $h/1.0472]
		set f [expr $h-floor($h)]
		set p [expr round(255*$v*(1.0-$s))]
		set q [expr round(255*$v*(1.0-$s*$f))]
		set t [expr round(255*$v*(1.0-$s*(1.0-$f)))]
		set v [expr round(255*$v)]
		switch [expr int($h)] {
			0 {set r $v; set g $t; set b $p}
			1 {set r $q; set g $v; set b $p}
			2 {set r $p; set g $v; set b $t}
			3 {set r $p; set g $q; set b $v}
			4 {set r $t; set g $p; set b $v}
			5 {set r $v; set g $p; set b $q}
		}
	}
	return [format "#%.2X%.2X%.2X" $r $g $b]
}
#
proc randomColor {} {format #%06x [expr {int(rand() * 0xFFFFFF)}]}
################################################################################


################################################################################
proc ColorProgressive2 {} {
	global A LM S LMdist IDT
	set DCOLOR(MAX) 0
	levenshteinMatrix
	set NewickStringColor [CAH $A($IDT,AnnotationList)]
	set S(NewickStringColor) $NewickStringColor
	#puts $NewickStringColor
	# liste tips hierarchic
	set S(HierarchicAnnotation) ""
	# G & D de ROOT
	set id [BgBdx $NewickStringColor] ;# id est l'index de la virgule separant deux fils
	set G [string range $NewickStringColor 1 [expr $id - 1]]
	set D [string range $NewickStringColor [expr $id + 1] [expr [string length $NewickStringColor] - 2]]
	# TOP DOWN Hue intervals
	set HueInf 0.001
	set HueSup 6.28
	set LMdist(profondeur) 1
	set S(HierarchicColorSaturation) 0.90
	set S(HierarchicColorLuminance) 0.50 
	set S(HierarchicColorLuminanceDelta) [expr 0.50 / $LMdist(profondeur)]
	# Hue intervals pondéré nombre de feuilles sous-jacentes
	set NbG [NewickColorCount $G]
	set NbD [NewickColorCount $D]
	set Total [expr $NbG + $NbD]
	set HueDelta [expr $HueSup - $HueInf]
	set HueMid [expr $HueInf + (($NbG * $HueDelta) / $Total)]
	set HueInfG $HueInf
	set ponderation [expr $LMdist($G,$D) * 0.02]
	set HueSupG [expr $HueMid - $ponderation]
	set HueInfD [expr $HueMid + $ponderation]  
	set HueSupD $HueSup
	if {$HueSupG < $HueInfG} {set HueSupG $HueInfG}
	if {$HueInfD > $HueSupD} {set HueInfD $HueSupD}
	# Go
	NewickColorGo $G $HueInfG $HueSupG
	NewickColorGo $D $HueInfD $HueSupD
}
#
proc NewickColorGo {s HueInf HueSup} {
	global S LMdist IDT
	if {[string match *(* $s]} {
		# NODE
		set id [BgBdx $s]
		set G [string range $s 1 [expr $id - 1]]
		set D [string range $s [expr $id + 1] [expr [string length $s] - 2]]
		# Hue intervals pondéré nombre de feuilles sous-jacentes		
		set NbG [NewickColorCount $G]
		set NbD [NewickColorCount $D]
		set Total [expr $NbG + $NbD]
		set HueDelta [expr $HueSup - $HueInf]
		set HueMid [expr $HueInf + (($NbG * $HueDelta) / $Total)]
		set HueInfG $HueInf
		# prise en compte distance pour mieux discriminer les nodes
		set ponderation [expr $LMdist($G,$D) * 0.02]
		set HueSupG [expr $HueMid - $ponderation]
		set HueInfD [expr $HueMid + $ponderation]
		#set HueSupG [expr $HueMid - 0.05]
		#set HueInfD [expr $HueMid + 0.05]
		#set HueSupG $HueMid 
		#set HueInfD $HueMid 
		set HueSupD $HueSup
		if {$HueSupG < $HueInfG} {set HueSupG $HueInfG}
		if {$HueInfD > $HueSupD} {set HueInfD $HueSupD}
		# Relance
		set S(HierarchicColorSaturation) [expr $S(HierarchicColorSaturation) - 0.050]
		set S(HierarchicColorLuminance) [expr $S(HierarchicColorLuminance) + $S(HierarchicColorLuminanceDelta)]
		if {$S(HierarchicColorSaturation) < 0.50} {set S(HierarchicColorSaturation) 0.50}
		if {$S(HierarchicColorLuminance) > 0.95} {set S(HierarchicColorLuminance) 0.95}
		NewickColorGo $G $HueInfG $HueSupG
		NewickColorGo $D $HueInfD $HueSupD
		return
	} {
		# TIP
		set Hue [expr ($HueInf + $HueSup) / 2.0]
		set S(ColorAnnotation,$s) [ColorPanel_hsb2rgbSpecific $Hue $S(HierarchicColorSaturation) $S(HierarchicColorLuminance)]
		set S(ColorAnnotationHue,$s) $Hue 
		append S(HierarchicAnnotation) " $s"
		return
	}
}
#
proc levenshteinMatrix {} {
	global S LM A IDT
	catch {unset LM}
	set annotations [lsort -dictionary $A($IDT,AnnotationList)]
	foreach a1 $annotations {
		foreach a2 $annotations {
			set LM($a1,$a2) [levenshteinDistance $a1 $a2]
		}
	}
}
#
proc levenshteinDistance {s t} {
    # Edge cases
    if {![set n [string length $t]]} {
		return [string length $s]
    } elseif {![set m [string length $s]]} {
		return $n
    }
    # Fastest way to initialize
    for {set i 0} {$i <= $m} {incr i} {
		lappend d 0
		lappend p $i
    }
    # Loop, computing the distance table (well, a moving section)
    for {set j 0} {$j < $n} {} {
		set tj [string index $t $j]
		lset d 0 [incr j]
		for {set i 0} {$i < $m} {} {
	    	set a [expr {[lindex $d $i]+1}]
	    	set b [expr {[lindex $p $i]+([string index $s $i] ne $tj)}]
	    	set c [expr {[lindex $p [incr i]]+1}]
	    	# Faster than min($a,$b,$c)
	    	lset d $i [expr {$a<$b ? $c<$a ? $c : $a : $c<$b ? $c : $b}]
		}
		# Swap
		set nd $p; set p $d; set d $nd
    }
    # The score is at the end of the last-computed row
    return [lindex $p end]
}

# Classification Ascendante Hierarchique agregation Average Linkage
proc CAH {annotations} {
	global A LM LMdist
	# recherche des deux element k1 et k2 de distance minimale
	set MaxD 0
	foreach a1 $annotations {
		foreach a2 $annotations {
		    if {$LM($a1,$a2) >  $MaxD && $a1 != $a2} {set MaxD $LM($a1,$a2)}
		}
	}
	set MinD $MaxD
	foreach a1 $annotations {
		foreach a2 $annotations {
		    if {$LM($a1,$a2) <  $MinD && $a1 != $a2 } {set MinD $LM($a1,$a2); set k1 $a1; set k2 $a2}
		}
	}
	# insertion de l'entreé k1,k2
	foreach a1 $annotations {
		# average linkage
		set newkey [format "%s%s%s%s%s" ( $k1 = $k2 )]
		
		# stockage distance NODE
		regsub -all = $k1 , kk1
		regsub -all = $k2 , kk2
		set LMdist($kk1,$kk2) $MinD  
		set LMdist($kk2,$kk1) $MinD  

		# NODE
		set LM($newkey,$a1) [expr ($LM($k1,$a1) + $LM($k2,$a1)) / 2.0]
		set LM($a1,$newkey) [expr ($LM($k1,$a1) + $LM($k2,$a1)) / 2.0]
		set LM($newkey,$newkey) 0
	}
	# suppression des entrées k1 et k2
	array unset LM $k1,* ; array unset LM *,$k1
	array unset LM $k2,* ; array unset LM *,$k2
	# nouvelle liste d'annotation
	set listeannotation ""
	foreach {k v} [array get LM *] {
		append listeannotation " [split $k ,]"
	}
	foreach {k v} [array get LM *] {
		foreach a [split $k ,] {
			lappend listeannotation $a
		}
	}
	set listeannotation [DelRep $listeannotation]
	# Relance ou Return
	if {[llength $listeannotation] > 2.0} {
		incr LMdist(profondeur)
		CAH $listeannotation
	} else {
		set newkey [format "%s%s%s%s%s" "(" [lindex $listeannotation 0] "=" [lindex $listeannotation 1] ")" ]
		# stockage distance NODE
		regsub -all = [lindex $listeannotation 0] , kk1
		regsub -all = [lindex $listeannotation 1] , kk2
		set LMdist($kk1,$kk2) $MinD  
		set LMdist($kk2,$kk1) $MinD 
		regsub -all = $newkey , output
		return $output
	}
}
#
proc NewickColorCount {s} {
	global S
	if {[string match *(* $s]} {
		# NODE
		set id [BgBdx $s]
		set G [string range $s [expr $id + 1] [expr [string length $s] - 2]]
		set D [string range $s 1 [expr $id - 1]]
		# G 
		set nbG [NewickColorCount $G]
		set nbD [NewickColorCount $D] 
		return [expr $nbG + $nbD]
	} {
		return 1.0
	}
}
proc ColorPanel_hsb2rgbSpecific {h s v} {
	if {$h == 0} {
		set v [expr round(255*$v)]
		set r $v
		set g $v
		set b $v
	} else {
		if {$h >= 6.28318} {set h [expr $h-6.28318]}
		set h [expr $h/1.0472]
		set f [expr $h-floor($h)]		
		set p [expr round(255*$v*(1.0-$s))]
		set q [expr round(255*$v*(1.0-$s*$f))]
		set t [expr round(255*$v*(1.0-$s*(1.0-$f)))]
		set v [expr round(255*$v)]
		switch [expr int($h)] {
			0 {set r $v; set g $t; set b $p}
			1 {set r $q; set g $v; set b $p}
			2 {set r $p; set g $v; set b $t}
			3 {set r $p; set g $q; set b $v}
			4 {set r $t; set g $p; set b $v}
			5 {set r $v; set g $p; set b $q}
		}
	}
	#puts ">>>> $h $s $v RGB = $r $g $b [format "#%.2X%.2X%.2X" $r $g $b]"
	return [format "#%.2X%.2X%.2X" $r $g $b]
}
##########################################################################################





################################################################################
# TAB "C" (Comparaison)
################################################################################

# IDT = reference (map, tree et AA distribution)
# S(AAIDTlist) reference la liste les sets d'annotation ancestrales a projeter/comparer
# pour chaque node, si annotations majoritaires égales => fg color du node
# sinon camemberts pour chaque IDT
# Version Bubble
proc AAcomparBubble {} {
	global IDT S A AA T
	set w $S(canvas)
	# reset
	$w delete bubblecircle
	$w delete Cammenb
	ResetTreeForegroundColor
	# ID dataset source
	set S(AAIDTlist) {}
	foreach {k v} [array get S ComparAA,*] {
		if {$v == 1} {
			set ID [lindex [split $k ,] end]
			lappend S(AAIDTlist) $ID
		}
	}
	lappend S(AAIDTlist) $IDT
	# majoritaire
	foreach IDTi $S(AAIDTlist) {
		foreach n $T($IDTi,allint_cod) {
			set max 0
			foreach {k v} [array get AA $IDTi,distribution,$n,*] {
				set a [lindex [split $k ,] end]
				if {$v > $max} {
					set max $v
					set AA($IDTi,majority,$n) $a
				}
			}
		}
	}
	# 
	foreach n $T($IDT,allint_cod) {
		set l {}
		foreach IDTi $S(AAIDTlist) {
			if {[lsearch -exact $l $AA($IDTi,majority,$n)] == -1} {
				lappend l $AA($IDTi,majority,$n)
			}
		}
		if {[llength $l] == 1} {
			# identité majoritaire 
			set a [lindex $l 0]
			$w itemconfigure H$n -fill $S(ColorAnnotation,$a)
			if {$T($IDT,typ,$n) != "f"} {
				$w itemconfigure V$n -fill $S(ColorAnnotation,$a)
			}
		} else {
			set x 0
			set y 0
			foreach IDTi $S(AAIDTlist) {
				AA-Bub $w $IDTi $x $y $n
				incr x [expr int($S(SurfaceMaxBubble))]
				incr y 0
			}
		}
	}
	# fg arretes terminales
	foreach i $T($IDT,ue_cod) {
		#set n $T($IDT,dbv,$i)
		set max 0
		foreach {k v} [array get AA $IDT,distribution,$i,*] {
			if {$v > $max} {set max $v ; set a [lindex [split $k ,] end]}
		}
		$w itemconfigure H$i -fill $S(ColorAnnotation,$a)
	}
}
#
proc AA-Bub {w IDTi xadd yadd n } {
	global IDT S T A AA
	set co [$w coords H$n]
	set x [expr ([lindex $co 0]+[lindex $co 2])/2.0 + $xadd ]
	set y [expr ([lindex $co 1]+[lindex $co 3])/2.0 + $yadd]
	set d [expr $S(SurfaceMaxBubble) / 2.0]
	set a $AA($IDTi,majority,$n)
	if [catch {set color $S(ColorAnnotation,$a)} result] {
		set color white
	} 
	$w create oval [expr $x - $d] [expr $y - $d] [expr $x + $d] [expr $y + $d] \
		-fill $color -outline black -tags "bubblecircle bubblecirclenode=$n bubbleannotation=$a"
	$w raise bubblecircle
	update idletasks
}
# version Piechart
proc AAcomparPieChart {} {
	global IDT S A AA T
	set w $S(canvas)
	# reset
	$w delete bubblecircle
	$w delete Cammenb
	ResetTreeForegroundColor
	# ID dataset source
	set S(AAIDTlist) {}
	foreach {k v} [array get S ComparAA,*] {
		if {$v == 1} {
			set ID [lindex [split $k ,] end]
			lappend S(AAIDTlist) $ID
		}
	}
	lappend S(AAIDTlist) $IDT
	# identification majoritaire
	foreach IDTi $S(AAIDTlist) {
		foreach n $T($IDTi,allint_cod) {
			set max 0
			foreach {k v} [array get AA $IDTi,distribution,$n,*] {
				set a [lindex [split $k ,] end]
				if {$v > $max} {
					set max $v
					#set AA($IDT,majority,$n) $a
					set AA($IDTi,majority,$n) $a
				}
			}
		}
	}
	# 
	foreach n $T($IDT,allint_cod) {
		set l {}
		foreach IDTi $S(AAIDTlist) {
			if {[lsearch -exact $l $AA($IDTi,majority,$n)] == -1} {
				lappend l $AA($IDTi,majority,$n)
			}
		}
		if {[llength $l] == 1} {
			# identité majoritaire 
			set a [lindex $l 0]
			$w itemconfigure H$n -fill $S(ColorAnnotation,$a)
			if {$T($IDT,typ,$n) != "f"} {
					$w itemconfigure V$n -fill $S(ColorAnnotation,$a)
			}
		} else {
		
		
			set x 0
			set y -8
			foreach IDTi $S(AAIDTlist) {
				AA-Pie $w $IDTi $x $y $n
				set x [expr $x + $S(deltaPieCompar)]
				incr y 0
			}
		}
	}
	# fg arretes terminales
	foreach i $T($IDT,ue_cod) {
		#set n $T($IDT,dbv,$i)
		set max 0
		foreach {k v} [array get AA $IDT,distribution,$i,*] {
			if {$v > $max} {set max $v ; set a [lindex [split $k ,] end]}
		}
		$w itemconfigure H$i -fill $S(ColorAnnotation,$a)
	}
}
#
proc AA-Pie {w IDTi xadd yadd n } {
	global IDT S T A AA
	set co [$w coords H$n]
	#set x [expr ([lindex $co 0]+[lindex $co 2])/2.0]
	#set y [expr ([lindex $co 1]+[lindex $co 3])/2.0]
	set x [expr ([lindex $co 0]+[lindex $co 2])/2.0]
	set y [expr ([lindex $co 1]+[lindex $co 3])/2.0]
	#set id $T($IDT,dbv,$n) ; # l'ID referancant les annotatios est dbv
	#### Camembert 
	set apc {} ; # liste annotation / proba / couleur
	set sommeproba 0
	foreach {k v} [array get AA $IDTi,distribution,$n,*] {
		set a [lindex [split $k ,] end]
		set sommeproba [expr $sommeproba + $v]
		lappend apc [list $a $v $S(ColorAnnotation,$a)]
	}
	# AJOUTER a la liste APC un element AUTRE si la somme n'est pas 1
	set somme [expr 1 - $sommeproba]
	if {$somme > 0.001} {
		lappend apc [list "?" [rnd $somme 2] grey]
	}
	# dessin
	piechart $w [expr $x + $xadd] [expr $y + $yadd] $S(deltaPieCompar) $S(deltaPieCompar) $n $apc
	$w raise Cammenb
	$w configure -scrollregion [$w bbox all]
	update idletasks
}
# on a une MAP, IDT, on met en selection le dataset AA d'une ou de plusieurs MAP IDTi en selection pour comparer
#
proc ComparTreeSelection {m} {
	global S IDT
	# liste des tree, selection (command) et affichage
	$m delete 0 end
	foreach id $S(MapList) {
		if {$id != $IDT} {
			$m add command -label "Tree from Map $id" -command "AddTreeFrom $id"
		}
	}
}
#
proc ComparAASelection {m} {
	global S IDT
	# liste des DISTRIBUTIONs aa, selection (checklist) et affichage
	$m delete 0 end
	foreach id $S(MapList) {
		if {$id != $IDT} {
			$m add checkbutton -label "AA distribution from Map $id" -variable S(ComparAA,$id) 
		}
	}
}
#
proc ZoomAAcomparPieChart {s} {
	global IDT S T
	set delta 1.0
	switch $s {
		-	{
			set S(deltaPieCompar) [expr $S(deltaPieCompar) -$delta]
		}
		+	{
			set S(deltaPieCompar) [expr $S(deltaPieCompar) +$delta]
		}
	}
	if {$S(deltaPieCompar) < 0.0} {
		set S(deltaPieCompar) 0.0
	}
	AAcomparPieChart 
}
#
proc AddTreeFrom {id} {
	global S T IDT
	$S(canvas) delete all
	set currentIDT $IDT
	set currentCanvas $S(canvas)
	set IDT $id
	TreeInit
	# copy data structure
	foreach {k v} [array get T $id,*] {
	 set newk [format "%s%s" $currentIDT [string range $k [string first , $k] end]]
	 set T($newk) $v
	}
	PhyNJ $currentCanvas
	$currentCanvas configure -scrollregion [$S(canvas) bbox all]
	set IDT $currentIDT
	update idletasks
	
}
################################################################################


################################################################################
# OngleAA comparaison majoritaire/pupko/parsimony (selon availability) et 
# affiche les bubbles pour un node si il y a des differences si tout pareil fg color
################################################################################
#
proc BubbleInconsistenciesCAP {} {
	global IDT T S A AA
	set w $S(canvas)
	set items [$w find withtag bubbleInconsistencies]
	if {$items == {}} {
		BubbleInconsistencies
	} else {
		$w delete bubbleInconsistencies
		ResetTreeForegroundColor
		return
		# reset foreground
	}
}
proc BubbleInconsistencies {} {
	global IDT T S A AA
	set w $S(canvas)

	# reset
	$w delete bubbleInconsistencies
	$w delete bubble
	$w delete Cammenb
	ResetTreeForegroundColor
	
	# majoritaire
	foreach n $T($IDT,allint_cod) {
		set max 0
		foreach {k v} [array get AA $IDT,distribution,$n,*] {
			set a [lindex [split $k ,] end]
			if {$v > $max} {set max $v ; set AA($IDT,majority,$n) $a}
		}
	}
	# majority $AA($IDTi,majority,$n)
	# pupko $AA($IDT,pupko,$n)
	# parsimony $AA($IDT,parsimony,$n)
	# construction ltype selon presence
	set ltype {}
	foreach t {majority pupko parsimony} {
		if [info exists AA($IDT,$t,0)] {lappend ltype $t}
	}
	# 
	foreach n $T($IDT,allint_cod) {
		set l {}
		foreach type $ltype {
			if {[lsearch -exact $l $AA($IDT,$type,$n)] == -1} {
				lappend l $AA($IDT,$type,$n)
			}
		}
		if {[llength $l] == 1} {
			# vraisemblance marginale majoritaire / vraisemblance jointe / parcimonie
			set a [lindex $l 0]
			$w itemconfigure H$n -fill $S(ColorAnnotation,$a)
			$w itemconfigure V$n -fill $S(ColorAnnotation,$a)
		} else {
			# au moins une difference
			set x 0
			set y 0
			foreach type $ltype {
				BubbleInconsistencies-Draw $w $type $n $x $y 
				incr x [expr int($S(SurfaceMaxBubble))]
				incr y 0
			}
		}
	}
	# fg arretes terminales
	foreach i $T($IDT,ue_cod) {
		set max 0
		foreach {k v} [array get AA $IDT,distribution,$i,*] {
			if {$v > $max} {set max $v ; set a [lindex [split $k ,] end]}
		}
		$w itemconfigure H$i -fill $S(ColorAnnotation,$a)
	}
}

#
proc BubbleInconsistencies-Draw {w type n xadd yadd } {
	global IDT S T A AA
	set co [$w coords H$n]
	set x [expr ([lindex $co 0]+[lindex $co 2])/2.0 + $xadd ]
	set y [expr ([lindex $co 1]+[lindex $co 3])/2.0 + $yadd]
	set d [expr $S(SurfaceMaxBubble) / 2.0]
	set a $AA($IDT,$type,$n)
	if [catch {set color $S(ColorAnnotation,$a)} result] {
		set color white
	} 
	$w create oval [expr $x - $d] [expr $y - $d] [expr $x + $d] [expr $y + $d] \
		-fill $color -outline black -tags " bubble bubbleInconsistencies bubblecircle bubblecirclenode=$n bubbleannotation=$a"
	$w raise bubblecircle
	update idletasks
}

proc BubbleInconsistenciesZoom {s} {
	global IDT S T P A Bubble
	set delta 1.0
	switch $s {
		-	{
			set S(SurfaceMaxBubble) [expr $S(SurfaceMaxBubble) -$delta]
		}
		+	{
			set S(SurfaceMaxBubble) [expr $S(SurfaceMaxBubble) +$delta]
		}
	}
	if {$S(SurfaceMaxBubble) < 0.0} {
		set S(SurfaceMaxBubble) 0.0
	}
	BubbleInconsistencies  
}

################################################################################


################################################################################
# Onglet C, comparaison AAs et Transitions
################################################################################
#  courbes proba comparaison plusieurs sets d'annotations (eg beast vs pam)
# node n
# remplacement par $S(AAIDTlist) pointant vers A
proc ScenarioMakeComparGraphic {w x y} {
	global IDT S T A
	set tags [$w gettags current]
	if {[lsearch $tags "LEGEND"] != -1} {
		set tag [lindex $tags [lsearch $tags LEGENDANNOTATION*]]
		set a [lindex [split $tag =] end]
		if {$a == "ALL"} {
			$S(canvas2) itemconfigure ACURVE -state normal
		} elseif {[string range $a 0 2] == "MAP"} {
			set state [lindex [$S(canvas2) itemconfigure CURVEIDT$a -state ] end]
			if {$state == "normal"} {
				$S(canvas2) itemconfigure CURVEIDT$a -state hidden
			} else {
				$S(canvas2) itemconfigure CURVEIDT$a -state normal
			}
		} else {
			set state [lindex [$S(canvas2) itemconfigure CURVEA$a -state ] end]
			if {$state == "normal"} {
				$S(canvas2) itemconfigure CURVEA$a -state hidden
			} else {
				$S(canvas2) itemconfigure CURVEA$a -state normal
			}
		}
	} else {
		set node [string range [lindex $tags [lsearch -glob $tags N*]] 1 end]
		if {$node != ""} {
			$S(canvas2) delete all
			set H [lindex [$S(canvas2) configure -width] end]
			# CANVAS 1 init taille trait 
			foreach n $T($IDT,all_cod) {
				$S(canvas) itemconfigure H$n -width 1 
			}
			# line width 3 scenario de la racine au noeud 
			LineWithNodeToRoot $node
			# line width 3 du noeud aux feuilles
			LineWidthNodeToTips $node
			# CANVAS 2 lignes horizontales marquant les noeuds internes
			set S(scenariopere) $node ;# init
			ScenarioPere $node
			foreach n $S(scenariopere) {
				set co [$S(canvas) coords H$n]
				set x1 [lindex $co 0] 
				set y1 0
				set y2 $H 
				$S(canvas2) create line $x1 $y1 $x1 $y2 -tags "V$n" -fill grey -dash {2 2}
			}
			# camemberts
			$S(canvas2) delete Cammenb
			set y -100
			foreach IDTi $S(AAIDTlist) {
				makeMapScenarioCamenb $S(canvas2) $S(scenariopere) $IDTi $y
				incr y 30
			}
			foreach IDTi $S(AAIDTlist) {
				ScenarioMakeComparCurve $IDTi $node $H
			}
			# scenario majoritaire
			ScenarioMakeMajoritaire $node $H
			# legende active
			set co [$S(canvas2) bbox all]
			set x [expr [lindex $co 2] + 10.0]
			set y 20
			set CommonAnnotationList {}
			foreach IDTi $S(AAIDTlist) {
				foreach a [lsort -dictionary $A($IDTi,AnnotationList)] {
					if {[lsearch -exact $CommonAnnotationList $a] == -1} {lappend CommonAnnotationList $a}
				}
			}
			foreach a [lsort -dictionary $CommonAnnotationList] {
				$S(canvas2) create text $x $y -text $a -fill $S(ColorAnnotation,$a) \
					-anchor w -tags "LEGEND LEGENDANNOTATION=$a"
				incr y 20
			}
			# toutes & map
			$S(canvas2) create text $x $y -text "*" -fill black \
					-anchor w -tags "LEGEND LEGENDANNOTATION=ALL"
			incr y 20
			foreach IDTi $S(AAIDTlist) {
				$S(canvas2) create text $x $y -text "MAP $IDTi" -fill black \
					-anchor w -tags "LEGEND LEGENDANNOTATION=MAP$IDTi"
				incr y 20
			}
			# scrollregion
			$S(canvas2) configure -scrollregion [$S(canvas2) bbox all]
		}
	}
}
#
# du noeud pointé à la racine
proc LineWithNodeToRoot {n} {
	global IDT T S
	$S(canvas) itemconfigure H$n -width 3 
	set p $T($IDT,pere,$n)
	if {$T($IDT,typ,$p) == "r"} {
		$S(canvas) itemconfigure H$p -width 3 
		return 
	} else {
		LineWithNodeToRoot $p
	}
}
# du noeud pointé à la racine
proc LineWidthNodeToTips {n} {
	global IDT T S
	$S(canvas) itemconfigure H$n -width 3 
	if {$T($IDT,typ,$n) == "f"} { 
		return 
	} else {
		set f1 $T($IDT,fils1,$n)
		set f2 $T($IDT,fils2,$n)
		LineWidthNodeToTips $f1
		LineWidthNodeToTips $f2
	}
}
proc makeMapScenarioCamenb {w ln IDTi ybase} {
	global IDT S T A 
	set delta [expr $S(SurfaceMax) +10]
	foreach n $ln {
		set co [$w coords V$n ]
		set x [lindex $co 0]
		set y [expr ([lindex $co 1]+[lindex $co 3])/2.0]
		# liste annotation proba couleur
		set apc {}
		# l'ID referancant les annotatios est dbv
		set nodea $T($IDT,dbv,$n)
		set sommeproba 0
		set sommeprobaINF 0
		set probaMax 0.0
		foreach {k v} [array get A $IDTi,$nodea,* ] {
			set a [lindex [split $k ,] end]
			lappend apc [list $a $v $S(ColorAnnotation,$a)]
			set sommeproba [expr $sommeproba + $v]
		}
		lappend apc [list "*" $sommeprobaINF black]
		##### AJOUTER a la liste APC un element AUTRE si la somme n'est pas 1
		#set somme [expr 1 - $sommeproba - $sommeprobaINF]
		set somme [expr 1 - $sommeproba ]
		if {$somme > 0.001} {
			lappend apc [list "?" [rnd $somme 2] grey]
		}
		# dessin
		piechart $w [expr $x - $delta/2.0 ] [expr $ybase + $y- $delta/2.0 -100] $delta $delta $n $apc
	}
	#
	$w raise Cammenb
	$w itemconfigure CammenbTXT -state hidden
	# affichage des ID nodes en dessous des camemberts
	foreach node $ln {
		set co [$w bbox Cammenb$node]
		set x [expr ([lindex $co 0] + [lindex $co 2]) / 2.0]
		set y [expr [lindex $co 3] + 10.0]
		$w create text $x $y -text $T($IDT,dbv,$node) -font {Helvetica 10 bold}
	}
	$w configure -scrollregion [$w bbox all]
	update idletasks
}
#
proc ScenarioMakeComparCurve {IDTi node H} {
	global IDT S T A
	set S(scenariopere) $node ;# init
	ScenarioPere $node ;# S(scenariopere)
	# du node à la racine 
	foreach a [lsort -dictionary $A($IDTi,AnnotationList)] {
		set lxy {}
		foreach n [lsort -dictionary -increasing $S(scenariopere)] {
			set co [$S(canvas) coords H$n]
			set x [lindex $co 0]
			set nodeA $T($IDT,dbv,$n)
			set y [expr $H - ($A($IDTi,$nodeA,$a) * $H)]
			lappend lxy $x $y
		}
		$S(canvas2) create line $lxy  \
			-fill $S(ColorAnnotation,$a) \
			-smooth 1 -splinesteps 100  \
			-tags "ACURVE CURVEA$a CURVEIDTMAP$IDTi"
	}
	# des feuilles au noeud
	if {$T($IDT,typ,$node) != "f"} {
		set feuilles [lsort -dictionary -increasing [Node2Leafs2 $node]]
		foreach f $feuilles {
			MultiScenarioMakeComparCurve $S(canvas2) $IDTi $node $T($IDT,ltc,$f) $H
		}
	}
}
proc MultiScenarioMakeComparCurve {w IDTi node f H} {
	global IDT S T AA A
	set S(scenariopere) $f
	ScenarioPere2 $node $f
	foreach a [lsort -dictionary  $A($IDTi,AnnotationList)] {
		set lxy {}
		foreach n [lsort -dictionary -increasing $S(scenariopere)] {
			set co [$S(canvas) coords H$n]
			set x [lindex $co 0]
			set nodeA $T($IDT,dbv,$n)
			set y [expr $H - ($A($IDTi,$nodeA,$a) * $H)]
			lappend lxy $x $y
		}
		$w create line $lxy -fill $S(ColorAnnotation,$a)  \
			-smooth 1 -splinesteps 100 -dash {2 2}  \
			-tags "ACURVE CURVEA$a CURVEIDTMAP$IDTi"
	}
}
# one tree several ancestral anotation sets, 
# une ligne de base (large width) par set d'annotation ancestrale
# une ligne finale pour la majoritaire inter set d'annotation
proc ScenarioMakeMajoritaire {node H} {
	global IDT S T A
	# identification majoritaire
	foreach IDTi $S(AAIDTlist) {
		foreach n $T($IDTi,all_cod) {
			set max 0
			# ID referancant les annotations est $T($IDT,dbv,$n)
			set nodei $T($IDTi,dbv,$n)
			foreach {k v} [array get A $IDTi,$nodei,*] {
				set a [lindex [split $k ,] end]
				if {$v > $max} {
					set max $v
					set A($IDTi,majoritaire,$n) $a
				}
			}
		}
	}
	# 
	set S(scenariopere) $node ;# init
	ScenarioPere $node ;# S(scenariopere)
	# du node à la racine
	foreach n [lsort -dictionary -increasing $S(scenariopere)] {
		set co [$S(canvas) coords H$n]
		set x1 [lindex $co 0]
		set x2 [lindex $co 2]
		set y 300
		foreach IDTi $S(AAIDTlist) {
			set a $A($IDTi,majoritaire,$n)
			#set p $A($IDTi,$n,$a)
			$S(canvas2) create rectangle $x1 $y $x2 [expr $y + 20] \
				-fill $S(ColorAnnotation,$a) -outline $S(ColorAnnotation,$a) \
				-tags "Scenario$IDTi"
			incr y 40
		}
	}
	#
	foreach IDTi $S(AAIDTlist) { 
		set co [$S(canvas2) bbox Scenario$IDTi]
		set x [lindex $co 2]
		set y [lindex $co 3]
		$S(canvas2) create text [expr $x + 20 ] [expr ([lindex $co 1] + [lindex $co 3]) / 2.0] \
			-text "MAP $IDTi" -font {Arial 10 normal}
	}
}
################################################################################


################################################################################
#  can2svg.tcl ---
#  This file provides translation from canvas commands to XML/SVG format.
#  Copyright (c) 2002  Mats Bengtsson
################################################################################
package provide can2svg 0.1

namespace eval ::can2svg:: {
	namespace export can2svg canvas2file
	variable formatArrowMarker
	variable formatArrowMarkerLast
	# The key into this array is 'arrowMarkerDef_$col_$a_$b_$c', where
	# col is color, and a, b, c are the arrow's shape.
	variable defsArrowMarkerArr
	# Similarly for stipple patterns.
	variable defsStipplePatternArr
	# This shouldn't be hardcoded!
	variable defaultFont {Helvetica 12}
	variable anglesToRadians [expr 3.14159265359/180.0]
	variable grayStipples {gray75 gray50 gray25 gray12}
	# Make 4x4 squares. Perhaps could be improved.
	variable stippleDataArr
	set stippleDataArr(gray75)  \
			{M 0 0 h3 M 0 1 h1 m 1 0 h2 M 0 2 h2 m 1 0 h1 M 0 3 h3}
	set stippleDataArr(gray50)  \
			{M 0 0 h1 m 1 0 h1 M 1 1 h1 m 1 0 h1 \
				M 0 2 h1 m 1 0 h1 M 1 3 h1 m 1 0 h1}
	set stippleDataArr(gray25)  \
			{M 0 0 h1 M 2 1 h1 M 1 2 h1 M 3 3 h1}
	set stippleDataArr(gray12) {M 0 0 h1 M 2 2 h1}
}
proc ::can2svg::can2svg {cmd args} {
	variable defsArrowMarkerArr
	variable defsStipplePatternArr
	variable anglesToRadians
	variable defaultFont
	variable grayStipples
	set nonum_ {[^0-9]}
	set wsp_ {[ ]+}
	set xml ""
	array set argsArr {-usetags all}
	array set argsArr $args
	switch -- [lindex $cmd 0] {
		create {
			set type [lindex $cmd 1]
			set rest [lrange $cmd 2 end]
			regexp -indices -- "-${nonum_}" $rest ind
			set ind1 [lindex $ind 0]
			set coo [string trim [string range $rest 0 [expr $ind1 - 1]]]
			set opts [string range $rest $ind1 end]
			array set optArr $opts
			# Figure out if we've got a spline.
			set haveSpline 0
			if {[info exists optArr(-smooth)] && ($optArr(-smooth) != "0") &&  \
						[info exists optArr(-splinesteps)] && ($optArr(-splinesteps) > 2)} {
				set haveSpline 1
			}
			if {[info exists optArr(-fill)]} {
				set fillValue $optArr(-fill)
			} else {
				set fillValue black
			}
			if {($argsArr(-usetags) != "0") && [info exists optArr(-tags)]} {
				switch -- $argsArr(-usetags) {
					all {
						set idAttr [list "id" $optArr(-tags)]
					}
					first {
						set idAttr [list "id" [lindex $optArr(-tags) 0]]
					}
					last {
						set idAttr [list "id" [lindex $optArr(-tags) end]]
					}
				}
			} else {
				set idAttr ""
			}
			# If we need a marker (arrow head) need to make that first.
			if {[info exists optArr(-arrow)]} {
				if {[info exists optArr(-arrowshape)]} {
					# Make a key of the arrowshape list into the array.
					regsub -all -- $wsp_ $optArr(-arrowshape) _ shapeKey
					set arrowKey ${fillValue}_${shapeKey}
					set arrowShape $optArr(-arrowshape)
				} else {
					set arrowKey ${fillValue}
					set arrowShape {8 10 3}
				}
				if {![info exists defsArrowMarkerArr($arrowKey)]} {
					set defsArrowMarkerArr($arrowKey)  \
							[eval {MakeArrowMarker} $arrowShape {$fillValue}]
					append xml $defsArrowMarkerArr($arrowKey)
					append xml "\n\t"
				}
			}
			# If we need a stipple bitmap, need to make that first. Limited!!!
			# Only: gray12, gray25, gray50, gray75
			foreach key {-stipple -outlinestipple} {
				if {[info exists optArr($key)] &&  \
							([lsearch $grayStipples $optArr($key)] >= 0)} {
					set stipple $optArr($key)
					if {![info exists defsStipplePatternArr($stipple)]} {
						set defsStipplePatternArr($stipple)  \
								[MakeGrayStippleDef $stipple]
					}
					append xml $defsStipplePatternArr($stipple)
					append xml "\n\t"
				}
			}
			switch -- $type {
				arc {
					
					# Had to do it the hard way! (?)
					# "Wrong" coordinate system :-(
					set elem "path"
					set style [MakeStyle $type $opts]
					foreach {x1 y1 x2 y2} $coo {}
					set cx [expr ($x1 + $x2)/2.0]
					set cy [expr ($y1 + $y2)/2.0]
					set rx [expr abs($x1 - $x2)/2.0]
					set ry [expr abs($y1 - $y2)/2.0]
					set rmin [expr $rx > $ry ? $ry : $rx]
					
					# This approximation gives a maximum half pixel error.
					set deltaPhi [expr 2.0/sqrt($rmin)]
					set extent [expr $anglesToRadians * $optArr(-extent)]
					set start [expr $anglesToRadians * $optArr(-start)]
					set nsteps [expr int(abs($extent)/$deltaPhi) + 2]
					set delta [expr $extent/$nsteps]
					set data [format "M %.1f %.1f L"   \
							[expr $cx + $rx*cos($start)] [expr $cy - $ry*sin($start)]]
					for {set i 0} {$i <= $nsteps} {incr i} {
						set phi [expr $start + $i * $delta]
						append data [format " %.1f %.1f"  \
								[expr $cx + $rx*cos($phi)] [expr $cy - $ry*sin($phi)]]
					}
					if {[info exists optArr(-style)]} {
						switch -- $optArr(-style) {
							chord {
								append data " Z"
							}
							pieslice {
								append data [format " %.1f %.1f Z" $cx $cy]
							}
						}
					} else {
						
						# Pieslice is the default.
						append data [format " %.1f %.1f Z" $cx $cy]
					}
					set attr [list "d" $data "style" $style]
					if {[string length $idAttr] > 0} {
						set attr [concat $attr $idAttr]
					}
					set xmlList [MakeXMLList $elem -attrlist $attr]
				}
				image - bitmap {
					set elem "image"
					set attr [MakeImageAttr $coo $opts]
					if {[string length $idAttr] > 0} {
						set attr [concat $attr $idAttr]
					}
					set xmlList [MakeXMLList $elem -attrlist $attr]
				}
				line {
					if {$haveSpline} {
						set elem "path"
						set style [MakeStyle $type $opts]
						set data "M [lrange $coo 0 1] Q"
						set i 4
						foreach {x y} [lrange $coo 2 end-4] {
							set x0 [expr ($x + [lindex $coo $i])/2.0]
							incr i
							set y0 [expr ($y + [lindex $coo $i])/2.0]
							incr i
							append data " $x $y $x0 $y0"
						}
						append data " [lrange $coo end-3 end]"
						set attr [list "d" $data "style" $style]
					} else {
						set elem "polyline"
						set style [MakeStyle $type $opts]
						set attr [list "points" $coo "style" $style]
					}
					if {[string length $idAttr] > 0} {
						set attr [concat $attr $idAttr]
					}
					set xmlList [MakeXMLList $elem -attrlist $attr]
				}
				oval {
					foreach {x y w h} [NormalizeRectCoords $coo] {}
					if {[expr $w == $h]} {
						set elem "circle"
						set attr [list  \
								"cx" [expr $x + $w/2.0]  \
								"cy" [expr $y + $h/2.0]  \
								"r"  [expr $w/2.0]]
					} else {
						set elem "ellipse"
						set attr [list  \
								"cx" [expr $x + $w/2.0]  \
								"cy" [expr $y + $h/2.0]  \
								"rx" [expr $w/2.0]	   \
								"ry" [expr $h/2.0]]
					}
					set style [MakeStyle $type $opts]
					lappend attr "style" $style
					if {[string length $idAttr] > 0} {
						set attr [concat $attr $idAttr]
					}
					set xmlList [MakeXMLList $elem -attrlist $attr]
				}
				polygon {
					
					if {$haveSpline} {
						set elem "path"
						set style [MakeStyle $type $opts]
						
						# Translating a closed polygon into a qubic bezier
						# path is a little bit tricky.
						set x0 [expr ([lindex $coo end-1] + [lindex $coo 0])/2.0]
						set y0 [expr ([lindex $coo end] + [lindex $coo 1])/2.0]
						set data "M $x0 $y0 Q"
						set i 2
						foreach {x y} [lrange $coo 0 end-2] {
							set x1 [expr ($x + [lindex $coo $i])/2.0]
							incr i
							set y1 [expr ($y + [lindex $coo $i])/2.0]
							incr i
							append data " $x $y $x1 $y1"
						}
						append data " [lrange $coo end-1 end] $x0 $y0"
						set attr [list "d" $data "style" $style]
					} else {
						
						set elem "polygon"
						set style [MakeStyle $type $opts]
						#puts "AVANT $type $opts APRES $style"
						set attr [list "points" $coo "style" $style]
					}
					if {[string length $idAttr] > 0} {
						set attr [concat $attr $idAttr]
					}
					set xmlList [MakeXMLList $elem -attrlist $attr]
				}
				rectangle {
					set elem "rect"
					set style [MakeStyle $type $opts]
					
					# width and height must be non-negative!
					foreach {x y w h} [NormalizeRectCoords $coo] {}
					set attr [list "x" $x "y" $y "width" $w "height" $h]
					lappend attr "style" $style
					if {[string length $idAttr] > 0} {
						set attr [concat $attr $idAttr]
					}
					set xmlList [MakeXMLList $elem -attrlist $attr]
				}
				text {
					set elem "text"
					set style [MakeStyle $type $opts]
					set nlines 1
					if {[info exists optArr(-text)]} {
						set chdata $optArr(-text)
						set nlines [expr [regexp -all "\n" $chdata] + 1]
					} else {
						set chdata ""
					}
					
					# Figure out the coords of the first baseline.
					set anchor center
					if {[info exists optArr(-anchor)]} {
						set anchor $optArr(-anchor)
					}
					if {[info exists optArr(-font)]} {
						set theFont $optArr(-font)
					} else {
						set theFont $defaultFont
					}
					set ascent [font metrics $theFont -ascent]
					set lineSpace [font metrics $theFont -linespace]
					
					foreach {xbase ybase}  \
							[GetTextSVGCoords $coo $anchor $chdata $theFont $nlines] {}
					
					set attr [list "x" $xbase "y" $ybase]
					lappend attr "style" $style
					if {[string length $idAttr] > 0} {
						set attr [concat $attr $idAttr]
					}
					set dy 0
					if {$nlines > 1} {
						
						# Use the 'tspan' trick here.
						set subList {}
						foreach line [split $chdata "\n"] {
							lappend subList [MakeXMLList "tspan"  \
									-attrlist [list "x" $xbase "dy" $dy] -chdata $line]
							set dy $lineSpace
						}
						set xmlList [MakeXMLList $elem -attrlist $attr \
								-subtags $subList]
					} else {
						set xmlList [MakeXMLList $elem -attrlist $attr \
								-chdata $chdata]
					}
				}
			}
		}
		move {
			foreach {tag dx dy} [lrange $cmd 1 3] {}
			set attr [list "transform" "translate($dx,$dy)"  \
					"xlink:href" "#$tag"]
			set xmlList [MakeXMLList "use" -attrlist $gattr]
		}
		scale {
			
		}
	}
	append xml [MakeXML $xmlList]
	return $xml
}
proc ::can2svg::MakeStyle {type opts} {
	# Defaults for everything except text.
	if {![string equal $type "text"]} {
		array set styleArr {fill none stroke black}
	}
	set fillCol black
	
	foreach {key value} $opts {
		
		switch -- $key {
			-arrow {
				set arrowValue $value
			}
			-arrowshape {
				set arrowShape $value
			}
			-capstyle {
				if {[string equal $value "projecting"]} {
					set value "square"
				}
				if {![string equal $value "butt"]} {
					set styleArr(stroke-linecap) $value
				}
			}
			-dash {
				set dashValue $value
			}
			-dashoffset {
				if {$value != 0} {
					set styleArr(stroke-dashoffset) $value
				}
			}
			-fill {
				set fillCol $value
				
				if {[string equal $type "line"]} {
					set styleArr(stroke) [MapEmptyToNone $value]
				} else {
					set styleArr(fill) [MapEmptyToNone $value]
				}
				
			}
			-font {
				set styleArr(font-family) [lindex $value 0]
				if {[llength $value] > 1} {
					set styleArr(font-size) [lindex $value 1]
				}
				if {[llength $value] > 2} {
					set tkstyle [lindex $value 2]
					switch -- $tkstyle {
						bold {
							set styleArr(font-weight) $tkstyle
						}
						italic {
							set styleArr(font-style) $tkstyle
						}
						underline {
							set styleArr(text-decoration) underline
						}
						overstrike {
							set styleArr(text-decoration) overline
						}
					}
				}
				
			}
			-joinstyle {
				set styleArr(stroke-linejoin) $value
			}
			-outline {
				set styleArr(stroke) [MapEmptyToNone $value]
			}
			-outlinestipple {
				set outlineStippleValue $value
			}
			-stipple {
				set stippleValue $value
			}
			-width {
				set styleArr(stroke-width) $value
			}
		}
	}
	
	# If any arrow specify its marker def url key.
	if {[info exists arrowValue]} {
		if {[info exists arrowShape]} {
			foreach {a b c} $arrowShape {}
			set arrowIdKey "arrowMarkerDef_${fillCol}_${a}_${b}_${c}"
			set arrowIdKeyLast "arrowMarkerLastDef_${fillCol}_${a}_${b}_${c}"
		} else {
			set arrowIdKey "arrowMarkerDef_${fillCol}"
		}
		switch -- $arrowValue {
			first {
				set styleArr(marker-start) "url(#$arrowIdKey)"
			}
			last {
				set styleArr(marker-end) "url(#$arrowIdKeyLast)"
			}
			both {
				set styleArr(marker-start) "url(#$arrowIdKey)"
				set styleArr(marker-end) "url(#$arrowIdKeyLast)"
			}
		}
	}
	
	if {[info exists stippleValue]} {
		# Overwrite any existing.
		set styleArr(fill) "url(#tile$stippleValue)"
	}
	if {[info exists outlineStippleValue]} {
		# Overwrite any existing.
		set styleArr(stroke) "url(#tile$stippleValue)"
	}
	# Transform dash value.
	if {[info exists dashValue]} {
		# Two different syntax here.
		if {[regexp {[\.,\-_ ]} $dashValue]} {
			# .=2 ,=4 -=6 space=4	times stroke width.
			# A space enlarges the... space.
			# Not foolproof!
			regsub -all -- {[^ ]} $dashValue "& " dash
			regsub -all -- "   "  $dash  "12 " dash
			regsub -all -- "  "   $dash  "8 " dash
			regsub -all -- " "	$dash  "4 " dash
			regsub -all -- {\.}   $dash  "2 " dash
			regsub -all -- {,}	$dash  "4 " dash
			regsub -all -- {-}	$dash  "6 " dash
			# Multiply with stroke width if > 1.
			if {[info exists styleArr(stroke-width)] &&  \
						($styleArr(stroke-width) > 1)} {
				set width $styleArr(stroke-width)
				set dashOrig $dash
				set dash {}
				foreach num $dashOrig {
					lappend dash [expr int($width * $num)]
				}
			}
			set styleArr(stroke-dasharray) [string trim $dash]
		} else {
			set styleArr(stroke-dasharray) $value
		}
	}
	if {[string equal $type "polygon"]} {
		set styleArr(fill-rule) "evenodd"
	}
	set style ""
	foreach {key value} [array get styleArr] {
		append style "${key}: ${value}; "
	}
	return [string trim $style]
}
proc ::can2svg::MakeImageAttr {coo opts} {
	array set optArr {-anchor nw}
	array set optArr $opts
	set theImage $optArr(-image)
	set w [image width $theImage]
	set h [image height $theImage]
	# We should make this an URI.
	set theFile [$theImage cget -file]
	set uri [UriFromLocalFile $theFile]
	foreach {x0 y0} $coo {}
	switch -- $optArr(-anchor) {
		nw {
			set x $x0
			set y $y0
		}
		n {
			set x [expr $x0 - $w/2.0]
			set y $y0
		}
		ne {
			set x [expr $x0 - $w]
			set y $y0
		}
		e {
			set x $x0
			set y [expr $y0 - $h/2.0]
		}
		se {
			set x [expr $x0 - $w]
			set y [expr $y0 - $h]
		}
		s {
			set x [expr $x0 - $w/2.0]
			set y [expr $y0 - $h]
		}
		sw {
			set x $x0
			set y [expr $y0 - $h]
		}
		w {
			set x $x0
			set y [expr $y0 - $h/2.0]
		}
		center {
			set x [expr $x0 - $w/2.0]
			set y [expr $y0 - $h/2.0]
		}
	}
	set attrList [list "x" $x "y" $y "width" $w "height" $h  \
			"xlink:href" $uri]
	return $attrList
}
proc ::can2svg::GetTextSVGCoords {coo anchor chdata theFont nlines} {
	foreach {x y} $coo {}
	set ascent [font metrics $theFont -ascent]
	set lineSpace [font metrics $theFont -linespace]
	# If not anchored to the west it gets more complicated.
	if {![string match $anchor "*w*"]} {
		# Need to figure out the extent of the text.
		if {$nlines <= 1} {
			set textWidth [font measure $theFont $chdata]
		} else {
			set textWidth 0
			foreach line [split $chdata "\n"] {
				set lineWidth [font measure $theFont $line]
				if {$lineWidth > $textWidth} {
					set textWidth $lineWidth
				}
			}
		}
	}
	switch -- $anchor {
		nw {
			set xbase $x
			set ybase [expr $y + $ascent]
		}
		w {
			set xbase $x
			set ybase [expr $y - $nlines*$lineSpace/2.0 + $ascent]
		}
		sw {
			set xbase $x
			set ybase [expr $y - $nlines*$lineSpace + $ascent]
		}
		s {
			set xbase [expr $x - $textWidth/2.0]
			set ybase [expr $y - $nlines*$lineSpace + $ascent]
		}
		se {
			set xbase [expr $x - $textWidth]
			set ybase [expr $y - $nlines*$lineSpace + $ascent]
		}
		e {
			set xbase [expr $x - $textWidth]
			set ybase [expr $y - $nlines*$lineSpace/2.0 + $ascent]
		}
		ne {
			set xbase [expr $x - $textWidth]
			set ybase [expr $y + $ascent]
		}
		n {
			set xbase [expr $x - $textWidth/2.0]
			set ybase [expr $y + $ascent]
		}
		center {
			set xbase [expr $x - $textWidth/2.0]
			set ybase [expr $y - $nlines*$lineSpace/2.0 + $ascent]
		}
	}
	
	return [list $xbase $ybase]
}
proc ::can2svg::MakeArrowMarker {a b c col} {
	variable formatArrowMarker
	variable formatArrowMarkerLast
	catch {unset formatArrowMarker}
	if {![info exists formatArrowMarker]} {
		# "M 0 c, b 0, a c, b 2*c Z" for the start marker.
		# "M 0 0, b c, 0 2*c, b-a c Z" for the last marker.
		set data "M 0 %s, %s 0, %s %s, %s %s Z"
		set style "fill: %s; stroke: %s;"
		set attr [list "d" $data "style" $style]
		set arrowList [MakeXMLList "path" -attrlist $attr]
		set markerAttr [list "id" %s "markerWidth" %s "markerHeight" %s  \
				"refX" %s "refY" %s "orient" "auto"]
		set defElemList [MakeXMLList "defs" -subtags  \
				[list [MakeXMLList "marker" -attrlist $markerAttr \
				-subtags [list $arrowList] ] ] ]
		set formatArrowMarker [MakeXML $defElemList]
		
		# ...and the last arrow marker.
		set dataLast "M 0 0, %s %s, 0 %s, %s %s Z"
		set attrLast [list "d" $dataLast "style" $style]
		set arrowLastList [MakeXMLList "path" -attrlist $attrLast]
		set defElemLastList [MakeXMLList "defs" -subtags  \
				[list [MakeXMLList "marker" -attrlist $markerAttr \
				-subtags [list $arrowLastList] ] ] ]
		set formatArrowMarkerLast [MakeXML $defElemLastList]
	}
	set idKey "arrowMarkerDef_${col}_${a}_${b}_${c}"
	set idKeyLast "arrowMarkerLastDef_${col}_${a}_${b}_${c}"
	
	# Figure out the order of all %s substitutions.
	set markerXML [format $formatArrowMarker $idKey  \
			$b [expr 2*$c] 0 $c  \
			$c $b $a $c $b [expr 2*$c] $col $col]
	set markerLastXML [format $formatArrowMarkerLast $idKeyLast  \
			$b [expr 2*$c] $b $c \
			$b $c [expr 2*$c] [expr $b-$a] $c $col $col]
	
	return "$markerXML\n\t$markerLastXML"
}
proc ::can2svg::MakeGrayStippleDef {stipple} {
	
	variable stippleDataArr
	
	set pathList [MakeXMLList "path" -attrlist  \
			[list "d" $stippleDataArr($stipple) "style" "stroke: black; fill: none;"]]
	set patterAttr [list "id" "tile$stipple" "x" 0 "y" 0 "width" 4 "height" 4 \
			"patternUnits" "userSpaceOnUse"]
	set defElemList [MakeXMLList "defs" -subtags  \
			[list [MakeXMLList "pattern" -attrlist $patterAttr \
			-subtags [list $pathList] ] ] ]
	
	return [MakeXML $defElemList]
}
proc ::can2svg::MapEmptyToNone {val} {
	
	if {[string length $val] == 0} {
		return "none"
	} else {
		return $val
	}
}
proc ::can2svg::NormalizeRectCoords {coo} {
	
	foreach {x1 y1 x2 y2} $coo {}
	return [list [expr $x2 > $x1 ? $x1 : $x2]  \
			[expr $y2 > $y1 ? $y1 : $y2]  \
			[expr abs($x1-$x2)]  \
			[expr abs($y1-$y2)]]
}
proc ::can2svg::makedocument {width height xml} {
	
	#set pre "<?xml version='1.0'?>\n\
	#		<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.0//EN\"\
	#		\"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\">"
	set pre "<?xml version='1.0' encoding=\"utf-8\"?>\n"
	set svgStart "<svg width='$width' height='$height'>"
	set svgEnd "</svg>"
	return "${pre}\n${svgStart}\n${xml}${svgEnd}"
}

proc ::can2svg::makedocument {width height xml} {
	set pre "<?xml version=\"1.0\"?>
	<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"
	\"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">"
	
	#		   <?xml version='1.1'?>\n\
	#				   <!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"\
	#				   \"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\">"
	#set svgStart "<svg width='$width' height='$height'>"
	set svgStart "<svg  xmlns=\"http://www.w3.org/2000/svg\"> \n <g id=\"g\" transform=\"translate(10, 10) scale(1.5)\">"
	set svgEnd " </g>\n</svg>"
	return "${pre}\n${svgStart}\n${xml}${svgEnd}"
}

proc ::can2svg::canvas2file {wcan path args} {
	variable defsArrowMarkerArr
	variable defsStipplePatternArr
	# Need to make a fresh start for marker def's.
	catch {unset defsArrowMarkerArr}
	catch {unset defsStipplePatternArr}
	#array set argsArr  \
	#		[list -width [winfo width $wcan] -height [winfo height $wcan]]
	# correction chevenet
	$wcan configure -scrollregion [$wcan bbox all]
	$wcan xview moveto 0
	$wcan yview moveto 0
	set width [expr [lindex [$wcan bbox all] 2] - [lindex [$wcan bbox all] 0]]
	set height [expr [lindex [$wcan bbox all] 3] - [lindex [$wcan bbox all] 1]]
	array set argsArr   [list -width $width -height $height]
	array set argsArr $args
	set fd [open $path w]
	set xml ""
	# ici pour la modification sur les tags // hidden
	foreach id [$wcan find all] {
		set type [$wcan type $id]
		set opts [$wcan itemconfigure $id]
		set opcmd {}
		foreach opt $opts {
			set op [lindex $opt 0]
			set val [lindex $opt 4]
			
			# Empty val's except -fill can be stripped off.
			if {![string equal $op "-fill"] && ([string length $val] == 0)} {
				continue
			}
			lappend opcmd $op $val
		}
		set co [$wcan coords $id]
		set cmd [concat "create" $type $co $opcmd]
		append xml "\t[can2svg $cmd]\n"
	}
	puts $fd [makedocument $argsArr(-width) $argsArr(-height) $xml]
	close $fd
}
proc ::can2svg::MakeXML {xmlList} {
	# Extract the XML data items.
	foreach {tag attrlist isempty chdata childlist} $xmlList {}
	set rawxml "<$tag"
	foreach {attr value} $attrlist {
		append rawxml " ${attr}='${value}'"
	}
	if {$isempty} {
		append rawxml "/>"
		return $rawxml
	} else {
		append rawxml ">"
	}
	# Call ourselves recursively for each child element.
	# There is an arbitrary choice here where childs are put before PCDATA.
	foreach child $childlist {
		append rawxml [MakeXML $child]
	}
	# Make standard entity replacements.
	if {[string length $chdata]} {
		append rawxml [XMLCrypt $chdata]
	}
	append rawxml "</$tag>"
	return $rawxml
}
proc ::can2svg::MakeXMLList {tagname args} {
	
	# Fill in the defaults.
	array set xmlarr {-isempty 1 -attrlist {} -chdata {} -subtags {}}
	
	# Override the defults with actual values.
	if {[llength $args] > 0} {
		array set xmlarr $args
	}
	if {!(($xmlarr(-chdata) == "") && ($xmlarr(-subtags) == ""))} {
		set xmlarr(-isempty) 0
	}
	
	# Build sub elements list.
	set sublist {}
	foreach child $xmlarr(-subtags) {
		lappend sublist $child
	}
	set xmlList [list $tagname $xmlarr(-attrlist) $xmlarr(-isempty)  \
			$xmlarr(-chdata) $sublist]
	return $xmlList
}
proc ::can2svg::XMLCrypt {chdata} {
	foreach from {\& < > {"} {'}}   \
			to {{\&amp;} {\&lt;} {\&gt;} {\&quot;} {\&apos;}} {
				regsub -all $from $chdata $to chdata
			}
	return $chdata
	#"
}
proc ::can2svg::UriFromLocalFile {path} {
	if {[string equal $::tcl_platform(platform) "windows"]} {
		# Trim the volume specifier.
		set vol_ {([A-Z]:/|[A-Z]:\\)}
		regexp "${vol_}+(.+)$" $path match x path
	}
	set pathList [split $path "/:\\"]
	set pathJoin [join $pathList /]
	# Any characters not 7bit encode as %xx hex???
	regsub -all -- " " $pathJoin "%20" pathJoin
	return file:///${pathJoin}
}
################################################################################

################################################################################
# TOOL
################################################################################
# ::math::random
#	   args	optional arguments that specify the range within which to
#			   choose a number:
#					   (null)		  choose a number between 0 and 1
#					   val			 choose a number between 0 and val
#					   val1 val2	   choose a number between val1 and val2
namespace eval ::math {
proc ::math::random {args} {
	set num [expr {rand()}]
	if { [llength $args] == 0 } {
		return $num
	} elseif { [llength $args] == 1 } {
		return [expr {int($num * [lindex $args 0])}]
	} elseif { [llength $args] == 2 } {
		foreach {lower upper} $args break
		set range [expr {$upper - $lower}]
		return [expr {int($num * $range) + $lower}]
	} else {
		set fn [lindex [info level 0] 0]
		error "wrong # args: should be \"$fn ?value1? ?value2?\""
	}
}
}

### DelRep suppression des repetitions d'une liste
proc DelRep {lin}   {
	set lou {}
	foreach i $lin {
		if {[lsearch -exact $lou $i] == -1} {lappend lou $i}
	}
	return $lou
}
#
proc rnd {value decimalplaces} {
	return [expr {round(10**$decimalplaces*$value)/10.0**$decimalplaces} ]
}

# n -> liste des labels feuilles
proc Node2Leafs2 {n} {
	global IDT A T S
	if {$T($IDT,typ,$n) == "f"} {
		return $T($IDT,ctl,$n)
	} else { 
		set l1 [Node2Leafs2 $T($IDT,fils1,$n)]
		set l2 [Node2Leafs2 $T($IDT,fils2,$n)]
		#lappend l $l1 $l2
		set l [concat $l1 " $l2"]
		return $l
	}
}
proc Node2LeafsCode2 {np n} {
	global IDT A T S
	if {$T($IDT,typ,$n) == f} {
		return $n
	} else {
		set l1 [Node2LeafsCode2 $np $T($IDT,fils1,$n)]
		set l2 [Node2LeafsCode2 $np $T($IDT,fils2,$n)]
		set l [concat $l1 " $l2"]
		return $l
	}
}
# renvoie que les feuilles sous n
proc Node2NodesLeaf {n} {
	global IDT A T S
	if {$T($IDT,typ,$n) == "f"} {
		return $n
	} else {
		set l1 [Node2NodesLeaf $T($IDT,fils1,$n)]
		set l2 [Node2NodesLeaf $T($IDT,fils2,$n)]
		set l [concat $l1 " $l2"]
		return $l
	}
}
# renvoie que les noeuds internes sous n
proc Node2SubTreeNodes {n} {
	global IDT A T S
	if {$T($IDT,typ,$n) == "f"} {
		return
	} else {
		set l1 [Node2SubTreeNodes $T($IDT,fils1,$n)]
		set l2 [Node2SubTreeNodes $T($IDT,fils2,$n)]
		set l [concat $n " $l1" " $l2"]
		return $l
	}
}
# renvoie les noeuds internes ET les feuilles sous n
proc Node2NodesAll {n} {
	global IDT A T S
	if {$T($IDT,typ,$n) == "f"} {
		return $n
	} else {
		set l1 [Node2NodesAll $T($IDT,fils1,$n)]
		set l2 [Node2NodesAll $T($IDT,fils2,$n)]
		set l [concat $n "$l1" " $l2"]
		return $l
	}
}

### prise en compte character missing data
proc operatorAND {l1 l2} {
	set lintersect {}
	if {[lsearch -exact $l1 *] != -1}  {
		return $l2
	} elseif {[lsearch -exact $l2 *] != -1} {
		return $l1
	} else  {
		foreach i $l1 {
			if {[lsearch -exact $l2 $i] != -1 && [lsearch -exact $lintersect $i] == -1} {lappend lintersect $i}
		}
		return $lintersect
	}
}
### prise en compte character missing data
proc operatorAND2 {l1 l2} {
	set lintersect {}
	foreach i $l1 {
		if {[lsearch -exact $l2 $i] != -1 && [lsearch -exact $lintersect $i] == -1} {lappend lintersect $i}
	}
	return $lintersect
}
################################################################################


################################################################################
# TOOLBOX ICONS
################################################################################
package require Tk
package require Ttk

image create photo zoom+ -data {iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADg
dz34AAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA
/mIAADn4AAAQm00sP1EAAAHKSURBVHja1NbLalRBEAbg73ghIoiIxER0IV5CFkZiFA0ERAVR
ELcuRB/GhY/iRlFEBAMmIoZARAkhIEQRBdGFl5UuvCBpN3Wkac6cmRGzsODQ3dXV/Vd11V8z
VUrJWso6aywb6klVVU374ziL0XDmDWYxj9W2i/+8TEpJwzPtwz2kDt8CjncDSCk1ApzEl7jo
J2ZwDVdxB19jbxVX+gXYi4+Zl2MNZ3cHUMIvnOoH4GYcfIotmf127CqK4kbYLuW5bAM4GmGn
mOdyF98xmOmG8Cnsz3cCyD06jQpzeFbYb8RA7NfyAdMxP9ELDw7EuBzjpkjwMqZC9xArmIz1
Upa7dh40SIXhePuB0O2Meb1O/TD5VYx15XyLXGzDo9AdjvVcRkR43Z0QHMuINFGY3Q/9jkw3
mJX0hV7L9HbGgc2Z/R4cLJ70etguFslvBdiPz3FwPnpQKcOZIwkvg3yNAFV9edbszuBWEO1H
VNJCsHYc57C1uO95NMX3vTa7ETxoaXZP8LbQvcgjaYsgL9OJaH6jWB/V8ji+sXBiKDuzEi/w
rlsEvcqRYHQZyUhbu+5XDjWAXPyXADXh6sZ3qdsPzt/KFC53TPJ/+6/i9wDiBh8YFeXDsgAA
AABJRU5ErkJggg==}

image create photo zoom-   -data {iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADg
dz34AAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA
/mIAADn4AAAQm00sP1EAAAGlSURBVHja1JbPK0RRFMc/D0VTstJMsfJjms1oGkIpoUTJ1kL8
MRb+FBsiCwuFDU2NSJMoI1Fi4ceKhUaaa/N9ur3uvPdGZuHU7d7eu+d+7znf8z3vecYYGmlN
NNha/IXnea73OWAGyOgyd8ABUACqYQf/ZMYYgyNNvcAOYGqMIjASBWCMcQJMAG866BPYB1aB
FWAbeNe7KrBcL0AP8GzdMuvw7RaQAb6AyXoANuR4ArRH8LeuvSWbyzCAIYVttI6yJPCi/XO1
AOwynQI84Ag4jQHwBOxqPR5HB/2azzW3ieALpcEfZWBUe0oWd+E6cJgHpIAukelbAmj1MxFb
aMCNZr9yPsSFS4EVS4gAt9GCgGFLSPkYHHRaJT0ft0y3LA0kIgDWtPfMFWUtgD7gVY4F9aCg
payLGOBa4nMCeP7hVrObBjYltIoqqSiic8As0BE471JN8TFus0sDeyHN7hi4Dzwr25GERWCX
aV7NLwM0q1oONbK6RNLyuVIGHqIiiGuDUnQwknRYu67XBhwgC38J4AvOb3yLUR+c39oYsFST
5H/7V/E9ANKqHH8m+SkbAAAAAElFTkSuQmCC}
global IDT zoom-

image create photo zoomf -data {iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADg
dz34AAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHnIAACA2AAA+xgAAH/3AAB6eAAA
+FcAADvgAAANTOIlrKwAAAFOSURBVHja7NW9SxxRFAXw36xfiIJWVqKFW/gPWAQC6QU7u4Bg
k0Is/AsE0UIQKwubkCAWm0ZLa21sLFKmsbUxKqKi+Dlp7sAwzK4bcRHBCw/e3HffOe++c2Ym
SdNUK6OixfH+CdqzSZIka5jGI5Im9ye4xxU+4ShbyLRtzxX3owcHeGiS5BEjGERbaUWaphlb
Dbfo+89bWEGKoTLcdyHyQ6sJFvAFFw1d9IL4jMm4/184f02RlwP4DH9jvl6K+wKC8QCcz1lz
LnJTr0FQw2FJfh+7RYK8Bk/oxAQuCy9OBdfYiQOclhAcY7iRBpvRZr1xhwHMxvNYDmY0PhlL
jTpYxVbB1zfhtCQ6usMPfMMefkbua9R1Neqg7lpxoBffw0En2MDv6GyxnshNgRfqO3OnHsKf
IJl5liA2Vopk6EB3HYdVsY1qVp98/JPfnODfAMdA4EqiojwfAAAAAElFTkSuQmCC}

image create photo zoomw -data {iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADg
dz34AAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHnIAACA2AAA+xgAAH/3AAB6eAAA
+FcAADvgAAANTOIlrKwAAAEwSURBVHja7NW7K8VhGAfwz09nOi4ZyKZMyialDBZiUAYlk0GR
rC6D/4MRKZNyGWWhFPInKIMRCymH3F7LSyeXc/g5kvKtt+cdnp7v+1ze75OEEPwkyvww/j5B
5vmSJMkYmksU9yCEsABCCGKjNxFKdFaf42byWK+iXcFhypc3oh+5NyXKw1zMJg26I0HBJme/
Ufvsh00ugnYMoQmPOMIitr87plVYxg5aMYkp1GMLa6hOS1CDPQzgFr2oxAxGcYY+7Ebfwv/g
HdzjOt5vol1C7aupC3hIk8EFOrERSzWNLrRgEA3YRwfO02QAl+jBMMbRFjOrwARmcfcpqSiC
+XjqkODky1qUh1wB/9Mi8XKfIRiJc59WKj4kKI+2vwRqmn2PYB3HpZLrlzXwv5N/neBpAPFu
WA8Fi9VRAAAAAElFTkSuQmCC}

image create photo zoomy+ -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAQAAABKfvVzAAAACXBIWXMAAAsTAAALEwEAmpwYAAAA
82lDQ1BQaG90b3Nob3AgSUNDIHByb2ZpbGUAAHjardA/SgNBHMXx70TULloE67mAgmu15eQPi2AR
YopsqkxmBw1hd4fZn3/2Bh7B43iDFIIXUVJbqARLwU/1eM2DB+rZDPujzjGUlcRsYvJZPteHGw7Y
AwDrmmDG4yuAqq48vynYvqEAXk/NsD/ib/ZdiAK8AEnhGwd8AKsHCQJqAfSW6yCgBOitp5MBqCeg
K/5RAAZ1aOPq5lb0eZqm2hT10uvrthFfNvqycnUMdbTiC+B7A+Aoi7bVmS1Lq5OzhH83y+f6K71P
UYA62ey6HXcX73/+7FzAJ0sbODDOTdGSAAAAIGNIUk0AAHolAACAgwAA+f8AAIDpAAB1MAAA6mAA
ADqYAAAXb5JfxUYAAAGOSURBVHjatNTPSxRhHAbwz2zGRiAhYRruIaxkD21stlQghAViEF07RP0x
HfpTuhSJRJDgFqEIG0WIEFiEQdRB7VSHfiD77bAzOzutgR563oF5Z/g+7/PO83zfScL+UNpnvQGS
/KluVlXJR8+saBdLo3DjpCeiZ7RczCp6R0aY9k34remeu+Z9F9ruZITOlRPGbQktta5+xbyw40qx
NiM8FF4ZBEeNpVY8EFYN5PvJCA1toZGu/dhPw2DEtnA9/9QQSrgqsex1SjionBq3aQGX+3M4jTUc
0rRmCs+tu4RVjEdqTGQ5dJEYNaaM48rKvZ4XFT6ghh8ahrzAOUOWUceGvvwuCGEyffFUOAaGbQk3
dsthTmg5DE44k270vvBG0m8rp3wVVlS7wqPmhPBeRSG4JDoeznhk0C9NLTvqrjmSUt+a9UVIumGk
mLBY6LKXPqWzdyp5G2YKHVsnTas6YMOSJTWLRsC6GZ//bu/dcN5mV2Wi2N7/wtku5ebeCNRtC7fY
K4Ept/OTl/z3v8afAQAKRMNgJfDHOgAAAABJRU5ErkJggg==}

image create photo zoomy- -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAQAAABKfvVzAAAACXBIWXMAAAsTAAALEwEAmpwYAAAA
82lDQ1BQaG90b3Nob3AgSUNDIHByb2ZpbGUAAHjardA/SgNBHMXx70TULloE67mAgmu15eQPi2AR
YopsqkxmBw1hd4fZn3/2Bh7B43iDFIIXUVJbqARLwU/1eM2DB+rZDPujzjGUlcRsYvJZPteHGw7Y
AwDrmmDG4yuAqq48vynYvqEAXk/NsD/ib/ZdiAK8AEnhGwd8AKsHCQJqAfSW6yCgBOitp5MBqCeg
K/5RAAZ1aOPq5lb0eZqm2hT10uvrthFfNvqycnUMdbTiC+B7A+Aoi7bVmS1Lq5OzhH83y+f6K71P
UYA62ey6HXcX73/+7FzAJ0sbODDOTdGSAAAAIGNIUk0AAHolAACAgwAA+f8AAIDpAAB1MAAA6mAA
ADqYAAAXb5JfxUYAAAFwSURBVHjatNQxaxRhEMbx354HSiBYSQKmihquOTnioYIgiRAiiK2F6Iex
8KPYKIYUFkLOJhI4McghBoxIBNEiiZUWEjluLO6yt+9uFFP4bLG7MP+Zd56Z3SwcT7VjxquTjd9a
ljXUfPLChkEaGsmNc56JwtV1pQyEyIEF34VfOh56YNUPYeB+MX0RmLUndDXzgBmrQt9imvwQeCK8
Nlnq77HQUx8f8xBoGwjtiiFT9oVb41ZDqOGGzEubFWDXc1yvzuEC3uKUjnd6enq2XUUPs6mt9QKc
mXZWH0w4WfQ8GZyPaOKndmGKB2hhR2V+l4UwX0l2xp5wu+oSK0LXRAl4JLyRHTWH874JGxp58LQV
IXwwk9qaxfDYS56adKCjq6/lptMjdMuyr4rUSHPWkuV75fPoaXtYJZIKQ1vnLWg4Yce6dU1rpsB7
S76U1/soXbKbV5lL1/tPupgjd/4NoGVfuFv+gP6ma+4lTf/nv8bvAQA8I7TSe6+ODQAAAABJRU5E
rkJggg==}

image create photo TMzoomy- -data {
iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAlwSFlzAAAL
EwAACxMBAJqcGAAAAwBpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADx4OnhtcG1ldGEgeG1sbnM6
eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IlhNUCBDb3JlIDUuNC4wIj4KICAgPHJkZjpSREYg
eG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4K
ICAgICAgPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIKICAgICAgICAgICAgeG1sbnM6dGlm
Zj0iaHR0cDovL25zLmFkb2JlLmNvbS90aWZmLzEuMC8iCiAgICAgICAgICAgIHhtbG5zOnhtcD0i
aHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wLyIKICAgICAgICAgICAgeG1sbnM6cGhvdG9zaG9w
PSJodHRwOi8vbnMuYWRvYmUuY29tL3Bob3Rvc2hvcC8xLjAvIj4KICAgICAgICAgPHRpZmY6WVJl
c29sdXRpb24+NzI8L3RpZmY6WVJlc29sdXRpb24+CiAgICAgICAgIDx0aWZmOlhSZXNvbHV0aW9u
PjcyPC90aWZmOlhSZXNvbHV0aW9uPgogICAgICAgICA8eG1wOkNyZWF0ZURhdGU+MjAxMy0wMS0w
OFQwMjowMzoxMjwveG1wOkNyZWF0ZURhdGU+CiAgICAgICAgIDx4bXA6TW9kaWZ5RGF0ZT4yMDEz
LTAxLTA4VDAyOjAzOjEyPC94bXA6TW9kaWZ5RGF0ZT4KICAgICAgICAgPHhtcDpDcmVhdG9yVG9v
bD5QaG90b3MgNC4wPC94bXA6Q3JlYXRvclRvb2w+CiAgICAgICAgIDxwaG90b3Nob3A6RGF0ZUNy
ZWF0ZWQ+MjAxMy0wMS0wOFQwMjowMzoxMjwvcGhvdG9zaG9wOkRhdGVDcmVhdGVkPgogICAgICA8
L3JkZjpEZXNjcmlwdGlvbj4KICAgPC9yZGY6UkRGPgo8L3g6eG1wbWV0YT4KXZYONwAAAO9JREFU
OBGVk4ENgjAQRVEXYARGYAScQDaQEdiAmZxANxAnECfABYz+l/awNjSmP/m937t/d5BAUfyi1/Uq
vj3RR/EvSjnCRhtgkRqeJEZVME9iK1ae6EmkxpBV9Mpac7niIDd5T6cIBhfcOSowoA2TkaYWPgV6
ARfIJtNxrIKaZPHiAFsXss+lD3Hz7Y3iJsHae8zrry70CjzyXeQ1YpCjhqcTweDC9xwlMWA8iJUn
mhw1PEmwxYZgjmm1s2p4k+hUMbNtJQcuIjk+qFrMBptt+CzdZE9QA5ufor1iJ52NRh02YN5lt7v/
4qG+vXj6ABNNUnMjwBKAAAAAAElFTkSuQmCC
}

image create photo TMzoomy+ -data {
iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAlwSFlzAAAL
EwAACxMBAJqcGAAAAwBpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADx4OnhtcG1ldGEgeG1sbnM6
eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IlhNUCBDb3JlIDUuNC4wIj4KICAgPHJkZjpSREYg
eG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4K
ICAgICAgPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIKICAgICAgICAgICAgeG1sbnM6dGlm
Zj0iaHR0cDovL25zLmFkb2JlLmNvbS90aWZmLzEuMC8iCiAgICAgICAgICAgIHhtbG5zOnhtcD0i
aHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wLyIKICAgICAgICAgICAgeG1sbnM6cGhvdG9zaG9w
PSJodHRwOi8vbnMuYWRvYmUuY29tL3Bob3Rvc2hvcC8xLjAvIj4KICAgICAgICAgPHRpZmY6WVJl
c29sdXRpb24+NzI8L3RpZmY6WVJlc29sdXRpb24+CiAgICAgICAgIDx0aWZmOlhSZXNvbHV0aW9u
PjcyPC90aWZmOlhSZXNvbHV0aW9uPgogICAgICAgICA8eG1wOkNyZWF0ZURhdGU+MjAxMy0wMS0w
OFQwMjowMzoxMjwveG1wOkNyZWF0ZURhdGU+CiAgICAgICAgIDx4bXA6TW9kaWZ5RGF0ZT4yMDEz
LTAxLTA4VDAyOjAzOjEyPC94bXA6TW9kaWZ5RGF0ZT4KICAgICAgICAgPHhtcDpDcmVhdG9yVG9v
bD5QaG90b3MgNC4wPC94bXA6Q3JlYXRvclRvb2w+CiAgICAgICAgIDxwaG90b3Nob3A6RGF0ZUNy
ZWF0ZWQ+MjAxMy0wMS0wOFQwMjowMzoxMjwvcGhvdG9zaG9wOkRhdGVDcmVhdGVkPgogICAgICA8
L3JkZjpEZXNjcmlwdGlvbj4KICAgPC9yZGY6UkRGPgo8L3g6eG1wbWV0YT4KXZYONwAAAPZJREFU
OBGdk40NgkAUg1EXYARGYAScQDeQEdiAmZxANxAnECfABYz2kyueRGKwSe+9a98PF2OSfKLS9SQ+
Asl34k+kqogbPcARj5pJNHIobsWtmAWStyIeQ76ikurmNFR4M1e0VkQrRVD3oT8bBcxtJMYDkPHQ
/BV3RMPFbHI+jlnk0TcMWHL7A0MfyTkMKBQXgUEa7nkQXGv/FSudfPJF5BnATyBHw0MrRVD34X02
SimgcCNmgeRoeNRMgi0eQvGY9g7yqJ1EKcfF3ooGjiIaP2UuzgabPbxTXsyeoAY230Q/sVQ+G4U6
PKBbzW7v/xdX9a3F/RMgEVhp+UgCJwAAAABJRU5ErkJggg==
}

image create photo TMzoomx- -data {
iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAlwSFlzAAAL
EwAACxMBAJqcGAAAAwBpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADx4OnhtcG1ldGEgeG1sbnM6
eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IlhNUCBDb3JlIDUuNC4wIj4KICAgPHJkZjpSREYg
eG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4K
ICAgICAgPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIKICAgICAgICAgICAgeG1sbnM6dGlm
Zj0iaHR0cDovL25zLmFkb2JlLmNvbS90aWZmLzEuMC8iCiAgICAgICAgICAgIHhtbG5zOnhtcD0i
aHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wLyIKICAgICAgICAgICAgeG1sbnM6cGhvdG9zaG9w
PSJodHRwOi8vbnMuYWRvYmUuY29tL3Bob3Rvc2hvcC8xLjAvIj4KICAgICAgICAgPHRpZmY6WVJl
c29sdXRpb24+NzI8L3RpZmY6WVJlc29sdXRpb24+CiAgICAgICAgIDx0aWZmOlhSZXNvbHV0aW9u
PjcyPC90aWZmOlhSZXNvbHV0aW9uPgogICAgICAgICA8eG1wOkNyZWF0ZURhdGU+MjAxMy0wMS0w
OFQwMjowMzoxMjwveG1wOkNyZWF0ZURhdGU+CiAgICAgICAgIDx4bXA6TW9kaWZ5RGF0ZT4yMDEz
LTAxLTA4VDAyOjAzOjEyPC94bXA6TW9kaWZ5RGF0ZT4KICAgICAgICAgPHhtcDpDcmVhdG9yVG9v
bD5QaG90b3MgNC4wPC94bXA6Q3JlYXRvclRvb2w+CiAgICAgICAgIDxwaG90b3Nob3A6RGF0ZUNy
ZWF0ZWQ+MjAxMy0wMS0wOFQwMjowMzoxMjwvcGhvdG9zaG9wOkRhdGVDcmVhdGVkPgogICAgICA8
L3JkZjpEZXNjcmlwdGlvbj4KICAgPC9yZGY6UkRGPgo8L3g6eG1wbWV0YT4KXZYONwAAAOhJREFU
OBGdkoENglAMRFEXYARGYAScQDbwj8AGzuQEuoE4gTgBLmD0XmwDwv8xeMmlpXe9QkKWfaPR40V8
Gen34k/kcowXPcArGp4kWimYO7EWCyN9J6IREkWjqS/nEQezzjxBdYZWEwLqmTIM0JJvgQC55P20
FiNN7YD10P7XEXC11Up1lWBpHvfa46c0KrzyTeQzpmCGhieIUbSaeshOfWGk92U8SXDFQwia0rWT
NLxJBCluJoSeGTiLzPihSnExuOzhvfoqlnDQ8GnkGmTm4PJDdC24sKRWMntAv1myad5O9S5uxeMb
iqZUbfajddcAAAAASUVORK5CYII=
}

image create photo TMzoomx+ -data {
iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAlwSFlzAAAL
EwAACxMBAJqcGAAAAwBpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADx4OnhtcG1ldGEgeG1sbnM6
eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IlhNUCBDb3JlIDUuNC4wIj4KICAgPHJkZjpSREYg
eG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4K
ICAgICAgPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIKICAgICAgICAgICAgeG1sbnM6dGlm
Zj0iaHR0cDovL25zLmFkb2JlLmNvbS90aWZmLzEuMC8iCiAgICAgICAgICAgIHhtbG5zOnhtcD0i
aHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wLyIKICAgICAgICAgICAgeG1sbnM6cGhvdG9zaG9w
PSJodHRwOi8vbnMuYWRvYmUuY29tL3Bob3Rvc2hvcC8xLjAvIj4KICAgICAgICAgPHRpZmY6WVJl
c29sdXRpb24+NzI8L3RpZmY6WVJlc29sdXRpb24+CiAgICAgICAgIDx0aWZmOlhSZXNvbHV0aW9u
PjcyPC90aWZmOlhSZXNvbHV0aW9uPgogICAgICAgICA8eG1wOkNyZWF0ZURhdGU+MjAxMy0wMS0w
OFQwMjowMzoxMjwveG1wOkNyZWF0ZURhdGU+CiAgICAgICAgIDx4bXA6TW9kaWZ5RGF0ZT4yMDEz
LTAxLTA4VDAyOjAzOjEyPC94bXA6TW9kaWZ5RGF0ZT4KICAgICAgICAgPHhtcDpDcmVhdG9yVG9v
bD5QaG90b3MgNC4wPC94bXA6Q3JlYXRvclRvb2w+CiAgICAgICAgIDxwaG90b3Nob3A6RGF0ZUNy
ZWF0ZWQ+MjAxMy0wMS0wOFQwMjowMzoxMjwvcGhvdG9zaG9wOkRhdGVDcmVhdGVkPgogICAgICA8
L3JkZjpEZXNjcmlwdGlvbj4KICAgPC9yZGY6UkRGPgo8L3g6eG1wbWV0YT4KXZYONwAAAPNJREFU
OBGdkwERglAQRFELEIEIRsAE0sAfgQZmMoE2EBOICbCAo/v0Tr74GQd3Zv33d/fuYByy7BO1rkfx
bqTeiD+RKxE3+gA/8ciMopFDuBUrsTBStyIeQ5KopXpzbgnfzBWtFdGC+IVGCmYVOfEAZDy05FN4
mE1eD88i8lT2mPflfxUDTtZa6pwZTXrflyZ41v3nWeuXRz6LvAbwV6BGw0MLYhKNVAIE12JhpPZm
MqNgiw9h0JDu7eWRHUWQ42GGUKOBg4jGX7kUJ4PNPrxTXaYmbCWy5WYnNZqDzVcRHQZxMkp1+IBu
Mbn99V1c1LcSdw+ecFpnUTehtwAAAABJRU5ErkJggg==
}

image create photo zoomx- -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAQAAABKfvVzAAAACXBIWXMAAAsTAAALEwEAmpwYAAAA
82lDQ1BQaG90b3Nob3AgSUNDIHByb2ZpbGUAAHjardA/SgNBHMXx70TULloE67mAgmu15eQPi2AR
YopsqkxmBw1hd4fZn3/2Bh7B43iDFIIXUVJbqARLwU/1eM2DB+rZDPujzjGUlcRsYvJZPteHGw7Y
AwDrmmDG4yuAqq48vynYvqEAXk/NsD/ib/ZdiAK8AEnhGwd8AKsHCQJqAfSW6yCgBOitp5MBqCeg
K/5RAAZ1aOPq5lb0eZqm2hT10uvrthFfNvqycnUMdbTiC+B7A+Aoi7bVmS1Lq5OzhH83y+f6K71P
UYA62ey6HXcX73/+7FzAJ0sbODDOTdGSAAAAIGNIUk0AAHolAACAgwAA+f8AAIDpAAB1MAAA6mAA
ADqYAAAXb5JfxUYAAAGDSURBVHjalNMxaxRRFIbhZ9aAEghWkoCpooZtVpa4qCBIIoQIYmsh+mMs
/Ck2iiGFhZC1iQRWFBlEIRGJIFoksdJCIssei92d3JlZRM/AcO/Mee/h+865Wfi/mILseN+2pqnh
sxe2DcrJMXwVNc55JpKn50oViARY9kP4reuhBzb8FAbup8kpsOBA6GkVx83bEPpW0vRj4Inw2kxF
4WMhNxU1oGMgdGqWzDoUbo0Fh9AAN2ReelMD9j3H9fTTELiAdzil671cLrfrKnIsJKYO+1BEZs5Z
fTDtZJE1+hkF8Akt/NJJuniENvbqzbgshKWahjMOhNsxwdZ1oWe6AjwS3som9eG878K2ZpE8Z10I
H82ntmYxHr5VT8040tXT13bT6RH6wZpvk4Zv0WZp+F75MlrtjqukFYbOLVnWdMKeLVtaNs2CHau+
VitMikv2iyqL5fFO25muLxbInToQFSBGt/BQuFu9QEqCE+Nxzb2S6LFiJQfGu7LGRmX4/HVPWuHf
4s8A4KLPvKiLt10AAAAASUVORK5CYII=}

image create photo TMzoomxfont+ -data {
iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAlwSFlzAAAL
EwAACxMBAJqcGAAAAwBpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADx4OnhtcG1ldGEgeG1sbnM6
eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IlhNUCBDb3JlIDUuNC4wIj4KICAgPHJkZjpSREYg
eG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4K
ICAgICAgPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIKICAgICAgICAgICAgeG1sbnM6dGlm
Zj0iaHR0cDovL25zLmFkb2JlLmNvbS90aWZmLzEuMC8iCiAgICAgICAgICAgIHhtbG5zOnhtcD0i
aHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wLyIKICAgICAgICAgICAgeG1sbnM6cGhvdG9zaG9w
PSJodHRwOi8vbnMuYWRvYmUuY29tL3Bob3Rvc2hvcC8xLjAvIj4KICAgICAgICAgPHRpZmY6WVJl
c29sdXRpb24+NzI8L3RpZmY6WVJlc29sdXRpb24+CiAgICAgICAgIDx0aWZmOlhSZXNvbHV0aW9u
PjcyPC90aWZmOlhSZXNvbHV0aW9uPgogICAgICAgICA8eG1wOkNyZWF0ZURhdGU+MjAxMy0wMS0w
OFQwMjowMzoxMjwveG1wOkNyZWF0ZURhdGU+CiAgICAgICAgIDx4bXA6TW9kaWZ5RGF0ZT4yMDEz
LTAxLTA4VDAyOjAzOjEyPC94bXA6TW9kaWZ5RGF0ZT4KICAgICAgICAgPHhtcDpDcmVhdG9yVG9v
bD5QaG90b3MgNC4wPC94bXA6Q3JlYXRvclRvb2w+CiAgICAgICAgIDxwaG90b3Nob3A6RGF0ZUNy
ZWF0ZWQ+MjAxMy0wMS0wOFQwMjowMzoxMjwvcGhvdG9zaG9wOkRhdGVDcmVhdGVkPgogICAgICA8
L3JkZjpEZXNjcmlwdGlvbj4KICAgPC9yZGY6UkRGPgo8L3g6eG1wbWV0YT4KXZYONwAAAQxJREFU
OBGlk7FNw1AURQNUdBnBDRJlRjATkA3wCB4gPbOkS/pIsAFmAkyREoUNyDnhX0gsUlhc6fjff9/7
79uKMpmcqmX7Al8F/QMMdU2whccUppjjgxmQ1Zo90Q1mlY1rBzb3MIeqoO/BmkOiDeYDFgYt5PDU
AOVmvVkPZg2oW1geHI8OLM4TlL1ZZM193uJkgAXxpvjhWh3VsL9vcOnuP3LAaxlQs14USvSzn5Ug
vakf1panr/wGfobKJ+jNrJk18Kc6UhtsvIeqoM9he87KWzLEQUNSe6Jm71k1VNLsEL2ZegYzf8oZ
jJY3Z/gOX4+ewAFv/oR8YoMfrZoTGbC7Gn38+3/xzrk7WO8B6NJcUXCL5JMAAAAASUVORK5CYII=
}


image create photo TMzoom- -data {
iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAlwSFlzAAAL
EwAACxMBAJqcGAAAAwBpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADx4OnhtcG1ldGEgeG1sbnM6
eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IlhNUCBDb3JlIDUuNC4wIj4KICAgPHJkZjpSREYg
eG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4K
ICAgICAgPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIKICAgICAgICAgICAgeG1sbnM6dGlm
Zj0iaHR0cDovL25zLmFkb2JlLmNvbS90aWZmLzEuMC8iCiAgICAgICAgICAgIHhtbG5zOnhtcD0i
aHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wLyIKICAgICAgICAgICAgeG1sbnM6cGhvdG9zaG9w
PSJodHRwOi8vbnMuYWRvYmUuY29tL3Bob3Rvc2hvcC8xLjAvIj4KICAgICAgICAgPHRpZmY6WVJl
c29sdXRpb24+NzI8L3RpZmY6WVJlc29sdXRpb24+CiAgICAgICAgIDx0aWZmOlhSZXNvbHV0aW9u
PjcyPC90aWZmOlhSZXNvbHV0aW9uPgogICAgICAgICA8eG1wOkNyZWF0ZURhdGU+MjAxMy0wMS0w
OFQwMjowMzoxMjwveG1wOkNyZWF0ZURhdGU+CiAgICAgICAgIDx4bXA6TW9kaWZ5RGF0ZT4yMDEz
LTAxLTA4VDAyOjAzOjEyPC94bXA6TW9kaWZ5RGF0ZT4KICAgICAgICAgPHhtcDpDcmVhdG9yVG9v
bD5QaG90b3MgNC4wPC94bXA6Q3JlYXRvclRvb2w+CiAgICAgICAgIDxwaG90b3Nob3A6RGF0ZUNy
ZWF0ZWQ+MjAxMy0wMS0wOFQwMjowMzoxMjwvcGhvdG9zaG9wOkRhdGVDcmVhdGVkPgogICAgICA8
L3JkZjpEZXNjcmlwdGlvbj4KICAgPC9yZGY6UkRGPgo8L3g6eG1wbWV0YT4KXZYONwAAARJJREFU
OBGdkr1Nw1AURs3PAKEnkiuERJOCAcwEyQbxCB6AkoaKMSgQQkwQNoiZIM4EYYLAOY93JRPFhfmk
8+7/fc+Si+KvGsI1fGf0l9BXSfAML3AFSRPO/mAsCGvNHnUP13AJj5DUctrcwQLKjH4H1lyivD2U
/IYohidR6VlzHdhTwwNM4QKeoGjB4sJgQNbiFTf4b/AKt5AKFr1Je4yyl8ctTuFM59zjH9rHjJs+
c1BhTwaY5Z7ozeGvaTA+ewN+xqHMWbOnhqNqycaSOX6Z0Y9hewblLbHERYdEbUXN3kHVVKLZJfrm
1AeYW8MMRsubY/kOvxq9gQFv/oL4xBp/tComYsEu/U0jV3T0b+EO3n8Ab4xXp0WA3IcAAAAASUVO
RK5CYII=
}

image create photo TMzoom+ -data {
iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAlwSFlzAAAL
EwAACxMBAJqcGAAAAwBpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADx4OnhtcG1ldGEgeG1sbnM6
eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IlhNUCBDb3JlIDUuNC4wIj4KICAgPHJkZjpSREYg
eG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4K
ICAgICAgPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIKICAgICAgICAgICAgeG1sbnM6dGlm
Zj0iaHR0cDovL25zLmFkb2JlLmNvbS90aWZmLzEuMC8iCiAgICAgICAgICAgIHhtbG5zOnhtcD0i
aHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wLyIKICAgICAgICAgICAgeG1sbnM6cGhvdG9zaG9w
PSJodHRwOi8vbnMuYWRvYmUuY29tL3Bob3Rvc2hvcC8xLjAvIj4KICAgICAgICAgPHRpZmY6WVJl
c29sdXRpb24+NzI8L3RpZmY6WVJlc29sdXRpb24+CiAgICAgICAgIDx0aWZmOlhSZXNvbHV0aW9u
PjcyPC90aWZmOlhSZXNvbHV0aW9uPgogICAgICAgICA8eG1wOkNyZWF0ZURhdGU+MjAxMy0wMS0w
OFQwMjowMzoxMjwveG1wOkNyZWF0ZURhdGU+CiAgICAgICAgIDx4bXA6TW9kaWZ5RGF0ZT4yMDEz
LTAxLTA4VDAyOjAzOjEyPC94bXA6TW9kaWZ5RGF0ZT4KICAgICAgICAgPHhtcDpDcmVhdG9yVG9v
bD5QaG90b3MgNC4wPC94bXA6Q3JlYXRvclRvb2w+CiAgICAgICAgIDxwaG90b3Nob3A6RGF0ZUNy
ZWF0ZWQ+MjAxMy0wMS0wOFQwMjowMzoxMjwvcGhvdG9zaG9wOkRhdGVDcmVhdGVkPgogICAgICA8
L3JkZjpEZXNjcmlwdGlvbj4KICAgPC9yZGY6UkRGPgo8L3g6eG1wbWV0YT4KXZYONwAAARZJREFU
OBGVk81NAlEURsefAiwBOpi1CcloA+rCtVPCdGLApVVYAXQgVCBWAAWY6DnDvTAQWbwvObz7vvvz
5jGZqjpWx/YTfgPjFxjqls0MpnCViRuCYWMOyNWcNeoVLuEZJgZqATV8wxOMA2M9c3NQ9jj4J+Kq
C2PNmqfkyVi9Z06vhXt4g3e4hmoJJh/dhIYDtMzpeRXVN+7CXcKkp2fj6Toa5AgPyv/g4BRGDlhF
T8N6EYS139dhZG3m+7Xj10f+Aq+h8grGeub0WvhXS1wLLHyAUWCczdaclafkEAedkrk5OWvPqiWT
xQ4x1lML0PNV1lAsT87hG+KmeAINnryFvGJLXKyGjhyw2X+OBWPW1PqB3cHHH/NQWbOXrkfPAAAA
AElFTkSuQmCC

}



image create photo TMzoomxfont- -data {
iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAAXNSR0IArs4c6QAAAAlwSFlzAAAL
EwAACxMBAJqcGAAAAwBpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADx4OnhtcG1ldGEgeG1sbnM6
eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IlhNUCBDb3JlIDUuNC4wIj4KICAgPHJkZjpSREYg
eG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4K
ICAgICAgPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIKICAgICAgICAgICAgeG1sbnM6dGlm
Zj0iaHR0cDovL25zLmFkb2JlLmNvbS90aWZmLzEuMC8iCiAgICAgICAgICAgIHhtbG5zOnhtcD0i
aHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wLyIKICAgICAgICAgICAgeG1sbnM6cGhvdG9zaG9w
PSJodHRwOi8vbnMuYWRvYmUuY29tL3Bob3Rvc2hvcC8xLjAvIj4KICAgICAgICAgPHRpZmY6WVJl
c29sdXRpb24+NzI8L3RpZmY6WVJlc29sdXRpb24+CiAgICAgICAgIDx0aWZmOlhSZXNvbHV0aW9u
PjcyPC90aWZmOlhSZXNvbHV0aW9uPgogICAgICAgICA8eG1wOkNyZWF0ZURhdGU+MjAxMy0wMS0w
OFQwMjowMzoxMjwveG1wOkNyZWF0ZURhdGU+CiAgICAgICAgIDx4bXA6TW9kaWZ5RGF0ZT4yMDEz
LTAxLTA4VDAyOjAzOjEyPC94bXA6TW9kaWZ5RGF0ZT4KICAgICAgICAgPHhtcDpDcmVhdG9yVG9v
bD5QaG90b3MgNC4wPC94bXA6Q3JlYXRvclRvb2w+CiAgICAgICAgIDxwaG90b3Nob3A6RGF0ZUNy
ZWF0ZWQ+MjAxMy0wMS0wOFQwMjowMzoxMjwvcGhvdG9zaG9wOkRhdGVDcmVhdGVkPgogICAgICA8
L3JkZjpEZXNjcmlwdGlvbj4KICAgPC9yZGY6UkRGPgo8L3g6eG1wbWV0YT4KXZYONwAAAQdJREFU
OBGlk7FNxFAQRA1EZFeCEyTCK8FUwHWAS3AB5NRCBjkSdICpABNciI4O4D35z2FO58BipNHO7s7u
97fsqvqLjvQVfheqb+AxXFK8TmOFmA5mQaI9PcE5YgvvUugRmge4gXWheoD2XBJcIB6SdIgMr1Kc
RGsD1NNC8QQ/4a1JD21uTGZgb/oU3v8+XhvSk6IPYz3pIav9glOz/8AFb2VBQzyZ4bp44i3pGDqC
j/wOvcYhrNnT00Kxv8KY/r5IjX4cdaE6wz16Fp6iwVOOMb1n+npn0dKJ2UVqa+IFWvODWsPF8OQs
36GbxRsY8OQvmGu26MVomMiC3dni8fG/+GDuCj7+AHMyVtAzqx0rAAAAAElFTkSuQmCC}



image create photo zoomx+ -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAQAAABKfvVzAAAACXBIWXMAAAsTAAALEwEAmpwYAAAA
82lDQ1BQaG90b3Nob3AgSUNDIHByb2ZpbGUAAHjardA/SgNBHMXx70TULloE67mAgmu15eQPi2AR
YopsqkxmBw1hd4fZn3/2Bh7B43iDFIIXUVJbqARLwU/1eM2DB+rZDPujzjGUlcRsYvJZPteHGw7Y
AwDrmmDG4yuAqq48vynYvqEAXk/NsD/ib/ZdiAK8AEnhGwd8AKsHCQJqAfSW6yCgBOitp5MBqCeg
K/5RAAZ1aOPq5lb0eZqm2hT10uvrthFfNvqycnUMdbTiC+B7A+Aoi7bVmS1Lq5OzhH83y+f6K71P
UYA62ey6HXcX73/+7FzAJ0sbODDOTdGSAAAAIGNIUk0AAHolAACAgwAA+f8AAIDpAAB1MAAA6mAA
ADqYAAAXb5JfxUYAAAGXSURBVHjalNRBaxNRFAXgb2IlIgYpUltpF1K1ZGE01qBCQapSKohbFyL4
48SNYpEiKLSKpBRSFCkFoYpUEF20da9i6HWRySSTxEXehWHem3veuffcwyRhuDUCSWdftais4KvX
1h3kk6P1yDjOeCG6ouFqO7ETSbQZ5i0r+atuQ9NFtxwTHnpE5GkCpu0JDZXsw5TnQtONPEMb8FR4
pwROmFQAT4RNI/2AmgOhlt697LcxMG5fuNPN0brppsSa9yngsGIq3K5XuN6tYwtwDls4YtWWObyx
7Ro2Md0qO+nMIVuJCZOKOKWoqEegDsMXVPBLzai3uGTUGqrY0SfrFSHMpkcvhZNgzJ5wN9UnJ+uS
0HAUnHY+LfWx8EEyaA5n/RTWlTPqCUtC+GxqsDUWPFPyx6qGpqrbjqfQjxb9iB5rwIyVnAs2fEvf
Pplq53YYWrLOmld2yI66uooV42Dbgu+99taje+Cy3Yxlprvpvvqy3YUMcq8XEAODqn3hfn4O/2eA
OQ9yTbc7RnQZM9pHObsN+9coDJnv3wBpr9piHh75tgAAAABJRU5ErkJggg==}

image create photo s-swap -data {iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAAD
gdz34AAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wA
A/mIAADn4AAAQm00sP1EAAAF3SURBVHja7NW/S1ZRGAfwz9UEJY02h4Z+qmtI2lBJ+gc0vVv
g0FSLODq5O7g4udXQ3NYkCFIOgtHW5A8QiYYiaDDN4rQ8b1xe3nve+wYiQQcunHN5nu/3Oef
7nO8pUkrOcvQ44/HvE1xoToqiyMWNYRqjkbOPN9iuSvijbUpJRugreIkTpDbfGm5XEaSUsgT
j+NgC+BVf2hDNdEtwHZ9KAC9wD5dxKapewk+sY7hbglcl8KcZbcYxGPNrdQnGorKE5zWb5Rm
+oVGHYD7ATzBSA3yutNtfeFxF0BcJqxG8E+u+DPgjHOK4VNQ+HrYjmMC76JSEI3zAkwzBYBS
wFTmv0Y+LVUc029J+qx12AEP4HPHLdTRo4BQrNQVeaHcfOrVp+XbeyoDP4HuAv0dvNzd5II7
oCIu4GT7UixvxrynuDzzo1irutOhxij3sxjyVwBt/40VwF28rjC5hE5M5syua4B3s+j6mcDX
WB9gI8qxdF//f5HMn+D0ALGYZGBn5bZIAAAAASUVORK5CYII=}

image create photo s-size-+ -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAABwSURBVHja
7NWxCsJgDIXRk9ah1M1B0Pd/NUHcWydJl78+gFgomjuFS+CDm5BEZtpSnY1VgD8AHNYiIr66
r5kZPxYRThjwwAsXBI6Ym3dvvVf0GDEhcWveGc939OupqBkU4GNFPZwC7B+wAAAA//8DADkK
GjpZAkUbAAAAAElFTkSuQmCC}

image create photo s-color -data {iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAA
Dgdz34AAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+w
AA/mIAADn4AAAQm00sP1EAAAEJSURBVHja7NU7SgRBEMbx34h4DK+hC6OosIHZJoqCsZ5CMD
ExFDUQERETH3gKb2Bmbuoj8IGytkkFzbK6IzObyBZMUM3w/b+q7q4uUkqGGWOGHCPA4EgpaW
CjF7HSV7cBQIkuEjpNA2bwhTNsBqjTFGAen7iMfC+q6GK5LmA2xK8jPwrxhCe06wDKaMtV5M
eZ+DOm6rRoIZxf9HH+gOk6m9zGC84jP8zEH6OyWsd0LcS2cdDTltaP96sCYAIbPZDceevXC1
wBMBliu5GvZs7LgROiAmApc7yfVTJXaQRlgB3cZd9J/HuaAd6w/pcZN56tfeA1hAq8x/o9tn
CD2+h75ShGT+b/B3wPAHOA5Y0MW+/1AAAAAElFTkSuQmCC}

image create photo infowhat -data {iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAAD
gdz34AAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wA
A/mIAADn4AAAQm00sP1EAAAIESURBVHja1NbPi45RFAfwzzPkR02MUQyNl1IjNprGjGzsmMl
OslMSaTY2in9gilJIsjEpKSXNQjEWdspGypStMGES86uYYiiPhfPo6fXcp7GYhVNv5773nnO
+9557v+c8WZ7nFlNaLLIsLQZZlqVsWrEXu9FAhgk8w2PMVjn9yUye5xJpWoGzeIc88ZvEEFZ
XAeR5ngTYhuelQA9wCgPoxyBG8CPWX6J3oQBdeB+OD7GzJsVduBO2s2WQFMBKvAiH64mg63A
ex0tz58LnLdrrAM6E4aOaXd8spW5raf52zF1KAbTiA75jew3AYKTjXpy4kA34jDl0VAEciB2
M1gRfgjasSazfiBjHirhlou0JPZrgy9F4WW9wucyhkhS+vVVM3hT6VYVjA8swEyc4ErpZxkN
3VAEU458Vjq9xCzvi/xNMVxG4KAxVABOhtyTyezKeKAyXgpWlM/SnKoCnoQcSl3sixpO4m9h
Ef+ixvyn9O6cz+Fpxiu7S278fd9DdZNOOKcyjkSLaUAQZaXI+VAL4EkVweZPNtVgfrmPyqrj
QPOhfSBsu4Ar6KlJzOnymgnC1xW5XMDWPl7O5hnjrcTVsv2HfQst1X+kkc8HQwwHeg4ORkum
w+Yj9/9IPYC0uRs5TDWc+qm5nquFkRfCalrkxdtcT+W2JHY9F1R2va5nZf/9V8WsAlWIkYWY
kRXgAAAAASUVORK5CYII=}

image create photo g-size -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAABwSURBVHja
7NWxCsJgDIXRk9ah1M1B0Pd/NUHcWydJl78+gFgomjuFS+CDm5BEZtpSnY1VgD8AHNYiIr66
r5kZPxYRThjwwAsXBI6Ym3dvvVf0GDEhcWveGc939OupqBkU4GNFPZwC7B+wAAAA//8DADkK
GjpZAkUbAAAAAElFTkSuQmCC}

image create photo g-color -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAEJSURBVHja
7NU7SgRBEMbx34h4DK+hC6OosIHZJoqCsZ5CMDExFDUQERETH3gKb2Bmbuoj8IGytkkFzbK6
IzObyBZMUM3w/b+q7q4uUkqGGWOGHCPA4EgpaWCjF7HSV7cBQIkuEjpNA2bwhTNsBqjTFGAe
n7iMfC+q6GK5LmA2xK8jPwrxhCe06wDKaMtV5MeZ+DOm6rRoIZxf9HH+gOk6m9zGC84jP8zE
H6OyWsd0LcS2cdDTltaP96sCYAIbPZDceevXC1wBMBliu5GvZs7LgROiAmApc7yfVTJXaQRl
gB3cZd9J/HuaAd6w/pcZN56tfeA1hAq8x/o9tnCD2+h75ShGT+b/B3wPAHOA5Y0MW+/1AAAA
AElFTkSuQmCC}

image create photo n-what -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAIESURBVHja
1NbPi45RFAfwzzPkR02MUQyNl1IjNprGjGzsmMlOslMSaTY2in9gilJIsjEpKSXNQjEWdspG
ypStMGES86uYYiiPhfPo6fXcp7GYhVNv5773nnO+9557v+c8WZ7nFlNaLLIsLQZZlqVsWrEX
u9FAhgk8w2PMVjn9yUye5xJpWoGzeIc88ZvEEFZXAeR5ngTYhuelQA9wCgPoxyBG8CPWX6J3
oQBdeB+OD7GzJsVduBO2s2WQFMBKvAiH64mg63Aex0tz58LnLdrrAM6E4aOaXd8spW5raf52
zF1KAbTiA75jew3AYKTjXpy4kA34jDl0VAEciB2M1gRfgjasSazfiBjHirhlou0JPZrgy9F4
WW9wucyhkhS+vVVM3hT6VYVjA8swEyc4ErpZxkN3VAEU458Vjq9xCzvi/xNMVxG4KAxVABOh
tyTyezKeKAyXgpWlM/SnKoCnoQcSl3sixpO4m9hEf+ixvyn9O6cz+Fpxiu7S278fd9DdZNOO
KcyjkSLaUAQZaXI+VAL4EkVweZPNtVgfrmPyqrjQPOhfSBsu4Ar6KlJzOnymgnC1xW5XMDWP
l7O5hnjrcTVsv2HfQst1X+kkc8HQwwHeg4ORkumw+Yj9/9IPYC0uRs5TDWc+qm5nquFkRfCa
lrkxdtcT+W2JHY9F1R2va5nZf/9V8WsAlWIkYWYkRXgAAAAASUVORK5CYII=}

image create photo g-dash -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAFwSURBVHja
7NS9a1VBEAXw3+bjGQwYYiFokUYULZR0wVrs/E8CEQRbIeRPUOxFG0EQ1MYmkrSKlfmwiFUE
sUiiaKK+xLGZJ+vNjV1AyBu4sOzZmTN75uwtEeEwY8AhR5/gCBAM9RallFuYQu3bTUxjFhca
uau4jXsYr/YLFiLiDogI+Rbms3j9dTGC5RZsGcfyTBN72qs7VDG/wFbeqmTiBr7jMS5iOM92
sYIfuI+TiQV2sfDnOr2XXEoZz24/YQ+nk2gU27n3MfPOYBDH8S0Lf8i9U9iJiK2mRE/y4A1M
tlw7cB1XD8CuYCbXD3t1axd1qsEf5K7Bf2ClMk2nTaIJjGEtJbmUSSfwNSVawi+cT81H8SW7
fpeSncVmRKz/ZVNcw+VMHMgBfk4r3sS5dA38xFvcxVw21knyLl7jQXMGL1t03c3Br7Rgr1Ky
vRbsWZtN57PrOjaywHO8b2BvstijtGkdi/tm0P/Z9Qn+X4LfAwCxIqRg0nQVGQAAAABJRU5E
rkJggg==}

image create photo moveall -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAF+SURBVHja
1JbPK0VREMc/5/2gJ6WXYkkKyYayQJYvys4/YCHxD9hbslQWsmFpYyNlQVnxB9iJhYVkQzaU
H11jM4d5p/veO7feW5g6Nd37ne/3zMw5c68TEVppOVpsBe8452Lx37rqmq9MlgwckKhfbEWJ
ivLXsEQFmybQLiKfPvsssTGgLtOnnO7ciUgScwKdB0U0OS8iSeyhiGlyLzBuYwy5ZR8B+mJK
VAIGgSXgCHgAxmpg88bvAW6AE2AZGADaqlLRdBaBL92pX/tKAFAwWC/QCWwEMQKs/mJNUAk4
SAE/AwsmC5/JFHCXgr8AutMEfD0/UoIEWDEic8B7DVylqjKBAMAucAUcBoEvWpJR4C14dw6c
6aKRwBAwq/6aIdnWZ5cB+ZY+nwAmYwRCW1eifmA+IN+rdw9iBTqAHfWPDfkjUG6GgBcpax+8
wGajm5xFAGA6KM9MjECWcT1s/FfgutnfAztvnvQCxn8yI+weONVxcmuHX9S4/rd/FT8DAKie
73KPw2zVAAAAAElFTkSuQmCC}

image create photo g-move -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAF+SURBVHja
1JbPK0VREMc/5/2gJ6WXYkkKyYayQJYvys4/YCHxD9hbslQWsmFpYyNlQVnxB9iJhYVkQzaU
H11jM4d5p/veO7feW5g6Nd37ne/3zMw5c68TEVppOVpsBe8452Lx37rqmq9MlgwckKhfbEWJ
ivLXsEQFmybQLiKfPvsssTGgLtOnnO7ciUgScwKdB0U0OS8iSeyhiGlyLzBuYwy5ZR8B+mJK
VAIGgSXgCHgAxmpg88bvAW6AE2AZGADaqlLRdBaBL92pX/tKAFAwWC/QCWwEMQKs/mJNUAk4
SAE/AwsmC5/JFHCXgr8AutMEfD0/UoIEWDEic8B7DVylqjKBAMAucAUcBoEvWpJR4C14dw6c
6aKRwBAwq/6aIdnWZ5cB+ZY+nwAmYwRCW1eifmA+IN+rdw9iBTqAHfWPDfkjUG6GgBcpax+8
wGajm5xFAGA6KM9MjECWcT1s/FfgutnfAztvnvQCxn8yI+weONVxcmuHX9S4/rd/FT8DAKie
73KPw2zVAAAAAElFTkSuQmCC}

image create photo g-move2 -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAF+SURBVHja
1JbPK0VREMc/5/2gJ6WXYkkKyYayQJYvys4/YCHxD9hbslQWsmFpYyNlQVnxB9iJhYVkQzaU
H11jM4d5p/veO7feW5g6Nd37ne/3zMw5c68TEVppOVpsBe8452Lx37rqmq9MlgwckKhfbEWJ
ivLXsEQFmybQLiKfPvsssTGgLtOnnO7ciUgScwKdB0U0OS8iSeyhiGlyLzBuYwy5ZR8B+mJK
VAIGgSXgCHgAxmpg88bvAW6AE2AZGADaqlLRdBaBL92pX/tKAFAwWC/QCWwEMQKs/mJNUAk4
SAE/AwsmC5/JFHCXgr8AutMEfD0/UoIEWDEic8B7DVylqjKBAMAucAUcBoEvWpJR4C14dw6c
6aKRwBAwq/6aIdnWZ5cB+ZY+nwAmYwRCW1eifmA+IN+rdw9iBTqAHfWPDfkjUG6GgBcpax+8
wGajm5xFAGA6KM9MjECWcT1s/FfgutnfAztvnvQCxn8yI+weONVxcmuHX9S4/rd/FT8DAKie
73KPw2zVAAAAAElFTkSuQmCC}

image create photo g-movesubtree -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAFtSURBVHja
1Na/S1dRGAbwjyGC4iKCLkIhqIhLgoNJOEWBm/+Ag4T+A+7Roo2CQ7lY0CCCS0gNBQ6S4uqk
2OAg4pDikqA4XJdXOFyufu+37nfohcM9vPec5+E8749zmrIs00h7pNGWZZk6TjGA8bpw6yBo
wS7O0dMIgk/IYuwEYWUE8wn43VgrQ1AmyB3x3U98J/iN3iqDnEq0VRb3oRN0Y7gE8SAel6mD
VvRhGl9ChqclCLpwiG94HbK1FEk0hZtcID8GALxN/J/D146FggSYLcqiVqwWLD7HZAB+wNeY
P8NRwfqf6LwvTQdxXbApw0wi6ytc3bPuRa06WMYe1nMbL0KSIVzm/m3iR4yahdaPlzGfS0CW
wredA18M/whG/6aS3wTQE0zkwFeqaBVteB/zjQT8NKnyf+5FbQF2kRC8q7qbjuXkeV5Vs0sv
mzv7g4Oqr8y035xFAda05joIjvE92smvkKmmNf33r4rbAQCb5hicW9khOwAAAABJRU5ErkJg
gg==}
image create photo g-shrink -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAADtSURBVHja
1NahSkRBFIfx3xVd9gEWNBusgtEX2G0bfAVNBruIT2AwikZBhK1Wg2jyBXwBwSAYBIuIY7kj
A16U3Z0T/Ofh+2bmnDNMk1ISmQXBCRdIKal8TWPsf3MrClZxgYRJTUEfB3ht4QnntQRjPBTg
aoINXHWA5xYMcIyPX+AzCZawjac/wD8EiwVkB+sd8BO84GyWIpWCLQw71lzjFu/ozTPJnxGD
XJ7gpt1lWZAGj+3Oe5FPRR+HeJu2yNO26RomkYKcEe4jBblBdvEcJchZwWmkIGcTd63gMkKQ
s4ejSMEyhpnb/PtfxdcARl8XG96wcqcAAAAASUVORK5CYII=}

image create photo fgauche  -data  {
iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAYAAABWdVznAAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAACJSURBVHja
lNIhDsIwFAbgbzPTGDSIYRAchovAKRAYJgnnAgGGeyAegpI0C2Ndk5rm/177XlpFhEkrIgyg
DTrMv5mIGAQrPBDYjYEW9xTuUP8DbVb51H92H2zxTOHjrz4jQp2dv9KGpnRKC1zTLWc+Bcea
ztEFTclYl7gltC8BsMYBsxxUU7/GewDUo6HnriVCegAAAABJRU5ErkJggg==}

image create photo fdroit -data {iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAYAAAB
WdVznAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wA
A/mIAADn4AAAQm00sP1EAAACGSURBVHjalNIxDsIwDIXhr5EQMwszDLAwcBPOAqdgBTbEuWC
AkzCZJUFVRUP7JMvL+23rJU1EGKWI+BZmOGLT5+sCBwQeWA4Bprhl6I7FPwASrl2oBhRdMvT
CrnhSJY937pNcP1MqOrWmr2onJZyz+VnMNWDfmrweEus8b9j2PVwz9mt8BgCCr6Dq5e/jJwA
AAABJRU5ErkJggg==}

image create photo fbas -data {iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAYAAABWd
VznAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/
mIAADn4AAAQm00sP1EAAACKSURBVHjatNEhFsIwEIThrxg0Bg2iGAQ3wXAROAUCAzge5wIBh
pssZukLpRUVrEledv9kMlNFhCE1MrAGA1LSFNueC8bYYRIRDXBC5NpWcM3evgRWeGbjWACXP
LtjWQKwwCMHDjjn/oZ5I79la1289Bmeff23I4carwRnPwb1BLfBusvR6u9JvwcAuq9GBcPoM
NgAAAAASUVORK5CYII=}

image create photo fhaut -data {iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAYAAABW
dVznAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA
/mIAADn4AAAQm00sP1EAAACBSURBVHjatNEhDsJQEIThrxg0Bl0EGAQ3wXAROAUCQ3GEc4Gg
hpss5olN04Y8wSabbDLzZ5LZJiLUzEztTCTscRj1jgAtenyw/gW0eCLK9hkaAqtkvuFS7jc2
Q2CLVzHcU+I1Je0ycC7CY6S5rmhdBhY4YT5R/RHLiND8/XHVwHcA0RdGBS0glsYAAAAASUVO
RK5CYII=}

image create photo g-expand -data {iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAA
ADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHnIAACA2AAA+xgAAH/3AAB6e
AAA+FcAADvgAAANTOIlrKwAAAGNSURBVHja7NXNSxZRFAbw3xu4CLSYkGyRUS1Mgz4kkBDaB
EVQazX6DwJJaFGr8g8oqEUQtA1yWbugaFlEBLnInRQhGdWiD03tg9vmTNyGeXmVeDfhA4dhn
jnPc+6dc89MI6WkndigzVgv0BoppT/xj+jEiVYFutCDrehGxyqMO1BgBxZxF73NCtxAwg98x
zxuY7DGeAh38BYrWA5twhIuYlcu2IQHWMBHvI7EUnQpy53M+G94FfEruC8Yx/ZSUGA6EyW8R
B/Ox45eRO6FeP4J57Al+M34iilsy1e+E/cq5mXcxwEcidc0GPwi9tc0ebiuSZNNzMu4muU+C
W5iLXNQtMjpiutZHMZzXF/L+T2O2Sarf4Ox6MXn4A5hAHsrPhuDr8XRJgVO4Qzex/3lyJ/GB
4xks9IZp+9atuu/cDCEC2E4kxmnEJa4lfHv8BTP8DO4OYyiuzpoNys7WMJDHKtZ0Ek8qsxKH
lcwUC3Qg37sifderKKHBXbHsV3GY+xr18fuNBo52Vj/Zf7/BX4PAMNasq33+hqHAAAAAElFT
kSuQmCC	}

image create photo compute -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAGwSURBVHja
7NU7aFRBFAbgbzWCGoRIfEDEB0SRiJVgZWGjaCepxMo+ilhooY29IiK+EDsFETsLG0GsLFII
omAKgyQGgvEJgi8kuTZn4XDZvXt3YTsPDHfmP2fmnzn/nLmNoij005bps/WdQFEUKtI0hGl8
wOae1m1BsAfjGMB5FNEuB3YY+3ol2IT5WHABi4lgKbAlfMfuOgQDJfxX6m8o+Rol7EcvIn/F
bBpP4zRO4HXC5/G+W5G34lJKyQK2pNBhvEv+29jejQZ30+QCD1vMu16KedyJIKdotsUVLdtw
aTzTbR3sxZd0a46n0CP4E76fOJB8g1hftw6epxQs4gUm8TfhbyJ2I3biJqYwhl1VBKP4Xcpz
uzaGU2kjTXyynQYwF8LN4E6Mm/YxsCk8iedjW/jO4iA+Yy1Gqt6i1VgT/f1pZ+OBrYzvscDn
sCqwW4Hdr9Ig2wpMRLEN4QKe4mQU6dVY8AwO4RPeYqQuQbaL8QY9Cp2OBlGlBnUJluMlrsX4
GR5EvkdxI7TZEeLr5QRX8A334gQTpTpYV7cO2tlgvFWvcK7T37C5buP/T7+T/RsAY80j2JC0
UDUAAAAASUVORK5CYII=}

image create photo misc -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAIgSURBVHja
1NZdaI5hGAfw38t8NIYT0xQSjaihmBwx047WRCjykXKElYM54mBHLJQoUZNGceTjxBJKOyIU
kciBfKWWr9LM59bj5Np6vL3vs6EduOqp++2+rv//vv73dV33m0uSxFDaMENsJekfuVwuy3cc
mjASh/C+mONvqiRJ0v9l2AhcQxLfHZRmEfRjDpJgZwDXYXGs9+T5jMXwgQjmoR1P4sRNmI2l
aE6BbcYSLEMrXuAnXmF9FsEzvMS5kKFPkvY4eZ/V437svcXpOMwNdGURdONoCmgyGvE4wC7h
eqw7sBqjU/770Z1FcBhfcAoVqcBRuJ3K6GqBeypHF9qyCLanQI5FYCVuoSfuYV/s38TcFMFl
fMbULIJVKYJtofsndMaF9llN9MFz5HAgYtYNpkxrsBxr8TXkmFhAkgrMQUuAN/5JH9SlMrmI
siJj5kT47MaMIByQoDqCzmMNvkXp1oYcMBN3w29DZN2DXjRkEYyPXngUnQlVeIgrUZJbopzf
hJRwMJVxcxbB3nCqKiDJgtQ8Ops3i6bH3ne0ZhF04kwe8KLo1ASvo1JqcSGymZDybclvtJIC
A6s8dK3ESszHB+zCyZBnRVRRW1RaR8hagx/FRysb8TFO2x1jYVPI05AK2xpgC2M83MM7PED9
QFVUhikYkwLcEaTVmBXrg8Uer795D0pjuvZG2T7N0/6fHxyYhCM4jmlZjmnM3H//r+LXACXj
0OduqNJDAAAAAElFTkSuQmCC}

image create photo input -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAE6SURBVHja
1JavTgNBEMZ/Q1sQNag2TTAkgCM8BAaBICGpAYfGYHmEvgQJpqICBJ4nIKAr6R9TUJgWmsFM
k+v1urPleoIvmezezszu3TffbE5UlSKxQcEopxdEpAXUAzkCPAId4Ah4SzoXGFHVOQOGgDrW
svQX4CK0XxZFXxFfPrZxE7gHzouqwbdR1gaaRRzwA0yBEnAHnLlFjsTUxhNgy+alxHruA/aB
Y2BiRU9SNvBU1I1Q0TIbxagoD8arUvQO9J2YOrA7I2TB61B0E/HWV4n4Xno/Sbe2iHSBPXsc
AZ/OAdtAzeZ9Vd1ZhaJn4MmaKQtqarqMvuxSGAKvTsxByOlRNDFth1CxOymTIq/It5YcsutQ
kT2KToGGU4PD4A5r7uReTCdXc3RyNUZFD0bLX/Dhqujf/VX8DgClz+mzCjMTrAAAAABJRU5E
rkJggg==}

image create photo savefile -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAHGSURBVHja
1NaxaxZBEAXw3yUBTQgqCKJi0ogRQSFgVCy0UgsL8U+InYWVhZWV1tqLoFjY21iolYWFKCoW
FiJJ5NMQSTAISgSJa7MXjv12l6RI4cDC3d7sm92ZeW+vCSHYTBuwyTbUPjRNU/IZxWmcwDga
fMUrPMdybtFaZkIICmnaimv4jFAYC7iB7bkAIYRigIN4UwFOx0ccW2+ACXzZAHg7lrtBWtym
BY81GMZLHElOdDvusmtbMunpYRLfW9yhZNGVDLhYhw+ZGv1J5sZwHVfXmqdzgtG4yz2ZAM/w
LdOBFzCSzP/EgRDCQtpF5yv5nSy0cK/gf6nF7aboZIUvD/AjQ9JdBf8p3E9rMFYJcA9vk7lh
PMTOjP/uPiZHhnbtb5xr8Bovku+DWI3Pq/G9D6urRfMJwG9Mx/y/z6kBzuAUZpNvi/2M42Km
WJ+wv5K6Bo8y6y7nmLwDSxnnHg5nwEfwOOO/gvGSVNwstN08DiUsflLwvVvTom2YqQSZiHUr
gS+1RK2J3VQUrpJqlsBXcHa9cn28cpLSvXBuI/eBSKBbldME/MId7CtdOKlc52xv3N3RyNCB
KHzv8BRztSuz+e//Kv4NAEi+SOzvHPFbAAAAAElFTkSuQmCC}

image create photo boule -data {
iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0
RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAACtSURBVDjL7dIxCwFxHMbxX8mm
DAa67tV4D7wDZTolMlzpn+QoouNlyMZ0Vyc5g1v/JQPdYlJGmb5egbpLNsOzfuqprwDyzeQP
fABc08A1FdOSZlLQODlFL2OkARThGGIfLhsIujxtUcmBWVETe3AcQNCGaMG9KTo5MMprTkvY
dsCzYD/kWk8D9LMK34ZoDqED6waHaooLL1uMR0vUzRJ9roneVUStymL8S/wR8AaM7e7IrixO
RwAAAABJRU5ErkJggg==
}

image create photo eye -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAGASURBVHja
7NUxSFdRFMfxzysHQxJECIk0orFCIh2kqcSCwqWtLZCGQJfmBhexpYYGF1cnqVHSElwUatIl
InCwoAykoKFSyG7L+cvl8X//lJCW/4UH7557zvlezvnde4uUksMcRxzyaAL+P6Cl9lMURZVP
BwbQh24cxSesYQWb9YL21JlSUiHV05jCd6Ts+12az+JiPUBKqRIwip+R4DXuoRfXMIx+3MZ8
BnqYl7wRYDoC1nEjbMfwAtux9jYgonTLYV9AWyNALfkztGfQ55jEGSyFz1d01VqIRxmktR5g
NBzmSuU6iV+xU3icleVOyfdJ2CfqAc7Hrj7iQhZ0CrvYwFPsZIC7mV979GsXV6tKdCVUsoXL
WfDLknISfqAn1k+EZBPG/tbkm1kz74fuO/EqS/4Z18N/CO/DPr5fmV7Cmwh6FzI9F6q6FevD
mUy3MXKQcwCteIAv2c6/leYJMzhbddCKWvIGV8VxDEZPeqJkm1jFIj40uiqK5ovWBPzz+DMA
Oh7pgMbKbGcAAAAASUVORK5CYII=}

image create photo book -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAFgSURBVHja
7NW9SxxRFAXw3zNr/OiWNYI2BiSNGGIRQQvBJmAR+0AglvEfsAjWIljZSZqkDqRNFQQrJZVh
2xBiEwJWFuJXos/m7jIMu4rCNmEPXGaGue/cO3fOeS/lnHUSPTqMboFbUWncpJSghlUc4QA/
8QOHbdbX8ATjeIxhbOB3Uzw5ZwUljSCX4gAfsVAgnsdWFC/nTxR5U4M8pfQG/4LsM3bwFDOY
DuJ9nGE2nr9jD3U8wzJeoz/n/KH8BV+wiXOslEYxEqO7iC7XMVbKWYp3a9hu8FYKCSdBAA9L
i//EwkUM4V2L/9EX17/BdS8V9SB1jdbxAimkeFec3ujkggKO8QqPwsn1iFNc4qqgtElMhZPn
QuIX7Qo8CEMN4nlEA8f4Gn4YwCe8QLVF07PobVXgG15iNwxTbmQuPCC2jXq520AV2815F7aK
22Y8irdhqPf4dVNyk7d7ZP7/Ba4HAPIUcL2BWpmpAAAAAElFTkSuQmCC}

image create photo s -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAIRSURBVHja
1NbNi09hFAfwzx2TqDEjiplJg40RC2W8lIWSvGRjvP0BFjQbK7FhoabGBhGmjJRIWQwiCUnZ
2CjEUklm0ORlxIjx0rU5P93Gvdf8pmbhqVvPPW/fc5/ne865SZqmxnPVGOdVW9kkSVJkU4eV
WI4WJHiFB7iHwTynPyeTpqmCY5qEvehDWvC8RSca8gDSNC0EaMXDTKDr2IX1WIcO9OJH6J9h
6WgB5qE/HG9gUckRt+Ji2A5mQYoAJuNJOPRUcZdd4fMS08oA9oThrTEQ5kL4HikCqMMbfAm2
VLua8AlDaMwD2BAZfMVVbMk4z8Ud7M/IpuMaTmbofiZibM8D6AxlX3zFT8wJxysZRq0I2aGM
bGvINsd7dx7A2VCuRnPs96EevyJgH85HsIHYP8bdkC0Ov8uVuLU5bSPBa9xHe1C2BsfQiFVR
2TPQHQkdwMRMVSd/FwQHA31HqHbG+7ugbvae+oMQEzA/U4w9sT+Vd0TtobwUwabgacg2ZnrX
zRGJCGp+z9xJRx7AVHwIFlUutyEqe2SPWpBD04X4iGG0FBVahUm9Y6iDo+F7uqyS6/E8DLuq
CL47c19N/2p2S4INKc5hdkngmTgett+wZrTtelnmS4aiQrcFeBs24QTeh80A1lYzDyqt4DA+
lwyc4aDmrKKBk1SCl4zM5siuLc63JjJ+hNt4UTYyk//+r+L3AFS3aQH+BubBAAAAAElFTkSu
QmCC}

image create photo r -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAAADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAAHISURBVHja
1NZPiE9RFAfwz5P8S5gsGMrSn7L8mdnZMU1pslCKhZUtTTRbZTY2LJVYKGVB2aopiYWSElYS
MkWN/C1mkHQtnKfb8+7P/MzMwqvTe+/77vl+7z33nHNflVKykNciC3wtrh+qqiqNWYmdGMQm
VHiFe7iN921OvyOTUlII0zKMYRKpYFMYx+o2gZRSUWAL7nchbtoT7JitwGa87IG8tg+5SElg
OR79A3ltk+jrJjA2B/LaTpcE+vB5HgQ+YX3Nm9fBDxzCcbxtbPp5DOFKA3+MERzBTJbWw3/u
9q8VrA14KJvRjQbpg8C/YXuGj2Y+Z9tCdAFfcTIcbsXgXSF+F0uwP/CrMe4y9mFFZFLCtTaB
S/HxXRAdwMdoJ4fj2wiWRjiGo8ASJrJQJlxvEziVLfFgEHfC8WHgN+N9MO7HAv+ObRH/Uexp
E9ibCbzA1iA50ciSo4EPxGpr/E70qmKhrYnsqR2m8ayQik9jk5v4TPj0lwptfB7q4GK3Sl6F
53Mgf4P+vzW7TpZuvdiXSOlZteuBHlcyhd29nAeiqs9ELZSIp3EOG0sHTlWTdzkyN8TsOlgX
9fE6WsZEpHTxyKz++7+KnwMACPPIdwvdDUAAAAAASUVORK5CYII=}

image create photo target -data {
iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAYAAABWdVznAAAACXBIWXMAAAsTAAALEwEAmpwY
AAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wAA/mIAADn4AAAQm00sP1EAAADfSURBVHja
lNE/K8VRGAfwz082I2aUG7omSVcGZVIWL4EymNy8ATOb4Q4yoczXG5CBVZFFZPKnZLGZ9Fie
X053u890zun7qe85p4oI/cwgVFVV7+exiVl84gJd/EBEqCKiBi0c4xsvGMY0NnJ9GxEfstIE
XtHuaTCGfQQ6EaEGHZxhAIsF2M3wHaZKcI1VHOAX69jJ8D3WMFKCmwQtfGUw8IAGrjBXgiOc
ZI2FvPgzRrGCJwyVoIF3bCeawTiaeKvPSwDLeMQlDnGe4b060PsPsIQtTGatLk7zIf5BP/M3
AKaDav5zj2dDAAAAAElFTkSuQmCC}

image create photo tresholdbar -data {iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAYAAAB
WdVznAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wA
A/mIAADn4AAAQm00sP1EAAACGSURBVHjalNIxDsIwDIXhr5EQMwszDLAwcBPOAqdgBTbEuWC
AkzCZJUFVRUP7JMvL+23rJU1EGKWI+BZmOGLT5+sCBwQeWA4Bprhl6I7FPwASrl2oBhRdMvT
CrnhSJY937pNcP1MqOrWmr2onJZyz+VnMNWDfmrweEus8b9j2PVwz9mt8BgCCr6Dq5e/jJwA
AAABJRU5ErkJggg==}

image create photo fgcoloronthefly -data {iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAYAAAB
WdVznAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHo/AACAbwAA/GAAAIFJAABt+wA
A/mIAADn4AAAQm00sP1EAAACGSURBVHjalNIxDsIwDIXhr5EQMwszDLAwcBPOAqdgBTbEuWC
AkzCZJUFVRUP7JMvL+23rJU1EGKWI+BZmOGLT5+sCBwQeWA4Bprhl6I7FPwASrl2oBhRdMvT
CrnhSJY937pNcP1MqOrWmr2onJZyz+VnMNWDfmrweEus8b9j2PVwz9mt8BgCCr6Dq5e/jJwA
AAABJRU5ErkJggg==}

image create photo root -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAA
ADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHnIAACA2AAA+xgAAH/3AAB6e
AAA+FcAADvgAAANTOIlrKwAAAGNSURBVHja7NXNSxZRFAbw3xu4CLSYkGyRUS1Mgz4kkBDaB
EVQazX6DwJJaFGr8g8oqEUQtA1yWbugaFlEBLnInRQhGdWiD03tg9vmTNyGeXmVeDfhA4dhn
jnPc+6dc89MI6WkndigzVgv0BoppT/xj+jEiVYFutCDrehGxyqMO1BgBxZxF73NCtxAwg98x
zxuY7DGeAh38BYrWA5twhIuYlcu2IQHWMBHvI7EUnQpy53M+G94FfEruC8Yx/ZSUGA6EyW8R
B/Ox45eRO6FeP4J57Al+M34iilsy1e+E/cq5mXcxwEcidc0GPwi9tc0ebiuSZNNzMu4muU+C
W5iLXNQtMjpiutZHMZzXF/L+T2O2Sarf4Ox6MXn4A5hAHsrPhuDr8XRJgVO4Qzex/3lyJ/GB
4xks9IZp+9atuu/cDCEC2E4kxmnEJa4lfHv8BTP8DO4OYyiuzpoNys7WMJDHKtZ0Ek8qsxKH
lcwUC3Qg37sifderKKHBXbHsV3GY+xr18fuNBo52Vj/Zf7/BX4PAMNasq33+hqHAAAAAElFT
kSuQmCC}

image create photo scenariobar -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAA
ADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHnIAACA2AAA+xgAAH/3AAB6e
AAA+FcAADvgAAANTOIlrKwAAAGNSURBVHja7NXNSxZRFAbw3xu4CLSYkGyRUS1Mgz4kkBDaB
EVQazX6DwJJaFGr8g8oqEUQtA1yWbugaFlEBLnInRQhGdWiD03tg9vmTNyGeXmVeDfhA4dhn
jnPc+6dc89MI6WkndigzVgv0BoppT/xj+jEiVYFutCDrehGxyqMO1BgBxZxF73NCtxAwg98x
zxuY7DGeAh38BYrWA5twhIuYlcu2IQHWMBHvI7EUnQpy53M+G94FfEruC8Yx/ZSUGA6EyW8R
B/Ox45eRO6FeP4J57Al+M34iilsy1e+E/cq5mXcxwEcidc0GPwi9tc0ebiuSZNNzMu4muU+C
W5iLXNQtMjpiutZHMZzXF/L+T2O2Sarf4Ox6MXn4A5hAHsrPhuDr8XRJgVO4Qzex/3lyJ/GB
4xks9IZp+9atuu/cDCEC2E4kxmnEJa4lfHv8BTP8DO4OYyiuzpoNys7WMJDHKtZ0Ek8qsxKH
lcwUC3Qg37sifderKKHBXbHsV3GY+xr18fuNBo52Vj/Zf7/BX4PAMNasq33+hqHAAAAAElFT
kSuQmCC}

image create photo scenariobarpupko -data {
iVBORw0KGgoAAAANSUhEUgAAABgAAAAYCAYAA
ADgdz34AAAACXBIWXMAAAsTAAALEwEAmpwYAAAAIGNIUk0AAHnIAACA2AAA+xgAAH/3AAB6e
AAA+FcAADvgAAANTOIlrKwAAAGNSURBVHja7NXNSxZRFAbw3xu4CLSYkGyRUS1Mgz4kkBDaB
EVQazX6DwJJaFGr8g8oqEUQtA1yWbugaFlEBLnInRQhGdWiD03tg9vmTNyGeXmVeDfhA4dhn
jnPc+6dc89MI6WkndigzVgv0BoppT/xj+jEiVYFutCDrehGxyqMO1BgBxZxF73NCtxAwg98x
zxuY7DGeAh38BYrWA5twhIuYlcu2IQHWMBHvI7EUnQpy53M+G94FfEruC8Yx/ZSUGA6EyW8R
B/Ox45eRO6FeP4J57Al+M34iilsy1e+E/cq5mXcxwEcidc0GPwi9tc0ebiuSZNNzMu4muU+C
W5iLXNQtMjpiutZHMZzXF/L+T2O2Sarf4Ox6MXn4A5hAHsrPhuDr8XRJgVO4Qzex/3lyJ/GB
4xks9IZp+9atuu/cDCEC2E4kxmnEJa4lfHv8BTP8DO4OYyiuzpoNys7WMJDHKtZ0Ek8qsxKH
lcwUC3Qg37sifderKKHBXbHsV3GY+xr18fuNBo52Vj/Zf7/BX4PAMNasq33+hqHAAAAAElFT
kSuQmCC}

image create photo ancestralannotation -data {
iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAABXUlEQVRYR+2WDQ3CMBCFNyXM
CUMBOGAOwAHDATgABTAHzAESQAncI21ya9b21lxDQmjSQOjY++63Vxb514okNrRrI3Wjzz1t
fBZlZv0TvX/t0QBEmxMAll8iBi5yAsDF8whAnxPgJQnvTwI0ZPmOdiXwQKfpgSnClk0lCX3C
T1K5017mKsOQcEui6AFYNe0tA+lR/7RRJUmNSCosSIEhwFjLPNJbruZNqsKWziYh3OVrmTiD
GyvHJMSYu1pksfsQACQtk/9PRZh7AMkQa5l4XlWYA4haZmLCRsOCEHwdQBqCh0m6c9SsCQ+k
JKEqiC3DA0FjbBpbsBhlOHMOVUD4ZQQR1LWtiI6+AwwhwmrMuSpIym2oCpICYCMhBck+FYdA
cB0DYGypT8U+EI/+52eVgcQVmALyn4qTJqJQTPkZ+kfsllWdil0wyZyRJQk5SKjFq5ehLzS1
aeE2HIOp+A0zjFX9k2v+uQAAAABJRU5ErkJggg==
}

image create photo inout -data {
iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAABNElEQVRYR+2XYRHCMAyFmQIk
MByAAxxgARQACgAHSBgOkAAOsIADHEDe7sp13bqkGaF/2F3u1ibt+5q00BWj5nOi5pKsDPpT
mlcKvpCdyZ7cwMILgPiGG5Dgh/iOrOob4wNgwDhBQBoKgHUs2Ad4SWdUxEUh+gB8n0TTLeBI
wduObHZCWABgzhkZNmNY0haEFQAyFoOYk+/uUmoJEIPAZp+S1UfUGiAGgeOJY/8TAOgcyPbe
Tr7R+8IKQHJiPtoWJcgGwAmHP3T14r+ZgT+AWQaG/km5Mqv3QHYALrVSvzoDUgEuTg2QvQTZ
AbjUSv3qEkgFuDg1QPYSZAfgUiv1q0sgFeDikgG4CYf6W/cBq0+zLtAHdZbhhSS8OA5dYd94
fD1Br3EjQhudK7KJkTpWXjlxaLwBzFFWIWWIvDkAAAAASUVORK5CYII=
}

image create photo transitions -data {
iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAACHElEQVRYR8WWjVHDMAxG2wlg
A8IEwASECYAJCBPABsAGbEA7AbBB2QA2CBuwAejlrJ7iOLbz0+I7XetE0fcsK4qXi92PlUhU
fTLL3esvfkUDiNuQ1r4A0A5C7BMgCLFvgA7EfwC0IACgSBgbsR+xT7F39zukRk/F+VKM30Ox
MvFwUxMWwPcH5FlsnQhUyf07JzwEuMlEDECDkRleodqLXsj8JWOlMah1DgAB2JoLMbKiA6Cj
oUs2/mS2skVYygVWxe9NIDAQxw6G21dir54ftfMmBhywPKM1Zl0bcS70vQWArMTOPYGNzMmE
Dub4IHzvhL1HOgBb8RiABgHCzwb1wHUGFY9w5auauc1ASzwHAB9dpcb0tyKi3dxSgI54LkAh
juzngVF6kv+PKWV3H4CgeC4Afog9GMEhWVjFtii3FdPZ6pFZmAWgLwtnDiy2G7OdB0JZoEDt
axkC0SIkE51DSWoLeIjvgXbASv7Tfu0IBjYO9jXs+MYakfZ5f5XMQw0q9L2Aw++ELQgFIL0n
YjSWUow2a8e1TGixjNBWqC/BAazFPtzFUCveQuR+jL4lWGGIAEXI9gaPOTltIHIAWEnlVmWj
AkQQfzuSysYheh74EkcKEJHYAI7vAVs4dLTOA6xUj2Tst/325wRmW6gdfqmTVGY654EckTE+
o84DY4T6nvEBBp0H5gCZfB6YCjH5PDAHwOTzwBQIXmNe1eD4AyR8g5+LFmlBAAAAAElFTkSu
QmCC
}
image create photo edition -data {
iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAACO0lEQVRYR+2W3VHDMBCESQVA
BTgVABVgKgAqwFQAVICpgFABoQKgApwOoALcAVAB7OfRMRdFjm1e8uKb2Yn+7m51WsmZbG3Y
JhvOvzUSGCswpAKZBHsk5ALtA2EniPhLv29CLVTCIrQ7Nd6HwKminAv8DrFnLX4U+G21dQTY
6U3Y8ZDE8VoqchsqsxKnjUAZkqcSf4fB7YGsiAmRJYsJcKZPLbsmMcfAjjAqRHmHEGH9hYBm
GvMESP4qIC4zxIQhPhznbo5mITxEY77r/W0csR4bCU8AdifOm3KVQi3sCbvm5NZA+nMNAXyn
IQ56MiPXGR0jcKX2nVvgd/sTxglEQG+ZOh9rCPgchTq+WtfqzyAQB7nXGITMjEA8zvxMuOxJ
ILV+CoFK4Iyxd8FrgDE/T+lIikGy621AA3lEEA3sh7EqJsDkYeRQqu/Pr2PDS9OmIz/oCSz6
HEEmb5yGXDcScm2pZu2yx0fWHIGVs02Eqfk+VWhE5hYWaidFaGvia0jp/cs1V59vQh/jG0BC
s3jnf/PxQ1TJwwSCM32uZB0iEZRgbcdB2REnZLFMYNe5I4PQ6TevYeopphJ2K8yvVAPWEOHx
Qf2ANkYw/ABtElMt/LwRA4JNciwmYOM4ppRPAgRZCezEAkGEyuUCwktdz9SNWPunlGAQiasR
baqzy1tAHEivWFsF/EJ2Uwj+O9GZVQtehLlA1VqtDwFzztSgKoA2pTYxIj6Opg47Zbe0O20I
gc5g/1kwEhgrsPEK/AKKLnbg9FzF+AAAAABJRU5ErkJggg==
}
image create photo compare -data {
iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAACDUlEQVRYR+2X4U0DMQyF6QSw
ATABMAGwAUxA2QAmgQ3oBrABZQLaDcoGZQJ473Q+fI7jJKci8YOTLF0Tn9+X2EmT2d7Pc4rX
B9g1bKvad/l6gGDPsHvYioFnfXSKv8LowI7LX4BgbGpQiwOkxooAWlxGW4JYwHFupsZrExct
Lm0dBAGWsHNnnnMQFLpRsyeffuGFfbcmlicuLm8EoAMhTiogRJyukj4NwHcNEYmv4XshQWog
HvuRi2AOQCBYaJJzO7ZOHLbVQSII5ov9+okA6Od9w/ZB3JvGCMKOogTgZHQs7gGwrRaiFWA0
8lwepZ0QG9i+N4y+rQWA6Tju0xLmUToXeOFSi54WAMZhTLtEk6UkjiVxL33cB0pPAmFHUTPy
XPpqAJKZ0AAt4lNnQKe4S4edgdIU7rz/H+BPzcAcCX5qSHLrPqBDswBZ9EkRtkBMBRjEc6ug
FmIKwEg8WoY1EK0AiXgEwD+jd9hRUBMtABvEOYMlp21vFUTHKM3TAsDv3DOmDVIrPnUrTiDs
kSx3hvuA4qFJR2kGvG+SmZAgxdMrvrwy+0QEwIJ7gS1h4WmbQWrEpXj06sgB6GovnrYZhJTe
xcQ9w8FXIDwAb6lFEN3FhFczQujzX05cyoAQC1MTXpu4eBCf6BwuJhqiJG50q39qiE4c1l1O
5SEEbz8stmTDqJaJHQnB4ryjOF2/AXeCjiGIBTexAAAAAElFTkSuQmCC
}
image create photo compare -data {
iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAYAAABzenr0AAABbUlEQVRYR+2WbRHCMAyGmQMc
MAdIYE7AATgAHIADcDIk4AAcIIG8u+YulHRNunH7Q+9yXbu179N89FbNJm7VxPqzIQANwe/D
AY7UtyWHKQFgYfSyAcAN4gHQhJ+BYCFIXCAWgJTwgUQvQXhDPcZukD4Ai3AcdjeIBlAiXAwi
AcYQdoMA4BfCZhAGQAKtxKpHSKprvFPhGLmBO6MW62/QiEMwNkhSmMRbwKSScChIVpg9kStD
L4hZWANAMi7JEPeXiBXmcyBW4TnttSa7yxBAAAmCHg3iJ7KzEQRragHcJRcLhHkIb8l2ZHhG
a8mOqSrwguB7qzCzmqrAAoJvLCf+EGYPWaugD0R4v3Nv7GpVmCe9VZACcQtrAA1NWqtAguBZ
JlffiUevAun+nPC/Cv5VgJvvq1l/SnHRyP+F0iooAuBFKFMNJN5Uu5K1w3dzFg/Ei1MgLmHL
TZikDi8YBEN4Ro1xbpMSD+T2dL2fHOANctGxoTs00gYAAAAASUVORK5CYII=
}
################################################################################


################################################################################
# GO
################################################################################
set S(pathuser) [file dirname [info script]] 
set S(pathuser) [pwd] 
set S(SCRIPTdir) [file dirname [info script]]
ttk::style theme use clam
proc bgerror {m} {}
MainPanel
puts $S(version)
################################################################################